@@ -17,38 +17,50 @@ import (
 
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
+	"github.com/juju/names"
 	"github.com/juju/utils"
 
+	"github.com/juju/juju/api"
+	"github.com/juju/juju/apiserver/params"
+	"github.com/juju/juju/cloudinit"
+	"github.com/juju/juju/juju/paths"
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
-	"github.com/juju/juju/state"
-	"github.com/juju/juju/state/api"
-	"github.com/juju/juju/state/api/params"
+	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/version"
 )
 
 var logger = loggo.GetLogger("juju.agent")
 
+// logDir returns a filesystem path to the location where juju
+// may create a folder containing its logs
+var logDir = paths.MustSucceed(paths.LogDir(version.Current.Series))
+
+// dataDir returns the default data directory for this running system
+var dataDir = paths.MustSucceed(paths.DataDir(version.Current.Series))
+
 // DefaultLogDir defines the default log directory for juju agents.
 // It's defined as a variable so it could be overridden in tests.
-var DefaultLogDir = "/var/log/juju"
+var DefaultLogDir = path.Join(logDir, "juju")
 
 // DefaultDataDir defines the default data directory for juju agents.
 // It's defined as a variable so it could be overridden in tests.
-var DefaultDataDir = "/var/lib/juju"
+var DefaultDataDir = dataDir
 
 // SystemIdentity is the name of the file where the environment SSH key is kept.
 const SystemIdentity = "system-identity"
 
 const (
-	LxcBridge        = "LXC_BRIDGE"
-	ProviderType     = "PROVIDER_TYPE"
-	ContainerType    = "CONTAINER_TYPE"
-	Namespace        = "NAMESPACE"
-	StorageDir       = "STORAGE_DIR"
-	StorageAddr      = "STORAGE_ADDR"
-	AgentServiceName = "AGENT_SERVICE_NAME"
-	MongoOplogSize   = "MONGO_OPLOG_SIZE"
+	LxcBridge              = "LXC_BRIDGE"
+	ProviderType           = "PROVIDER_TYPE"
+	ContainerType          = "CONTAINER_TYPE"
+	Namespace              = "NAMESPACE"
+	StorageDir             = "STORAGE_DIR"
+	StorageAddr            = "STORAGE_ADDR"
+	AgentServiceName       = "AGENT_SERVICE_NAME"
+	MongoOplogSize         = "MONGO_OPLOG_SIZE"
+	NumaCtlPreference      = "NUMA_CTL_PREFERENCE"
+	AllowsSecureConnection = "SECURE_STATESERVER_CONNECTION"
 )
 
 // The Config interface is the sole way that the agent gets access to the
@@ -75,11 +87,11 @@ type Config interface {
 	SystemIdentityPath() string
 
 	// Jobs returns a list of MachineJobs that need to run.
-	Jobs() []params.MachineJob
+	Jobs() []multiwatcher.MachineJob
 
 	// Tag returns the tag of the entity on whose behalf the state connection
 	// will be made.
-	Tag() string
+	Tag() names.Tag
 
 	// Dir returns the agent's directory.
 	Dir() string
@@ -98,7 +110,7 @@ type Config interface {
 	// WriteCommands returns shell commands to write the agent configuration.
 	// It returns an error if the configuration does not have all the right
 	// elements.
-	WriteCommands() ([]string, error)
+	WriteCommands(series string) ([]string, error)
 
 	// StateServingInfo returns the details needed to run
 	// a state server and reports whether those details
@@ -108,9 +120,9 @@ type Config interface {
 	// APIInfo returns details for connecting to the API server.
 	APIInfo() *api.Info
 
-	// StateInfo returns details for connecting to the state server and reports
-	// whether those details are available
-	StateInfo() (*state.Info, bool)
+	// MongoInfo returns details for connecting to the state server's mongo
+	// database and reports whether those details are available
+	MongoInfo() (*mongo.MongoInfo, bool)
 
 	// OldPassword returns the fallback password when connecting to the
 	// API server.
@@ -123,6 +135,10 @@ type Config interface {
 	// Value returns the value associated with the key, or an empty string if
 	// the key is not found.
 	Value(key string) string
+
+	// PreferIPv6 returns whether to prefer using IPv6 addresses (if
+	// available) when connecting to the state or API server.
+	PreferIPv6() bool
 }
 
 type ConfigSetterOnly interface {
@@ -143,7 +159,7 @@ type ConfigSetterOnly interface {
 	// SetValue updates the value for the specified key.
 	SetValue(key, value string)
 
-	// SetUpgradedToVerson sets the version that
+	// SetUpgradedToVersion sets the version that
 	// the agent has successfully upgraded to.
 	SetUpgradedToVersion(newVersion version.Number)
 
@@ -193,7 +209,7 @@ type ConfigSetterWriter interface {
 type MigrateParams struct {
 	DataDir      string
 	LogDir       string
-	Jobs         []params.MachineJob
+	Jobs         []multiwatcher.MachineJob
 	DeleteValues []string
 	Values       map[string]string
 }
@@ -219,9 +235,9 @@ type configInternal struct {
 	configFilePath    string
 	dataDir           string
 	logDir            string
-	tag               string
+	tag               names.Tag
 	nonce             string
-	jobs              []params.MachineJob
+	jobs              []multiwatcher.MachineJob
 	upgradedToVersion version.Number
 	caCert            string
 	stateDetails      *connectionDetails
@@ -229,20 +245,22 @@ type configInternal struct {
 	oldPassword       string
 	servingInfo       *params.StateServingInfo
 	values            map[string]string
+	preferIPv6        bool
 }
 
 type AgentConfigParams struct {
 	DataDir           string
 	LogDir            string
-	Jobs              []params.MachineJob
+	Jobs              []multiwatcher.MachineJob
 	UpgradedToVersion version.Number
-	Tag               string
+	Tag               names.Tag
 	Password          string
 	Nonce             string
 	StateAddresses    []string
 	APIAddresses      []string
 	CACert            string
 	Values            map[string]string
+	PreferIPv6        bool
 }
 
 // NewAgentConfig returns a new config object suitable for use for a
@@ -255,9 +273,15 @@ func NewAgentConfig(configParams AgentConfigParams) (ConfigSetterWriter, error)
 	if configParams.LogDir != "" {
 		logDir = configParams.LogDir
 	}
-	if configParams.Tag == "" {
+	if configParams.Tag == nil {
 		return nil, errors.Trace(requiredError("entity tag"))
 	}
+	switch configParams.Tag.(type) {
+	case names.MachineTag, names.UnitTag:
+		// these are the only two type of tags that can represent an agent
+	default:
+		return nil, errors.Errorf("entity tag must be MachineTag or UnitTag, got %T", configParams.Tag)
+	}
 	if configParams.UpgradedToVersion == version.Zero {
 		return nil, errors.Trace(requiredError("upgradedToVersion"))
 	}
@@ -279,6 +303,7 @@ func NewAgentConfig(configParams AgentConfigParams) (ConfigSetterWriter, error)
 		caCert:            configParams.CACert,
 		oldPassword:       configParams.Password,
 		values:            configParams.Values,
+		preferIPv6:        configParams.PreferIPv6,
 	}
 	if len(configParams.StateAddresses) > 0 {
 		config.stateDetails = &connectionDetails{
@@ -309,6 +334,9 @@ func NewStateMachineConfig(configParams AgentConfigParams, serverInfo params.Sta
 	if serverInfo.PrivateKey == "" {
 		return nil, errors.Trace(requiredError("state server key"))
 	}
+	if serverInfo.CAPrivateKey == "" {
+		return nil, errors.Trace(requiredError("ca cert key"))
+	}
 	if serverInfo.StatePort == 0 {
 		return nil, errors.Trace(requiredError("state port"))
 	}
@@ -324,17 +352,17 @@ func NewStateMachineConfig(configParams AgentConfigParams, serverInfo params.Sta
 }
 
 // Dir returns the agent-specific data directory.
-func Dir(dataDir, agentName string) string {
+func Dir(dataDir string, tag names.Tag) string {
 	// Note: must use path, not filepath, as this
 	// function is used by the client on Windows.
-	return path.Join(dataDir, "agents", agentName)
+	return path.Join(dataDir, "agents", tag.String())
 }
 
 // ConfigPath returns the full path to the agent config file.
 // NOTE: Delete this once all agents accept --config instead
 // of --data-dir - it won't be needed anymore.
-func ConfigPath(dataDir, agentName string) string {
-	return filepath.Join(Dir(dataDir, agentName), agentConfigFilename)
+func ConfigPath(dataDir string, tag names.Tag) string {
+	return filepath.Join(Dir(dataDir, tag), agentConfigFilename)
 }
 
 // ReadConfig reads configuration data from the given location.
@@ -397,7 +425,7 @@ func (c0 *configInternal) Clone() Config {
 	// by ConfigSetter methods.
 	c1.stateDetails = c0.stateDetails.clone()
 	c1.apiDetails = c0.apiDetails.clone()
-	c1.jobs = append([]params.MachineJob{}, c0.jobs...)
+	c1.jobs = append([]multiwatcher.MachineJob{}, c0.jobs...)
 	c1.values = make(map[string]string, len(c0.values))
 	for key, val := range c0.values {
 		c1.values[key] = val
@@ -414,7 +442,7 @@ func (config *configInternal) Migrate(newParams MigrateParams) error {
 		config.logDir = newParams.LogDir
 	}
 	if len(newParams.Jobs) > 0 {
-		config.jobs = make([]params.MachineJob, len(newParams.Jobs))
+		config.jobs = make([]multiwatcher.MachineJob, len(newParams.Jobs))
 		copy(config.jobs, newParams.Jobs)
 	}
 	for _, key := range newParams.DeleteValues {
@@ -504,7 +532,7 @@ func (c *configInternal) SystemIdentityPath() string {
 	return filepath.Join(c.dataDir, SystemIdentity)
 }
 
-func (c *configInternal) Jobs() []params.MachineJob {
+func (c *configInternal) Jobs() []multiwatcher.MachineJob {
 	return c.jobs
 }
 
@@ -524,6 +552,10 @@ func (c *configInternal) Value(key string) string {
 	return c.values[key]
 }
 
+func (c *configInternal) PreferIPv6() bool {
+	return c.preferIPv6
+}
+
 func (c *configInternal) StateServingInfo() (params.StateServingInfo, bool) {
 	if c.servingInfo == nil {
 		return params.StateServingInfo{}, false
@@ -546,7 +578,7 @@ func (c *configInternal) OldPassword() string {
 	return c.oldPassword
 }
 
-func (c *configInternal) Tag() string {
+func (c *configInternal) Tag() names.Tag {
 	return c.tag
 }
 
@@ -596,13 +628,17 @@ func (c *configInternal) fileContents() ([]byte, error) {
 	return buf.Bytes(), nil
 }
 
-func (c *configInternal) WriteCommands() ([]string, error) {
+func (c *configInternal) WriteCommands(series string) ([]string, error) {
+	renderer, err := cloudinit.NewRenderer(series)
+	if err != nil {
+		return nil, err
+	}
 	data, err := c.fileContents()
 	if err != nil {
 		return nil, err
 	}
-	commands := []string{"mkdir -p " + utils.ShQuote(c.Dir())}
-	commands = append(commands, writeFileCommands(c.File(agentConfigFilename), data, 0600)...)
+	commands := renderer.Mkdir(c.Dir())
+	commands = append(commands, renderer.WriteFile(c.File(agentConfigFilename), string(data), 0600)...)
 	return commands, nil
 }
 
@@ -611,16 +647,19 @@ func (c *configInternal) APIInfo() *api.Info {
 	addrs := c.apiDetails.addresses
 	if isStateServer {
 		port := servingInfo.APIPort
-		localApiAddr := fmt.Sprintf("localhost:%d", port)
+		localAPIAddr := net.JoinHostPort("localhost", strconv.Itoa(port))
+		if c.preferIPv6 {
+			localAPIAddr = net.JoinHostPort("::1", strconv.Itoa(port))
+		}
 		addrInAddrs := false
 		for _, addr := range addrs {
-			if addr == localApiAddr {
+			if addr == localAPIAddr {
 				addrInAddrs = true
 				break
 			}
 		}
 		if !addrInAddrs {
-			addrs = append(addrs, localApiAddr)
+			addrs = append(addrs, localAPIAddr)
 		}
 	}
 	return &api.Info{
@@ -632,13 +671,16 @@ func (c *configInternal) APIInfo() *api.Info {
 	}
 }
 
-func (c *configInternal) StateInfo() (info *state.Info, ok bool) {
+func (c *configInternal) MongoInfo() (info *mongo.MongoInfo, ok bool) {
 	ssi, ok := c.StateServingInfo()
 	if !ok {
 		return nil, false
 	}
 	addr := net.JoinHostPort("127.0.0.1", strconv.Itoa(ssi.StatePort))
-	return &state.Info{
+	if c.preferIPv6 {
+		addr = net.JoinHostPort("::1", strconv.Itoa(ssi.StatePort))
+	}
+	return &mongo.MongoInfo{
 		Info: mongo.Info{
 			Addrs:  []string{addr},
 			CACert: c.caCert,
