@@ -1,21 +1,24 @@
-// Copyright 2013 Canonical Ltd.
+// Copyright 2014 Canonical Ltd.
+// Copyright 2014 Cloudbase Solutions SRL
 // Licensed under the AGPLv3, see LICENCE file for details.
 
 package agent_test
 
 import (
 	"fmt"
+	"path/filepath"
 	"reflect"
 
+	"github.com/juju/names"
 	jc "github.com/juju/testing/checkers"
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/agent"
+	"github.com/juju/juju/api"
+	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
-	"github.com/juju/juju/state"
-	"github.com/juju/juju/state/api"
-	"github.com/juju/juju/state/api/params"
+	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/testing"
 	"github.com/juju/juju/version"
 )
@@ -44,14 +47,14 @@ var agentConfigTests = []struct {
 	about: "missing upgraded to version",
 	params: agent.AgentConfigParams{
 		DataDir: "/data/dir",
-		Tag:     "omg",
+		Tag:     names.NewMachineTag("1"),
 	},
 	checkErr: "upgradedToVersion not found in configuration",
 }, {
 	about: "missing password",
 	params: agent.AgentConfigParams{
 		DataDir:           "/data/dir",
-		Tag:               "omg",
+		Tag:               names.NewMachineTag("1"),
 		UpgradedToVersion: version.Current.Number,
 	},
 	checkErr: "password not found in configuration",
@@ -59,7 +62,7 @@ var agentConfigTests = []struct {
 	about: "missing CA cert",
 	params: agent.AgentConfigParams{
 		DataDir:           "/data/dir",
-		Tag:               "omg",
+		Tag:               names.NewMachineTag("1"),
 		UpgradedToVersion: version.Current.Number,
 		Password:          "sekrit",
 	},
@@ -68,7 +71,7 @@ var agentConfigTests = []struct {
 	about: "need either state or api addresses",
 	params: agent.AgentConfigParams{
 		DataDir:           "/data/dir",
-		Tag:               "omg",
+		Tag:               names.NewMachineTag("1"),
 		UpgradedToVersion: version.Current.Number,
 		Password:          "sekrit",
 		CACert:            "ca cert",
@@ -78,7 +81,7 @@ var agentConfigTests = []struct {
 	about: "invalid state address",
 	params: agent.AgentConfigParams{
 		DataDir:           "/data/dir",
-		Tag:               "omg",
+		Tag:               names.NewMachineTag("1"),
 		UpgradedToVersion: version.Current.Number,
 		Password:          "sekrit",
 		CACert:            "ca cert",
@@ -89,7 +92,7 @@ var agentConfigTests = []struct {
 	about: "invalid api address",
 	params: agent.AgentConfigParams{
 		DataDir:           "/data/dir",
-		Tag:               "omg",
+		Tag:               names.NewMachineTag("1"),
 		UpgradedToVersion: version.Current.Number,
 		Password:          "sekrit",
 		CACert:            "ca cert",
@@ -100,7 +103,7 @@ var agentConfigTests = []struct {
 	about: "good state addresses",
 	params: agent.AgentConfigParams{
 		DataDir:           "/data/dir",
-		Tag:               "omg",
+		Tag:               names.NewMachineTag("1"),
 		UpgradedToVersion: version.Current.Number,
 		Password:          "sekrit",
 		CACert:            "ca cert",
@@ -110,7 +113,7 @@ var agentConfigTests = []struct {
 	about: "good api addresses",
 	params: agent.AgentConfigParams{
 		DataDir:           "/data/dir",
-		Tag:               "omg",
+		Tag:               names.NewMachineTag("1"),
 		UpgradedToVersion: version.Current.Number,
 		Password:          "sekrit",
 		CACert:            "ca cert",
@@ -120,7 +123,7 @@ var agentConfigTests = []struct {
 	about: "both state and api addresses",
 	params: agent.AgentConfigParams{
 		DataDir:           "/data/dir",
-		Tag:               "omg",
+		Tag:               names.NewMachineTag("1"),
 		UpgradedToVersion: version.Current.Number,
 		Password:          "sekrit",
 		CACert:            "ca cert",
@@ -131,7 +134,7 @@ var agentConfigTests = []struct {
 	about: "everything...",
 	params: agent.AgentConfigParams{
 		DataDir:           "/data/dir",
-		Tag:               "omg",
+		Tag:               names.NewMachineTag("1"),
 		Password:          "sekrit",
 		UpgradedToVersion: version.Current.Number,
 		CACert:            "ca cert",
@@ -143,7 +146,7 @@ var agentConfigTests = []struct {
 	about: "missing logDir sets default",
 	params: agent.AgentConfigParams{
 		DataDir:           "/data/dir",
-		Tag:               "omg",
+		Tag:               names.NewMachineTag("1"),
 		Password:          "sekrit",
 		UpgradedToVersion: version.Current.Number,
 		CACert:            "ca cert",
@@ -154,15 +157,71 @@ var agentConfigTests = []struct {
 	inspectConfig: func(c *gc.C, cfg agent.Config) {
 		c.Check(cfg.LogDir(), gc.Equals, agent.DefaultLogDir)
 	},
+}, {
+	about: "agentConfig must not be a User tag",
+	params: agent.AgentConfigParams{
+		DataDir:           "/data/dir",
+		Tag:               names.NewUserTag("admin"), // this is a joke, the admin user is nil.
+		UpgradedToVersion: version.Current.Number,
+		Password:          "sekrit",
+	},
+	checkErr: "entity tag must be MachineTag or UnitTag, got names.UserTag",
+}, {
+	about: "agentConfig accepts a Unit tag",
+	params: agent.AgentConfigParams{
+		DataDir:           "/data/dir",
+		Tag:               names.NewUnitTag("ubuntu/1"),
+		Password:          "sekrit",
+		UpgradedToVersion: version.Current.Number,
+		CACert:            "ca cert",
+		StateAddresses:    []string{"localhost:1234"},
+		APIAddresses:      []string{"localhost:1235"},
+	},
+	inspectConfig: func(c *gc.C, cfg agent.Config) {
+		c.Check(cfg.Dir(), gc.Equals, "/data/dir/agents/unit-ubuntu-1")
+	},
+}, {
+	about: "prefer-ipv6 parsed when set",
+	params: agent.AgentConfigParams{
+		DataDir:           "/data/dir",
+		Tag:               names.NewMachineTag("1"),
+		Password:          "sekrit",
+		UpgradedToVersion: version.Current.Number,
+		CACert:            "ca cert",
+		StateAddresses:    []string{"localhost:1234"},
+		APIAddresses:      []string{"localhost:1235"},
+		Nonce:             "a nonce",
+		PreferIPv6:        true,
+	},
+	inspectConfig: func(c *gc.C, cfg agent.Config) {
+		c.Check(cfg.PreferIPv6(), jc.IsTrue)
+	},
+}, {
+	about: "missing prefer-ipv6 defaults to false",
+	params: agent.AgentConfigParams{
+		DataDir:           "/data/dir",
+		Tag:               names.NewMachineTag("1"),
+		Password:          "sekrit",
+		UpgradedToVersion: version.Current.Number,
+		CACert:            "ca cert",
+		StateAddresses:    []string{"localhost:1234"},
+		APIAddresses:      []string{"localhost:1235"},
+		Nonce:             "a nonce",
+	},
+	inspectConfig: func(c *gc.C, cfg agent.Config) {
+		c.Check(cfg.PreferIPv6(), jc.IsFalse)
+	},
 }}
 
 func (*suite) TestNewAgentConfig(c *gc.C) {
-
 	for i, test := range agentConfigTests {
 		c.Logf("%v: %s", i, test.about)
-		_, err := agent.NewAgentConfig(test.params)
+		config, err := agent.NewAgentConfig(test.params)
 		if test.checkErr == "" {
-			c.Assert(err, gc.IsNil)
+			c.Assert(err, jc.ErrorIsNil)
+			if test.inspectConfig != nil {
+				test.inspectConfig(c, config)
+			}
 		} else {
 			c.Assert(err, gc.ErrorMatches, test.checkErr)
 		}
@@ -173,13 +232,13 @@ func (*suite) TestMigrate(c *gc.C) {
 	initialParams := agent.AgentConfigParams{
 		DataDir:           c.MkDir(),
 		LogDir:            c.MkDir(),
-		Tag:               "omg",
+		Tag:               names.NewMachineTag("1"),
 		Nonce:             "nonce",
 		Password:          "secret",
 		UpgradedToVersion: version.MustParse("1.16.5"),
-		Jobs: []params.MachineJob{
-			params.JobManageEnviron,
-			params.JobHostUnits,
+		Jobs: []multiwatcher.MachineJob{
+			multiwatcher.JobManageEnviron,
+			multiwatcher.JobHostUnits,
 		},
 		CACert:         "ca cert",
 		StateAddresses: []string{"localhost:1234"},
@@ -215,7 +274,7 @@ func (*suite) TestMigrate(c *gc.C) {
 	}, {
 		fields: []string{"Jobs"},
 		newParams: agent.MigrateParams{
-			Jobs: []params.MachineJob{params.JobHostUnits},
+			Jobs: []multiwatcher.MachineJob{multiwatcher.JobHostUnits},
 		},
 	}, {
 		comment:   "invalid/immutable field specified",
@@ -247,25 +306,25 @@ func (*suite) TestMigrate(c *gc.C) {
 		c.Logf("test %d: %s %v", i, summary, test.fields)
 
 		initialConfig, err := agent.NewAgentConfig(initialParams)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 
 		newConfig, err := agent.NewAgentConfig(initialParams)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 
 		c.Assert(initialConfig.Write(), gc.IsNil)
 		c.Assert(agent.ConfigFileExists(initialConfig), jc.IsTrue)
 
 		err = newConfig.Migrate(test.newParams)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		err = newConfig.Write()
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		c.Assert(agent.ConfigFileExists(newConfig), jc.IsTrue)
 
 		// Make sure we can read it back successfully and it
 		// matches what we wrote.
 		configPath := agent.ConfigPath(newConfig.DataDir(), newConfig.Tag())
 		readConfig, err := agent.ReadConfig(configPath)
-		c.Check(err, gc.IsNil)
+		c.Check(err, jc.ErrorIsNil)
 		c.Check(newConfig, jc.DeepEquals, readConfig)
 
 		// Make sure only the specified fields were changed and
@@ -274,15 +333,15 @@ func (*suite) TestMigrate(c *gc.C) {
 			switch field {
 			case "Values":
 				err = agent.PatchConfig(initialConfig, field, test.expectValues)
-				c.Check(err, gc.IsNil)
+				c.Check(err, jc.ErrorIsNil)
 			case "DeleteValues":
 				err = agent.PatchConfig(initialConfig, field, test.newParams.DeleteValues)
-				c.Check(err, gc.IsNil)
+				c.Check(err, jc.ErrorIsNil)
 			default:
 				value := reflect.ValueOf(test.newParams).FieldByName(field)
 				if value.IsValid() && test.expectErr == "" {
 					err = agent.PatchConfig(initialConfig, field, value.Interface())
-					c.Check(err, gc.IsNil)
+					c.Check(err, jc.ErrorIsNil)
 				} else {
 					err = agent.PatchConfig(initialConfig, field, value)
 					c.Check(err, gc.ErrorMatches, test.expectErr)
@@ -293,6 +352,18 @@ func (*suite) TestMigrate(c *gc.C) {
 	}
 }
 
+func stateServingInfo() params.StateServingInfo {
+	return params.StateServingInfo{
+		Cert:           "cert",
+		PrivateKey:     "key",
+		CAPrivateKey:   "ca key",
+		StatePort:      69,
+		APIPort:        47,
+		SharedSecret:   "shared",
+		SystemIdentity: "identity",
+	}
+}
+
 func (*suite) TestNewStateMachineConfig(c *gc.C) {
 	type testStruct struct {
 		about         string
@@ -311,32 +382,36 @@ func (*suite) TestNewStateMachineConfig(c *gc.C) {
 		},
 		checkErr: "state server key not found in configuration",
 	}, {
-		about: "missing state port",
+		about: "missing ca cert key",
 		servingInfo: params.StateServingInfo{
 			Cert:       "server cert",
 			PrivateKey: "server key",
 		},
+		checkErr: "ca cert key not found in configuration",
+	}, {
+		about: "missing state port",
+		servingInfo: params.StateServingInfo{
+			Cert:         "server cert",
+			PrivateKey:   "server key",
+			CAPrivateKey: "ca key",
+		},
 		checkErr: "state port not found in configuration",
 	}, {
 		about: "params api port",
 		servingInfo: params.StateServingInfo{
-			Cert:       "server cert",
-			PrivateKey: "server key",
-			StatePort:  69,
+			Cert:         "server cert",
+			PrivateKey:   "server key",
+			CAPrivateKey: "ca key",
+			StatePort:    69,
 		},
 		checkErr: "api port not found in configuration",
 	}}
 	for _, test := range agentConfigTests {
 		tests = append(tests, testStruct{
-			about:  test.about,
-			params: test.params,
-			servingInfo: params.StateServingInfo{
-				Cert:       "server cert",
-				PrivateKey: "server key",
-				StatePort:  3171,
-				APIPort:    300,
-			},
-			checkErr: test.checkErr,
+			about:       test.about,
+			params:      test.params,
+			servingInfo: stateServingInfo(),
+			checkErr:    test.checkErr,
 		})
 	}
 
@@ -344,7 +419,7 @@ func (*suite) TestNewStateMachineConfig(c *gc.C) {
 		c.Logf("%v: %s", i, test.about)
 		cfg, err := agent.NewStateMachineConfig(test.params, test.servingInfo)
 		if test.checkErr == "" {
-			c.Assert(err, gc.IsNil)
+			c.Assert(err, jc.ErrorIsNil)
 			if test.inspectConfig != nil {
 				test.inspectConfig(c, cfg)
 			}
@@ -356,7 +431,7 @@ func (*suite) TestNewStateMachineConfig(c *gc.C) {
 
 var attributeParams = agent.AgentConfigParams{
 	DataDir:           "/data/dir",
-	Tag:               "omg",
+	Tag:               names.NewMachineTag("1"),
 	UpgradedToVersion: version.Current.Number,
 	Password:          "sekrit",
 	CACert:            "ca cert",
@@ -367,26 +442,21 @@ var attributeParams = agent.AgentConfigParams{
 
 func (*suite) TestAttributes(c *gc.C) {
 	conf, err := agent.NewAgentConfig(attributeParams)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(conf.DataDir(), gc.Equals, "/data/dir")
-	c.Assert(conf.SystemIdentityPath(), gc.Equals, "/data/dir/system-identity")
-	c.Assert(conf.Tag(), gc.Equals, "omg")
-	c.Assert(conf.Dir(), gc.Equals, "/data/dir/agents/omg")
+	compareSystemIdentityPath := filepath.FromSlash("/data/dir/system-identity")
+	systemIdentityPath := filepath.FromSlash(conf.SystemIdentityPath())
+	c.Assert(systemIdentityPath, gc.Equals, compareSystemIdentityPath)
+	c.Assert(conf.Tag(), gc.Equals, names.NewMachineTag("1"))
+	c.Assert(conf.Dir(), gc.Equals, "/data/dir/agents/machine-1")
 	c.Assert(conf.Nonce(), gc.Equals, "a nonce")
 	c.Assert(conf.UpgradedToVersion(), jc.DeepEquals, version.Current.Number)
 }
 
 func (*suite) TestStateServingInfo(c *gc.C) {
-	servingInfo := params.StateServingInfo{
-		Cert:           "old cert",
-		PrivateKey:     "old key",
-		StatePort:      69,
-		APIPort:        47,
-		SharedSecret:   "shared",
-		SystemIdentity: "identity",
-	}
+	servingInfo := stateServingInfo()
 	conf, err := agent.NewStateMachineConfig(attributeParams, servingInfo)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	gotInfo, ok := conf.StateServingInfo()
 	c.Assert(ok, jc.IsTrue)
 	c.Assert(gotInfo, jc.DeepEquals, servingInfo)
@@ -395,6 +465,7 @@ func (*suite) TestStateServingInfo(c *gc.C) {
 		StatePort:      169,
 		Cert:           "new cert",
 		PrivateKey:     "new key",
+		CAPrivateKey:   "new ca key",
 		SharedSecret:   "new shared",
 		SystemIdentity: "new identity",
 	}
@@ -406,22 +477,22 @@ func (*suite) TestStateServingInfo(c *gc.C) {
 
 func (*suite) TestStateServingInfoNotAvailable(c *gc.C) {
 	conf, err := agent.NewAgentConfig(attributeParams)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	_, available := conf.StateServingInfo()
-	c.Assert(available, gc.Equals, false)
+	c.Assert(available, jc.IsFalse)
 }
 
 func (s *suite) TestAPIAddressesCannotWriteBack(c *gc.C) {
 	conf, err := agent.NewAgentConfig(attributeParams)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	value, err := conf.APIAddresses()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(value, jc.DeepEquals, []string{"localhost:1235"})
 	value[0] = "invalidAdr"
 	//Check out change hasn't gone back into the internals
 	newValue, err := conf.APIAddresses()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(newValue, jc.DeepEquals, []string{"localhost:1235"})
 }
 
@@ -430,31 +501,24 @@ func (*suite) TestWriteAndRead(c *gc.C) {
 	testParams.DataDir = c.MkDir()
 	testParams.LogDir = c.MkDir()
 	conf, err := agent.NewAgentConfig(testParams)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	c.Assert(conf.Write(), gc.IsNil)
 	reread, err := agent.ReadConfig(agent.ConfigPath(conf.DataDir(), conf.Tag()))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(reread, jc.DeepEquals, conf)
 }
 
-func (*suite) TestAPIInfoAddsLocalhostWhenServingInfoPesent(c *gc.C) {
+func (*suite) TestAPIInfoAddsLocalhostWhenServingInfoPresent(c *gc.C) {
 	attrParams := attributeParams
-	servingInfo := params.StateServingInfo{
-		Cert:           "old cert",
-		PrivateKey:     "old key",
-		StatePort:      69,
-		APIPort:        1492,
-		SharedSecret:   "shared",
-		SystemIdentity: "identity",
-	}
+	servingInfo := stateServingInfo()
 	conf, err := agent.NewStateMachineConfig(attrParams, servingInfo)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	apiinfo := conf.APIInfo()
 	c.Check(apiinfo.Addrs, gc.HasLen, len(attrParams.APIAddresses)+1)
 	localhostAddressFound := false
 	for _, eachApiAddress := range apiinfo.Addrs {
-		if eachApiAddress == "localhost:1492" {
+		if eachApiAddress == "localhost:47" {
 			localhostAddressFound = true
 			break
 		}
@@ -462,26 +526,67 @@ func (*suite) TestAPIInfoAddsLocalhostWhenServingInfoPesent(c *gc.C) {
 	c.Assert(localhostAddressFound, jc.IsTrue)
 }
 
-func (*suite) TestAPIInfoDoesntAddLocalhostWhenNoServingInfo(c *gc.C) {
+func (*suite) TestAPIInfoAddsLocalhostWhenServingInfoPresentAndPreferIPv6On(c *gc.C) {
 	attrParams := attributeParams
+	attrParams.PreferIPv6 = true
+	servingInfo := stateServingInfo()
+	conf, err := agent.NewStateMachineConfig(attrParams, servingInfo)
+	c.Assert(err, jc.ErrorIsNil)
+	apiinfo := conf.APIInfo()
+	c.Check(apiinfo.Addrs, gc.HasLen, len(attrParams.APIAddresses)+1)
+	localhostAddressFound := false
+	for _, eachApiAddress := range apiinfo.Addrs {
+		if eachApiAddress == "[::1]:47" {
+			localhostAddressFound = true
+			break
+		}
+		c.Check(eachApiAddress, gc.Not(gc.Equals), "localhost:47")
+	}
+	c.Assert(localhostAddressFound, jc.IsTrue)
+}
+
+func (*suite) TestMongoInfoHonorsPreferIPv6(c *gc.C) {
+	attrParams := attributeParams
+	attrParams.PreferIPv6 = true
+	servingInfo := stateServingInfo()
+	conf, err := agent.NewStateMachineConfig(attrParams, servingInfo)
+	c.Assert(err, jc.ErrorIsNil)
+	mongoInfo, ok := conf.MongoInfo()
+	c.Assert(ok, jc.IsTrue)
+	c.Check(mongoInfo.Info.Addrs, jc.DeepEquals, []string{"[::1]:69"})
+
+	attrParams.PreferIPv6 = false
+	conf, err = agent.NewStateMachineConfig(attrParams, servingInfo)
+	c.Assert(err, jc.ErrorIsNil)
+	mongoInfo, ok = conf.MongoInfo()
+	c.Assert(ok, jc.IsTrue)
+	c.Check(mongoInfo.Info.Addrs, jc.DeepEquals, []string{"127.0.0.1:69"})
+}
+
+func (*suite) TestAPIInfoDoesntAddLocalhostWhenNoServingInfoPreferIPv6Off(c *gc.C) {
+	attrParams := attributeParams
+	attrParams.PreferIPv6 = false
 	conf, err := agent.NewAgentConfig(attrParams)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
+	apiinfo := conf.APIInfo()
+	c.Assert(apiinfo.Addrs, gc.DeepEquals, attrParams.APIAddresses)
+}
+
+func (*suite) TestAPIInfoDoesntAddLocalhostWhenNoServingInfoPreferIPv6On(c *gc.C) {
+	attrParams := attributeParams
+	attrParams.PreferIPv6 = true
+	conf, err := agent.NewAgentConfig(attrParams)
+	c.Assert(err, jc.ErrorIsNil)
 	apiinfo := conf.APIInfo()
 	c.Assert(apiinfo.Addrs, gc.DeepEquals, attrParams.APIAddresses)
 }
 
 func (*suite) TestSetPassword(c *gc.C) {
 	attrParams := attributeParams
-	servingInfo := params.StateServingInfo{
-		Cert:           "old cert",
-		PrivateKey:     "old key",
-		StatePort:      1234,
-		APIPort:        1235,
-		SharedSecret:   "shared",
-		SystemIdentity: "identity",
-	}
+	servingInfo := stateServingInfo()
+	servingInfo.APIPort = 1235
 	conf, err := agent.NewStateMachineConfig(attrParams, servingInfo)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	expectAPIInfo := &api.Info{
 		Addrs:    attrParams.APIAddresses,
@@ -492,7 +597,7 @@ func (*suite) TestSetPassword(c *gc.C) {
 	}
 	c.Assert(conf.APIInfo(), jc.DeepEquals, expectAPIInfo)
 	addr := fmt.Sprintf("127.0.0.1:%d", servingInfo.StatePort)
-	expectStateInfo := &state.Info{
+	expectStateInfo := &mongo.MongoInfo{
 		Info: mongo.Info{
 			Addrs:  []string{addr},
 			CACert: attrParams.CACert,
@@ -500,7 +605,7 @@ func (*suite) TestSetPassword(c *gc.C) {
 		Tag:      attrParams.Tag,
 		Password: "",
 	}
-	info, ok := conf.StateInfo()
+	info, ok := conf.MongoInfo()
 	c.Assert(ok, jc.IsTrue)
 	c.Assert(info, jc.DeepEquals, expectStateInfo)
 
@@ -510,14 +615,14 @@ func (*suite) TestSetPassword(c *gc.C) {
 	expectStateInfo.Password = "newpassword"
 
 	c.Assert(conf.APIInfo(), jc.DeepEquals, expectAPIInfo)
-	info, ok = conf.StateInfo()
+	info, ok = conf.MongoInfo()
 	c.Assert(ok, jc.IsTrue)
 	c.Assert(info, jc.DeepEquals, expectStateInfo)
 }
 
 func (*suite) TestSetOldPassword(c *gc.C) {
 	conf, err := agent.NewAgentConfig(attributeParams)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	c.Assert(conf.OldPassword(), gc.Equals, attributeParams.Password)
 	conf.SetOldPassword("newoldpassword")
@@ -526,7 +631,7 @@ func (*suite) TestSetOldPassword(c *gc.C) {
 
 func (*suite) TestSetUpgradedToVersion(c *gc.C) {
 	conf, err := agent.NewAgentConfig(attributeParams)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	c.Assert(conf.UpgradedToVersion(), gc.Equals, version.Current.Number)
 
@@ -537,10 +642,10 @@ func (*suite) TestSetUpgradedToVersion(c *gc.C) {
 
 func (*suite) TestSetAPIHostPorts(c *gc.C) {
 	conf, err := agent.NewAgentConfig(attributeParams)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	addrs, err := conf.APIAddresses()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(addrs, gc.DeepEquals, attributeParams.APIAddresses)
 
 	// The first cloud-local address for each server is used,
@@ -564,6 +669,6 @@ func (*suite) TestSetAPIHostPorts(c *gc.C) {
 		network.AddressesWithPort(server3, 125),
 	})
 	addrs, err = conf.APIAddresses()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(addrs, gc.DeepEquals, []string{"0.1.2.3:123", "0.1.2.5:125"})
 }
