@@ -6,7 +6,7 @@ package cloudinit_test
 import (
 	"regexp"
 
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/cloudinit"
 )
