@@ -4,6 +4,7 @@
 package envcmd
 
 var (
-	GetDefaultEnvironment         = getDefaultEnvironment
 	GetCurrentEnvironmentFilePath = getCurrentEnvironmentFilePath
+	GetConfigStore                = &getConfigStore
+	EndpointRefresher             = &endpointRefresher
 )
