@@ -6,12 +6,11 @@ package main
 import (
 	"errors"
 	"fmt"
-
-	"github.com/juju/cmd"
 	"launchpad.net/gnuflag"
 
-	"github.com/juju/juju/cmd/envcmd"
-	"github.com/juju/juju/juju"
+	"github.com/juju/cmd"
+
+	"github.com/juju/juju/cmd/juju/block"
 )
 
 var addKeysDoc = `
@@ -20,7 +19,7 @@ Add new authorized ssh keys to allow the holder of those keys to log on to Juju
 
 // AddKeysCommand is used to add a new authorized ssh key for a user.
 type AddKeysCommand struct {
-	envcmd.EnvCommandBase
+	AuthorizedKeysBase
 	user    string
 	sshKeys []string
 }
@@ -49,7 +48,7 @@ func (c *AddKeysCommand) SetFlags(f *gnuflag.FlagSet) {
 }
 
 func (c *AddKeysCommand) Run(context *cmd.Context) error {
-	client, err := juju.NewKeyManagerClient(c.EnvName)
+	client, err := c.NewKeyManagerClient()
 	if err != nil {
 		return err
 	}
@@ -57,7 +56,7 @@ func (c *AddKeysCommand) Run(context *cmd.Context) error {
 
 	results, err := client.AddKeys(c.user, c.sshKeys...)
 	if err != nil {
-		return err
+		return block.ProcessBlockedError(err, block.BlockChange)
 	}
 	for i, result := range results {
 		if result.Error != nil {
