@@ -9,7 +9,7 @@ import (
 	"github.com/juju/cmd"
 
 	"github.com/juju/juju/cmd/envcmd"
-	"github.com/juju/juju/juju"
+	"github.com/juju/juju/cmd/juju/block"
 )
 
 // RemoveRelationCommand causes an existing service relation to be shut down.
@@ -36,10 +36,10 @@ func (c *RemoveRelationCommand) Init(args []string) error {
 }
 
 func (c *RemoveRelationCommand) Run(_ *cmd.Context) error {
-	client, err := juju.NewAPIClientFromName(c.EnvName)
+	client, err := c.NewAPIClient()
 	if err != nil {
 		return err
 	}
 	defer client.Close()
-	return client.DestroyRelation(c.Endpoints...)
+	return block.ProcessBlockedError(client.DestroyRelation(c.Endpoints...), block.BlockRemove)
 }
