@@ -10,7 +10,7 @@ import (
 	"github.com/juju/names"
 
 	"github.com/juju/juju/cmd/envcmd"
-	"github.com/juju/juju/juju"
+	"github.com/juju/juju/cmd/juju/block"
 )
 
 // RemoveServiceCommand causes an existing service to be destroyed.
@@ -33,7 +33,7 @@ func (c *RemoveServiceCommand) Init(args []string) error {
 	if len(args) == 0 {
 		return fmt.Errorf("no service specified")
 	}
-	if !names.IsService(args[0]) {
+	if !names.IsValidService(args[0]) {
 		return fmt.Errorf("invalid service name %q", args[0])
 	}
 	c.ServiceName, args = args[0], args[1:]
@@ -41,10 +41,10 @@ func (c *RemoveServiceCommand) Init(args []string) error {
 }
 
 func (c *RemoveServiceCommand) Run(_ *cmd.Context) error {
-	client, err := juju.NewAPIClientFromName(c.EnvName)
+	client, err := c.NewAPIClient()
 	if err != nil {
 		return err
 	}
 	defer client.Close()
-	return client.ServiceDestroy(c.ServiceName)
+	return block.ProcessBlockedError(client.ServiceDestroy(c.ServiceName), block.BlockRemove)
 }
