@@ -11,7 +11,7 @@ import (
 	"launchpad.net/gnuflag"
 
 	"github.com/juju/juju/cmd/envcmd"
-	"github.com/juju/juju/juju"
+	"github.com/juju/juju/cmd/juju/block"
 )
 
 // ResolvedCommand marks a unit in an error state as ready to continue.
@@ -37,7 +37,7 @@ func (c *ResolvedCommand) SetFlags(f *gnuflag.FlagSet) {
 func (c *ResolvedCommand) Init(args []string) error {
 	if len(args) > 0 {
 		c.UnitName = args[0]
-		if !names.IsUnit(c.UnitName) {
+		if !names.IsValidUnit(c.UnitName) {
 			return fmt.Errorf("invalid unit name %q", c.UnitName)
 		}
 		args = args[1:]
@@ -48,10 +48,10 @@ func (c *ResolvedCommand) Init(args []string) error {
 }
 
 func (c *ResolvedCommand) Run(_ *cmd.Context) error {
-	client, err := juju.NewAPIClientFromName(c.EnvName)
+	client, err := c.NewAPIClient()
 	if err != nil {
 		return err
 	}
 	defer client.Close()
-	return client.Resolved(c.UnitName, c.Retry)
+	return block.ProcessBlockedError(client.Resolved(c.UnitName, c.Retry), block.BlockChange)
 }
