@@ -10,14 +10,14 @@ import (
 	"github.com/juju/names"
 
 	"github.com/juju/juju/cmd/envcmd"
-	"github.com/juju/juju/juju"
+	"github.com/juju/juju/cmd/juju/block"
 )
 
 // RetryProvisioningCommand updates machines' error status to tell
 // the provisoner that it should try to re-provision the machine.
 type RetryProvisioningCommand struct {
 	envcmd.EnvCommandBase
-	Machines []string
+	Machines []names.MachineTag
 }
 
 func (c *RetryProvisioningCommand) Info() *cmd.Info {
@@ -32,25 +32,26 @@ func (c *RetryProvisioningCommand) Init(args []string) error {
 	if len(args) == 0 {
 		return fmt.Errorf("no machine specified")
 	}
-	c.Machines = make([]string, len(args))
+	c.Machines = make([]names.MachineTag, len(args))
 	for i, arg := range args {
-		if !names.IsMachine(arg) {
+		if !names.IsValidMachine(arg) {
 			return fmt.Errorf("invalid machine %q", arg)
 		}
-		c.Machines[i] = names.NewMachineTag(arg).String()
+		c.Machines[i] = names.NewMachineTag(arg)
 	}
 	return nil
 }
 
 func (c *RetryProvisioningCommand) Run(context *cmd.Context) error {
-	client, err := juju.NewAPIClientFromName(c.EnvName)
+	client, err := c.NewAPIClient()
 	if err != nil {
 		return err
 	}
 	defer client.Close()
+
 	results, err := client.RetryProvisioning(c.Machines...)
 	if err != nil {
-		return err
+		return block.ProcessBlockedError(err, block.BlockChange)
 	}
 	for i, result := range results {
 		if result.Error != nil {
