@@ -4,12 +4,16 @@
 package main
 
 import (
-	"github.com/juju/charm"
-	charmtesting "github.com/juju/charm/testing"
-	gc "launchpad.net/gocheck"
+	"strings"
+
+	"github.com/juju/cmd"
+	jc "github.com/juju/testing/checkers"
+	gc "gopkg.in/check.v1"
+	"gopkg.in/juju/charm.v4"
 
 	"github.com/juju/juju/cmd/envcmd"
 	jujutesting "github.com/juju/juju/juju/testing"
+	"github.com/juju/juju/testcharms"
 	"github.com/juju/juju/testing"
 )
 
@@ -26,26 +30,42 @@ func runUnexpose(c *gc.C, args ...string) error {
 
 func (s *UnexposeSuite) assertExposed(c *gc.C, service string, expected bool) {
 	svc, err := s.State.Service(service)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	actual := svc.IsExposed()
 	c.Assert(actual, gc.Equals, expected)
 }
 
 func (s *UnexposeSuite) TestUnexpose(c *gc.C) {
-	charmtesting.Charms.BundlePath(s.SeriesPath, "dummy")
+	testcharms.Repo.CharmArchivePath(s.SeriesPath, "dummy")
 	err := runDeploy(c, "local:dummy", "some-service-name")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	curl := charm.MustParseURL("local:trusty/dummy-1")
 	s.AssertService(c, "some-service-name", curl, 1, 0)
 
 	err = runExpose(c, "some-service-name")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	s.assertExposed(c, "some-service-name", true)
 
 	err = runUnexpose(c, "some-service-name")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	s.assertExposed(c, "some-service-name", false)
 
 	err = runUnexpose(c, "nonexistent-service")
 	c.Assert(err, gc.ErrorMatches, `service "nonexistent-service" not found`)
 }
+
+func (s *UnexposeSuite) TestBlockUnexpose(c *gc.C) {
+	testcharms.Repo.CharmArchivePath(s.SeriesPath, "dummy")
+	err := runDeploy(c, "local:dummy", "some-service-name")
+	c.Assert(err, jc.ErrorIsNil)
+	curl := charm.MustParseURL("local:trusty/dummy-1")
+	s.AssertService(c, "some-service-name", curl, 1, 0)
+
+	// Block operation
+	s.AssertConfigParameterUpdated(c, "block-all-changes", true)
+	err = runExpose(c, "some-service-name")
+	c.Assert(err, gc.ErrorMatches, cmd.ErrSilent.Error())
+	// msg is logged
+	stripped := strings.Replace(c.GetTestLog(), "\n", "", -1)
+	c.Check(stripped, gc.Matches, ".*To unblock changes.*")
+}
