@@ -9,7 +9,7 @@ import (
 	"github.com/juju/cmd"
 
 	"github.com/juju/juju/cmd/envcmd"
-	"github.com/juju/juju/juju"
+	"github.com/juju/juju/cmd/juju/block"
 )
 
 // UnsetCommand sets configuration values of a service back
@@ -49,10 +49,10 @@ func (c *UnsetCommand) Init(args []string) error {
 
 // Run resets the configuration of a service.
 func (c *UnsetCommand) Run(ctx *cmd.Context) error {
-	apiclient, err := juju.NewAPIClientFromName(c.EnvName)
+	apiclient, err := c.NewAPIClient()
 	if err != nil {
 		return err
 	}
 	defer apiclient.Close()
-	return apiclient.ServiceUnset(c.ServiceName, c.Options)
+	return block.ProcessBlockedError(apiclient.ServiceUnset(c.ServiceName, c.Options), block.BlockChange)
 }
