@@ -11,19 +11,23 @@ import (
 	"io/ioutil"
 	"strings"
 
+	jujucmd "github.com/juju/cmd"
 	jc "github.com/juju/testing/checkers"
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
+	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/cmd/envcmd"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/environs/filestorage"
-	"github.com/juju/juju/environs/storage"
 	"github.com/juju/juju/environs/sync"
 	envtesting "github.com/juju/juju/environs/testing"
-	envtools "github.com/juju/juju/environs/tools"
+	"github.com/juju/juju/environs/tools"
 	toolstesting "github.com/juju/juju/environs/tools/testing"
 	"github.com/juju/juju/juju/testing"
+	"github.com/juju/juju/network"
+	"github.com/juju/juju/state"
 	coretesting "github.com/juju/juju/testing"
+	coretools "github.com/juju/juju/tools"
 	"github.com/juju/juju/version"
 )
 
@@ -285,6 +289,7 @@ func (s *UpgradeJujuSuite) TestUpgradeJuju(c *gc.C) {
 	for i, test := range upgradeJujuTests {
 		c.Logf("\ntest %d: %s", i, test.about)
 		s.Reset(c)
+		tools.DefaultBaseURL = ""
 
 		// Set up apparent CLI version and initialize the command.
 		version.Current = version.MustParseBinary(test.currentVersion)
@@ -293,7 +298,7 @@ func (s *UpgradeJujuSuite) TestUpgradeJuju(c *gc.C) {
 			if test.expectInitErr != "" {
 				c.Check(err, gc.ErrorMatches, test.expectInitErr)
 			} else {
-				c.Check(err, gc.IsNil)
+				c.Check(err, jc.ErrorIsNil)
 			}
 			continue
 		}
@@ -302,58 +307,63 @@ func (s *UpgradeJujuSuite) TestUpgradeJuju(c *gc.C) {
 		toolsDir := c.MkDir()
 		updateAttrs := map[string]interface{}{
 			"agent-version":      test.agentVersion,
-			"tools-metadata-url": "file://" + toolsDir,
+			"agent-metadata-url": "file://" + toolsDir + "/tools",
 		}
 		err := s.State.UpdateEnvironConfig(updateAttrs, nil, nil)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		versions := make([]version.Binary, len(test.tools))
 		for i, v := range test.tools {
 			versions[i] = version.MustParseBinary(v)
 		}
 		if len(versions) > 0 {
-			envtesting.MustUploadFakeToolsVersions(s.Conn.Environ.Storage(), versions...)
 			stor, err := filestorage.NewFileStorageWriter(toolsDir)
-			c.Assert(err, gc.IsNil)
-			envtesting.MustUploadFakeToolsVersions(stor, versions...)
+			c.Assert(err, jc.ErrorIsNil)
+			envtesting.MustUploadFakeToolsVersions(stor, s.Environ.Config().AgentStream(), versions...)
 		}
 
 		err = com.Run(coretesting.Context(c))
 		if test.expectErr != "" {
 			c.Check(err, gc.ErrorMatches, test.expectErr)
 			continue
-		} else if !c.Check(err, gc.IsNil) {
+		} else if !c.Check(err, jc.ErrorIsNil) {
 			continue
 		}
 
 		// Check expected changes to environ/state.
 		cfg, err := s.State.EnvironConfig()
-		c.Check(err, gc.IsNil)
+		c.Check(err, jc.ErrorIsNil)
 		agentVersion, ok := cfg.AgentVersion()
-		c.Check(ok, gc.Equals, true)
+		c.Check(ok, jc.IsTrue)
 		c.Check(agentVersion, gc.Equals, version.MustParse(test.expectVersion))
 
 		for _, uploaded := range test.expectUploaded {
 			// Substitute latest LTS for placeholder in expected series for uploaded tools
 			uploaded = strings.Replace(uploaded, "%LTS%", config.LatestLtsSeries(), 1)
-
 			vers := version.MustParseBinary(uploaded)
-			r, err := storage.Get(s.Conn.Environ.Storage(), envtools.StorageName(vers))
-			if !c.Check(err, gc.IsNil) {
-				continue
-			}
-			data, err := ioutil.ReadAll(r)
-			r.Close()
-			c.Check(err, gc.IsNil)
-			expectContent := version.Current
-			expectContent.Number = agentVersion
-			checkToolsContent(c, data, "jujud contents "+expectContent.String())
+			s.checkToolsUploaded(c, vers, agentVersion)
 		}
 	}
 }
 
+func (s *UpgradeJujuSuite) checkToolsUploaded(c *gc.C, vers version.Binary, agentVersion version.Number) {
+	storage, err := s.State.ToolsStorage()
+	c.Assert(err, jc.ErrorIsNil)
+	defer storage.Close()
+	_, r, err := storage.Tools(vers)
+	if !c.Check(err, jc.ErrorIsNil) {
+		return
+	}
+	data, err := ioutil.ReadAll(r)
+	r.Close()
+	c.Check(err, jc.ErrorIsNil)
+	expectContent := version.Current
+	expectContent.Number = agentVersion
+	checkToolsContent(c, data, "jujud contents "+expectContent.String())
+}
+
 func checkToolsContent(c *gc.C, data []byte, uploaded string) {
 	zr, err := gzip.NewReader(bytes.NewReader(data))
-	c.Check(err, gc.IsNil)
+	c.Check(err, jc.ErrorIsNil)
 	defer zr.Close()
 	tr := tar.NewReader(zr)
 	found := false
@@ -362,7 +372,7 @@ func checkToolsContent(c *gc.C, data []byte, uploaded string) {
 		if err == io.EOF {
 			break
 		}
-		c.Check(err, gc.IsNil)
+		c.Check(err, jc.ErrorIsNil)
 		if strings.ContainsAny(hdr.Name, "/\\") {
 			c.Fail()
 		}
@@ -370,7 +380,7 @@ func checkToolsContent(c *gc.C, data []byte, uploaded string) {
 			c.Fail()
 		}
 		content, err := ioutil.ReadAll(tr)
-		c.Check(err, gc.IsNil)
+		c.Check(err, jc.ErrorIsNil)
 		c.Check(string(content), gc.Equals, uploaded)
 		found = true
 	}
@@ -383,26 +393,46 @@ func checkToolsContent(c *gc.C, data []byte, uploaded string) {
 // in the environment state.
 func (s *UpgradeJujuSuite) Reset(c *gc.C) {
 	s.JujuConnSuite.Reset(c)
-	envtesting.RemoveTools(c, s.Conn.Environ.Storage())
+	envtesting.RemoveTools(c, s.DefaultToolsStorage, s.Environ.Config().AgentStream())
 	updateAttrs := map[string]interface{}{
 		"default-series": "raring",
 		"agent-version":  "1.2.3",
 	}
 	err := s.State.UpdateEnvironConfig(updateAttrs, nil, nil)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	s.PatchValue(&sync.BuildToolsTarball, toolstesting.GetMockBuildTools(c))
+
+	// Set API host ports so FindTools works.
+	hostPorts := [][]network.HostPort{{{
+		Address: network.NewAddress("0.1.2.3", network.ScopeUnknown),
+		Port:    1234,
+	}}}
+	err = s.State.SetAPIHostPorts(hostPorts)
+	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *UpgradeJujuSuite) TestUpgradeJujuWithRealUpload(c *gc.C) {
 	s.Reset(c)
 	cmd := envcmd.Wrap(&UpgradeJujuCommand{})
 	_, err := coretesting.RunCommand(c, cmd, "--upload-tools")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	vers := version.Current
 	vers.Build = 1
-	tools, err := envtools.FindInstanceTools(s.Conn.Environ, vers.Number, vers.Series, &vers.Arch)
-	c.Assert(err, gc.IsNil)
-	c.Assert(len(tools), gc.Equals, 1)
+	s.checkToolsUploaded(c, vers, vers.Number)
+	//_, err = envtools.FindExactTools(s.Environ, vers.Number, vers.Series, vers.Arch)
+	//c.Assert(err, jc.ErrorIsNil)
+}
+
+func (s *UpgradeJujuSuite) TestBlockUpgradeJujuWithRealUpload(c *gc.C) {
+	s.Reset(c)
+	cmd := envcmd.Wrap(&UpgradeJujuCommand{})
+	// Block operation
+	s.AssertConfigParameterUpdated(c, "block-all-changes", true)
+	_, err := coretesting.RunCommand(c, cmd, "--upload-tools")
+	c.Assert(err, gc.ErrorMatches, jujucmd.ErrSilent.Error())
+	// msg is logged
+	stripped := strings.Replace(c.GetTestLog(), "\n", "", -1)
+	c.Check(stripped, gc.Matches, ".*To unblock changes.*")
 }
 
 type DryRunTest struct {
@@ -456,40 +486,209 @@ upgrade to this version by running
 
 	for i, test := range tests {
 		c.Logf("\ntest %d: %s", i, test.about)
+		s.Reset(c)
+		tools.DefaultBaseURL = ""
+
 		s.PatchValue(&version.Current, version.MustParseBinary(test.currentVersion))
 		com := &UpgradeJujuCommand{}
 		err := coretesting.InitCommand(envcmd.Wrap(com), test.cmdArgs)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		toolsDir := c.MkDir()
 		updateAttrs := map[string]interface{}{
 			"agent-version":      test.agentVersion,
-			"tools-metadata-url": "file://" + toolsDir,
+			"agent-metadata-url": "file://" + toolsDir + "/tools",
 		}
 
 		err = s.State.UpdateEnvironConfig(updateAttrs, nil, nil)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		versions := make([]version.Binary, len(test.tools))
 		for i, v := range test.tools {
 			versions[i] = version.MustParseBinary(v)
 		}
 		if len(versions) > 0 {
-			envtesting.MustUploadFakeToolsVersions(s.Conn.Environ.Storage(), versions...)
 			stor, err := filestorage.NewFileStorageWriter(toolsDir)
-			c.Assert(err, gc.IsNil)
-			envtesting.MustUploadFakeToolsVersions(stor, versions...)
+			c.Assert(err, jc.ErrorIsNil)
+			envtesting.MustUploadFakeToolsVersions(stor, s.Environ.Config().AgentStream(), versions...)
 		}
 
 		ctx := coretesting.Context(c)
 		err = com.Run(ctx)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 
 		// Check agent version doesn't change
 		cfg, err := s.State.EnvironConfig()
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		agentVer, ok := cfg.AgentVersion()
-		c.Assert(ok, gc.Equals, true)
+		c.Assert(ok, jc.IsTrue)
 		c.Assert(agentVer, gc.Equals, version.MustParse(test.agentVersion))
 		output := coretesting.Stderr(ctx)
 		c.Assert(output, gc.Equals, test.expectedCmdOutput)
 	}
 }
+
+func (s *UpgradeJujuSuite) TestUpgradeInProgress(c *gc.C) {
+	fakeAPI := NewFakeUpgradeJujuAPI(c, s.State)
+	fakeAPI.setVersionErr = &params.Error{
+		Message: "a message from the server about the problem",
+		Code:    params.CodeUpgradeInProgress,
+	}
+	fakeAPI.patch(s)
+	cmd := &UpgradeJujuCommand{}
+	err := coretesting.InitCommand(envcmd.Wrap(cmd), []string{})
+	c.Assert(err, jc.ErrorIsNil)
+
+	err = cmd.Run(coretesting.Context(c))
+	c.Assert(err, gc.ErrorMatches, "a message from the server about the problem\n"+
+		"\n"+
+		"Please wait for the upgrade to complete or if there was a problem with\n"+
+		"the last upgrade that has been resolved, consider running the\n"+
+		"upgrade-juju command with the --reset-previous-upgrade flag.",
+	)
+}
+
+func (s *UpgradeJujuSuite) TestBlockUpgradeInProgress(c *gc.C) {
+	fakeAPI := NewFakeUpgradeJujuAPI(c, s.State)
+	fakeAPI.setVersionErr = &params.Error{
+		Code:    params.CodeOperationBlocked,
+		Message: "The operation has been blocked.",
+	}
+	fakeAPI.patch(s)
+	cmd := &UpgradeJujuCommand{}
+	err := coretesting.InitCommand(envcmd.Wrap(cmd), []string{})
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Block operation
+	s.AssertConfigParameterUpdated(c, "block-all-changes", true)
+	err = cmd.Run(coretesting.Context(c))
+	c.Assert(err, gc.ErrorMatches, jujucmd.ErrSilent.Error())
+	// msg is logged
+	stripped := strings.Replace(c.GetTestLog(), "\n", "", -1)
+	c.Check(stripped, gc.Matches, ".*To unblock changes.*")
+}
+
+func (s *UpgradeJujuSuite) TestResetPreviousUpgrade(c *gc.C) {
+	fakeAPI := NewFakeUpgradeJujuAPI(c, s.State)
+	fakeAPI.patch(s)
+
+	ctx := coretesting.Context(c)
+	var stdin bytes.Buffer
+	ctx.Stdin = &stdin
+
+	run := func(answer string, expect bool, args ...string) {
+		stdin.Reset()
+		if answer != "" {
+			stdin.WriteString(answer)
+		}
+
+		fakeAPI.reset()
+
+		cmd := &UpgradeJujuCommand{}
+		err := coretesting.InitCommand(envcmd.Wrap(cmd),
+			append([]string{"--reset-previous-upgrade"}, args...))
+		c.Assert(err, jc.ErrorIsNil)
+		err = cmd.Run(ctx)
+		if expect {
+			c.Assert(err, jc.ErrorIsNil)
+		} else {
+			c.Assert(err, gc.ErrorMatches, "previous upgrade not reset and no new upgrade triggered")
+		}
+
+		c.Assert(fakeAPI.abortCurrentUpgradeCalled, gc.Equals, expect)
+		expectedVersion := version.Number{}
+		if expect {
+			expectedVersion = fakeAPI.nextVersion.Number
+		}
+		c.Assert(fakeAPI.setVersionCalledWith, gc.Equals, expectedVersion)
+	}
+
+	const expectUpgrade = true
+	const expectNoUpgrade = false
+
+	// EOF on stdin - equivalent to answering no.
+	run("", expectNoUpgrade)
+
+	// -y on command line - no confirmation required
+	run("", expectUpgrade, "-y")
+
+	// --yes on command line - no confirmation required
+	run("", expectUpgrade, "--yes")
+
+	// various ways of saying "yes" to the prompt
+	for _, answer := range []string{"y", "Y", "yes", "YES"} {
+		run(answer, expectUpgrade)
+	}
+
+	// various ways of saying "no" to the prompt
+	for _, answer := range []string{"n", "N", "no", "foo"} {
+		run(answer, expectNoUpgrade)
+	}
+}
+
+func NewFakeUpgradeJujuAPI(c *gc.C, st *state.State) *fakeUpgradeJujuAPI {
+	nextVersion := version.Current
+	nextVersion.Minor++
+	return &fakeUpgradeJujuAPI{
+		c:           c,
+		st:          st,
+		nextVersion: nextVersion,
+	}
+}
+
+type fakeUpgradeJujuAPI struct {
+	c                         *gc.C
+	st                        *state.State
+	nextVersion               version.Binary
+	setVersionErr             error
+	abortCurrentUpgradeCalled bool
+	setVersionCalledWith      version.Number
+}
+
+func (a *fakeUpgradeJujuAPI) reset() {
+	a.setVersionErr = nil
+	a.abortCurrentUpgradeCalled = false
+	a.setVersionCalledWith = version.Number{}
+}
+
+func (a *fakeUpgradeJujuAPI) patch(s *UpgradeJujuSuite) {
+	s.PatchValue(&getUpgradeJujuAPI, func(*UpgradeJujuCommand) (upgradeJujuAPI, error) {
+		return a, nil
+	})
+}
+
+func (a *fakeUpgradeJujuAPI) EnvironmentGet() (map[string]interface{}, error) {
+	config, err := a.st.EnvironConfig()
+	if err != nil {
+		return make(map[string]interface{}), err
+	}
+	return config.AllAttrs(), nil
+}
+
+func (a *fakeUpgradeJujuAPI) FindTools(majorVersion, minorVersion int, series, arch string) (
+	result params.FindToolsResult, err error,
+) {
+	tools := toolstesting.MakeTools(a.c, a.c.MkDir(), "released", []string{a.nextVersion.String()})
+	return params.FindToolsResult{
+		List:  tools,
+		Error: nil,
+	}, nil
+}
+
+func (a *fakeUpgradeJujuAPI) UploadTools(r io.Reader, vers version.Binary, additionalSeries ...string) (
+	*coretools.Tools, error,
+) {
+	panic("not implemented")
+}
+
+func (a *fakeUpgradeJujuAPI) AbortCurrentUpgrade() error {
+	a.abortCurrentUpgradeCalled = true
+	return nil
+}
+
+func (a *fakeUpgradeJujuAPI) SetEnvironAgentVersion(v version.Number) error {
+	a.setVersionCalledWith = v
+	return a.setVersionErr
+}
+
+func (a *fakeUpgradeJujuAPI) Close() error {
+	return nil
+}
