@@ -4,17 +4,27 @@
 package main
 
 import (
+	"bytes"
 	"encoding/base64"
 	"fmt"
+	"io/ioutil"
 	"net"
-	"strconv"
+	"os"
+	"path/filepath"
+	"strings"
 	"time"
 
 	"github.com/juju/cmd"
+	"github.com/juju/errors"
+	"github.com/juju/loggo"
+	"github.com/juju/names"
 	goyaml "gopkg.in/yaml.v1"
 	"launchpad.net/gnuflag"
 
 	"github.com/juju/juju/agent"
+	agenttools "github.com/juju/juju/agent/tools"
+	agentcmd "github.com/juju/juju/cmd/jujud/agent"
+	cmdutil "github.com/juju/juju/cmd/jujud/util"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
@@ -22,22 +32,32 @@ import (
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/state"
-	"github.com/juju/juju/state/api/params"
+	"github.com/juju/juju/state/multiwatcher"
+	"github.com/juju/juju/state/storage"
+	"github.com/juju/juju/state/toolstorage"
+	"github.com/juju/juju/utils/ssh"
+	"github.com/juju/juju/version"
 	"github.com/juju/juju/worker/peergrouper"
 )
 
 var (
-	agentInitializeState = agent.InitializeState
-	minSocketTimeout     = 1 * time.Minute
+	maybeInitiateMongoServer = peergrouper.MaybeInitiateMongoServer
+	agentInitializeState     = agent.InitializeState
+	sshGenerateKey           = ssh.GenerateKey
+	newStateStorage          = storage.NewStorage
+	minSocketTimeout         = 1 * time.Minute
+	logger                   = loggo.GetLogger("juju.cmd.jujud")
 )
 
 type BootstrapCommand struct {
 	cmd.CommandBase
-	AgentConf
-	EnvConfig   map[string]interface{}
-	Constraints constraints.Value
-	Hardware    instance.HardwareCharacteristics
-	InstanceId  string
+	agentcmd.AgentConf
+	EnvConfig        map[string]interface{}
+	Constraints      constraints.Value
+	Hardware         instance.HardwareCharacteristics
+	InstanceId       string
+	AdminUsername    string
+	ImageMetadataDir string
 }
 
 // Info returns a decription of the command.
@@ -54,15 +74,20 @@ func (c *BootstrapCommand) SetFlags(f *gnuflag.FlagSet) {
 	f.Var(constraints.ConstraintsValue{Target: &c.Constraints}, "constraints", "initial environment constraints (space-separated strings)")
 	f.Var(&c.Hardware, "hardware", "hardware characteristics (space-separated strings)")
 	f.StringVar(&c.InstanceId, "instance-id", "", "unique instance-id for bootstrap machine")
+	f.StringVar(&c.AdminUsername, "admin-user", "admin", "set the name for the juju admin user")
+	f.StringVar(&c.ImageMetadataDir, "image-metadata", "", "custom image metadata source dir")
 }
 
 // Init initializes the command for running.
 func (c *BootstrapCommand) Init(args []string) error {
 	if len(c.EnvConfig) == 0 {
-		return requiredError("env-config")
+		return cmdutil.RequiredError("env-config")
 	}
 	if c.InstanceId == "" {
-		return requiredError("instance-id")
+		return cmdutil.RequiredError("instance-id")
+	}
+	if !names.IsValidUser(c.AdminUsername) {
+		return errors.Errorf("%q is not a valid username", c.AdminUsername)
 	}
 	return c.AgentConf.CheckArgs(args)
 }
@@ -78,15 +103,17 @@ func (c *BootstrapCommand) Run(_ *cmd.Context) error {
 		return err
 	}
 	agentConfig := c.CurrentConfig()
+	network.InitializeFromConfig(agentConfig)
 
 	// agent.Jobs is an optional field in the agent config, and was
 	// introduced after 1.17.2. We default to allowing units on
 	// machine-0 if missing.
 	jobs := agentConfig.Jobs()
 	if len(jobs) == 0 {
-		jobs = []params.MachineJob{
-			params.JobManageEnviron,
-			params.JobHostUnits,
+		jobs = []multiwatcher.MachineJob{
+			multiwatcher.JobManageEnviron,
+			multiwatcher.JobHostUnits,
+			multiwatcher.JobManageNetworking,
 		}
 	}
 
@@ -105,9 +132,19 @@ func (c *BootstrapCommand) Run(_ *cmd.Context) error {
 		return err
 	}
 
-	// Create system-identity file
-	if err := agent.WriteSystemIdentityFile(agentConfig); err != nil {
-		return err
+	// Generate a private SSH key for the state servers, and add
+	// the public key to the environment config. We'll add the
+	// private key to StateServingInfo below.
+	privateKey, publicKey, err := sshGenerateKey(config.JujuSystemKey)
+	if err != nil {
+		return errors.Annotate(err, "failed to generate system key")
+	}
+	authorizedKeys := config.ConcatAuthKeys(envCfg.AuthorizedKeys(), publicKey)
+	envCfg, err = env.Config().Apply(map[string]interface{}{
+		config.AuthKeysConfig: authorizedKeys,
+	})
+	if err != nil {
+		return errors.Annotate(err, "failed to add public key to environment config")
 	}
 
 	// Generate a shared secret for the Mongo replica set, and write it out.
@@ -120,14 +157,21 @@ func (c *BootstrapCommand) Run(_ *cmd.Context) error {
 		return fmt.Errorf("bootstrap machine config has no state serving info")
 	}
 	info.SharedSecret = sharedSecret
-	err = c.ChangeConfig(func(agentConfig agent.ConfigSetter) {
+	info.SystemIdentity = privateKey
+	err = c.ChangeConfig(func(agentConfig agent.ConfigSetter) error {
 		agentConfig.SetStateServingInfo(info)
+		return nil
 	})
 	if err != nil {
 		return fmt.Errorf("cannot write agent config: %v", err)
 	}
 	agentConfig = c.CurrentConfig()
 
+	// Create system-identity file
+	if err := agent.WriteSystemIdentityFile(agentConfig); err != nil {
+		return err
+	}
+
 	if err := c.startMongo(addrs, agentConfig); err != nil {
 		return err
 	}
@@ -136,8 +180,8 @@ func (c *BootstrapCommand) Run(_ *cmd.Context) error {
 	// Initialise state, and store any agent config (e.g. password) changes.
 	var st *state.State
 	var m *state.Machine
-	err = nil
-	writeErr := c.ChangeConfig(func(agentConfig agent.ConfigSetter) {
+	err = c.ChangeConfig(func(agentConfig agent.ConfigSetter) error {
+		var stateErr error
 		dialOpts := mongo.DefaultDialOpts()
 
 		// Set a longer socket timeout than usual, as the machine
@@ -152,7 +196,9 @@ func (c *BootstrapCommand) Run(_ *cmd.Context) error {
 		// We shouldn't attempt to dial peers until we have some.
 		dialOpts.Direct = true
 
-		st, m, err = agentInitializeState(
+		adminTag := names.NewLocalUserTag(c.AdminUsername)
+		st, m, stateErr = agentInitializeState(
+			adminTag,
 			agentConfig,
 			envCfg,
 			agent.BootstrapMachineConfig{
@@ -166,50 +212,34 @@ func (c *BootstrapCommand) Run(_ *cmd.Context) error {
 			dialOpts,
 			environs.NewStatePolicy(),
 		)
+		return stateErr
 	})
-	if writeErr != nil {
-		return fmt.Errorf("cannot write initial configuration: %v", err)
-	}
 	if err != nil {
 		return err
 	}
 	defer st.Close()
 
-	// bootstrap machine always gets the vote
-	return m.SetHasVote(true)
-}
+	// Populate the tools catalogue.
+	if err := c.populateTools(st, env); err != nil {
+		return err
+	}
 
-// newEnsureServerParams creates an EnsureServerParams from an agent configuration.
-func newEnsureServerParams(agentConfig agent.Config) (mongo.EnsureServerParams, error) {
-	// If oplog size is specified in the agent configuration, use that.
-	// Otherwise leave the default zero value to indicate to EnsureServer
-	// that it should calculate the size.
-	var oplogSize int
-	if oplogSizeString := agentConfig.Value(agent.MongoOplogSize); oplogSizeString != "" {
-		var err error
-		if oplogSize, err = strconv.Atoi(oplogSizeString); err != nil {
-			return mongo.EnsureServerParams{}, fmt.Errorf("invalid oplog size: %q", oplogSizeString)
+	// Add custom image metadata to environment storage.
+	if c.ImageMetadataDir != "" {
+		stor := newStateStorage(st.EnvironUUID(), st.MongoSession())
+		if err := c.storeCustomImageMetadata(stor); err != nil {
+			return err
 		}
 	}
 
-	servingInfo, ok := agentConfig.StateServingInfo()
-	if !ok {
-		return mongo.EnsureServerParams{}, fmt.Errorf("agent config has no state serving info")
-	}
-
-	params := mongo.EnsureServerParams{
-		StateServingInfo: servingInfo,
-		DataDir:          agentConfig.DataDir(),
-		Namespace:        agentConfig.Value(agent.Namespace),
-		OplogSize:        oplogSize,
-	}
-	return params, nil
+	// bootstrap machine always gets the vote
+	return m.SetHasVote(true)
 }
 
 func (c *BootstrapCommand) startMongo(addrs []network.Address, agentConfig agent.Config) error {
 	logger.Debugf("starting mongo")
 
-	info, ok := agentConfig.StateInfo()
+	info, ok := agentConfig.MongoInfo()
 	if !ok {
 		return fmt.Errorf("no state info available")
 	}
@@ -233,11 +263,11 @@ func (c *BootstrapCommand) startMongo(addrs []network.Address, agentConfig agent
 	}
 
 	logger.Debugf("calling ensureMongoServer")
-	ensureServerParams, err := newEnsureServerParams(agentConfig)
+	ensureServerParams, err := cmdutil.NewEnsureServerParams(agentConfig)
 	if err != nil {
 		return err
 	}
-	err = ensureMongoServer(ensureServerParams)
+	err = cmdutil.EnsureMongoServer(ensureServerParams)
 	if err != nil {
 		return err
 	}
@@ -254,6 +284,84 @@ func (c *BootstrapCommand) startMongo(addrs []network.Address, agentConfig agent
 	})
 }
 
+// populateTools stores uploaded tools in provider storage
+// and updates the tools metadata.
+func (c *BootstrapCommand) populateTools(st *state.State, env environs.Environ) error {
+	agentConfig := c.CurrentConfig()
+	dataDir := agentConfig.DataDir()
+	tools, err := agenttools.ReadTools(dataDir, version.Current)
+	if err != nil {
+		return err
+	}
+
+	data, err := ioutil.ReadFile(filepath.Join(
+		agenttools.SharedToolsDir(dataDir, version.Current),
+		"tools.tar.gz",
+	))
+	if err != nil {
+		return err
+	}
+
+	storage, err := st.ToolsStorage()
+	if err != nil {
+		return err
+	}
+	defer storage.Close()
+
+	var toolsVersions []version.Binary
+	if strings.HasPrefix(tools.URL, "file://") {
+		// Tools were uploaded: clone for each series of the same OS.
+		osSeries := version.OSSupportedSeries(tools.Version.OS)
+		for _, series := range osSeries {
+			toolsVersion := tools.Version
+			toolsVersion.Series = series
+			toolsVersions = append(toolsVersions, toolsVersion)
+		}
+	} else {
+		// Tools were downloaded from an external source: don't clone.
+		toolsVersions = []version.Binary{tools.Version}
+	}
+
+	for _, toolsVersion := range toolsVersions {
+		metadata := toolstorage.Metadata{
+			Version: toolsVersion,
+			Size:    tools.Size,
+			SHA256:  tools.SHA256,
+		}
+		logger.Debugf("Adding tools: %v", toolsVersion)
+		if err := storage.AddTools(bytes.NewReader(data), metadata); err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
+// storeCustomImageMetadata reads the custom image metadata from disk,
+// and stores the files in environment storage with the same relative
+// paths.
+func (c *BootstrapCommand) storeCustomImageMetadata(stor storage.Storage) error {
+	logger.Debugf("storing custom image metadata from %q", c.ImageMetadataDir)
+	return filepath.Walk(c.ImageMetadataDir, func(abspath string, info os.FileInfo, err error) error {
+		if err != nil {
+			return err
+		}
+		if !info.Mode().IsRegular() {
+			return nil
+		}
+		relpath, err := filepath.Rel(c.ImageMetadataDir, abspath)
+		if err != nil {
+			return err
+		}
+		f, err := os.Open(abspath)
+		if err != nil {
+			return err
+		}
+		defer f.Close()
+		logger.Debugf("storing %q in environment storage (%d bytes)", relpath, info.Size())
+		return stor.Put(relpath, f, info.Size())
+	})
+}
+
 // yamlBase64Value implements gnuflag.Value on a map[string]interface{}.
 type yamlBase64Value map[string]interface{}
 
