@@ -19,10 +19,12 @@ import (
 	"github.com/juju/cmd"
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
+	"github.com/juju/names"
 	"github.com/juju/utils"
 	goyaml "gopkg.in/yaml.v1"
 	"launchpad.net/gnuflag"
 
+	"github.com/juju/juju/api"
 	"github.com/juju/juju/cmd/envcmd"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
@@ -31,11 +33,9 @@ import (
 	"github.com/juju/juju/environs/configstore"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju"
-	"github.com/juju/juju/mongo"
-	"github.com/juju/juju/network"
 	_ "github.com/juju/juju/provider/all"
-	"github.com/juju/juju/state"
-	"github.com/juju/juju/state/api"
+	"github.com/juju/juju/provider/common"
+	"github.com/juju/juju/state/backups"
 	"github.com/juju/juju/utils/ssh"
 )
 
@@ -109,10 +109,25 @@ var updateBootstrapMachineTemplate = mustParseTemplate(`
 	export LC_ALL=C
 	tar xzf juju-backup.tgz
 	test -d juju-backup
-	apt-get --option=Dpkg::Options::=--force-confold --option=Dpkg::options::=--force-unsafe-io --assume-yes --quiet install mongodb-clients
-	
+
+
 	initctl stop jujud-machine-0
 
+	#The code apt-get throws when lock is taken
+	APTOUTPUT=100 
+	while [ $APTOUTPUT -gt 0 ]
+	do
+		# We will try to run apt-get and it can fail if other dpkg is in use
+		# the subshell call is not reached by -e so we can have apt-get fail
+		APTOUTPUT=$(apt-get --option=Dpkg::Options::=--force-confold --option=Dpkg::options::=--force-unsafe-io --assume-yes --quiet install mongodb-clients &> /dev/null; echo $?)
+		if [ $APTOUTPUT -gt 0 ] && [ $APTOUTPUT -ne 100 ]; then
+			echo "apt-get failed with an irrecoverable error $APTOUTPUT";
+			exit 1
+		fi
+	done
+	
+
+
 	initctl stop juju-db
 	rm -r /var/lib/juju
 	rm -r /var/log/juju
@@ -152,11 +167,12 @@ var updateBootstrapMachineTemplate = mustParseTemplate(`
 	# Remove all state machines but 0, to restore HA
 	mongoAdminEval '
 		db = db.getSiblingDB("juju")
-		db.machines.update({_id: "0"}, {$set: {instanceid: {{.NewInstanceId | printf "%q" }} } })
+		db.machines.update({machineid: "0"}, {$set: {instanceid: {{.NewInstanceId | printf "%q" }} } })
+		db.machines.update({machineid: "0"}, {$set: {"addresses": ["{{.Address}}"] } })
 		db.instanceData.update({_id: "0"}, {$set: {instanceid: {{.NewInstanceId | printf "%q" }} } })
-		db.machines.remove({_id: {$ne:"0"}, hasvote: true})
-		db.stateServers.update({"_id":"e"}, {$set:{"machineids" : [0]}})
-		db.stateServers.update({"_id":"e"}, {$set:{"votingmachineids" : [0]}})
+		db.machines.remove({machineid: {$ne:"0"}, hasvote: true})
+		db.stateServers.update({"_id":"e"}, {$set:{"machineids" : ["0"]}})
+		db.stateServers.update({"_id":"e"}, {$set:{"votingmachineids" : ["0"]}})
 	'
 
 	# Give time to replset to initiate
@@ -209,81 +225,56 @@ func (c *restoreCommand) Run(ctx *cmd.Context) error {
 	}
 	agentConf, err := extractConfig(c.backupFile)
 	if err != nil {
-		return fmt.Errorf("cannot extract configuration from backup file: %v", err)
+		return errors.Annotate(err, "cannot extract configuration from backup file")
 	}
 	progress("extracted credentials from backup file")
 	store, err := configstore.Default()
 	if err != nil {
 		return err
 	}
-	cfg, _, err := environs.ConfigForName(c.EnvName, store)
+	cfg, err := c.Config(store)
 	if err != nil {
 		return err
 	}
 	env, err := rebootstrap(cfg, ctx, c.Constraints)
 	if err != nil {
-		return fmt.Errorf("cannot re-bootstrap environment: %v", err)
+		return errors.Annotate(err, "cannot re-bootstrap environment")
 	}
 	progress("connecting to newly bootstrapped instance")
-	var conn *juju.APIConn
+	var apiState *api.State
 	// The state server backend may not be ready to accept logins so we retry.
 	// We'll do up to 8 retries over 2 minutes to give the server time to come up.
 	// Typically we expect only 1 retry will be needed.
 	attempt := utils.AttemptStrategy{Delay: 15 * time.Second, Min: 8}
+	// While specifying the admin user will work for now, as soon as we allow
+	// the users to have a different initial user name, or they have changed
+	// the password for the admin user, this will fail.
+	owner := names.NewUserTag("admin")
 	for a := attempt.Start(); a.Next(); {
-		conn, err = juju.NewAPIConn(env, api.DefaultDialOpts())
+		apiState, err = juju.NewAPIState(owner, env, api.DefaultDialOpts())
 		if err == nil || errors.Cause(err).Error() != "EOF" {
 			break
 		}
 		progress("bootstrapped instance not ready - attempting to redial")
 	}
 	if err != nil {
-		return fmt.Errorf("cannot connect to bootstrap instance: %v", err)
+		return errors.Annotate(err, "cannot connect to bootstrap instance")
 	}
 	progress("restoring bootstrap machine")
-	newInstId, machine0Addr, err := restoreBootstrapMachine(conn, c.backupFile, agentConf)
+	machine0Addr, err := restoreBootstrapMachine(apiState, c.backupFile, agentConf)
 	if err != nil {
-		return fmt.Errorf("cannot restore bootstrap machine: %v", err)
+		return errors.Annotate(err, "cannot restore bootstrap machine")
 	}
 	progress("restored bootstrap machine")
-	// Update the environ state to point to the new instance.
-	if err := bootstrap.SaveState(env.Storage(), &bootstrap.BootstrapState{
-		StateInstances: []instance.Id{newInstId},
-	}); err != nil {
-		return fmt.Errorf("cannot update environ bootstrap state storage: %v", err)
-	}
-	// Construct our own state info rather than using juju.NewConn so
-	// that we can avoid storage eventual-consistency issues
-	// (and it's faster too).
-	caCert, ok := cfg.CACert()
-	if !ok {
-		return fmt.Errorf("configuration has no CA certificate")
-	}
+
+	apiState, err = juju.NewAPIState(owner, env, api.DefaultDialOpts())
 	progress("opening state")
-	// We need to retry here to allow mongo to come up on the restored state server.
-	// The connection might succeed due to the mongo dial retries but there may still
-	// be a problem issuing database commands.
-	var st *state.State
-	for a := attempt.Start(); a.Next(); {
-		st, err = state.Open(&state.Info{
-			Info: mongo.Info{
-				Addrs:  []string{fmt.Sprintf("%s:%d", machine0Addr, cfg.StatePort())},
-				CACert: caCert,
-			},
-			Tag:      agentConf.Credentials.Tag,
-			Password: agentConf.Credentials.Password,
-		}, mongo.DefaultDialOpts(), environs.NewStatePolicy())
-		if err == nil {
-			break
-		}
-		progress("state server not ready - attempting to re-connect")
-	}
 	if err != nil {
-		return fmt.Errorf("cannot open state: %v", err)
+		return errors.Annotate(err, "cannot connect to api server")
 	}
 	progress("updating all machines")
-	if err := updateAllMachines(st, machine0Addr); err != nil {
-		return fmt.Errorf("cannot update machines: %v", err)
+	if err := updateAllMachines(apiState, machine0Addr); err != nil {
+		return errors.Annotate(err, "cannot update machines")
 	}
 	return nil
 }
@@ -300,32 +291,40 @@ func rebootstrap(cfg *config.Config, ctx *cmd.Context, cons constraints.Value) (
 		"provisioner-safe-mode": true,
 	})
 	if err != nil {
-		return nil, fmt.Errorf("cannot enable provisioner-safe-mode: %v", err)
+		return nil, errors.Annotate(err, "cannot enable provisioner-safe-mode")
 	}
 	env, err := environs.New(cfg)
 	if err != nil {
 		return nil, err
 	}
-	st, err := bootstrap.LoadState(env.Storage())
-	if err != nil {
-		return nil, fmt.Errorf("cannot retrieve environment storage; perhaps the environment was not bootstrapped: %v", err)
+	instanceIds, err := env.StateServerInstances()
+	switch errors.Cause(err) {
+	case nil, environs.ErrNoInstances:
+		// Some providers will return a nil error even
+		// if there are no live state server instances.
+		break
+	case environs.ErrNotBootstrapped:
+		return nil, errors.Trace(err)
+	default:
+		return nil, errors.Annotate(err, "cannot determine state server instances")
 	}
-	if len(st.StateInstances) == 0 {
-		return nil, fmt.Errorf("no instances found on bootstrap state; perhaps the environment was not bootstrapped")
-	}
-	if len(st.StateInstances) > 1 {
-		return nil, fmt.Errorf("restore does not support HA juju configurations yet")
-	}
-	inst, err := env.Instances(st.StateInstances)
-	if err == nil {
-		return nil, fmt.Errorf("old bootstrap instance %q still seems to exist; will not replace", inst)
-	}
-	if err != environs.ErrNoInstances {
-		return nil, fmt.Errorf("cannot detect whether old instance is still running: %v", err)
+	if len(instanceIds) > 0 {
+		instances, err := env.Instances(instanceIds)
+		switch errors.Cause(err) {
+		case nil, environs.ErrPartialInstances:
+			return nil, fmt.Errorf("old bootstrap instances %q still seems to exist; will not replace", instances)
+		case environs.ErrNoInstances:
+			// No state server instances, so keep running.
+			break
+		default:
+			return nil, errors.Annotate(err, "cannot detect whether old instance is still running")
+		}
 	}
 	// Remove the storage so that we can bootstrap without the provider complaining.
-	if err := env.Storage().Remove(bootstrap.StateFile); err != nil {
-		return nil, fmt.Errorf("cannot remove %q from storage: %v", bootstrap.StateFile, err)
+	if env, ok := env.(environs.EnvironStorage); ok {
+		if err := env.Storage().Remove(common.StateFile); err != nil {
+			return nil, errors.Annotate(err, fmt.Sprintf("cannot remove %q from storage", common.StateFile))
+		}
 	}
 
 	// TODO If we fail beyond here, then we won't have a state file and
@@ -334,42 +333,42 @@ func rebootstrap(cfg *config.Config, ctx *cmd.Context, cons constraints.Value) (
 	// error-prone) or we could provide a --no-check flag to make
 	// it go ahead anyway without the check.
 
-	args := environs.BootstrapParams{Constraints: cons}
-	if err := bootstrap.Bootstrap(ctx, env, args); err != nil {
-		return nil, fmt.Errorf("cannot bootstrap new instance: %v", err)
+	args := bootstrap.BootstrapParams{Constraints: cons}
+	if err := bootstrap.Bootstrap(envcmd.BootstrapContextNoVerify(ctx), env, args); err != nil {
+		return nil, errors.Annotate(err, "cannot bootstrap new instance")
 	}
 	return env, nil
 }
 
-func restoreBootstrapMachine(conn *juju.APIConn, backupFile string, agentConf agentConfig) (newInstId instance.Id, addr string, err error) {
-	client := conn.State.Client()
+func restoreBootstrapMachine(st *api.State, backupFile string, agentConf agentConfig) (addr string, err error) {
+	client := st.Client()
 	addr, err = client.PublicAddress("0")
 	if err != nil {
-		return "", "", fmt.Errorf("cannot get public address of bootstrap machine: %v", err)
+		return "", errors.Annotate(err, "cannot get public address of bootstrap machine")
 	}
 	paddr, err := client.PrivateAddress("0")
 	if err != nil {
-		return "", "", fmt.Errorf("cannot get private address of bootstrap machine: %v", err)
+		return "", errors.Annotate(err, "cannot get private address of bootstrap machine")
 	}
 	status, err := client.Status(nil)
 	if err != nil {
-		return "", "", fmt.Errorf("cannot get environment status: %v", err)
+		return "", errors.Annotate(err, "cannot get environment status")
 	}
 	info, ok := status.Machines["0"]
 	if !ok {
-		return "", "", fmt.Errorf("cannot find bootstrap machine in status")
+		return "", fmt.Errorf("cannot find bootstrap machine in status")
 	}
-	newInstId = instance.Id(info.InstanceId)
+	newInstId := instance.Id(info.InstanceId)
 
 	progress("copying backup file to bootstrap host")
 	if err := sendViaScp(backupFile, addr, "~/juju-backup.tgz"); err != nil {
-		return "", "", fmt.Errorf("cannot copy backup file to bootstrap instance: %v", err)
+		return "", errors.Annotate(err, "cannot copy backup file to bootstrap instance")
 	}
 	progress("updating bootstrap machine")
 	if err := runViaSsh(addr, updateBootstrapMachineScript(newInstId, agentConf, addr, paddr)); err != nil {
-		return "", "", fmt.Errorf("update script failed: %v", err)
+		return "", errors.Annotate(err, "update script failed")
 	}
-	return newInstId, addr, nil
+	return addr, nil
 }
 
 type credentials struct {
@@ -384,32 +383,73 @@ type agentConfig struct {
 	StatePort   string
 }
 
+func extractMachineID(archive *os.File) (string, error) {
+	paths := backups.NewCanonicalArchivePaths()
+
+	gzr, err := gzip.NewReader(archive)
+	if err != nil {
+		return "", errors.Annotate(err, fmt.Sprintf("cannot unzip %q", archive.Name()))
+	}
+	defer gzr.Close()
+
+	metaFile, err := findFileInTar(gzr, paths.MetadataFile)
+	if errors.IsNotFound(err) {
+		// Older archives don't have a metadata file and always have machine-0.
+		return "0", nil
+	}
+	if err != nil {
+		return "", errors.Trace(err)
+	}
+	meta, err := backups.NewMetadataJSONReader(metaFile)
+	if err != nil {
+		return "", errors.Trace(err)
+	}
+	return meta.Origin.Machine, nil
+}
+
 func extractConfig(backupFile string) (agentConfig, error) {
 	f, err := os.Open(backupFile)
 	if err != nil {
 		return agentConfig{}, err
 	}
 	defer f.Close()
+
+	// Extract the machine tag.
+	machineID, err := extractMachineID(f)
+	if err != nil {
+		return agentConfig{}, err
+	}
+	_, err = f.Seek(0, os.SEEK_SET)
+	if err != nil {
+		return agentConfig{}, err
+	}
+	tag := names.NewMachineTag(machineID)
+
+	// Extract the config file.
 	gzr, err := gzip.NewReader(f)
 	if err != nil {
-		return agentConfig{}, fmt.Errorf("cannot unzip %q: %v", backupFile, err)
+		return agentConfig{}, errors.Annotate(err, fmt.Sprintf("cannot unzip %q", backupFile))
 	}
 	defer gzr.Close()
 	outerTar, err := findFileInTar(gzr, "juju-backup/root.tar")
 	if err != nil {
 		return agentConfig{}, err
 	}
-	agentConf, err := findFileInTar(outerTar, "var/lib/juju/agents/machine-0/agent.conf")
+	// TODO(ericsnowcurrently) This should come from an authoritative source.
+	const confFilename = "var/lib/juju/agents/%s/agent.conf"
+	agentConf, err := findFileInTar(outerTar, fmt.Sprintf(confFilename, tag))
 	if err != nil {
 		return agentConfig{}, err
 	}
+
+	// Extract the config data.
 	data, err := ioutil.ReadAll(agentConf)
 	if err != nil {
-		return agentConfig{}, fmt.Errorf("failed to read agent config file: %v", err)
+		return agentConfig{}, errors.Annotate(err, "failed to read agent config file")
 	}
 	var conf interface{}
 	if err := goyaml.Unmarshal(data, &conf); err != nil {
-		return agentConfig{}, fmt.Errorf("cannot unmarshal agent config file: %v", err)
+		return agentConfig{}, errors.Annotate(err, "cannot unmarshal agent config file")
 	}
 	m, ok := conf.(map[interface{}]interface{})
 	if !ok {
@@ -450,8 +490,11 @@ func findFileInTar(r io.Reader, name string) (io.Reader, error) {
 	tarr := tar.NewReader(r)
 	for {
 		hdr, err := tarr.Next()
+		if err == io.EOF {
+			return nil, errors.NotFoundf(name)
+		}
 		if err != nil {
-			return nil, fmt.Errorf("%q not found: %v", name, err)
+			return nil, errors.Annotatef(err, "while looking for %q", name)
 		}
 		if path.Clean(hdr.Name) == name {
 			return tarr, nil
@@ -492,27 +535,28 @@ func setAgentAddressScript(stateAddr string) string {
 
 // updateAllMachines finds all machines and resets the stored state address
 // in each of them. The address does not include the port.
-func updateAllMachines(st *state.State, stateAddr string) error {
-	machines, err := st.AllMachines()
+func updateAllMachines(apiState *api.State, stateAddr string) error {
+	client := apiState.Client()
+	status, err := client.Status(nil)
 	if err != nil {
-		return err
+		return errors.Annotate(err, "cannot get status")
 	}
 	pendingMachineCount := 0
 	done := make(chan error)
-	for _, machine := range machines {
+	for _, machineStatus := range status.Machines {
 		// A newly resumed state server requires no updating, and more
 		// than one state server is not yet support by this plugin.
-		if machine.IsManager() || machine.Life() == state.Dead {
+		if machineStatus.HasVote || machineStatus.WantsVote || machineStatus.Life == "dead" {
 			continue
 		}
 		pendingMachineCount++
-		machine := machine
+		machine := machineStatus
 		go func() {
-			err := runMachineUpdate(machine, setAgentAddressScript(stateAddr))
+			err := runMachineUpdate(client, machine.Id, setAgentAddressScript(stateAddr))
 			if err != nil {
-				logger.Errorf("failed to update machine %s: %v", machine, err)
+				logger.Errorf("failed to update machine %s: %v", machine.Id, err)
 			} else {
-				progress("updated machine %s", machine)
+				progress("updated machine %s", machine.Id)
 			}
 			done <- err
 		}()
@@ -520,18 +564,18 @@ func updateAllMachines(st *state.State, stateAddr string) error {
 	err = nil
 	for ; pendingMachineCount > 0; pendingMachineCount-- {
 		if updateErr := <-done; updateErr != nil && err == nil {
-			err = fmt.Errorf("machine update failed")
+			err = errors.Annotate(updateErr, "machine update failed")
 		}
 	}
 	return err
 }
 
 // runMachineUpdate connects via ssh to the machine and runs the update script
-func runMachineUpdate(m *state.Machine, sshArg string) error {
-	progress("updating machine: %v\n", m)
-	addr := network.SelectPublicAddress(m.Addresses())
-	if addr == "" {
-		return fmt.Errorf("no appropriate public address found")
+func runMachineUpdate(client *api.Client, id string, sshArg string) error {
+	progress("updating machine: %v\n", id)
+	addr, err := client.PublicAddress(id)
+	if err != nil {
+		return errors.Annotate(err, "no public address found")
 	}
 	return runViaSsh(addr, sshArg)
 }
@@ -546,7 +590,7 @@ func runViaSsh(addr string, script string) error {
 	userCmd.Stdout = &stdoutBuf
 	err := userCmd.Run()
 	if err != nil {
-		return fmt.Errorf("ssh command failed: %v (%q)", err, stderrBuf.String())
+		return errors.Annotate(err, fmt.Sprintf("ssh command failed: (%q)", stderrBuf.String()))
 	}
 	progress("ssh command succedded: %q", stdoutBuf.String())
 	return nil
@@ -555,7 +599,7 @@ func runViaSsh(addr string, script string) error {
 func sendViaScp(file, host, destFile string) error {
 	err := ssh.Copy([]string{file, "ubuntu@" + host + ":" + destFile}, nil)
 	if err != nil {
-		return fmt.Errorf("scp command failed: %v", err)
+		return errors.Annotate(err, "scp command failed")
 	}
 	return nil
 }
@@ -571,7 +615,7 @@ func execTemplate(tmpl *template.Template, data interface{}) string {
 	var buf bytes.Buffer
 	err := tmpl.Execute(&buf, data)
 	if err != nil {
-		panic(fmt.Errorf("template error: %v", err))
+		panic(errors.Annotate(err, "template error"))
 	}
 	return buf.String()
 }
