@@ -17,6 +17,7 @@ import (
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/container"
 	"github.com/juju/juju/environs/cloudinit"
+	"github.com/juju/juju/environs/imagemetadata"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/version"
 )
@@ -96,10 +97,14 @@ type containerManager struct {
 
 var _ container.Manager = (*containerManager)(nil)
 
+// Exposed so tests can observe our side-effects
+var startParams StartParams
+
 func (manager *containerManager) CreateContainer(
 	machineConfig *cloudinit.MachineConfig,
 	series string,
-	network *container.NetworkConfig) (instance.Instance, *instance.HardwareCharacteristics, error) {
+	network *container.NetworkConfig,
+) (instance.Instance, *instance.HardwareCharacteristics, error) {
 
 	name := names.NewMachineTag(machineConfig.MachineId).String()
 	if manager.name != "" {
@@ -118,15 +123,23 @@ func (manager *containerManager) CreateContainer(
 	logger.Tracef("write cloud-init")
 	userDataFilename, err := container.WriteUserData(machineConfig, directory)
 	if err != nil {
-		return nil, nil, errors.LoggedErrorf(logger, "failed to write user data: %v", err)
+		err = errors.Annotate(err, "failed to write user data")
+		logger.Infof(err.Error())
+		return nil, nil, err
 	}
 	// Create the container.
-	startParams := ParseConstraintsToStartParams(machineConfig.Constraints)
+	startParams = ParseConstraintsToStartParams(machineConfig.Constraints)
 	startParams.Arch = version.Current.Arch
 	startParams.Series = series
 	startParams.Network = network
 	startParams.UserDataFile = userDataFilename
 
+	// If the Simplestream requested is anything but released, update
+	// our StartParams to request it.
+	if machineConfig.ImageStream != imagemetadata.ReleasedStream {
+		startParams.ImageDownloadUrl = imagemetadata.UbuntuCloudImagesURL + "/" + machineConfig.ImageStream
+	}
+
 	var hardware instance.HardwareCharacteristics
 	hardware, err = instance.ParseHardware(
 		fmt.Sprintf("arch=%s mem=%vM root-disk=%vG cpu-cores=%v",
@@ -137,12 +150,27 @@ func (manager *containerManager) CreateContainer(
 
 	logger.Tracef("create the container, constraints: %v", machineConfig.Constraints)
 	if err := kvmContainer.Start(startParams); err != nil {
-		return nil, nil, errors.LoggedErrorf(logger, "kvm container creation failed: %v", err)
+		err = errors.Annotate(err, "kvm container creation failed")
+		logger.Infof(err.Error())
+		return nil, nil, err
 	}
 	logger.Tracef("kvm container created")
 	return &kvmInstance{kvmContainer, name}, &hardware, nil
 }
 
+func (manager *containerManager) IsInitialized() bool {
+	requiredBinaries := []string{
+		"virsh",
+		"uvt-kvm",
+	}
+	for _, bin := range requiredBinaries {
+		if _, err := exec.LookPath(bin); err != nil {
+			return false
+		}
+	}
+	return true
+}
+
 func (manager *containerManager) DestroyContainer(id instance.Id) error {
 	name := string(id)
 	kvmContainer := KvmObjectFactory.New(name)
