@@ -3,16 +3,21 @@
 
 package lxc
 
-import "github.com/juju/juju/container"
+import (
+	"github.com/juju/juju/container"
+)
 
 var (
 	ContainerConfigFilename = containerConfigFilename
 	ContainerDirFilesystem  = containerDirFilesystem
 	GenerateNetworkConfig   = generateNetworkConfig
+	DiscoverHostNIC         = &discoverHostNIC
 	NetworkConfigTemplate   = networkConfigTemplate
 	RestartSymlink          = restartSymlink
 	ReleaseVersion          = &releaseVersion
 	PreferFastLXC           = preferFastLXC
+	InitProcessCgroupFile   = &initProcessCgroupFile
+	RuntimeGOOS             = &runtimeGOOS
 )
 
 func GetCreateWithCloneValue(mgr container.Manager) bool {
