@@ -4,33 +4,44 @@
 package lxc
 
 import (
+	"bufio"
+	"bytes"
 	"fmt"
 	"io/ioutil"
+	"net"
 	"os"
 	"os/exec"
 	"path/filepath"
+	"runtime"
 	"strconv"
 	"strings"
+	"text/template"
 	"time"
 
 	"github.com/juju/loggo"
 	"github.com/juju/names"
+	"github.com/juju/utils/symlink"
 	"launchpad.net/golxc"
 
+	"github.com/juju/errors"
 	"github.com/juju/juju/agent"
 	"github.com/juju/juju/container"
 	"github.com/juju/juju/environs/cloudinit"
 	"github.com/juju/juju/instance"
+	"github.com/juju/juju/juju/arch"
 	"github.com/juju/juju/version"
+	"github.com/juju/utils/keyvalues"
 )
 
 var logger = loggo.GetLogger("juju.container.lxc")
 
 var (
-	defaultTemplate  = "ubuntu-cloud"
-	LxcContainerDir  = "/var/lib/lxc"
-	LxcRestartDir    = "/etc/lxc/auto"
-	LxcObjectFactory = golxc.Factory()
+	defaultTemplate       = "ubuntu-cloud"
+	LxcContainerDir       = golxc.GetDefaultLXCContainerDir()
+	LxcRestartDir         = "/etc/lxc/auto"
+	LxcObjectFactory      = golxc.Factory()
+	initProcessCgroupFile = "/proc/1/cgroup"
+	runtimeGOOS           = runtime.GOOS
 )
 
 const (
@@ -65,22 +76,56 @@ func containerDirFilesystem() (string, error) {
 	return lines[1], nil
 }
 
+// IsLXCSupported returns a boolean value indicating whether or not
+// we can run LXC containers
+func IsLXCSupported() (bool, error) {
+	if runtimeGOOS != "linux" {
+		return false, nil
+	}
+
+	file, err := os.Open(initProcessCgroupFile)
+	if err != nil {
+		return false, errors.Trace(err)
+	}
+	defer file.Close()
+
+	scanner := bufio.NewScanner(file)
+	for scanner.Scan() {
+		line := scanner.Text()
+		fields := strings.Split(line, ":")
+		if len(fields) != 3 {
+			return false, errors.Errorf("Malformed cgroup file")
+		}
+		if fields[2] != "/" {
+			// When running in a container the anchor point will be something
+			// other then "/". Return false here as we do not support nested LXC
+			// containers
+			return false, nil
+		}
+	}
+
+	if err := scanner.Err(); err != nil {
+		return false, errors.Errorf("Failed to read cgroup file")
+	}
+	return true, nil
+}
+
 type containerManager struct {
 	name              string
 	logdir            string
-	toolsdir          string
 	createWithClone   bool
 	useAUFS           bool
 	backingFilesystem string
+	imageURLGetter    container.ImageURLGetter
 }
 
 // containerManager implements container.Manager.
 var _ container.Manager = (*containerManager)(nil)
 
-// NewContainerManager returns a manager object that can start and stop lxc
-// containers. The containers that are created are namespaced by the name
-// parameter.
-func NewContainerManager(conf container.ManagerConfig) (container.Manager, error) {
+// NewContainerManager returns a manager object that can start and
+// stop lxc containers. The containers that are created are namespaced
+// by the name parameter inside the given ManagerConfig.
+func NewContainerManager(conf container.ManagerConfig, imageURLGetter container.ImageURLGetter) (container.Manager, error) {
 	name := conf.PopValue(container.ConfigName)
 	if name == "" {
 		return nil, fmt.Errorf("name is required")
@@ -89,7 +134,6 @@ func NewContainerManager(conf container.ManagerConfig) (container.Manager, error
 	if logDir == "" {
 		logDir = agent.DefaultLogDir
 	}
-	toolsDir := conf.PopValue(container.ConfigToolsDir)
 	var useClone bool
 	useCloneVal := conf.PopValue("use-clone")
 	if useCloneVal != "" {
@@ -117,10 +161,10 @@ func NewContainerManager(conf container.ManagerConfig) (container.Manager, error
 	return &containerManager{
 		name:              name,
 		logdir:            logDir,
-		toolsdir:          toolsDir,
 		createWithClone:   useClone,
 		useAUFS:           useAUFS,
 		backingFilesystem: backingFS,
+		imageURLGetter:    imageURLGetter,
 	}, nil
 }
 
@@ -141,36 +185,43 @@ func preferFastLXC(release string) bool {
 	return value >= 14.04
 }
 
+// CreateContainer creates or clones an LXC container.
 func (manager *containerManager) CreateContainer(
 	machineConfig *cloudinit.MachineConfig,
 	series string,
 	network *container.NetworkConfig,
-) (instance.Instance, *instance.HardwareCharacteristics, error) {
-	start := time.Now()
+) (inst instance.Instance, _ *instance.HardwareCharacteristics, err error) {
+
+	// Check our preconditions
+	if manager == nil {
+		panic("manager is nil")
+	} else if series == "" {
+		panic("series not set")
+	} else if network == nil {
+		panic("network is nil")
+	}
+
+	// Log how long the start took
+	defer func(start time.Time) {
+		if err == nil {
+			logger.Tracef("container %q started: %v", inst.Id(), time.Now().Sub(start))
+		}
+	}(time.Now())
+
 	name := names.NewMachineTag(machineConfig.MachineId).String()
 	if manager.name != "" {
 		name = fmt.Sprintf("%s-%s", manager.name, name)
 	}
+
 	// Create the cloud-init.
 	directory, err := container.NewDirectory(name)
 	if err != nil {
-		return nil, nil, err
+		return nil, nil, errors.Annotate(err, "failed to create a directory for the container")
 	}
 	logger.Tracef("write cloud-init")
-	if manager.createWithClone {
-		// If we are using clone, disable the apt-get steps
-		machineConfig.DisablePackageCommands = true
-	}
 	userDataFilename, err := container.WriteUserData(machineConfig, directory)
 	if err != nil {
-		logger.Errorf("failed to write user data: %v", err)
-		return nil, nil, err
-	}
-	logger.Tracef("write the lxc.conf file")
-	configFile, err := writeLxcConfig(network, directory)
-	if err != nil {
-		logger.Errorf("failed to write config file: %v", err)
-		return nil, nil, err
+		return nil, nil, errors.Annotate(err, "failed to write user data")
 	}
 
 	var lxcContainer golxc.Container
@@ -181,9 +232,13 @@ func (manager *containerManager) CreateContainer(
 			network,
 			machineConfig.AuthorizedKeys,
 			machineConfig.AptProxySettings,
+			machineConfig.AptMirror,
+			machineConfig.EnableOSRefreshUpdate,
+			machineConfig.EnableOSUpgrade,
+			manager.imageURLGetter,
 		)
 		if err != nil {
-			return nil, nil, err
+			return nil, nil, errors.Annotate(err, "failed to retrieve the template to clone")
 		}
 		templateParams := []string{
 			"--debug",                      // Debug errors in the cloud image
@@ -200,13 +255,12 @@ func (manager *containerManager) CreateContainer(
 
 		lock, err := AcquireTemplateLock(templateContainer.Name(), "clone")
 		if err != nil {
-			return nil, nil, fmt.Errorf("failed to acquire lock on template: %v", err)
+			return nil, nil, errors.Annotate(err, "failed to acquire lock on template")
 		}
 		defer lock.Unlock()
 		lxcContainer, err = templateContainer.Clone(name, extraCloneArgs, templateParams)
 		if err != nil {
-			logger.Errorf("lxc container cloning failed: %v", err)
-			return nil, nil, err
+			return nil, nil, errors.Annotate(err, "lxc container cloning failed")
 		}
 	} else {
 		// Note here that the lxcObjectFacotry only returns a valid container
@@ -219,42 +273,136 @@ func (manager *containerManager) CreateContainer(
 			"--hostid", name, // Use the container name as the hostid
 			"-r", series,
 		}
-		// Create the container.
-		logger.Tracef("create the container")
-		if err := lxcContainer.Create(configFile, defaultTemplate, nil, templateParams); err != nil {
-			logger.Errorf("lxc container creation failed: %v", err)
+		var caCert []byte
+		if manager.imageURLGetter != nil {
+			arch := arch.HostArch()
+			imageURL, err := manager.imageURLGetter.ImageURL(instance.LXC, series, arch)
+			if err != nil {
+				return nil, nil, errors.Annotatef(err, "cannot determine cached image URL")
+			}
+			templateParams = append(templateParams, "-T", imageURL)
+			caCert = manager.imageURLGetter.CACert()
+		}
+		if err = createContainer(lxcContainer, network, directory, nil, templateParams, caCert); err != nil {
 			return nil, nil, err
 		}
-		logger.Tracef("lxc container created")
 	}
+
 	if err := autostartContainer(name); err != nil {
-		return nil, nil, err
+		return nil, nil, errors.Annotate(err, "failed to configure the container for autostart")
 	}
 	if err := mountHostLogDir(name, manager.logdir); err != nil {
-		return nil, nil, err
-	}
-	if err := mountHostToolsDir(name, manager.toolsdir); err != nil {
-		return nil, nil, err
+		return nil, nil, errors.Annotate(err, "failed to mount the directory to log to")
 	}
 	// Start the lxc container with the appropriate settings for grabbing the
 	// console output and a log file.
 	consoleFile := filepath.Join(directory, "console.log")
 	lxcContainer.SetLogFile(filepath.Join(directory, "container.log"), golxc.LogDebug)
 	logger.Tracef("start the container")
+
 	// We explicitly don't pass through the config file to the container.Start
 	// method as we have passed it through at container creation time.  This
 	// is necessary to get the appropriate rootfs reference without explicitly
 	// setting it ourselves.
 	if err = lxcContainer.Start("", consoleFile); err != nil {
-		logger.Errorf("container failed to start: %v", err)
+		logger.Warningf("container failed to start %v", err)
+		// if the container fails to start we should try to destroy it
+		// check if the container has been constructed
+		if lxcContainer.IsConstructed() {
+			// if so, then we need to destroy the leftover container
+			if derr := lxcContainer.Destroy(); derr != nil {
+				// if an error is reported there is probably a leftover
+				// container that the user should clean up manually
+				logger.Errorf("container failed to start and failed to destroy: %v", derr)
+				return nil, nil, errors.Annotate(err, "container failed to start and failed to destroy: manual cleanup of containers needed")
+			}
+			logger.Warningf("container failed to start and was destroyed - safe to retry")
+			return nil, nil, errors.Wrap(err, instance.NewRetryableCreationError("container failed to start and was destroyed: "+lxcContainer.Name()))
+		}
+		logger.Warningf("container failed to start: %v", err)
+		return nil, nil, errors.Annotate(err, "container failed to start")
+	}
+
+	hardware := &instance.HardwareCharacteristics{
+		Arch: &version.Current.Arch,
+	}
+
+	return &lxcInstance{lxcContainer, name}, hardware, nil
+}
+
+func createContainer(lxcContainer golxc.Container, network *container.NetworkConfig, containerDirectory string,
+	extraCreateArgs, templateParams []string, caCert []byte,
+) error {
+	logger.Tracef("write the lxc.conf file")
+	configFile, err := writeLxcConfig(network, containerDirectory)
+	if err != nil {
+		return errors.Annotatef(err, "failed to write config file")
+	}
+
+	var execEnv []string = nil
+	var closer func()
+	if caCert != nil {
+		execEnv, closer, err = wgetEnvironment(caCert)
+		if err != nil {
+			return errors.Annotatef(err, "failed to get environment for wget execution")
+		}
+		defer closer()
+	}
+
+	// Create the container.
+	logger.Debugf("create the lxc container")
+	logger.Debugf("lxc-create template params: %v", templateParams)
+	if err := lxcContainer.Create(configFile, defaultTemplate, extraCreateArgs, templateParams, execEnv); err != nil {
+		return errors.Annotatef(err, "lxc container creation failed")
+	}
+	return nil
+}
+
+// wgetEnvironment creates a script to call wget with the
+// --no-check-certificate argument, patching the PATH to ensure
+// the script is invoked by the lxc template bash script.
+// It returns a slice of env variables to pass to the lxc create command.
+func wgetEnvironment(caCert []byte) (execEnv []string, closer func(), _ error) {
+	env := os.Environ()
+	kv, err := keyvalues.Parse(env, true)
+	if err != nil {
 		return nil, nil, err
 	}
-	arch := version.Current.Arch
-	hardware := &instance.HardwareCharacteristics{
-		Arch: &arch,
+	// Create a wget bash script in a temporary directory.
+	tmpDir, err := ioutil.TempDir("", "wget")
+	if err != nil {
+		return nil, nil, err
 	}
-	logger.Tracef("container %q started: %v", name, time.Now().Sub(start))
-	return &lxcInstance{lxcContainer, name}, hardware, nil
+	closer = func() {
+		os.RemoveAll(tmpDir)
+	}
+	// Write the ca cert.
+	caCertPath := filepath.Join(tmpDir, "ca-cert.pem")
+	err = ioutil.WriteFile(caCertPath, caCert, 0755)
+	if err != nil {
+		defer closer()
+		return nil, nil, err
+	}
+
+	// Write the wget script.
+	wgetTmpl := `#!/bin/bash
+/usr/bin/wget --ca-certificate=%s $*
+`
+	wget := fmt.Sprintf(wgetTmpl, caCertPath)
+	err = ioutil.WriteFile(filepath.Join(tmpDir, "wget"), []byte(wget), 0755)
+	if err != nil {
+		defer closer()
+		return nil, nil, err
+	}
+
+	// Update the path to point to the script.
+	for k, v := range kv {
+		if strings.ToUpper(k) == "PATH" {
+			v = strings.Join([]string{tmpDir, v}, string(os.PathListSeparator))
+		}
+		execEnv = append(execEnv, fmt.Sprintf("%s=%s", k, v))
+	}
+	return execEnv, closer, nil
 }
 
 func appendToContainerConfig(name, line string) error {
@@ -274,7 +422,7 @@ func autostartContainer(name string) error {
 	// option should be set in the LXC config file, this is done in the networkConfigTemplate
 	// function below.
 	if useRestartDir() {
-		if err := os.Symlink(
+		if err := symlink.New(
 			containerConfigFilename(name),
 			restartSymlink(name),
 		); err != nil {
@@ -290,8 +438,9 @@ func autostartContainer(name string) error {
 
 func mountHostLogDir(name, logDir string) error {
 	// Make sure that the mount dir has been created.
-	logger.Tracef("make the mount dir for the shared logs")
-	if err := os.MkdirAll(internalLogDir(name), 0755); err != nil {
+	internalDir := internalLogDir(name)
+	logger.Tracef("make the mount dir for the shared logs: %s", internalDir)
+	if err := os.MkdirAll(internalDir, 0755); err != nil {
 		logger.Errorf("failed to create internal /var/log/juju mount dir: %v", err)
 		return err
 	}
@@ -301,22 +450,6 @@ func mountHostLogDir(name, logDir string) error {
 	return appendToContainerConfig(name, line)
 }
 
-func mountHostToolsDir(name, toolsDir string) error {
-	if toolsDir == "" {
-		return nil
-	}
-	// Make sure that the mount dir has been created.
-	logger.Tracef("make the mount dir for the tools")
-	if err := os.MkdirAll(internalToolsDir(name), 0755); err != nil {
-		logger.Errorf("failed to create internal /tmp/juju/tools mount dir: %v", err)
-		return err
-	}
-	line := fmt.Sprintf(
-		"lxc.mount.entry=%s tmp/juju/tools none defaults,bind 0 0\n",
-		toolsDir)
-	return appendToContainerConfig(name, line)
-}
-
 func (manager *containerManager) DestroyContainer(id instance.Id) error {
 	start := time.Now()
 	name := string(id)
@@ -362,18 +495,24 @@ func (manager *containerManager) ListContainers() (result []instance.Instance, e
 	return
 }
 
+func (manager *containerManager) IsInitialized() bool {
+	requiredBinaries := []string{
+		"lxc-ls",
+	}
+	for _, bin := range requiredBinaries {
+		if _, err := exec.LookPath(bin); err != nil {
+			return false
+		}
+	}
+	return true
+}
+
 const internalLogDirTemplate = "%s/%s/rootfs/var/log/juju"
 
 func internalLogDir(containerName string) string {
 	return fmt.Sprintf(internalLogDirTemplate, LxcContainerDir, containerName)
 }
 
-const internalToolsDirTemplate = "%s/%s/rootfs/tmp/juju/tools"
-
-func internalToolsDir(containerName string) string {
-	return fmt.Sprintf(internalToolsDirTemplate, LxcContainerDir, containerName)
-}
-
 func restartSymlink(name string) string {
 	return filepath.Join(LxcRestartDir, name+".conf")
 }
@@ -383,13 +522,75 @@ func containerConfigFilename(name string) string {
 }
 
 const networkTemplate = `
-lxc.network.type = %s
-lxc.network.link = %s
+lxc.network.type = {{.Type}}
+lxc.network.link = {{.Link}}
 lxc.network.flags = up
+{{if gt .MTU 0}}lxc.network.mtu = {{.MTU}}{{end}}
 `
 
 func networkConfigTemplate(networkType, networkLink string) string {
-	return fmt.Sprintf(networkTemplate, networkType, networkLink)
+	type config struct {
+		Type string
+		Link string
+		MTU  int
+	}
+	values := config{
+		Type: networkType,
+		Link: networkLink,
+	}
+
+	tmpl, err := template.New("config").Parse(networkTemplate)
+	if err != nil {
+		logger.Errorf("cannot parse container config template: %v", err)
+		return ""
+	}
+
+	primaryNIC, err := discoverHostNIC()
+	if err != nil {
+		logger.Warningf("cannot determine primary NIC MTU, not setting for container: %v", err)
+	} else {
+		logger.Debugf("setting container network interface MTU to %d", primaryNIC.MTU)
+		values.MTU = primaryNIC.MTU
+	}
+	var buf bytes.Buffer
+	if err := tmpl.Execute(&buf, values); err != nil {
+		logger.Errorf("cannot render container config: %v", err)
+		return ""
+	}
+	return buf.String()
+}
+
+// discoverHostNIC detects and returns the primary network interface
+// on the machine. Out of all interfaces, the first non-loopback
+// device which is up and has address is considered the primary.
+var discoverHostNIC = func() (net.Interface, error) {
+	interfaces, err := net.Interfaces()
+	if err != nil {
+		return net.Interface{}, errors.Annotatef(err, "cannot get network interfaces")
+	}
+	logger.Tracef("trying to discover primary network interface")
+	for _, iface := range interfaces {
+		if iface.Flags&net.FlagLoopback != 0 {
+			// Skip the loopback.
+			logger.Tracef("not using loopback interface %q", iface.Name)
+			continue
+		}
+		if iface.Flags&net.FlagUp != 0 {
+			// Possibly the primary, but ensure it has an address as
+			// well.
+			logger.Tracef("verifying interface %q has addresses", iface.Name)
+			addrs, err := iface.Addrs()
+			if err != nil {
+				return net.Interface{}, errors.Annotatef(err, "cannot get %q addresses", iface.Name)
+			}
+			if len(addrs) > 0 {
+				// We found it.
+				logger.Tracef("primary network interface is %q", iface.Name)
+				return iface, nil
+			}
+		}
+	}
+	return net.Interface{}, errors.Errorf("cannot detect the primary network interface")
 }
 
 func generateNetworkConfig(network *container.NetworkConfig) string {
