@@ -13,12 +13,12 @@ import (
 	"strings"
 	"time"
 
-	"github.com/juju/charm"
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	"github.com/juju/schema"
 	"github.com/juju/utils"
 	"github.com/juju/utils/proxy"
+	"gopkg.in/juju/charm.v4"
 
 	"github.com/juju/juju/cert"
 	"github.com/juju/juju/juju/osenv"
@@ -36,6 +36,12 @@ const (
 	// port opened.
 	FwGlobal = "global"
 
+	// FwNone requests that no firewalling should be performed inside
+	// the environment. No firewaller worker will be started. It's
+	// useful for clouds without support for either global or per
+	// instance security groups.
+	FwNone = "none"
+
 	// DefaultStatePort is the default port the state server is listening on.
 	DefaultStatePort int = 37017
 
@@ -62,8 +68,179 @@ const (
 	// fallbackLtsSeries is the latest LTS series we'll use, if we fail to
 	// obtain this information from the system.
 	fallbackLtsSeries string = "trusty"
+
+	// DefaultNumaControlPolicy should not be used by default.
+	// Only use numactl if user specifically requests it
+	DefaultNumaControlPolicy = false
+
+	// DefaultPreventDestroyEnvironment should not be used by default.
+	// Only prevent destroy-environment from running
+	// if user specifically requests it. Otherwise, let it run.
+	DefaultPreventDestroyEnvironment = false
+
+	// DefaultPreventRemoveObject should not be used by default.
+	// Only prevent remove-object from running
+	// if user specifically requests it. Otherwise, let it run.
+	// Object here is a juju artifact - machine, service, unit or relation.
+	DefaultPreventRemoveObject = false
+
+	// DefaultPreventAllChanges should not be used by default.
+	// Only prevent all-changes from running
+	// if user specifically requests it. Otherwise, let them run.
+	DefaultPreventAllChanges = false
 )
 
+// TODO(katco-): Please grow this over time.
+// Centralized place to store values of config keys. This transitions
+// mistakes in referencing key-values to a compile-time error.
+const (
+	//
+	// Settings Attributes
+	//
+
+	// ProvisionerHarvestModeKey stores the key for this setting.
+	ProvisionerHarvestModeKey = "provisioner-harvest-mode"
+
+	// AgentStreamKey stores the key for this setting.
+	AgentStreamKey = "agent-stream"
+
+	// AgentMetadataURLKey stores the key for this setting.
+	AgentMetadataURLKey = "agent-metadata-url"
+
+	// HttpProxyKey stores the key for this setting.
+	HttpProxyKey = "http-proxy"
+
+	// HttpsProxyKey stores the key for this setting.
+	HttpsProxyKey = "https-proxy"
+
+	// FtpProxyKey stores the key for this setting.
+	FtpProxyKey = "ftp-proxy"
+
+	// AptHttpProxyKey stores the key for this setting.
+	AptHttpProxyKey = "apt-http-proxy"
+
+	// AptHttpsProxyKey stores the key for this setting.
+	AptHttpsProxyKey = "apt-https-proxy"
+
+	// AptFtpProxyKey stores the key for this setting.
+	AptFtpProxyKey = "apt-ftp-proxy"
+
+	// NoProxyKey stores the key for this setting.
+	NoProxyKey = "no-proxy"
+
+	// LxcClone stores the value for this setting.
+	LxcClone = "lxc-clone"
+
+	// NumaControlPolicyKey stores the value for this setting
+	SetNumaControlPolicyKey = "set-numa-control-policy"
+
+	// BlockKeyPrefix is the prefix used for environment variables that block commands
+	BlockKeyPrefix = "block-"
+
+	// PreventDestroyEnvironmentKey stores the value for this setting
+	PreventDestroyEnvironmentKey = BlockKeyPrefix + "destroy-environment"
+
+	// PreventRemoveObjectKey stores the value for this setting
+	PreventRemoveObjectKey = BlockKeyPrefix + "remove-object"
+
+	// PreventAllChangesKey stores the value for this setting
+	PreventAllChangesKey = BlockKeyPrefix + "all-changes"
+
+	//
+	// Deprecated Settings Attributes
+	//
+
+	// Deprecated by provisioner-harvest-mode
+	// ProvisionerSafeModeKey stores the key for this setting.
+	ProvisionerSafeModeKey = "provisioner-safe-mode"
+
+	// Deprecated by agent-stream
+	// ToolsStreamKey stores the key for this setting.
+	ToolsStreamKey = "tools-stream"
+
+	// Deprecated by agent-metadata-url
+	// ToolsMetadataURLKey stores the key for this setting.
+	ToolsMetadataURLKey = "tools-metadata-url"
+
+	// Deprecated by use-clone
+	// LxcUseClone stores the key for this setting.
+	LxcUseClone = "lxc-use-clone"
+)
+
+// ParseHarvestMode parses description of harvesting method and
+// returns the representation.
+func ParseHarvestMode(description string) (HarvestMode, error) {
+	description = strings.ToLower(description)
+	for method, descr := range harvestingMethodToFlag {
+		if description == descr {
+			return method, nil
+		}
+	}
+	return 0, fmt.Errorf("unknown harvesting method: %s", description)
+}
+
+// HarvestMode is a bit field which is used to store the harvesting
+// behavior for Juju.
+type HarvestMode uint32
+
+const (
+	// HarvestNone signifies that Juju should not harvest any
+	// machines.
+	HarvestNone HarvestMode = 1 << iota
+	// HarvestUnknown signifies that Juju should only harvest machines
+	// which exist, but we don't know about.
+	HarvestUnknown
+	// HarvestDestroyed signifies that Juju should only harvest
+	// machines which have been explicitly released by the user
+	// through a destroy of a service/environment/unit.
+	HarvestDestroyed
+	// HarvestAll signifies that Juju should harvest both unknown and
+	// destroyed instances. ♫ Don't fear the reaper. ♫
+	HarvestAll HarvestMode = HarvestUnknown | HarvestDestroyed
+)
+
+// A mapping from method to description. Going this way will be the
+// more common operation, so we want this type of lookup to be O(1).
+var harvestingMethodToFlag = map[HarvestMode]string{
+	HarvestAll:       "all",
+	HarvestNone:      "none",
+	HarvestUnknown:   "unknown",
+	HarvestDestroyed: "destroyed",
+}
+
+// proxyAttrs contains attribute names that could contain loopback URLs, pointing to localhost
+var ProxyAttributes = []string{
+	HttpProxyKey,
+	HttpsProxyKey,
+	FtpProxyKey,
+	AptHttpProxyKey,
+	AptHttpsProxyKey,
+	AptFtpProxyKey,
+}
+
+// String returns the description of the harvesting mode.
+func (method HarvestMode) String() string {
+	if description, ok := harvestingMethodToFlag[method]; ok {
+		return description
+	}
+	panic("Unknown harvesting method.")
+}
+
+// None returns whether or not the None harvesting flag is set.
+func (method HarvestMode) HarvestNone() bool {
+	return method&HarvestNone != 0
+}
+
+// Destroyed returns whether or not the Destroyed harvesting flag is set.
+func (method HarvestMode) HarvestDestroyed() bool {
+	return method&HarvestDestroyed != 0
+}
+
+// Unknown returns whether or not the Unknown harvesting flag is set.
+func (method HarvestMode) HarvestUnknown() bool {
+	return method&HarvestUnknown != 0
+}
+
 var latestLtsSeries string
 
 type HasDefaultSeries interface {
@@ -245,36 +422,75 @@ func (c *Config) fillInStringDefault(attr string) {
 	}
 }
 
-// processDeprecatedAttributes ensures that the config is set up so that it works
-// correctly when used with older versions of Juju which require that deprecated
-// attribute values still be used.
-func (cfg *Config) processDeprecatedAttributes() {
+// ProcessDeprecatedAttributes gathers any deprecated attributes in attrs and adds or replaces
+// them with new name value pairs for the replacement attrs.
+// Ths ensures that older versions of Juju which require that deprecated
+// attribute values still be used will work as expected.
+func ProcessDeprecatedAttributes(attrs map[string]interface{}) map[string]interface{} {
+	processedAttrs := make(map[string]interface{}, len(attrs))
+	for k, v := range attrs {
+		processedAttrs[k] = v
+	}
 	// The tools url has changed so ensure that both old and new values are in the config so that
-	// upgrades work. "tools-url" is the old attribute name.
-	if oldToolsURL := cfg.defined["tools-url"]; oldToolsURL != nil && oldToolsURL.(string) != "" {
-		_, newToolsSpecified := cfg.ToolsURL()
-		// Ensure the new attribute name "tools-metadata-url" is set.
-		if !newToolsSpecified {
-			cfg.defined["tools-metadata-url"] = oldToolsURL
+	// upgrades work. "agent-metadata-url" is the old attribute name.
+	if oldToolsURL, ok := attrs[ToolsMetadataURLKey]; ok && oldToolsURL.(string) != "" {
+		if newTools, ok := attrs[AgentMetadataURLKey]; !ok || newTools.(string) == "" {
+			// Ensure the new attribute name "agent-metadata-url" is set.
+			processedAttrs[AgentMetadataURLKey] = oldToolsURL
 		}
+		// Even if the user has edited their environment yaml to remove the deprecated tools-metadata-url value,
+		// we still want it in the config for upgrades.
+		processedAttrs[ToolsMetadataURLKey] = processedAttrs[AgentMetadataURLKey]
 	}
-	// Even if the user has edited their environment yaml to remove the deprecated tools-url value,
-	// we still want it in the config for upgrades.
-	cfg.defined["tools-url"], _ = cfg.ToolsURL()
 
 	// Copy across lxc-use-clone to lxc-clone.
-	if lxcUseClone, ok := cfg.defined["lxc-use-clone"]; ok {
-		_, newValSpecified := cfg.LXCUseClone()
+	if lxcUseClone, ok := attrs[LxcUseClone]; ok {
+		_, newValSpecified := attrs[LxcClone]
 		// Ensure the new attribute name "lxc-clone" is set.
 		if !newValSpecified {
-			cfg.defined["lxc-clone"] = lxcUseClone
+			processedAttrs[LxcClone] = lxcUseClone
 		}
 	}
 
 	// Update the provider type from null to manual.
-	if cfg.Type() == "null" {
-		cfg.defined["type"] = "manual"
+	if attrs["type"] == "null" {
+		processedAttrs["type"] = "manual"
 	}
+
+	if _, ok := attrs[ProvisionerHarvestModeKey]; !ok {
+		if safeMode, ok := attrs[ProvisionerSafeModeKey].(bool); ok {
+
+			var harvestModeDescr string
+			if safeMode {
+				harvestModeDescr = HarvestDestroyed.String()
+			} else {
+				harvestModeDescr = HarvestAll.String()
+			}
+
+			processedAttrs[ProvisionerHarvestModeKey] = harvestModeDescr
+
+			logger.Infof(
+				`Based on your "%s" setting, configuring "%s" to "%s".`,
+				ProvisionerSafeModeKey,
+				ProvisionerHarvestModeKey,
+				harvestModeDescr,
+			)
+		}
+	}
+
+	//Update agent-stream from tools-stream if agent-stream was not specified but tools-stream was.
+	if _, ok := attrs[AgentStreamKey]; !ok {
+		if toolsKey, ok := attrs[ToolsStreamKey]; ok {
+			processedAttrs[AgentStreamKey] = toolsKey
+			logger.Infof(
+				`Based on your "%s" setting, configuring "%s" to "%s".`,
+				ToolsStreamKey,
+				AgentStreamKey,
+				toolsKey,
+			)
+		}
+	}
+	return processedAttrs
 }
 
 // Validate ensures that config is a valid configuration.  If old is not nil,
@@ -325,7 +541,9 @@ func Validate(cfg, old *Config) error {
 	}
 
 	// Check firewall mode.
-	if mode := cfg.FirewallMode(); mode != FwInstance && mode != FwGlobal {
+	switch mode := cfg.FirewallMode(); mode {
+	case FwInstance, FwGlobal, FwNone:
+	default:
 		return fmt.Errorf("invalid firewall mode in environment configuration: %q", mode)
 	}
 
@@ -345,11 +563,33 @@ func Validate(cfg, old *Config) error {
 			" of key-value pairs, not %q", authToken)
 	}
 
+	// Ensure that the given harvesting method is valid.
+	if hvstMeth, ok := cfg.defined[ProvisionerHarvestModeKey].(string); ok {
+		if _, err := ParseHarvestMode(hvstMeth); err != nil {
+			return err
+		}
+	}
+
 	// Check the immutable config values.  These can't change
 	if old != nil {
 		for _, attr := range immutableAttributes {
-			if newv, oldv := cfg.defined[attr], old.defined[attr]; newv != oldv {
-				return fmt.Errorf("cannot change %s from %#v to %#v", attr, oldv, newv)
+			switch attr {
+			case "uuid":
+				// uuid is special cased because currently (24/July/2014) there exist no juju
+				// environments whose environment configuration's contain a uuid key so we must
+				// treat uuid as field that can be updated from non existant to a valid uuid.
+				// We do not need to deal with the case of the uuid key being blank as the schema
+				// only permits valid uuids in that field.
+				oldv, oldexists := old.defined[attr]
+				newv := cfg.defined[attr]
+				if oldexists && oldv != newv {
+					newv := cfg.defined[attr]
+					return fmt.Errorf("cannot change %s from %#v to %#v", attr, oldv, newv)
+				}
+			default:
+				if newv, oldv := cfg.defined[attr], old.defined[attr]; newv != oldv {
+					return fmt.Errorf("cannot change %s from %#v to %#v", attr, oldv, newv)
+				}
 			}
 		}
 		if _, oldFound := old.AgentVersion(); oldFound {
@@ -359,7 +599,7 @@ func Validate(cfg, old *Config) error {
 		}
 	}
 
-	cfg.processDeprecatedAttributes()
+	cfg.defined = ProcessDeprecatedAttributes(cfg.defined)
 	return nil
 }
 
@@ -462,6 +702,16 @@ func (c *Config) Name() string {
 	return c.mustString("name")
 }
 
+// UUID returns the uuid for the environment.
+// For backwards compatability with 1.20 and earlier the value may be blank if
+// no uuid is present in this configuration. Once all enviroment configurations
+// have been upgraded, this relaxation will be dropped. The absence of a uuid
+// is indicated by a result of "", false.
+func (c *Config) UUID() (string, bool) {
+	value, exists := c.defined["uuid"].(string)
+	return value, exists
+}
+
 // DefaultSeries returns the configured default Ubuntu series for the environment,
 // and whether the default series was explicitly configured on the environment.
 func (c *Config) DefaultSeries() (string, bool) {
@@ -490,6 +740,44 @@ func (c *Config) SyslogPort() int {
 	return c.mustInt("syslog-port")
 }
 
+// NumaCtlPreference returns if numactl is preferred.
+func (c *Config) NumaCtlPreference() bool {
+	if numa, ok := c.defined[SetNumaControlPolicyKey]; ok {
+		return numa.(bool)
+	}
+	return DefaultNumaControlPolicy
+}
+
+// PreventDestroyEnvironment returns if destroy-environment
+// should be blocked from proceeding, thus preventing the operation.
+func (c *Config) PreventDestroyEnvironment() bool {
+	if attrValue, ok := c.defined[PreventDestroyEnvironmentKey]; ok {
+		return attrValue.(bool)
+	}
+	return DefaultPreventDestroyEnvironment
+}
+
+// PreventRemoveObject returns if remove-object
+// should be blocked from proceeding, thus preventing the operation.
+// Object in this context is a juju artifact: either a machine,
+// a service, a unit or a relation.
+func (c *Config) PreventRemoveObject() bool {
+	if attrValue, ok := c.defined[PreventRemoveObjectKey]; ok {
+		return attrValue.(bool)
+	}
+	return DefaultPreventRemoveObject
+}
+
+// PreventAllChanges returns if all-changes
+// should be blocked from proceeding, thus preventing the operation.
+// Changes in this context are any alterations to current environment.
+func (c *Config) PreventAllChanges() bool {
+	if attrValue, ok := c.defined[PreventAllChangesKey]; ok {
+		return attrValue.(bool)
+	}
+	return DefaultPreventAllChanges
+}
+
 // RsyslogCACert returns the certificate of the CA that signed the
 // rsyslog certificate, in PEM format, or nil if one hasn't been
 // generated yet.
@@ -500,6 +788,16 @@ func (c *Config) RsyslogCACert() string {
 	return ""
 }
 
+// RsyslogCAKey returns the key of the CA that signed the
+// rsyslog certificate, in PEM format, or nil if one hasn't been
+// generated yet.
+func (c *Config) RsyslogCAKey() string {
+	if s, ok := c.defined["rsyslog-ca-key"]; ok {
+		return s.(string)
+	}
+	return ""
+}
+
 // AuthorizedKeys returns the content for ssh's authorized_keys file.
 func (c *Config) AuthorizedKeys() string {
 	return c.mustString("authorized-keys")
@@ -525,22 +823,22 @@ func (c *Config) ProxySettings() proxy.Settings {
 
 // HttpProxy returns the http proxy for the environment.
 func (c *Config) HttpProxy() string {
-	return c.asString("http-proxy")
+	return c.asString(HttpProxyKey)
 }
 
 // HttpsProxy returns the https proxy for the environment.
 func (c *Config) HttpsProxy() string {
-	return c.asString("https-proxy")
+	return c.asString(HttpsProxyKey)
 }
 
 // FtpProxy returns the ftp proxy for the environment.
 func (c *Config) FtpProxy() string {
-	return c.asString("ftp-proxy")
+	return c.asString(FtpProxyKey)
 }
 
 // NoProxy returns the 'no proxy' for the environment.
 func (c *Config) NoProxy() string {
-	return c.asString("no-proxy")
+	return c.asString(NoProxyKey)
 }
 
 func (c *Config) getWithFallback(key, fallback string) string {
@@ -551,6 +849,14 @@ func (c *Config) getWithFallback(key, fallback string) string {
 	return value
 }
 
+// addSchemeIfMissing adds a scheme to a URL if it is missing
+func addSchemeIfMissing(defaultScheme string, url string) string {
+	if url != "" && !strings.Contains(url, "://") {
+		url = defaultScheme + "://" + url
+	}
+	return url
+}
+
 // AptProxySettings returns all three proxy settings; http, https and ftp.
 func (c *Config) AptProxySettings() proxy.Settings {
 	return proxy.Settings{
@@ -563,19 +869,24 @@ func (c *Config) AptProxySettings() proxy.Settings {
 // AptHttpProxy returns the apt http proxy for the environment.
 // Falls back to the default http-proxy if not specified.
 func (c *Config) AptHttpProxy() string {
-	return c.getWithFallback("apt-http-proxy", "http-proxy")
+	return addSchemeIfMissing("http", c.getWithFallback(AptHttpProxyKey, HttpProxyKey))
 }
 
 // AptHttpsProxy returns the apt https proxy for the environment.
 // Falls back to the default https-proxy if not specified.
 func (c *Config) AptHttpsProxy() string {
-	return c.getWithFallback("apt-https-proxy", "https-proxy")
+	return addSchemeIfMissing("https", c.getWithFallback(AptHttpsProxyKey, HttpsProxyKey))
 }
 
 // AptFtpProxy returns the apt ftp proxy for the environment.
 // Falls back to the default ftp-proxy if not specified.
 func (c *Config) AptFtpProxy() string {
-	return c.getWithFallback("apt-ftp-proxy", "ftp-proxy")
+	return addSchemeIfMissing("ftp", c.getWithFallback(AptFtpProxyKey, FtpProxyKey))
+}
+
+// AptMirror sets the apt mirror for the environment.
+func (c *Config) AptMirror() string {
+	return c.asString("apt-mirror")
 }
 
 // BootstrapSSHOpts returns the SSH timeout and retry delays used
@@ -626,8 +937,8 @@ func (c *Config) AdminSecret() string {
 }
 
 // FirewallMode returns whether the firewall should
-// manage ports per machine or global
-// (FwInstance or FwGlobal)
+// manage ports per machine, globally, or not at all.
+// (FwInstance, FwGlobal, or FwNone).
 func (c *Config) FirewallMode() string {
 	return c.mustString("firewall-mode")
 }
@@ -646,10 +957,10 @@ func (c *Config) AgentVersion() (version.Number, bool) {
 	return version.Zero, false
 }
 
-// ToolsURL returns the URL that locates the tools tarballs and metadata,
+// AgentMetadataURL returns the URL that locates the agent tarballs and metadata,
 // and whether it has been set.
-func (c *Config) ToolsURL() (string, bool) {
-	if url, ok := c.defined["tools-metadata-url"]; ok && url != "" {
+func (c *Config) AgentMetadataURL() (string, bool) {
+	if url, ok := c.defined[AgentMetadataURLKey]; ok && url != "" {
 		return url.(string), true
 	}
 	return "", false
@@ -669,6 +980,33 @@ func (c *Config) Development() bool {
 	return c.defined["development"].(bool)
 }
 
+// PreferIPv6 returns whether IPv6 addresses for API endpoints and
+// machines will be preferred (when available) over IPv4.
+func (c *Config) PreferIPv6() bool {
+	v, _ := c.defined["prefer-ipv6"].(bool)
+	return v
+}
+
+// EnableOSRefreshUpdate returns whether or not newly provisioned
+// instances should run their respective OS's update capability.
+func (c *Config) EnableOSRefreshUpdate() bool {
+	if val, ok := c.defined["enable-os-refresh-update"].(bool); !ok {
+		return true
+	} else {
+		return val
+	}
+}
+
+// EnableOSUpgrade returns whether or not newly provisioned instances
+// should run their respective OS's upgrade capability.
+func (c *Config) EnableOSUpgrade() bool {
+	if val, ok := c.defined["enable-os-upgrade"].(bool); !ok {
+		return true
+	} else {
+		return val
+	}
+}
+
 // SSLHostnameVerification returns weather the environment has requested
 // SSL hostname verification to be enabled.
 func (c *Config) SSLHostnameVerification() bool {
@@ -686,11 +1024,20 @@ func (c *Config) CharmStoreAuth() (string, bool) {
 	return auth, auth != ""
 }
 
-// ProvisionerSafeMode reports whether the provisioner should not
-// destroy machines it does not know about.
-func (c *Config) ProvisionerSafeMode() bool {
-	v, _ := c.defined["provisioner-safe-mode"].(bool)
-	return v
+// ProvisionerHarvestMode reports the harvesting methodology the
+// provisioner should take.
+func (c *Config) ProvisionerHarvestMode() HarvestMode {
+	if v, ok := c.defined[ProvisionerHarvestModeKey].(string); ok {
+		if method, err := ParseHarvestMode(v); err != nil {
+			// This setting should have already been validated. Don't
+			// burden the caller with handling any errors.
+			panic(err)
+		} else {
+			return method
+		}
+	} else {
+		return HarvestDestroyed
+	}
 }
 
 // ImageStream returns the simplestreams stream
@@ -704,6 +1051,17 @@ func (c *Config) ImageStream() string {
 	return "released"
 }
 
+// AgentStream returns the simplestreams stream
+// used to identify which tools to use when
+// when bootstrapping or upgrading an environment.
+func (c *Config) AgentStream() string {
+	v, _ := c.defined[AgentStreamKey].(string)
+	if v != "" {
+		return v
+	}
+	return "released"
+}
+
 // TestMode indicates if the environment is intended for testing.
 // In this case, accessing the charm store does not affect statistical
 // data of the store.
@@ -714,7 +1072,7 @@ func (c *Config) TestMode() bool {
 // LXCUseClone reports whether the LXC provisioner should create a
 // template and use cloning to speed up container provisioning.
 func (c *Config) LXCUseClone() (bool, bool) {
-	v, ok := c.defined["lxc-clone"].(bool)
+	v, ok := c.defined[LxcClone].(bool)
 	return v, ok
 }
 
@@ -774,10 +1132,12 @@ func (c *Config) Apply(attrs map[string]interface{}) (*Config, error) {
 var fields = schema.Fields{
 	"type":                       schema.String(),
 	"name":                       schema.String(),
+	"uuid":                       schema.UUID(),
 	"default-series":             schema.String(),
-	"tools-metadata-url":         schema.String(),
+	AgentMetadataURLKey:          schema.String(),
 	"image-metadata-url":         schema.String(),
 	"image-stream":               schema.String(),
+	AgentStreamKey:               schema.String(),
 	"authorized-keys":            schema.String(),
 	"authorized-keys-path":       schema.String(),
 	"firewall-mode":              schema.String(),
@@ -793,28 +1153,39 @@ var fields = schema.Fields{
 	"api-port":                   schema.ForceInt(),
 	"syslog-port":                schema.ForceInt(),
 	"rsyslog-ca-cert":            schema.String(),
+	"rsyslog-ca-key":             schema.String(),
 	"logging-config":             schema.String(),
 	"charm-store-auth":           schema.String(),
-	"provisioner-safe-mode":      schema.Bool(),
-	"http-proxy":                 schema.String(),
-	"https-proxy":                schema.String(),
-	"ftp-proxy":                  schema.String(),
-	"no-proxy":                   schema.String(),
-	"apt-http-proxy":             schema.String(),
-	"apt-https-proxy":            schema.String(),
-	"apt-ftp-proxy":              schema.String(),
+	ProvisionerHarvestModeKey:    schema.String(),
+	HttpProxyKey:                 schema.String(),
+	HttpsProxyKey:                schema.String(),
+	FtpProxyKey:                  schema.String(),
+	NoProxyKey:                   schema.String(),
+	AptHttpProxyKey:              schema.String(),
+	AptHttpsProxyKey:             schema.String(),
+	AptFtpProxyKey:               schema.String(),
+	"apt-mirror":                 schema.String(),
 	"bootstrap-timeout":          schema.ForceInt(),
 	"bootstrap-retry-delay":      schema.ForceInt(),
 	"bootstrap-addresses-delay":  schema.ForceInt(),
 	"test-mode":                  schema.Bool(),
 	"proxy-ssh":                  schema.Bool(),
-	"lxc-clone":                  schema.Bool(),
+	LxcClone:                     schema.Bool(),
 	"lxc-clone-aufs":             schema.Bool(),
+	"prefer-ipv6":                schema.Bool(),
+	"enable-os-refresh-update":   schema.Bool(),
+	"enable-os-upgrade":          schema.Bool(),
 	"disable-network-management": schema.Bool(),
+	SetNumaControlPolicyKey:      schema.Bool(),
+	PreventDestroyEnvironmentKey: schema.Bool(),
+	PreventRemoveObjectKey:       schema.Bool(),
+	PreventAllChangesKey:         schema.Bool(),
 
 	// Deprecated fields, retain for backwards compatibility.
-	"tools-url":     schema.String(),
-	"lxc-use-clone": schema.Bool(),
+	ToolsMetadataURLKey:    schema.String(),
+	LxcUseClone:            schema.Bool(),
+	ProvisionerSafeModeKey: schema.Bool(),
+	ToolsStreamKey:         schema.String(),
 }
 
 // alwaysOptional holds configuration defaults for attributes that may
@@ -833,24 +1204,33 @@ var alwaysOptional = schema.Defaults{
 	"ca-cert-path":               schema.Omit,
 	"ca-private-key-path":        schema.Omit,
 	"logging-config":             schema.Omit,
-	"provisioner-safe-mode":      schema.Omit,
+	ProvisionerHarvestModeKey:    schema.Omit,
 	"bootstrap-timeout":          schema.Omit,
 	"bootstrap-retry-delay":      schema.Omit,
 	"bootstrap-addresses-delay":  schema.Omit,
 	"rsyslog-ca-cert":            schema.Omit,
-	"http-proxy":                 schema.Omit,
-	"https-proxy":                schema.Omit,
-	"ftp-proxy":                  schema.Omit,
-	"no-proxy":                   schema.Omit,
-	"apt-http-proxy":             schema.Omit,
-	"apt-https-proxy":            schema.Omit,
-	"apt-ftp-proxy":              schema.Omit,
-	"lxc-clone":                  schema.Omit,
+	"rsyslog-ca-key":             schema.Omit,
+	HttpProxyKey:                 schema.Omit,
+	HttpsProxyKey:                schema.Omit,
+	FtpProxyKey:                  schema.Omit,
+	NoProxyKey:                   schema.Omit,
+	AptHttpProxyKey:              schema.Omit,
+	AptHttpsProxyKey:             schema.Omit,
+	AptFtpProxyKey:               schema.Omit,
+	"apt-mirror":                 schema.Omit,
+	LxcClone:                     schema.Omit,
 	"disable-network-management": schema.Omit,
+	AgentStreamKey:               schema.Omit,
+	SetNumaControlPolicyKey:      DefaultNumaControlPolicy,
+	PreventDestroyEnvironmentKey: DefaultPreventDestroyEnvironment,
+	PreventRemoveObjectKey:       DefaultPreventRemoveObject,
+	PreventAllChangesKey:         DefaultPreventAllChanges,
 
 	// Deprecated fields, retain for backwards compatibility.
-	"tools-url":     "",
-	"lxc-use-clone": schema.Omit,
+	ToolsMetadataURLKey:    "",
+	LxcUseClone:            schema.Omit,
+	ProvisionerSafeModeKey: schema.Omit,
+	ToolsStreamKey:         schema.Omit,
 
 	// For backward compatibility reasons, the following
 	// attributes default to empty strings rather than being
@@ -860,7 +1240,7 @@ var alwaysOptional = schema.Defaults{
 	"admin-secret":       "", // TODO(rog) omit
 	"ca-private-key":     "", // TODO(rog) omit
 	"image-metadata-url": "", // TODO(rog) omit
-	"tools-metadata-url": "", // TODO(rog) omit
+	AgentMetadataURLKey:  "", // TODO(rog) omit
 
 	"default-series": "",
 
@@ -872,10 +1252,16 @@ var alwaysOptional = schema.Defaults{
 	// Authentication string sent with requests to the charm store
 	"charm-store-auth": "",
 	// Previously image-stream could be set to an empty value
-	"image-stream":   "",
-	"test-mode":      false,
-	"proxy-ssh":      false,
-	"lxc-clone-aufs": false,
+	"image-stream":             "",
+	"test-mode":                false,
+	"proxy-ssh":                false,
+	"lxc-clone-aufs":           false,
+	"prefer-ipv6":              false,
+	"enable-os-refresh-update": schema.Omit,
+	"enable-os-upgrade":        schema.Omit,
+
+	// uuid may be missing for backwards compatability.
+	"uuid": schema.Omit,
 }
 
 func allowEmpty(attr string) bool {
@@ -899,7 +1285,12 @@ func allDefaults() schema.Defaults {
 		"bootstrap-retry-delay":      DefaultBootstrapSSHRetryDelay,
 		"bootstrap-addresses-delay":  DefaultBootstrapSSHAddressesDelay,
 		"proxy-ssh":                  true,
+		"prefer-ipv6":                false,
 		"disable-network-management": false,
+		SetNumaControlPolicyKey:      DefaultNumaControlPolicy,
+		PreventDestroyEnvironmentKey: DefaultPreventDestroyEnvironment,
+		PreventRemoveObjectKey:       DefaultPreventRemoveObject,
+		PreventAllChangesKey:         DefaultPreventAllChanges,
 	}
 	for attr, val := range alwaysOptional {
 		if _, ok := d[attr]; !ok {
@@ -931,15 +1322,17 @@ var mandatoryWithoutDefaults = []string{
 var immutableAttributes = []string{
 	"name",
 	"type",
+	"uuid",
 	"firewall-mode",
 	"state-port",
 	"api-port",
 	"bootstrap-timeout",
 	"bootstrap-retry-delay",
 	"bootstrap-addresses-delay",
-	"lxc-clone",
+	LxcClone,
 	"lxc-clone-aufs",
 	"syslog-port",
+	"prefer-ipv6",
 }
 
 var (
@@ -965,7 +1358,10 @@ func (cfg *Config) ValidateUnknownAttrs(fields schema.Fields, defaults schema.De
 	result := coerced.(map[string]interface{})
 	for name, value := range attrs {
 		if fields[name] == nil {
-			logger.Warningf("unknown config field %q", name)
+			if val, isString := value.(string); isString && val != "" {
+				// only warn about attributes with non-empty string values
+				logger.Warningf("unknown config field %q", name)
+			}
 			result[name] = value
 		}
 	}
@@ -973,8 +1369,8 @@ func (cfg *Config) ValidateUnknownAttrs(fields schema.Fields, defaults schema.De
 }
 
 // GenerateStateServerCertAndKey makes sure that the config has a CACert and
-// CAPrivateKey, generates and retruns new certificate and key.
-func (cfg *Config) GenerateStateServerCertAndKey() (string, string, error) {
+// CAPrivateKey, generates and returns new certificate and key.
+func (cfg *Config) GenerateStateServerCertAndKey(hostAddresses []string) (string, string, error) {
 	caCert, hasCACert := cfg.CACert()
 	if !hasCACert {
 		return "", "", fmt.Errorf("environment configuration has no ca-cert")
@@ -983,28 +1379,25 @@ func (cfg *Config) GenerateStateServerCertAndKey() (string, string, error) {
 	if !hasCAKey {
 		return "", "", fmt.Errorf("environment configuration has no ca-private-key")
 	}
-	var noHostnames []string
-	return cert.NewServer(caCert, caKey, time.Now().UTC().AddDate(10, 0, 0), noHostnames)
+	return cert.NewDefaultServer(caCert, caKey, hostAddresses)
 }
 
-type Specializer interface {
-	WithAuthAttrs(string) charm.Repository
-	WithTestMode(testMode bool) charm.Repository
-}
-
-// SpecializeCharmRepo returns a repository customized for given configuration.
+// SpecializeCharmRepo customizes a repository for a given configuration.
 // It adds authentication if necessary and sets a charm store's testMode flag.
-func SpecializeCharmRepo(repo charm.Repository, cfg *Config) charm.Repository {
+func SpecializeCharmRepo(repo charm.Repository, cfg *Config) {
+	type Specializer interface {
+		SetAuthAttrs(string)
+		SetTestMode(testMode bool)
+	}
 	// If a charm store auth token is set, pass it on to the charm store
 	if auth, authSet := cfg.CharmStoreAuth(); authSet {
 		if CS, isCS := repo.(Specializer); isCS {
-			repo = CS.WithAuthAttrs(auth)
+			CS.SetAuthAttrs(auth)
 		}
 	}
 	if CS, isCS := repo.(Specializer); isCS {
-		repo = CS.WithTestMode(cfg.TestMode())
+		CS.SetTestMode(cfg.TestMode())
 	}
-	return repo
 }
 
 // SSHTimeoutOpts lists the amount of time we will wait for various
@@ -1035,10 +1428,10 @@ func addIfNotEmpty(settings map[string]interface{}, key, value string) {
 // proxy settings.
 func ProxyConfigMap(proxySettings proxy.Settings) map[string]interface{} {
 	settings := make(map[string]interface{})
-	addIfNotEmpty(settings, "http-proxy", proxySettings.Http)
-	addIfNotEmpty(settings, "https-proxy", proxySettings.Https)
-	addIfNotEmpty(settings, "ftp-proxy", proxySettings.Ftp)
-	addIfNotEmpty(settings, "no-proxy", proxySettings.NoProxy)
+	addIfNotEmpty(settings, HttpProxyKey, proxySettings.Http)
+	addIfNotEmpty(settings, HttpsProxyKey, proxySettings.Https)
+	addIfNotEmpty(settings, FtpProxyKey, proxySettings.Ftp)
+	addIfNotEmpty(settings, NoProxyKey, proxySettings.NoProxy)
 	return settings
 }
 
@@ -1046,8 +1439,8 @@ func ProxyConfigMap(proxySettings proxy.Settings) map[string]interface{} {
 // proxy settings.
 func AptProxyConfigMap(proxySettings proxy.Settings) map[string]interface{} {
 	settings := make(map[string]interface{})
-	addIfNotEmpty(settings, "apt-http-proxy", proxySettings.Http)
-	addIfNotEmpty(settings, "apt-https-proxy", proxySettings.Https)
-	addIfNotEmpty(settings, "apt-ftp-proxy", proxySettings.Ftp)
+	addIfNotEmpty(settings, AptHttpProxyKey, proxySettings.Http)
+	addIfNotEmpty(settings, AptHttpsProxyKey, proxySettings.Https)
+	addIfNotEmpty(settings, AptFtpProxyKey, proxySettings.Ftp)
 	return settings
 }
