@@ -6,6 +6,7 @@ package config_test
 import (
 	"fmt"
 	"regexp"
+	"strings"
 	stdtesting "testing"
 	"time"
 
@@ -14,7 +15,7 @@ import (
 	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils/proxy"
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/cert"
 	"github.com/juju/juju/environs/config"
@@ -75,6 +76,32 @@ var configTests = []configTest{
 			"type": "my-type",
 			"name": "my-name",
 		},
+	}, {
+		about:       "Agent Stream",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":               "my-type",
+			"name":               "my-name",
+			"image-metadata-url": "image-url",
+			"agent-stream":       "agent-stream-value",
+		},
+	}, {
+		about:       "Deprecated tools-stream used",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":         "my-type",
+			"name":         "my-name",
+			"tools-stream": "tools-stream-value",
+		},
+	}, {
+		about:       "Deprecated tools-stream ignored",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":         "my-type",
+			"name":         "my-name",
+			"agent-stream": "agent-stream-value",
+			"tools-stream": "ignore-me",
+		},
 	}, {
 		about:       "Metadata URLs",
 		useDefaults: config.UseDefaults,
@@ -82,15 +109,15 @@ var configTests = []configTest{
 			"type":               "my-type",
 			"name":               "my-name",
 			"image-metadata-url": "image-url",
-			"tools-metadata-url": "tools-metadata-url-value",
+			"agent-metadata-url": "agent-metadata-url-value",
 		},
 	}, {
 		about:       "Deprecated tools metadata URL used",
 		useDefaults: config.UseDefaults,
 		attrs: testing.Attrs{
-			"type":      "my-type",
-			"name":      "my-name",
-			"tools-url": "tools-metadata-url-value",
+			"type":               "my-type",
+			"name":               "my-name",
+			"tools-metadata-url": "tools-metadata-url-value",
 		},
 	}, {
 		about:       "Deprecated tools metadata URL ignored",
@@ -98,8 +125,8 @@ var configTests = []configTest{
 		attrs: testing.Attrs{
 			"type":               "my-type",
 			"name":               "my-name",
-			"tools-metadata-url": "tools-metadata-url-value",
-			"tools-url":          "ignore-me",
+			"agent-metadata-url": "agent-metadata-url-value",
+			"tools-metadata-url": "ignore-me",
 		},
 	}, {
 		about:       "Explicit series",
@@ -196,25 +223,7 @@ var configTests = []configTest{
 			"ca-cert-path":        "~/othercert.pem",
 			"ca-private-key-path": "~/otherkey.pem",
 		},
-	}, /* {
-		about: "CA cert only from ~ path",
-		useDefaults: config.UseDefaults,
-		attrs: testing.Attrs{
-			"type":           "my-type",
-			"name":           "my-name",
-			"ca-cert-path":   "~/othercert.pem",
-			"ca-private-key": "",
-		},
 	}, {
-		about: "CA cert only as attribute",
-		useDefaults: config.UseDefaults,
-		attrs: testing.Attrs{
-			"type":           "my-type",
-			"name":           "my-name",
-			"ca-cert":        caCert,
-			"ca-private-key": "",
-		},
-	}, */{
 		about:       "CA cert and key as attributes",
 		useDefaults: config.UseDefaults,
 		attrs: testing.Attrs{
@@ -252,36 +261,7 @@ var configTests = []configTest{
 			"ca-private-key": invalidCAKey,
 		},
 		err: "bad CA certificate/key in configuration: crypto/tls:.*",
-	}, /* {
-		about: "No CA cert or key",
-		useDefaults: config.UseDefaults,
-		attrs: testing.Attrs{
-			"type":           "my-type",
-			"name":           "my-name",
-			"ca-cert":        "",
-			"ca-private-key": "",
-		},
 	}, {
-		about: "CA key but no cert",
-		useDefaults: config.UseDefaults,
-		attrs: testing.Attrs{
-			"type":           "my-type",
-			"name":           "my-name",
-			"ca-cert":        "",
-			"ca-private-key": caKey,
-		},
-		err: "bad CA certificate/key in configuration: crypto/tls:.*",
-	}, {
-		about: "No CA key",
-		useDefaults: config.UseDefaults,
-		attrs: testing.Attrs{
-			"type":           "my-type",
-			"name":           "my-name",
-			"ca-cert":        "foo",
-			"ca-private-key": "",
-		},
-		err: "bad CA certificate/key in configuration: no certificates found",
-	}, */{
 		about:       "CA cert specified as non-existent file",
 		useDefaults: config.UseDefaults,
 		attrs: testing.Attrs{
@@ -363,6 +343,96 @@ var configTests = []configTest{
 			"name": "my-name",
 			"disable-network-management": true,
 		},
+	}, {
+		about:       "set-numa-control-policy on",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"set-numa-control-policy": true,
+		},
+	}, {
+		about:       "set-numa-control-policy off",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"set-numa-control-policy": false,
+		},
+	}, {
+		about:       "block-destroy-environment on",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"block-destroy-environment": true,
+		},
+	}, {
+		about:       "block-destroy-environment off",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"block-destroy-environment": false,
+		},
+	}, {
+		about:       "block-remove-object on",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":                "my-type",
+			"name":                "my-name",
+			"block-remove-object": true,
+		},
+	}, {
+		about:       "block-remove-object off",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":                "my-type",
+			"name":                "my-name",
+			"block-remove-object": false,
+		},
+	}, {
+		about:       "block-all-changes on",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":              "my-type",
+			"name":              "my-name",
+			"block-all-changes": true,
+		},
+	}, {
+		about:       "block-all-changes off",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":               "my-type",
+			"name":               "my-name",
+			"block-all-changest": false,
+		},
+	}, {
+		about:       "Invalid prefer-ipv6 flag",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":            "my-type",
+			"name":            "my-name",
+			"authorized-keys": testing.FakeAuthKeys,
+			"prefer-ipv6":     "invalid",
+		},
+		err: `prefer-ipv6: expected bool, got string\("invalid"\)`,
+	}, {
+		about:       "prefer-ipv6 off",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":        "my-type",
+			"name":        "my-name",
+			"prefer-ipv6": false,
+		},
+	}, {
+		about:       "prefer-ipv6 on",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":        "my-type",
+			"name":        "my-name",
+			"prefer-ipv6": true,
+		},
 	}, {
 		about:       "Invalid agent version",
 		useDefaults: config.UseDefaults,
@@ -450,6 +520,14 @@ var configTests = []configTest{
 			"name":          "my-name",
 			"firewall-mode": config.FwGlobal,
 		},
+	}, {
+		about:       "None firewall mode",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":          "my-type",
+			"name":          "my-name",
+			"firewall-mode": config.FwNone,
+		},
 	}, {
 		about:       "Illegal firewall mode",
 		useDefaults: config.UseDefaults,
@@ -477,30 +555,66 @@ var configTests = []configTest{
 		},
 		err: `ssl-hostname-verification: expected bool, got string\("yes please"\)`,
 	}, {
-		about:       "provisioner-safe-mode off",
+		about: fmt.Sprintf(
+			"%s: %s",
+			"provisioner-harvest-mode",
+			config.HarvestAll.String(),
+		),
 		useDefaults: config.UseDefaults,
 		attrs: testing.Attrs{
-			"type":                  "my-type",
-			"name":                  "my-name",
-			"provisioner-safe-mode": false,
+			"type": "my-type",
+			"name": "my-name",
+			"provisioner-harvest-mode": config.HarvestAll.String(),
 		},
 	}, {
-		about:       "provisioner-safe-mode on",
+		about: fmt.Sprintf(
+			"%s: %s",
+			"provisioner-harvest-mode",
+			config.HarvestDestroyed.String(),
+		),
 		useDefaults: config.UseDefaults,
 		attrs: testing.Attrs{
-			"type":                  "my-type",
-			"name":                  "my-name",
-			"provisioner-safe-mode": true,
+			"type": "my-type",
+			"name": "my-name",
+			"provisioner-harvest-mode": config.HarvestDestroyed.String(),
 		},
 	}, {
-		about:       "provisioner-safe-mode incorrect",
+		about: fmt.Sprintf(
+			"%s: %s",
+			"provisioner-harvest-mode",
+			config.HarvestUnknown.String(),
+		),
 		useDefaults: config.UseDefaults,
 		attrs: testing.Attrs{
-			"type":                  "my-type",
-			"name":                  "my-name",
-			"provisioner-safe-mode": "yes please",
+			"type": "my-type",
+			"name": "my-name",
+			"provisioner-harvest-mode": config.HarvestUnknown.String(),
 		},
-		err: `provisioner-safe-mode: expected bool, got string\("yes please"\)`,
+	}, {
+		about: fmt.Sprintf(
+			"%s: %s",
+			"provisioner-harvest-mode",
+			config.HarvestNone.String(),
+		),
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"provisioner-harvest-mode": config.HarvestNone.String(),
+		},
+	}, {
+		about: fmt.Sprintf(
+			"%s: %s",
+			"provisioner-harvest-mode",
+			"incorrect",
+		),
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"provisioner-harvest-mode": "yes please",
+		},
+		err: `unknown harvesting method: yes please`,
 	}, {
 		about:       "default image stream",
 		useDefaults: config.UseDefaults,
@@ -516,6 +630,14 @@ var configTests = []configTest{
 			"name":         "my-name",
 			"image-stream": "daily",
 		},
+	}, {
+		about:       "explicit tools stream",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":         "my-type",
+			"name":         "my-name",
+			"agent-stream": "proposed",
+		},
 	}, {
 		about:       "Explicit state port",
 		useDefaults: config.UseDefaults,
@@ -669,7 +791,7 @@ var configTests = []configTest{
 			"image-metadata-url":        "",
 			"ca-private-key":            "",
 			"default-series":            "precise",
-			"tools-metadata-url":        "",
+			"agent-metadata-url":        "",
 			"secret-key":                "a-secret-key",
 			"access-key":                "an-access-key",
 			"agent-version":             "1.13.2",
@@ -692,6 +814,41 @@ var configTests = []configTest{
 			"name":      "my-name",
 			"test-mode": true,
 		},
+	}, {
+		about:       "valid uuid",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"uuid": "dcfbdb4a-bca2-49ad-aa7c-f011424e0fe4",
+		},
+	}, {
+		about:       "invalid uuid 1",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"uuid": "dcfbdb4abca249adaa7cf011424e0fe4",
+		},
+		err: "uuid: expected uuid, got string\\(\"dcfbdb4abca249adaa7cf011424e0fe4\"\\)",
+	}, {
+		about:       "invalid uuid 2",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"uuid": "uuid",
+		},
+		err: "uuid: expected uuid, got string\\(\"uuid\"\\)",
+	}, {
+		about:       "blank uuid",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type": "my-type",
+			"name": "my-name",
+			"uuid": "",
+		},
+		err: "uuid: expected uuid, got string\\(\"\"\\)",
 	},
 	authTokenConfigTest("token=value, tokensecret=value", true),
 	authTokenConfigTest("token=value, ", true),
@@ -714,6 +871,25 @@ var configTests = []configTest{
 	// backward compatibility with pre-1.13 config.
 	// missingAttributeNoDefault("state-port"),
 	// missingAttributeNoDefault("api-port"),
+	{
+		about:       "Deprecated safe-mode failover",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":                     "my-type",
+			"name":                     "my-name",
+			"provisioner-safe-mode":    true,
+			"provisioner-harvest-mode": config.HarvestNone.String(),
+		},
+	},
+	{
+		about:       "Explicit apt-mirror",
+		useDefaults: config.UseDefaults,
+		attrs: testing.Attrs{
+			"type":       "my-type",
+			"name":       "my-name",
+			"apt-mirror": "http://my.archive.ubuntu.com",
+		},
+	},
 }
 
 // authTokenConfigTest returns a config test that checks
@@ -868,6 +1044,35 @@ func (s *ConfigSuite) TestConfigEmptyCertFiles(c *gc.C) {
 	}
 }
 
+func (s *ConfigSuite) TestSafeModeDeprecatesGracefully(c *gc.C) {
+
+	cfg, err := config.New(config.UseDefaults, testing.Attrs{
+		"name":                  "name",
+		"type":                  "type",
+		"provisioner-safe-mode": false,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Check(
+		cfg.ProvisionerHarvestMode().String(),
+		gc.Equals,
+		config.HarvestAll.String(),
+	)
+
+	cfg, err = config.New(config.UseDefaults, testing.Attrs{
+		"name":                  "name",
+		"type":                  "type",
+		"provisioner-safe-mode": true,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+
+	c.Check(
+		cfg.ProvisionerHarvestMode().String(),
+		gc.Equals,
+		config.HarvestDestroyed.String(),
+	)
+}
+
 func (test configTest) check(c *gc.C, home *gitjujutesting.FakeHome) {
 	cfg, err := config.New(test.useDefaults, test.attrs)
 	if test.err != "" {
@@ -875,7 +1080,7 @@ func (test configTest) check(c *gc.C, home *gitjujutesting.FakeHome) {
 		c.Assert(err, gc.ErrorMatches, test.err)
 		return
 	}
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	typ, _ := test.attrs["type"].(string)
 	// "null" has been deprecated in favour of "manual",
@@ -904,6 +1109,11 @@ func (test configTest) check(c *gc.C, home *gitjujutesting.FakeHome) {
 	if syslogPort, ok := test.attrs["syslog-port"]; ok {
 		c.Assert(cfg.SyslogPort(), gc.Equals, syslogPort)
 	}
+	if expected, ok := test.attrs["uuid"]; ok {
+		got, exists := cfg.UUID()
+		c.Assert(exists, gc.Equals, ok)
+		c.Assert(got, gc.Equals, expected)
+	}
 
 	dev, _ := test.attrs["development"].(bool)
 	c.Assert(cfg.Development(), gc.Equals, dev)
@@ -976,10 +1186,12 @@ func (test configTest) check(c *gc.C, home *gitjujutesting.FakeHome) {
 		c.Assert(cfg.SSLHostnameVerification(), gc.Equals, v)
 	}
 
-	if v, ok := test.attrs["provisioner-safe-mode"]; ok {
-		c.Assert(cfg.ProvisionerSafeMode(), gc.Equals, v)
+	if v, ok := test.attrs["provisioner-harvest-mode"]; ok {
+		hvstMeth, err := config.ParseHarvestMode(v.(string))
+		c.Assert(err, jc.ErrorIsNil)
+		c.Assert(cfg.ProvisionerHarvestMode(), gc.Equals, hvstMeth)
 	} else {
-		c.Assert(cfg.ProvisionerSafeMode(), gc.Equals, false)
+		c.Assert(cfg.ProvisionerHarvestMode(), gc.Equals, config.HarvestDestroyed)
 	}
 	sshOpts := cfg.BootstrapSSHOpts()
 	test.assertDuration(
@@ -1014,24 +1226,38 @@ func (test configTest) check(c *gc.C, home *gitjujutesting.FakeHome) {
 	} else {
 		c.Assert(urlPresent, jc.IsFalse)
 	}
-	toolsURL, urlPresent := cfg.ToolsURL()
-	oldToolsURL, oldURLPresent := cfg.AllAttrs()["tools-url"]
-	oldToolsURLAttrValue, oldURLAttrPresent := test.attrs["tools-url"]
-	expectedToolsURLValue := test.attrs["tools-metadata-url"]
-	if expectedToolsURLValue == nil {
-		expectedToolsURLValue = oldToolsURLAttrValue
+
+	toolsURL, urlPresent := cfg.AgentMetadataURL()
+	oldToolsURL := cfg.AllAttrs()["tools-metadata-url"]
+	oldToolsURLAttrValue, oldTSTPresent := test.attrs["tools-metadata-url"]
+	expectedToolsURLValue := test.attrs["agent-metadata-url"]
+	expectedToolsURLPresent := true
+	if expectedToolsURLValue == nil || expectedToolsURLValue == "" {
+		if oldTSTPresent {
+			expectedToolsURLValue = oldToolsURLAttrValue
+		} else {
+			expectedToolsURLValue = oldToolsURL
+			expectedToolsURLPresent = false
+		}
 	}
-	if expectedToolsURLValue != nil && expectedToolsURLValue != "" {
-		c.Assert(expectedToolsURLValue, gc.Equals, "tools-metadata-url-value")
-		c.Assert(toolsURL, gc.Equals, expectedToolsURLValue)
-		c.Assert(urlPresent, jc.IsTrue)
-		c.Assert(oldToolsURL, gc.Equals, expectedToolsURLValue)
-		c.Assert(oldURLPresent, jc.IsTrue)
-	} else {
-		c.Assert(urlPresent, jc.IsFalse)
-		c.Assert(oldURLAttrPresent, jc.IsFalse)
-		c.Assert(oldToolsURL, gc.Equals, "")
+	c.Assert(toolsURL, gc.Equals, expectedToolsURLValue)
+	c.Assert(urlPresent, gc.Equals, expectedToolsURLPresent)
+
+	// assertions for deprecated tools-stream attribute used with new agent-stream
+	agentStreamValue := cfg.AgentStream()
+	oldTstToolsStreamAttr, oldTstOk := test.attrs["tools-stream"]
+	expectedAgentStreamAttr := test.attrs["agent-stream"]
+
+	// When no agent-stream provided, look for tools-stream
+	if expectedAgentStreamAttr == nil {
+		if oldTstOk {
+			expectedAgentStreamAttr = oldTstToolsStreamAttr
+		} else {
+			// If it's still nil, then hard-coded default is used
+			expectedAgentStreamAttr = "released"
+		}
 	}
+	c.Assert(agentStreamValue, gc.Equals, expectedAgentStreamAttr)
 
 	useLxcClone, useLxcClonePresent := cfg.LXCUseClone()
 	oldUseClone, oldUseClonePresent := cfg.AllAttrs()["lxc-use-clone"]
@@ -1044,14 +1270,14 @@ func (test configTest) check(c *gc.C, home *gitjujutesting.FakeHome) {
 			c.Assert(useLxcClone, gc.Equals, oldUseClone)
 		} else {
 			c.Assert(useLxcClonePresent, jc.IsFalse)
-			c.Assert(useLxcClone, gc.Equals, false)
+			c.Assert(useLxcClone, jc.IsFalse)
 		}
 	}
 	useLxcCloneAufs, ok := cfg.LXCUseCloneAUFS()
 	if v, ok := test.attrs["lxc-clone-aufs"]; ok {
 		c.Assert(useLxcCloneAufs, gc.Equals, v)
 	} else {
-		c.Assert(useLxcCloneAufs, gc.Equals, false)
+		c.Assert(useLxcCloneAufs, jc.IsFalse)
 	}
 }
 
@@ -1070,6 +1296,7 @@ func (s *ConfigSuite) TestConfigAttrs(c *gc.C) {
 	attrs := map[string]interface{}{
 		"type":                      "my-type",
 		"name":                      "my-name",
+		"uuid":                      "90168e4c-2f10-4e9c-83c2-1fb55a58e5a9",
 		"authorized-keys":           testing.FakeAuthKeys,
 		"firewall-mode":             config.FwInstance,
 		"admin-secret":              "foo",
@@ -1077,7 +1304,6 @@ func (s *ConfigSuite) TestConfigAttrs(c *gc.C) {
 		"ca-cert":                   caCert,
 		"ssl-hostname-verification": true,
 		"development":               false,
-		"provisioner-safe-mode":     false,
 		"state-port":                1234,
 		"api-port":                  4321,
 		"syslog-port":               2345,
@@ -1089,31 +1315,41 @@ func (s *ConfigSuite) TestConfigAttrs(c *gc.C) {
 		"test-mode":                 false,
 	}
 	cfg, err := config.New(config.NoDefaults, attrs)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	// These attributes are added if not set.
 	attrs["development"] = false
 	attrs["logging-config"] = "<root>=WARNING;unit=DEBUG"
 	attrs["ca-private-key"] = ""
 	attrs["image-metadata-url"] = ""
+	attrs["agent-metadata-url"] = ""
 	attrs["tools-metadata-url"] = ""
-	attrs["tools-url"] = ""
 	attrs["image-stream"] = ""
 	attrs["proxy-ssh"] = false
 	attrs["lxc-clone-aufs"] = false
+	attrs["prefer-ipv6"] = false
+	attrs["set-numa-control-policy"] = false
+	attrs["block-destroy-environment"] = false
+	attrs["block-remove-object"] = false
+	attrs["block-all-changes"] = false
 
 	// Default firewall mode is instance
 	attrs["firewall-mode"] = string(config.FwInstance)
 	c.Assert(cfg.AllAttrs(), jc.DeepEquals, attrs)
 	c.Assert(cfg.UnknownAttrs(), jc.DeepEquals, map[string]interface{}{"unknown": "my-unknown"})
 
+	// Verify that default provisioner-harvest-mode is good.
+	c.Assert(cfg.ProvisionerHarvestMode(), gc.Equals, config.HarvestDestroyed)
+
 	newcfg, err := cfg.Apply(map[string]interface{}{
 		"name":        "new-name",
+		"uuid":        "6216dfc3-6e82-408f-9f74-8565e63e6158",
 		"new-unknown": "my-new-unknown",
 	})
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	attrs["name"] = "new-name"
+	attrs["uuid"] = "6216dfc3-6e82-408f-9f74-8565e63e6158"
 	attrs["new-unknown"] = "my-new-unknown"
 	c.Assert(newcfg.AllAttrs(), jc.DeepEquals, attrs)
 }
@@ -1145,10 +1381,15 @@ var validationTests = []validationTest{{
 	old:   testing.Attrs{"agent-version": "1.9.27"},
 	err:   `cannot clear agent-version`,
 }, {
-	about: "Can't change the firewall-mode",
+	about: "Can't change the firewall-mode (global->instance)",
 	old:   testing.Attrs{"firewall-mode": config.FwGlobal},
 	new:   testing.Attrs{"firewall-mode": config.FwInstance},
 	err:   `cannot change firewall-mode from "global" to "instance"`,
+}, {
+	about: "Can't change the firewall-mode (global->none)",
+	old:   testing.Attrs{"firewall-mode": config.FwGlobal},
+	new:   testing.Attrs{"firewall-mode": config.FwNone},
+	err:   `cannot change firewall-mode from "global" to "none"`,
 }, {
 	about: "Cannot change the state-port",
 	old:   testing.Attrs{"state-port": config.DefaultStatePort},
@@ -1197,6 +1438,19 @@ var validationTests = []validationTest{{
 	old:   testing.Attrs{"lxc-clone-aufs": false},
 	new:   testing.Attrs{"lxc-clone-aufs": true},
 	err:   `cannot change lxc-clone-aufs from false to true`,
+}, {
+	about: "Cannot change prefer-ipv6",
+	old:   testing.Attrs{"prefer-ipv6": false},
+	new:   testing.Attrs{"prefer-ipv6": true},
+	err:   `cannot change prefer-ipv6 from false to true`,
+}, {
+	about: "Can change uuid from unset to set",
+	new:   testing.Attrs{"uuid": "dcfbdb4a-bca2-49ad-aa7c-f011424e0fe4"},
+}, {
+	about: "Cannot change uuid",
+	old:   testing.Attrs{"uuid": "90168e4c-2f10-4e9c-83c2-1fb55a58e5a9"},
+	new:   testing.Attrs{"uuid": "dcfbdb4a-bca2-49ad-aa7c-f011424e0fe4"},
+	err:   "cannot change uuid from \"90168e4c-2f10-4e9c-83c2-1fb55a58e5a9\" to \"dcfbdb4a-bca2-49ad-aa7c-f011424e0fe4\"",
 }}
 
 func (s *ConfigSuite) TestValidateChange(c *gc.C) {
@@ -1211,7 +1465,7 @@ func (s *ConfigSuite) TestValidateChange(c *gc.C) {
 		oldConfig := newTestConfig(c, test.old)
 		err := config.Validate(newConfig, oldConfig)
 		if test.err == "" {
-			c.Check(err, gc.IsNil)
+			c.Check(err, jc.ErrorIsNil)
 		} else {
 			c.Check(err, gc.ErrorMatches, test.err)
 		}
@@ -1234,10 +1488,11 @@ func (s *ConfigSuite) TestValidateUnknownAttrs(c *gc.C) {
 		"known":   "this",
 		"unknown": "that",
 	})
+	c.Assert(err, jc.ErrorIsNil)
 
 	// No fields: all attrs passed through.
 	attrs, err := cfg.ValidateUnknownAttrs(nil, nil)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(attrs, gc.DeepEquals, map[string]interface{}{
 		"known":   "this",
 		"unknown": "that",
@@ -1246,7 +1501,7 @@ func (s *ConfigSuite) TestValidateUnknownAttrs(c *gc.C) {
 	// Valid field: that and other attrs passed through.
 	fields := schema.Fields{"known": schema.String()}
 	attrs, err = cfg.ValidateUnknownAttrs(fields, nil)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(attrs, gc.DeepEquals, map[string]interface{}{
 		"known":   "this",
 		"unknown": "that",
@@ -1256,7 +1511,7 @@ func (s *ConfigSuite) TestValidateUnknownAttrs(c *gc.C) {
 	fields["default"] = schema.String()
 	defaults := schema.Defaults{"default": "the other"}
 	attrs, err = cfg.ValidateUnknownAttrs(fields, defaults)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(attrs, gc.DeepEquals, map[string]interface{}{
 		"known":   "this",
 		"unknown": "that",
@@ -1269,13 +1524,58 @@ func (s *ConfigSuite) TestValidateUnknownAttrs(c *gc.C) {
 	c.Assert(err, gc.ErrorMatches, `known: expected int, got string\("this"\)`)
 }
 
+type testAttr struct {
+	message string
+	aKey    string
+	aValue  string
+	checker gc.Checker
+}
+
+var emptyAttributeTests = []testAttr{
+	{
+		message: "Warning message about unknown attribute (%v) is expected because attribute value exists",
+		aKey:    "unknown",
+		aValue:  "unknown value",
+		checker: gc.Matches,
+	}, {
+		message: "Warning message about unknown attribute (%v) is unexpected because attribute value is empty",
+		aKey:    "unknown-empty",
+		aValue:  "",
+		checker: gc.Not(gc.Matches),
+	},
+}
+
+func (s *ConfigSuite) TestValidateUnknownEmptyAttr(c *gc.C) {
+	s.addJujuFiles(c)
+	cfg, err := config.New(config.UseDefaults, map[string]interface{}{
+		"name": "myenv",
+		"type": "other",
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	warningTxt := `.* unknown config field %q.*`
+
+	for i, test := range emptyAttributeTests {
+		c.Logf("test %d: %v\n", i, fmt.Sprintf(test.message, test.aKey))
+		testCfg, err := cfg.Apply(map[string]interface{}{test.aKey: test.aValue})
+		c.Assert(err, jc.ErrorIsNil)
+		attrs, err := testCfg.ValidateUnknownAttrs(nil, nil)
+		c.Assert(err, jc.ErrorIsNil)
+		// all attrs passed through
+		c.Assert(attrs, gc.DeepEquals, map[string]interface{}{test.aKey: test.aValue})
+		expectedWarning := fmt.Sprintf(warningTxt, test.aKey)
+		logOutputText := strings.Replace(c.GetTestLog(), "\n", "", -1)
+		// warning displayed or not based on test expectation
+		c.Assert(logOutputText, test.checker, expectedWarning, gc.Commentf(test.message, test.aKey))
+	}
+}
+
 func newTestConfig(c *gc.C, explicit testing.Attrs) *config.Config {
 	final := testing.Attrs{"type": "my-type", "name": "my-name"}
 	for key, value := range explicit {
 		final[key] = value
 	}
 	result, err := config.New(config.UseDefaults, final)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	return result
 }
 
@@ -1319,6 +1619,24 @@ func (s *ConfigSuite) TestProxyValuesWithFallback(c *gc.C) {
 	c.Assert(config.NoProxy(), gc.Equals, "localhost,10.0.3.1")
 }
 
+func (s *ConfigSuite) TestProxyValuesWithFallbackNoScheme(c *gc.C) {
+	s.addJujuFiles(c)
+
+	config := newTestConfig(c, testing.Attrs{
+		"http-proxy":  "user@10.0.0.1",
+		"https-proxy": "user@10.0.0.1",
+		"ftp-proxy":   "user@10.0.0.1",
+		"no-proxy":    "localhost,10.0.3.1",
+	})
+	c.Assert(config.HttpProxy(), gc.Equals, "user@10.0.0.1")
+	c.Assert(config.AptHttpProxy(), gc.Equals, "http://user@10.0.0.1")
+	c.Assert(config.HttpsProxy(), gc.Equals, "user@10.0.0.1")
+	c.Assert(config.AptHttpsProxy(), gc.Equals, "https://user@10.0.0.1")
+	c.Assert(config.FtpProxy(), gc.Equals, "user@10.0.0.1")
+	c.Assert(config.AptFtpProxy(), gc.Equals, "ftp://user@10.0.0.1")
+	c.Assert(config.NoProxy(), gc.Equals, "localhost,10.0.3.1")
+}
+
 func (s *ConfigSuite) TestProxyValues(c *gc.C) {
 	s.addJujuFiles(c)
 	config := newTestConfig(c, testing.Attrs{
@@ -1358,24 +1676,29 @@ func (s *ConfigSuite) TestProxyConfigMap(c *gc.C) {
 		Ftp:     "ftp proxy",
 		NoProxy: "no proxy",
 	}
+	expectedProxySettings := proxy.Settings{
+		Http:    "http://http proxy",
+		Https:   "https://https proxy",
+		Ftp:     "ftp://ftp proxy",
+		NoProxy: "",
+	}
 	cfg, err := cfg.Apply(config.ProxyConfigMap(proxySettings))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cfg.ProxySettings(), gc.DeepEquals, proxySettings)
-	// Apt proxy and proxy differ by the content of the no-proxy values.
-	proxySettings.NoProxy = ""
-	c.Assert(cfg.AptProxySettings(), gc.DeepEquals, proxySettings)
+	// Apt proxy settings always include the scheme. NoProxy is empty.
+	c.Assert(cfg.AptProxySettings(), gc.DeepEquals, expectedProxySettings)
 }
 
 func (s *ConfigSuite) TestAptProxyConfigMap(c *gc.C) {
 	s.addJujuFiles(c)
 	cfg := newTestConfig(c, testing.Attrs{})
 	proxySettings := proxy.Settings{
-		Http:  "http proxy",
-		Https: "https proxy",
-		Ftp:   "ftp proxy",
+		Http:  "http://httpproxy",
+		Https: "https://httpsproxy",
+		Ftp:   "ftp://ftpproxy",
 	}
 	cfg, err := cfg.Apply(config.AptProxyConfigMap(proxySettings))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	// The default proxy settings should still be empty.
 	c.Assert(cfg.ProxySettings(), gc.DeepEquals, proxy.Settings{})
 	c.Assert(cfg.AptProxySettings(), gc.DeepEquals, proxySettings)
@@ -1387,6 +1710,7 @@ func (s *ConfigSuite) TestGenerateStateServerCertAndKey(c *gc.C) {
 
 	for _, test := range []struct {
 		configValues map[string]interface{}
+		sanValues    []string
 		errMatch     string
 	}{{
 		configValues: map[string]interface{}{
@@ -1408,22 +1732,37 @@ func (s *ConfigSuite) TestGenerateStateServerCertAndKey(c *gc.C) {
 			"ca-cert":        testing.CACert,
 			"ca-private-key": testing.CAKey,
 		},
+	}, {
+		configValues: map[string]interface{}{
+			"name":           "test-no-certs",
+			"type":           "dummy",
+			"ca-cert":        testing.CACert,
+			"ca-private-key": testing.CAKey,
+		},
+		sanValues: []string{"10.0.0.1", "192.168.1.1"},
 	}} {
 		cfg, err := config.New(config.UseDefaults, test.configValues)
-		c.Assert(err, gc.IsNil)
-		certPEM, keyPEM, err := cfg.GenerateStateServerCertAndKey()
+		c.Assert(err, jc.ErrorIsNil)
+		certPEM, keyPEM, err := cfg.GenerateStateServerCertAndKey(test.sanValues)
 		if test.errMatch == "" {
-			c.Assert(err, gc.IsNil)
+			c.Assert(err, jc.ErrorIsNil)
 
 			_, _, err = cert.ParseCertAndKey(certPEM, keyPEM)
-			c.Check(err, gc.IsNil)
+			c.Check(err, jc.ErrorIsNil)
 
 			err = cert.Verify(certPEM, testing.CACert, time.Now())
-			c.Assert(err, gc.IsNil)
+			c.Assert(err, jc.ErrorIsNil)
 			err = cert.Verify(certPEM, testing.CACert, time.Now().AddDate(9, 0, 0))
-			c.Assert(err, gc.IsNil)
+			c.Assert(err, jc.ErrorIsNil)
 			err = cert.Verify(certPEM, testing.CACert, time.Now().AddDate(10, 0, 1))
 			c.Assert(err, gc.NotNil)
+			srvCert, err := cert.ParseCert(certPEM)
+			c.Assert(err, jc.ErrorIsNil)
+			sanIPs := make([]string, len(srvCert.IPAddresses))
+			for i, ip := range srvCert.IPAddresses {
+				sanIPs[i] = ip.String()
+			}
+			c.Assert(sanIPs, jc.SameContents, test.sanValues)
 		} else {
 			c.Assert(err, gc.ErrorMatches, test.errMatch)
 			c.Assert(certPEM, gc.Equals, "")
