@@ -4,38 +4,36 @@
 package jujutest
 
 import (
-	"bytes"
 	"fmt"
-	"io"
-	"io/ioutil"
-	"strings"
 	"time"
 
-	"github.com/juju/charm"
-	charmtesting "github.com/juju/charm/testing"
 	"github.com/juju/errors"
+	gitjujutesting "github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
+	"gopkg.in/juju/charm.v4"
 
+	"github.com/juju/juju/api"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/bootstrap"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/environs/configstore"
+	"github.com/juju/juju/environs/filestorage"
 	"github.com/juju/juju/environs/storage"
 	"github.com/juju/juju/environs/sync"
 	envtesting "github.com/juju/juju/environs/testing"
 	envtools "github.com/juju/juju/environs/tools"
+	envtoolstesting "github.com/juju/juju/environs/tools/testing"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju"
-	"github.com/juju/juju/juju/testing"
 	jujutesting "github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/dummy"
 	"github.com/juju/juju/state"
-	"github.com/juju/juju/state/api"
 	statetesting "github.com/juju/juju/state/testing"
+	"github.com/juju/juju/testcharms"
 	coretesting "github.com/juju/juju/testing"
 	coretools "github.com/juju/juju/tools"
 	"github.com/juju/juju/version"
@@ -45,6 +43,8 @@ import (
 // (e.g. Amazon EC2).  The Environ is opened once only for all the tests
 // in the suite, stored in Env, and Destroyed after the suite has completed.
 type LiveTests struct {
+	gitjujutesting.CleanupSuite
+
 	envtesting.ToolsFixture
 
 	// TestConfig contains the configuration attributes for opening an environment.
@@ -73,12 +73,26 @@ type LiveTests struct {
 
 	prepared     bool
 	bootstrapped bool
+	toolsStorage storage.Storage
 }
 
 func (t *LiveTests) SetUpSuite(c *gc.C) {
+	t.CleanupSuite.SetUpSuite(c)
 	t.ConfigStore = configstore.NewMem()
 }
 
+func (t *LiveTests) SetUpTest(c *gc.C) {
+	t.CleanupSuite.SetUpTest(c)
+	storageDir := c.MkDir()
+	t.DefaultBaseURL = "file://" + storageDir + "/tools"
+	t.ToolsFixture.SetUpTest(c)
+	stor, err := filestorage.NewFileStorageWriter(storageDir)
+	c.Assert(err, jc.ErrorIsNil)
+	t.UploadFakeTools(c, stor, "released", "released")
+	t.toolsStorage = stor
+	t.CleanupSuite.PatchValue(&envtools.BundleTools, envtoolstesting.GetMockBundleTools(c))
+}
+
 func publicAttrs(e environs.Environ) map[string]interface{} {
 	cfg := e.Config()
 	secrets, err := e.Provider().SecretAttrs(cfg)
@@ -93,9 +107,13 @@ func publicAttrs(e environs.Environ) map[string]interface{} {
 }
 
 func (t *LiveTests) TearDownSuite(c *gc.C) {
-	if t.Env != nil {
-		t.Destroy(c)
-	}
+	t.Destroy(c)
+	t.CleanupSuite.TearDownSuite(c)
+}
+
+func (t *LiveTests) TearDownTest(c *gc.C) {
+	t.ToolsFixture.TearDownTest(c)
+	t.CleanupSuite.TearDownTest(c)
 }
 
 // PrepareOnce ensures that the environment is
@@ -105,8 +123,8 @@ func (t *LiveTests) PrepareOnce(c *gc.C) {
 		return
 	}
 	cfg, err := config.New(config.NoDefaults, t.TestConfig)
-	c.Assert(err, gc.IsNil)
-	e, err := environs.Prepare(cfg, coretesting.Context(c), t.ConfigStore)
+	c.Assert(err, jc.ErrorIsNil)
+	e, err := environs.Prepare(cfg, envtesting.BootstrapContext(c), t.ConfigStore)
 	c.Assert(err, gc.IsNil, gc.Commentf("preparing environ %#v", t.TestConfig))
 	c.Assert(e, gc.NotNil)
 	t.Env = e
@@ -122,14 +140,13 @@ func (t *LiveTests) BootstrapOnce(c *gc.C) {
 	// we could connect to (actual live tests, rather than local-only)
 	cons := constraints.MustParse("mem=2G")
 	if t.CanOpenState {
-		_, err := sync.Upload(t.Env.Storage(), nil, coretesting.FakeDefaultSeries)
-		c.Assert(err, gc.IsNil)
+		_, err := sync.Upload(t.toolsStorage, "released", nil, coretesting.FakeDefaultSeries)
+		c.Assert(err, jc.ErrorIsNil)
 	}
-	t.UploadFakeTools(c, t.Env.Storage())
 	err := bootstrap.EnsureNotBootstrapped(t.Env)
-	c.Assert(err, gc.IsNil)
-	err = bootstrap.Bootstrap(coretesting.Context(c), t.Env, environs.BootstrapParams{Constraints: cons})
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
+	err = bootstrap.Bootstrap(envtesting.BootstrapContext(c), t.Env, bootstrap.BootstrapParams{Constraints: cons})
+	c.Assert(err, jc.ErrorIsNil)
 	t.bootstrapped = true
 }
 
@@ -138,7 +155,7 @@ func (t *LiveTests) Destroy(c *gc.C) {
 		return
 	}
 	err := environs.Destroy(t.Env, t.ConfigStore)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	t.bootstrapped = false
 	t.prepared = false
 	t.Env = nil
@@ -153,21 +170,20 @@ func (t *LiveTests) TestPrechecker(c *gc.C) {
 	}
 	const placement = ""
 	err := prechecker.PrecheckInstance("precise", constraints.Value{}, placement)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 }
 
 // TestStartStop is similar to Tests.TestStartStop except
 // that it does not assume a pristine environment.
 func (t *LiveTests) TestStartStop(c *gc.C) {
-	t.PrepareOnce(c)
-	t.UploadFakeTools(c, t.Env.Storage())
+	t.BootstrapOnce(c)
 
-	inst, _ := testing.AssertStartInstance(c, t.Env, "0")
+	inst, _ := jujutesting.AssertStartInstance(c, t.Env, "0")
 	c.Assert(inst, gc.NotNil)
 	id0 := inst.Id()
 
 	insts, err := t.Env.Instances([]instance.Id{id0, id0})
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(insts, gc.HasLen, 2)
 	c.Assert(insts[0].Id(), gc.Equals, id0)
 	c.Assert(insts[1].Id(), gc.Equals, id0)
@@ -177,7 +193,7 @@ func (t *LiveTests) TestStartStop(c *gc.C) {
 	// off if other instances have been created or destroyed in the same
 	// time frame. Instead, just check the instance we created exists.
 	insts, err = t.Env.AllInstances()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	found := false
 	for _, inst := range insts {
 		if inst.Id() == id0 {
@@ -188,7 +204,7 @@ func (t *LiveTests) TestStartStop(c *gc.C) {
 	c.Assert(found, gc.Equals, true, gc.Commentf("expected %v in %v", inst, insts))
 
 	addresses, err := jujutesting.WaitInstanceAddresses(t.Env, inst.Id())
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(addresses, gc.Not(gc.HasLen), 0)
 
 	insts, err = t.Env.Instances([]instance.Id{id0, ""})
@@ -198,7 +214,7 @@ func (t *LiveTests) TestStartStop(c *gc.C) {
 	c.Check(insts[1], gc.IsNil)
 
 	err = t.Env.StopInstances(inst.Id())
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	// The machine may not be marked as shutting down
 	// immediately. Repeat a few times to ensure we get the error.
@@ -213,88 +229,89 @@ func (t *LiveTests) TestStartStop(c *gc.C) {
 }
 
 func (t *LiveTests) TestPorts(c *gc.C) {
-	t.PrepareOnce(c)
-	t.UploadFakeTools(c, t.Env.Storage())
+	t.BootstrapOnce(c)
 
-	inst1, _ := testing.AssertStartInstance(c, t.Env, "1")
+	inst1, _ := jujutesting.AssertStartInstance(c, t.Env, "1")
 	c.Assert(inst1, gc.NotNil)
 	defer t.Env.StopInstances(inst1.Id())
 	ports, err := inst1.Ports("1")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ports, gc.HasLen, 0)
 
-	inst2, _ := testing.AssertStartInstance(c, t.Env, "2")
+	inst2, _ := jujutesting.AssertStartInstance(c, t.Env, "2")
 	c.Assert(inst2, gc.NotNil)
 	ports, err = inst2.Ports("2")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ports, gc.HasLen, 0)
 	defer t.Env.StopInstances(inst2.Id())
 
 	// Open some ports and check they're there.
-	err = inst1.OpenPorts("1", []network.Port{{"udp", 67}, {"tcp", 45}})
-	c.Assert(err, gc.IsNil)
+	err = inst1.OpenPorts("1", []network.PortRange{{67, 67, "udp"}, {45, 45, "tcp"}, {80, 100, "tcp"}})
+	c.Assert(err, jc.ErrorIsNil)
 	ports, err = inst1.Ports("1")
-	c.Assert(err, gc.IsNil)
-	c.Assert(ports, gc.DeepEquals, []network.Port{{"tcp", 45}, {"udp", 67}})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {80, 100, "tcp"}, {67, 67, "udp"}})
 	ports, err = inst2.Ports("2")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ports, gc.HasLen, 0)
 
-	err = inst2.OpenPorts("2", []network.Port{{"tcp", 89}, {"tcp", 45}})
-	c.Assert(err, gc.IsNil)
+	err = inst2.OpenPorts("2", []network.PortRange{{89, 89, "tcp"}, {45, 45, "tcp"}, {20, 30, "tcp"}})
+	c.Assert(err, jc.ErrorIsNil)
 
 	// Check there's no crosstalk to another machine
 	ports, err = inst2.Ports("2")
-	c.Assert(err, gc.IsNil)
-	c.Assert(ports, gc.DeepEquals, []network.Port{{"tcp", 45}, {"tcp", 89}})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ports, gc.DeepEquals, []network.PortRange{{20, 30, "tcp"}, {45, 45, "tcp"}, {89, 89, "tcp"}})
 	ports, err = inst1.Ports("1")
-	c.Assert(err, gc.IsNil)
-	c.Assert(ports, gc.DeepEquals, []network.Port{{"tcp", 45}, {"udp", 67}})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {80, 100, "tcp"}, {67, 67, "udp"}})
 
 	// Check that opening the same port again is ok.
 	oldPorts, err := inst2.Ports("2")
-	c.Assert(err, gc.IsNil)
-	err = inst2.OpenPorts("2", []network.Port{{"tcp", 45}})
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
+	err = inst2.OpenPorts("2", []network.PortRange{{45, 45, "tcp"}})
+	c.Assert(err, jc.ErrorIsNil)
+	err = inst2.OpenPorts("2", []network.PortRange{{20, 30, "tcp"}})
+	c.Assert(err, jc.ErrorIsNil)
 	ports, err = inst2.Ports("2")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ports, gc.DeepEquals, oldPorts)
 
 	// Check that opening the same port again and another port is ok.
-	err = inst2.OpenPorts("2", []network.Port{{"tcp", 45}, {"tcp", 99}})
-	c.Assert(err, gc.IsNil)
+	err = inst2.OpenPorts("2", []network.PortRange{{45, 45, "tcp"}, {99, 99, "tcp"}})
+	c.Assert(err, jc.ErrorIsNil)
 	ports, err = inst2.Ports("2")
-	c.Assert(err, gc.IsNil)
-	c.Assert(ports, gc.DeepEquals, []network.Port{{"tcp", 45}, {"tcp", 89}, {"tcp", 99}})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ports, gc.DeepEquals, []network.PortRange{{20, 30, "tcp"}, {45, 45, "tcp"}, {89, 89, "tcp"}, {99, 99, "tcp"}})
 
-	err = inst2.ClosePorts("2", []network.Port{{"tcp", 45}, {"tcp", 99}})
-	c.Assert(err, gc.IsNil)
+	err = inst2.ClosePorts("2", []network.PortRange{{45, 45, "tcp"}, {99, 99, "tcp"}, {20, 30, "tcp"}})
+	c.Assert(err, jc.ErrorIsNil)
 
 	// Check that we can close ports and that there's no crosstalk.
 	ports, err = inst2.Ports("2")
-	c.Assert(err, gc.IsNil)
-	c.Assert(ports, gc.DeepEquals, []network.Port{{"tcp", 89}})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ports, gc.DeepEquals, []network.PortRange{{89, 89, "tcp"}})
 	ports, err = inst1.Ports("1")
-	c.Assert(err, gc.IsNil)
-	c.Assert(ports, gc.DeepEquals, []network.Port{{"tcp", 45}, {"udp", 67}})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {80, 100, "tcp"}, {67, 67, "udp"}})
 
 	// Check that we can close multiple ports.
-	err = inst1.ClosePorts("1", []network.Port{{"tcp", 45}, {"udp", 67}})
-	c.Assert(err, gc.IsNil)
+	err = inst1.ClosePorts("1", []network.PortRange{{45, 45, "tcp"}, {67, 67, "udp"}, {80, 100, "tcp"}})
+	c.Assert(err, jc.ErrorIsNil)
 	ports, err = inst1.Ports("1")
 	c.Assert(ports, gc.HasLen, 0)
 
 	// Check that we can close ports that aren't there.
-	err = inst2.ClosePorts("2", []network.Port{{"tcp", 111}, {"udp", 222}})
-	c.Assert(err, gc.IsNil)
+	err = inst2.ClosePorts("2", []network.PortRange{{111, 111, "tcp"}, {222, 222, "udp"}, {600, 700, "tcp"}})
+	c.Assert(err, jc.ErrorIsNil)
 	ports, err = inst2.Ports("2")
-	c.Assert(ports, gc.DeepEquals, []network.Port{{"tcp", 89}})
+	c.Assert(ports, gc.DeepEquals, []network.PortRange{{89, 89, "tcp"}})
 
 	// Check errors when acting on environment.
-	err = t.Env.OpenPorts([]network.Port{{"tcp", 80}})
+	err = t.Env.OpenPorts([]network.PortRange{{80, 80, "tcp"}})
 	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "instance" for opening ports on environment`)
 
-	err = t.Env.ClosePorts([]network.Port{{"tcp", 80}})
+	err = t.Env.ClosePorts([]network.PortRange{{80, 80, "tcp"}})
 	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "instance" for closing ports on environment`)
 
 	_, err = t.Env.Ports()
@@ -302,64 +319,63 @@ func (t *LiveTests) TestPorts(c *gc.C) {
 }
 
 func (t *LiveTests) TestGlobalPorts(c *gc.C) {
-	t.PrepareOnce(c)
-	t.UploadFakeTools(c, t.Env.Storage())
+	t.BootstrapOnce(c)
 
 	// Change configuration.
 	oldConfig := t.Env.Config()
 	defer func() {
 		err := t.Env.SetConfig(oldConfig)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 	}()
 
 	attrs := t.Env.Config().AllAttrs()
-	attrs["firewall-mode"] = "global"
+	attrs["firewall-mode"] = config.FwGlobal
 	newConfig, err := t.Env.Config().Apply(attrs)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	err = t.Env.SetConfig(newConfig)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	// Create instances and check open ports on both instances.
-	inst1, _ := testing.AssertStartInstance(c, t.Env, "1")
+	inst1, _ := jujutesting.AssertStartInstance(c, t.Env, "1")
 	defer t.Env.StopInstances(inst1.Id())
 	ports, err := t.Env.Ports()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ports, gc.HasLen, 0)
 
-	inst2, _ := testing.AssertStartInstance(c, t.Env, "2")
+	inst2, _ := jujutesting.AssertStartInstance(c, t.Env, "2")
 	ports, err = t.Env.Ports()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ports, gc.HasLen, 0)
 	defer t.Env.StopInstances(inst2.Id())
 
-	err = t.Env.OpenPorts([]network.Port{{"udp", 67}, {"tcp", 45}, {"tcp", 89}, {"tcp", 99}})
-	c.Assert(err, gc.IsNil)
+	err = t.Env.OpenPorts([]network.PortRange{{67, 67, "udp"}, {45, 45, "tcp"}, {89, 89, "tcp"}, {99, 99, "tcp"}, {100, 110, "tcp"}})
+	c.Assert(err, jc.ErrorIsNil)
 
 	ports, err = t.Env.Ports()
-	c.Assert(err, gc.IsNil)
-	c.Assert(ports, gc.DeepEquals, []network.Port{{"tcp", 45}, {"tcp", 89}, {"tcp", 99}, {"udp", 67}})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {89, 89, "tcp"}, {99, 99, "tcp"}, {100, 110, "tcp"}, {67, 67, "udp"}})
 
 	// Check closing some ports.
-	err = t.Env.ClosePorts([]network.Port{{"tcp", 99}, {"udp", 67}})
-	c.Assert(err, gc.IsNil)
+	err = t.Env.ClosePorts([]network.PortRange{{99, 99, "tcp"}, {67, 67, "udp"}})
+	c.Assert(err, jc.ErrorIsNil)
 
 	ports, err = t.Env.Ports()
-	c.Assert(err, gc.IsNil)
-	c.Assert(ports, gc.DeepEquals, []network.Port{{"tcp", 45}, {"tcp", 89}})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {89, 89, "tcp"}, {100, 110, "tcp"}})
 
 	// Check that we can close ports that aren't there.
-	err = t.Env.ClosePorts([]network.Port{{"tcp", 111}, {"udp", 222}})
-	c.Assert(err, gc.IsNil)
+	err = t.Env.ClosePorts([]network.PortRange{{111, 111, "tcp"}, {222, 222, "udp"}, {2000, 2500, "tcp"}})
+	c.Assert(err, jc.ErrorIsNil)
 
 	ports, err = t.Env.Ports()
-	c.Assert(err, gc.IsNil)
-	c.Assert(ports, gc.DeepEquals, []network.Port{{"tcp", 45}, {"tcp", 89}})
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(ports, gc.DeepEquals, []network.PortRange{{45, 45, "tcp"}, {89, 89, "tcp"}, {100, 110, "tcp"}})
 
 	// Check errors when acting on instances.
-	err = inst1.OpenPorts("1", []network.Port{{"tcp", 80}})
+	err = inst1.OpenPorts("1", []network.PortRange{{80, 80, "tcp"}})
 	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "global" for opening ports on instance`)
 
-	err = inst1.ClosePorts("1", []network.Port{{"tcp", 80}})
+	err = inst1.ClosePorts("1", []network.PortRange{{80, 80, "tcp"}})
 	c.Assert(err, gc.ErrorMatches, `invalid firewall mode "global" for closing ports on instance`)
 
 	_, err = inst1.Ports("1")
@@ -377,7 +393,8 @@ func (t *LiveTests) TestBootstrapMultiple(c *gc.C) {
 	c.Logf("destroy env")
 	env := t.Env
 	t.Destroy(c)
-	env.Destroy() // Again, should work fine and do nothing.
+	err = env.Destroy() // Again, should work fine and do nothing.
+	c.Assert(err, jc.ErrorIsNil)
 
 	// check that we can bootstrap after destroy
 	t.BootstrapOnce(c)
@@ -391,40 +408,42 @@ func (t *LiveTests) TestBootstrapAndDeploy(c *gc.C) {
 
 	// TODO(niemeyer): Stop growing this kitchen sink test and split it into proper parts.
 
-	c.Logf("opening connection")
-	conn, err := juju.NewConn(t.Env)
-	c.Assert(err, gc.IsNil)
-	defer conn.Close()
+	c.Logf("opening state")
+	st := t.Env.(jujutesting.GetStater).GetStateInAPIServer()
+
+	env, err := st.Environment()
+	c.Assert(err, jc.ErrorIsNil)
+	owner := env.Owner()
 
 	c.Logf("opening API connection")
-	apiConn, err := juju.NewAPIConn(t.Env, api.DefaultDialOpts())
-	c.Assert(err, gc.IsNil)
-	defer conn.Close()
+	apiState, err := juju.NewAPIState(owner, t.Env, api.DefaultDialOpts())
+	c.Assert(err, jc.ErrorIsNil)
+	defer apiState.Close()
 
 	// Check that the agent version has made it through the
 	// bootstrap process (it's optional in the config.Config)
-	cfg, err := conn.State.EnvironConfig()
-	c.Assert(err, gc.IsNil)
+	cfg, err := st.EnvironConfig()
+	c.Assert(err, jc.ErrorIsNil)
 	agentVersion, ok := cfg.AgentVersion()
-	c.Check(ok, gc.Equals, true)
+	c.Check(ok, jc.IsTrue)
 	c.Check(agentVersion, gc.Equals, version.Current.Number)
 
 	// Check that the constraints have been set in the environment.
-	cons, err := conn.State.EnvironConstraints()
-	c.Assert(err, gc.IsNil)
+	cons, err := st.EnvironConstraints()
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cons.String(), gc.Equals, "mem=2048M")
 
 	// Wait for machine agent to come up on the bootstrap
 	// machine and find the deployed series from that.
-	m0, err := conn.State.Machine("0")
-	c.Assert(err, gc.IsNil)
+	m0, err := st.Machine("0")
+	c.Assert(err, jc.ErrorIsNil)
 
 	instId0, err := m0.InstanceId()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	// Check that the API connection is working.
-	status, err := apiConn.State.Client().Status(nil)
-	c.Assert(err, gc.IsNil)
+	status, err := apiState.Client().Status(nil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(status.Machines["0"].InstanceId, gc.Equals, string(instId0))
 
 	mw0 := newMachineToolWaiter(m0)
@@ -439,29 +458,29 @@ func (t *LiveTests) TestBootstrapAndDeploy(c *gc.C) {
 	// Create a new service and deploy a unit of it.
 	c.Logf("deploying service")
 	repoDir := c.MkDir()
-	url := charmtesting.Charms.ClonedURL(repoDir, mtools0.Version.Series, "dummy")
-	sch, err := conn.PutCharm(url, &charm.LocalRepository{Path: repoDir}, false)
-	c.Assert(err, gc.IsNil)
-	svc, err := conn.State.AddService("dummy", "user-admin", sch, nil)
-	c.Assert(err, gc.IsNil)
-	units, err := juju.AddUnits(conn.State, svc, 1, "")
-	c.Assert(err, gc.IsNil)
+	url := testcharms.Repo.ClonedURL(repoDir, mtools0.Version.Series, "dummy")
+	sch, err := jujutesting.PutCharm(st, url, &charm.LocalRepository{Path: repoDir}, false)
+	c.Assert(err, jc.ErrorIsNil)
+	svc, err := st.AddService("dummy", owner.String(), sch, nil)
+	c.Assert(err, jc.ErrorIsNil)
+	units, err := juju.AddUnits(st, svc, 1, "")
+	c.Assert(err, jc.ErrorIsNil)
 	unit := units[0]
 
 	// Wait for the unit's machine and associated agent to come up
 	// and announce itself.
 	mid1, err := unit.AssignedMachineId()
-	c.Assert(err, gc.IsNil)
-	m1, err := conn.State.Machine(mid1)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
+	m1, err := st.Machine(mid1)
+	c.Assert(err, jc.ErrorIsNil)
 	mw1 := newMachineToolWaiter(m1)
 	defer mw1.Stop()
 	waitAgentTools(c, mw1, mtools0.Version)
 
 	err = m1.Refresh()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	instId1, err := m1.InstanceId()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	uw := newUnitToolWaiter(unit)
 	defer uw.Stop()
 	utools := waitAgentTools(c, uw, expectedVersion)
@@ -469,7 +488,7 @@ func (t *LiveTests) TestBootstrapAndDeploy(c *gc.C) {
 	// Check that we can upgrade the environment.
 	newVersion := utools.Version
 	newVersion.Patch++
-	t.checkUpgrade(c, conn, newVersion, mw0, mw1, uw)
+	t.checkUpgrade(c, st, newVersion, mw0, mw1, uw)
 
 	// BUG(niemeyer): Logic below is very much wrong. Must be:
 	//
@@ -484,7 +503,7 @@ func (t *LiveTests) TestBootstrapAndDeploy(c *gc.C) {
 	// check that the PA removes it.
 	c.Logf("removing unit")
 	err = unit.Destroy()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	// Wait until unit is dead
 	uwatch := unit.Watch()
@@ -498,7 +517,7 @@ func (t *LiveTests) TestBootstrapAndDeploy(c *gc.C) {
 			c.Logf("unit has been removed")
 			break
 		}
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 	}
 	for {
 		c.Logf("destroying machine")
@@ -512,98 +531,10 @@ func (t *LiveTests) TestBootstrapAndDeploy(c *gc.C) {
 		if errors.IsNotFound(err) {
 			break
 		}
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 	}
 	c.Logf("waiting for instance to be removed")
-	t.assertStopInstance(c, conn.Environ, instId1)
-}
-
-func (t *LiveTests) TestBootstrapVerifyStorage(c *gc.C) {
-	// Bootstrap automatically verifies that storage is writable.
-	t.BootstrapOnce(c)
-	environ := t.Env
-	stor := environ.Storage()
-	reader, err := storage.Get(stor, "bootstrap-verify")
-	c.Assert(err, gc.IsNil)
-	defer reader.Close()
-	contents, err := ioutil.ReadAll(reader)
-	c.Assert(err, gc.IsNil)
-	c.Check(string(contents), gc.Equals,
-		"juju-core storage writing verified: ok\n")
-}
-
-func restoreBootstrapVerificationFile(c *gc.C, stor storage.Storage) {
-	content := "juju-core storage writing verified: ok\n"
-	contentReader := strings.NewReader(content)
-	err := stor.Put("bootstrap-verify", contentReader,
-		int64(len(content)))
-	c.Assert(err, gc.IsNil)
-}
-
-func (t *LiveTests) TestCheckEnvironmentOnConnect(c *gc.C) {
-	// When new connection is established to a bootstraped environment,
-	// it is checked that we are running against a juju-core environment.
-	if !t.CanOpenState {
-		c.Skip("CanOpenState is false; cannot open state connection")
-	}
-	t.BootstrapOnce(c)
-
-	conn, err := juju.NewConn(t.Env)
-	c.Assert(err, gc.IsNil)
-	conn.Close()
-
-	apiConn, err := juju.NewAPIConn(t.Env, api.DefaultDialOpts())
-	c.Assert(err, gc.IsNil)
-	apiConn.Close()
-}
-
-func (t *LiveTests) TestCheckEnvironmentOnConnectNoVerificationFile(c *gc.C) {
-	// When new connection is established to a bootstraped environment,
-	// it is checked that we are running against a juju-core environment.
-	//
-	// Absence of a verification file means it is a juju-core environment
-	// with an older version, which is fine.
-	if !t.CanOpenState {
-		c.Skip("CanOpenState is false; cannot open state connection")
-	}
-	t.BootstrapOnce(c)
-	environ := t.Env
-	stor := environ.Storage()
-	err := stor.Remove("bootstrap-verify")
-	c.Assert(err, gc.IsNil)
-	defer restoreBootstrapVerificationFile(c, stor)
-
-	conn, err := juju.NewConn(t.Env)
-	c.Assert(err, gc.IsNil)
-	conn.Close()
-}
-
-func (t *LiveTests) TestCheckEnvironmentOnConnectBadVerificationFile(c *gc.C) {
-	// When new connection is established to a bootstraped environment,
-	// it is checked that we are running against a juju-core environment.
-	//
-	// If the verification file has unexpected content, it is not
-	// a juju-core environment (likely to a Python juju environment).
-	if !t.CanOpenState {
-		c.Skip("CanOpenState is false; cannot open state connection")
-	}
-	t.BootstrapOnce(c)
-	environ := t.Env
-	stor := environ.Storage()
-
-	// Finally, replace the content with an arbitrary string.
-	badVerificationContent := "bootstrap storage verification"
-	reader := strings.NewReader(badVerificationContent)
-	err := stor.Put(
-		"bootstrap-verify",
-		reader,
-		int64(len(badVerificationContent)))
-	c.Assert(err, gc.IsNil)
-	defer restoreBootstrapVerificationFile(c, stor)
-
-	// Running NewConn() should fail.
-	_, err = juju.NewConn(t.Env)
-	c.Assert(err, gc.Equals, environs.InvalidEnvironmentError)
+	t.assertStopInstance(c, t.Env, instId1)
 }
 
 type tooler interface {
@@ -697,17 +628,17 @@ func (w *toolsWaiter) NextTools(c *gc.C) (*coretools.Tools, error) {
 func waitAgentTools(c *gc.C, w *toolsWaiter, expect version.Binary) *coretools.Tools {
 	c.Logf("waiting for %v to signal agent version", w.tooler.String())
 	tools, err := w.NextTools(c)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Check(tools.Version, gc.Equals, expect)
 	return tools
 }
 
 // checkUpgrade sets the environment agent version and checks that
 // all the provided watchers upgrade to the requested version.
-func (t *LiveTests) checkUpgrade(c *gc.C, conn *juju.Conn, newVersion version.Binary, waiters ...*toolsWaiter) {
+func (t *LiveTests) checkUpgrade(c *gc.C, st *state.State, newVersion version.Binary, waiters ...*toolsWaiter) {
 	c.Logf("putting testing version of juju tools")
-	upgradeTools, err := sync.Upload(t.Env.Storage(), &newVersion.Number, newVersion.Series)
-	c.Assert(err, gc.IsNil)
+	upgradeTools, err := sync.Upload(t.toolsStorage, "released", &newVersion.Number, newVersion.Series)
+	c.Assert(err, jc.ErrorIsNil)
 	// sync.Upload always returns tools for the series on which the tests are running.
 	// We are only interested in checking the version.Number below so need to fake the
 	// upgraded tools series to match that of newVersion.
@@ -715,8 +646,8 @@ func (t *LiveTests) checkUpgrade(c *gc.C, conn *juju.Conn, newVersion version.Bi
 
 	// Check that the put version really is the version we expect.
 	c.Assert(upgradeTools.Version, gc.Equals, newVersion)
-	err = statetesting.SetAgentVersion(conn.State, newVersion.Number)
-	c.Assert(err, gc.IsNil)
+	err = statetesting.SetAgentVersion(st, newVersion.Number)
+	c.Assert(err, jc.ErrorIsNil)
 
 	for i, w := range waiters {
 		c.Logf("waiting for upgrade of %d: %v", i, w.tooler.String())
@@ -735,14 +666,14 @@ func (t *LiveTests) assertStartInstance(c *gc.C, m *state.Machine) {
 	// Wait for machine to get an instance id.
 	for a := waitAgent.Start(); a.Next(); {
 		err := m.Refresh()
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		instId, err := m.InstanceId()
 		if err != nil {
-			c.Assert(err, jc.Satisfies, state.IsNotProvisionedError)
+			c.Assert(err, jc.Satisfies, errors.IsNotProvisioned)
 			continue
 		}
 		_, err = t.Env.Instances([]instance.Id{instId})
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		return
 	}
 	c.Fatalf("provisioner failed to start machine after %v", waitAgent.Total)
@@ -774,10 +705,10 @@ func assertInstanceId(c *gc.C, m *state.Machine, inst instance.Instance) {
 	}
 	for a := waitAgent.Start(); a.Next(); {
 		err := m.Refresh()
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		gotId, err = m.InstanceId()
 		if err != nil {
-			c.Assert(err, jc.Satisfies, state.IsNotProvisionedError)
+			c.Assert(err, jc.Satisfies, errors.IsNotProvisioned)
 			if inst == nil {
 				return
 			}
@@ -785,79 +716,30 @@ func assertInstanceId(c *gc.C, m *state.Machine, inst instance.Instance) {
 		}
 		break
 	}
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(gotId, gc.Equals, wantId)
 }
 
-// TODO check that binary data works ok?
-var contents = []byte("hello\n")
-var contents2 = []byte("goodbye\n\n")
-
-func (t *LiveTests) TestFile(c *gc.C) {
-	t.PrepareOnce(c)
-	name := fmt.Sprint("testfile", time.Now().UnixNano())
-	stor := t.Env.Storage()
-
-	checkFileDoesNotExist(c, stor, name, t.Attempt)
-	checkPutFile(c, stor, name, contents)
-	checkFileHasContents(c, stor, name, contents, t.Attempt)
-	checkPutFile(c, stor, name, contents2) // check that we can overwrite the file
-	checkFileHasContents(c, stor, name, contents2, t.Attempt)
-
-	// check that the listed contents include the
-	// expected name.
-	found := false
-	var names []string
-attempt:
-	for a := t.Attempt.Start(); a.Next(); {
-		var err error
-		names, err = stor.List("")
-		c.Assert(err, gc.IsNil)
-		for _, lname := range names {
-			if lname == name {
-				found = true
-				break attempt
-			}
-		}
-	}
-	if !found {
-		c.Errorf("file name %q not found in file list %q", name, names)
-	}
-	err := stor.Remove(name)
-	c.Check(err, gc.IsNil)
-	checkFileDoesNotExist(c, stor, name, t.Attempt)
-	// removing a file that does not exist should not be an error.
-	err = stor.Remove(name)
-	c.Check(err, gc.IsNil)
-
-	// RemoveAll deletes all files from storage.
-	checkPutFile(c, stor, "file-1.txt", contents)
-	checkPutFile(c, stor, "file-2.txt", contents)
-	err = stor.RemoveAll()
-	c.Check(err, gc.IsNil)
-	checkFileDoesNotExist(c, stor, "file-1.txt", t.Attempt)
-	checkFileDoesNotExist(c, stor, "file-2.txt", t.Attempt)
-}
-
 // Check that we get a consistent error when asking for an instance without
 // a valid machine config.
 func (t *LiveTests) TestStartInstanceWithEmptyNonceFails(c *gc.C) {
 	machineId := "4"
-	stateInfo := testing.FakeStateInfo(machineId)
-	apiInfo := testing.FakeAPIInfo(machineId)
-	machineConfig := environs.NewMachineConfig(machineId, "", nil, stateInfo, apiInfo)
+	stateInfo := jujutesting.FakeStateInfo(machineId)
+	apiInfo := jujutesting.FakeAPIInfo(machineId)
+	machineConfig, err := environs.NewMachineConfig(machineId, "", "released", "quantal", true, nil, stateInfo, apiInfo)
+	c.Assert(err, jc.ErrorIsNil)
 
 	t.PrepareOnce(c)
-	possibleTools := envtesting.AssertUploadFakeToolsVersions(c, t.Env.Storage(), version.MustParseBinary("5.4.5-trusty-amd64"))
-	inst, _, _, err := t.Env.StartInstance(environs.StartInstanceParams{
+	possibleTools := envtesting.AssertUploadFakeToolsVersions(c, t.toolsStorage, "released", "released", version.MustParseBinary("5.4.5-trusty-amd64"))
+	result, err := t.Env.StartInstance(environs.StartInstanceParams{
 		Tools:         possibleTools,
 		MachineConfig: machineConfig,
 	})
-	if inst != nil {
-		err := t.Env.StopInstances(inst.Id())
-		c.Check(err, gc.IsNil)
+	if result != nil && result.Instance != nil {
+		err := t.Env.StopInstances(result.Instance.Id())
+		c.Check(err, jc.ErrorIsNil)
 	}
-	c.Assert(inst, gc.IsNil)
+	c.Assert(result, gc.IsNil)
 	c.Assert(err, gc.ErrorMatches, ".*missing machine nonce")
 }
 
@@ -877,62 +759,29 @@ func (t *LiveTests) TestBootstrapWithDefaultSeries(c *gc.C) {
 		"state-server": false,
 		"name":         "dummy storage",
 	}))
-	dummyenv, err := environs.Prepare(dummyCfg, coretesting.Context(c), configstore.NewMem())
-	c.Assert(err, gc.IsNil)
+	dummyenv, err := environs.Prepare(dummyCfg, envtesting.BootstrapContext(c), configstore.NewMem())
+	c.Assert(err, jc.ErrorIsNil)
 	defer dummyenv.Destroy()
 
 	t.Destroy(c)
 
 	attrs := t.TestConfig.Merge(coretesting.Attrs{"default-series": other.Series})
 	cfg, err := config.New(config.NoDefaults, attrs)
-	c.Assert(err, gc.IsNil)
-	env, err := environs.Prepare(cfg, coretesting.Context(c), t.ConfigStore)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
+	env, err := environs.Prepare(cfg, envtesting.BootstrapContext(c), t.ConfigStore)
+	c.Assert(err, jc.ErrorIsNil)
 	defer environs.Destroy(env, t.ConfigStore)
 
-	currentName := envtools.StorageName(current)
-	otherName := envtools.StorageName(other)
-	envStorage := env.Storage()
-	dummyStorage := dummyenv.Storage()
-
-	defer envStorage.Remove(otherName)
-
-	_, err = sync.Upload(dummyStorage, &current.Number)
-	c.Assert(err, gc.IsNil)
-
-	// This will only work while cross-compiling across releases is safe,
-	// which depends on external elements. Tends to be safe for the last
-	// few releases, but we may have to refactor some day.
-	err = storageCopy(dummyStorage, currentName, envStorage, otherName)
-	c.Assert(err, gc.IsNil)
-
-	err = bootstrap.Bootstrap(coretesting.Context(c), env, environs.BootstrapParams{})
-	c.Assert(err, gc.IsNil)
-
-	conn, err := juju.NewConn(env)
-	c.Assert(err, gc.IsNil)
-	defer conn.Close()
+	err = bootstrap.Bootstrap(envtesting.BootstrapContext(c), env, bootstrap.BootstrapParams{})
+	c.Assert(err, jc.ErrorIsNil)
 
+	st := t.Env.(jujutesting.GetStater).GetStateInAPIServer()
 	// Wait for machine agent to come up on the bootstrap
 	// machine and ensure it deployed the proper series.
-	m0, err := conn.State.Machine("0")
-	c.Assert(err, gc.IsNil)
+	m0, err := st.Machine("0")
+	c.Assert(err, jc.ErrorIsNil)
 	mw0 := newMachineToolWaiter(m0)
 	defer mw0.Stop()
 
 	waitAgentTools(c, mw0, other)
 }
-
-func storageCopy(source storage.Storage, sourcePath string, target storage.Storage, targetPath string) error {
-	rc, err := storage.Get(source, sourcePath)
-	if err != nil {
-		return err
-	}
-	var buf bytes.Buffer
-	_, err = io.Copy(&buf, rc)
-	rc.Close()
-	if err != nil {
-		return err
-	}
-	return target.Put(targetPath, &buf, int64(buf.Len()))
-}
