@@ -67,14 +67,16 @@ type HasRegion interface {
 }
 
 type LookupConstraint interface {
-	// Generates a string array representing product ids formed similarly to an ISCSI qualified name (IQN).
-	Ids() ([]string, error)
-	// Returns the constraint parameters.
+	// IndexIds generates a string array representing index ids formed similarly to an ISCSI qualified name (IQN).
+	IndexIds() []string
+	// ProductIds generates a string array representing product ids formed similarly to an ISCSI qualified name (IQN).
+	ProductIds() ([]string, error)
+	// Params returns the constraint parameters.
 	Params() LookupParams
 }
 
 // LookupParams defines criteria used to find a metadata record.
-// Derived structs implement the Ids() method.
+// Derived structs implement the IndexIds() and ProductIds() method.
 type LookupParams struct {
 	CloudSpec
 	Series []string
@@ -168,7 +170,7 @@ type MirrorReference struct {
 	Format   string      `json:"format"`
 	DataType string      `json:"datatype"`
 	Path     string      `json:"path"`
-	Clouds   []CloudSpec `json:"clouds"`
+	Clouds   []CloudSpec `json:"clouds,omitempty"`
 }
 
 type MirrorMetadata struct {
@@ -224,10 +226,20 @@ func (metadata *CloudMetadata) extractCatalogsForProducts(productIds []string) [
 }
 
 // extractIndexes returns just the array of indexes, in arbitrary order.
-func (ind *Indices) extractIndexes() IndexMetadataSlice {
+func (ind *Indices) extractIndexes(indexIds []string) IndexMetadataSlice {
 	result := make(IndexMetadataSlice, 0, len(ind.Indexes))
-	for _, metadata := range ind.Indexes {
-		result = append(result, metadata)
+	if len(indexIds) == 0 {
+		// No ids specified so return everything.
+		for _, metadata := range ind.Indexes {
+			result = append(result, metadata)
+		}
+	} else {
+		// Return metadata for just the specified ids.
+		for _, id := range indexIds {
+			if metadata, ok := ind.Indexes[id]; ok {
+				result = append(result, metadata)
+			}
+		}
 	}
 	return result
 }
@@ -290,15 +302,24 @@ func newNoMatchingProductsError(message string, args ...interface{}) error {
 }
 
 const (
-	streamsDir       = "streams/%s"
-	unsignedIndex    = "streams/%s/index.json"
-	signedIndex      = "streams/%s/index.sjson"
-	unsignedMirror   = "streams/%s/mirrors.json"
-	defaultIndexPath = "streams/%s/index"
+	// These constants are used to specify the filenames used
+	// when generating simplestreams metadata for local consumption
+	// or for uploading to streams.canonical.com.
+	unsignedIndex  = "streams/%s/index%s.json"
+	unsignedMirror = "streams/%s/mirrors.json"
+	MirrorFile     = "streams/v1/cpc-mirrors.json"
 
-	defaultMirrorsPath = "streams/%s/mirrors"
-	signedSuffix       = ".sjson"
-	UnsignedSuffix     = ".json"
+	// These constants are used when searching for simplestreams metadata.
+	defaultLegacyIndexPath = "streams/%s/index"
+	defaultIndexPath       = "streams/%s/index2"
+	defaultMirrorsPath     = "streams/%s/mirrors"
+	SignedSuffix           = ".sjson"
+	UnsignedSuffix         = ".json"
+
+	// These constants define the currently supported simplestreams data formats.
+	IndexFormat   = "index:1.0"
+	ProductFormat = "products:1.0"
+	MirrorFormat  = "mirrors:1.0"
 )
 
 type appendMatchingFunc func(DataSource, []interface{}, map[string]interface{}, LookupConstraint) []interface{}
@@ -317,24 +338,18 @@ type ValueParams struct {
 	PublicKey string
 }
 
-// StreamsDir returns the streams directory for streamsVersion.
-func StreamsDir(streamsVersion string) string {
-	return fmt.Sprintf(streamsDir, streamsVersion)
-}
-
 // MirrorsPath returns the mirrors path for streamsVersion.
 func MirrorsPath(streamsVersion string) string {
 	return fmt.Sprintf(defaultMirrorsPath, streamsVersion)
 }
 
 // UnsignedIndex returns an unsigned index file name for streamsVersion.
-func UnsignedIndex(streamsVersion string) string {
-	return fmt.Sprintf(unsignedIndex, streamsVersion)
-}
-
-// SignedIndex returns a signed index file name for streamsVersion.
-func SignedIndex(streamsVersion string) string {
-	return fmt.Sprintf(signedIndex, streamsVersion)
+func UnsignedIndex(streamsVersion string, indexFileVersion int) string {
+	indexFileSuffix := ""
+	if indexFileVersion > 1 {
+		indexFileSuffix = fmt.Sprintf("%d", indexFileVersion)
+	}
+	return fmt.Sprintf(unsignedIndex, streamsVersion, indexFileSuffix)
 }
 
 // UnsignedMirror returns an unsigned mirror file name for streamsVersion.
@@ -342,20 +357,25 @@ func UnsignedMirror(streamsVersion string) string {
 	return fmt.Sprintf(unsignedMirror, streamsVersion)
 }
 
+// GetMetadataParams defines parameters used to load simplestreams metadata.
+type GetMetadataParams struct {
+	StreamsVersion   string
+	LookupConstraint LookupConstraint
+	OnlySigned       bool
+	ValueParams      ValueParams
+}
+
 // GetMetadata returns metadata records matching the specified constraint,looking in each source for signed metadata.
 // If onlySigned is false and no signed metadata is found in a source, the source is used to look for unsigned metadata.
 // Each source is tried in turn until at least one signed (or unsigned) match is found.
-func GetMetadata(
-	sources []DataSource, streamsVersion string, cons LookupConstraint, onlySigned bool,
-	params ValueParams) (items []interface{}, resolveInfo *ResolveInfo, err error) {
+func GetMetadata(sources []DataSource, params GetMetadataParams) (items []interface{}, resolveInfo *ResolveInfo, err error) {
 
-	baseIndexPath := fmt.Sprintf(defaultIndexPath, streamsVersion)
-	mirrorsPath := fmt.Sprintf(defaultMirrorsPath, streamsVersion)
 	for _, source := range sources {
-		items, resolveInfo, err = getMaybeSignedMetadata(source, baseIndexPath, mirrorsPath, cons, true, params)
+		logger.Debugf("searching for metadata in datasource %q", source.Description())
+		items, resolveInfo, err = getMaybeSignedMetadata(source, params, true)
 		// If no items are found using signed metadata, check unsigned.
-		if err != nil && len(items) == 0 && !onlySigned {
-			items, resolveInfo, err = getMaybeSignedMetadata(source, baseIndexPath, mirrorsPath, cons, false, params)
+		if err != nil && len(items) == 0 && !params.OnlySigned {
+			items, resolveInfo, err = getMaybeSignedMetadata(source, params, false)
 		}
 		if err == nil {
 			break
@@ -368,26 +388,37 @@ func GetMetadata(
 	return items, resolveInfo, err
 }
 
-// getMaybeSignedMetadata returns metadata records matching the specified constraint.
-func getMaybeSignedMetadata(source DataSource, baseIndexPath, mirrorsPath string, cons LookupConstraint,
-	signed bool, params ValueParams) ([]interface{}, *ResolveInfo, error) {
+// getMaybeSignedMetadata returns metadata records matching the specified constraint in params.
+func getMaybeSignedMetadata(source DataSource, params GetMetadataParams, signed bool) ([]interface{}, *ResolveInfo, error) {
+
+	makeIndexPath := func(basePath string) string {
+		pathNoSuffix := fmt.Sprintf(basePath, params.StreamsVersion)
+		indexPath := pathNoSuffix + UnsignedSuffix
+		if signed {
+			indexPath = pathNoSuffix + SignedSuffix
+		}
+		return indexPath
+	}
 
 	resolveInfo := &ResolveInfo{}
-	indexPath := baseIndexPath + UnsignedSuffix
-	if signed {
-		indexPath = baseIndexPath + signedSuffix
-	}
-	var items []interface{}
-	indexURL, err := source.URL(indexPath)
-	if err != nil {
-		// Some providers return an error if asked for the URL of a non-existent file.
-		// So the best we can do is use the relative path for the URL when logging messages.
-		indexURL = indexPath
-	}
 	resolveInfo.Source = source.Description()
 	resolveInfo.Signed = signed
+	indexPath := makeIndexPath(defaultIndexPath)
+
+	mirrorsPath := fmt.Sprintf(defaultMirrorsPath, params.StreamsVersion)
+	cons := params.LookupConstraint
+
+	indexRef, indexURL, err := fetchIndex(
+		source, indexPath, mirrorsPath, cons.Params().CloudSpec, signed, params.ValueParams,
+	)
+	if errors.IsNotFound(err) || errors.IsUnauthorized(err) {
+		logger.Debugf("%s not found, trying legacy index file", indexPath)
+		indexPath = makeIndexPath(defaultLegacyIndexPath)
+		indexRef, indexURL, err = fetchIndex(
+			source, indexPath, mirrorsPath, cons.Params().CloudSpec, signed, params.ValueParams,
+		)
+	}
 	resolveInfo.IndexURL = indexURL
-	indexRef, err := GetIndexWithFormat(source, indexPath, "index:1.0", mirrorsPath, signed, cons.Params().CloudSpec, params)
 	if err != nil {
 		if errors.IsNotFound(err) || errors.IsUnauthorized(err) {
 			logger.Debugf("cannot load index %q: %v", indexURL, err)
@@ -395,7 +426,7 @@ func getMaybeSignedMetadata(source DataSource, baseIndexPath, mirrorsPath string
 		return nil, resolveInfo, err
 	}
 	logger.Debugf("read metadata index at %q", indexURL)
-	items, err = indexRef.getLatestMetadataWithFormat(cons, "products:1.0", signed)
+	items, err := indexRef.getLatestMetadataWithFormat(cons, ProductFormat, signed)
 	if err != nil {
 		if errors.IsNotFound(err) {
 			logger.Debugf("skipping index because of error getting latest metadata %q: %v", indexURL, err)
@@ -411,6 +442,21 @@ func getMaybeSignedMetadata(source DataSource, baseIndexPath, mirrorsPath string
 	return items, resolveInfo, err
 }
 
+// fetchIndex attempts to load the index file at indexPath in source.
+func fetchIndex(source DataSource, indexPath string, mirrorsPath string, cloudSpec CloudSpec,
+	signed bool, params ValueParams) (indexRef *IndexReference, indexURL string, _ error) {
+	indexURL, err := source.URL(indexPath)
+	if err != nil {
+		// Some providers return an error if asked for the URL of a non-existent file.
+		// So the best we can do is use the relative path for the URL when logging messages.
+		indexURL = indexPath
+	}
+	indexRef, err = GetIndexWithFormat(
+		source, indexPath, IndexFormat, mirrorsPath, signed, cloudSpec, params,
+	)
+	return indexRef, indexURL, err
+}
+
 // fetchData gets all the data from the given source located at the specified path.
 // It returns the data found and the full URL used.
 func fetchData(source DataSource, path string, requireSigned bool, publicKey string) (data []byte, dataURL string, err error) {
@@ -487,7 +533,7 @@ func getMirrorRefs(source DataSource, baseMirrorsPath string, requireSigned bool
 
 	mirrorsPath := baseMirrorsPath + UnsignedSuffix
 	if requireSigned {
-		mirrorsPath = baseMirrorsPath + signedSuffix
+		mirrorsPath = baseMirrorsPath + SignedSuffix
 	}
 	var mirrors MirrorRefs
 	data, url, err := fetchData(source, mirrorsPath, requireSigned, params.PublicKey)
@@ -513,7 +559,7 @@ func getMirror(source DataSource, mirrors MirrorRefs, datatype, contentId string
 	if err != nil {
 		return nil, err
 	}
-	mirrorInfo, err := mirrorRef.getMirrorInfo(source, contentId, cloudSpec, "mirrors:1.0", requireSigned, publicKey)
+	mirrorInfo, err := mirrorRef.getMirrorInfo(source, contentId, cloudSpec, MirrorFormat, requireSigned, publicKey)
 	if err != nil {
 		return nil, err
 	}
@@ -529,12 +575,12 @@ func (indexRef *IndexReference) GetProductsPath(cons LookupConstraint) (string,
 	if indexRef.MirroredProductsPath != "" {
 		return indexRef.MirroredProductsPath, nil
 	}
-	prodIds, err := cons.Ids()
+	prodIds, err := cons.ProductIds()
 	if err != nil {
 		return "", err
 	}
-	candidates := indexRef.extractIndexes()
-	// Restrict to image-ids entries.
+	candidates := indexRef.extractIndexes(cons.IndexIds())
+	// Restrict to the relevant data type entries.
 	dataTypeMatches := func(metadata *IndexMetadata) bool {
 		return metadata.DataType == indexRef.valueParams.DataType
 	}
@@ -925,7 +971,7 @@ func (indexRef *IndexReference) getLatestMetadataWithFormat(cons LookupConstrain
 
 // GetLatestMetadata extracts and returns the metadata records matching the given criteria.
 func GetLatestMetadata(metadata *CloudMetadata, cons LookupConstraint, source DataSource, filterFunc appendMatchingFunc) ([]interface{}, error) {
-	prodIds, err := cons.Ids()
+	prodIds, err := cons.ProductIds()
 	if err != nil {
 		return nil, err
 	}
