@@ -9,9 +9,11 @@ package tools
 import (
 	"bytes"
 	"crypto/sha256"
+	"encoding/json"
 	"fmt"
 	"hash"
 	"io"
+	"io/ioutil"
 	"path"
 	"sort"
 	"strings"
@@ -22,9 +24,9 @@ import (
 
 	"github.com/juju/juju/environs/simplestreams"
 	"github.com/juju/juju/environs/storage"
+	"github.com/juju/juju/juju/arch"
 	coretools "github.com/juju/juju/tools"
 	"github.com/juju/juju/version"
-	"github.com/juju/juju/version/ubuntu"
 )
 
 func init() {
@@ -37,6 +39,9 @@ const (
 
 	// StreamsVersionV1 is used to construct the path for accessing streams data.
 	StreamsVersionV1 = "v1"
+
+	// IndexFileVersion is used to construct the streams index file.
+	IndexFileVersion = 2
 )
 
 var currentStreamsVersion = StreamsVersionV1
@@ -102,8 +107,20 @@ qsH+JQgcphKkC+JH0Dw7Q/0e16LClkPPa21NseVGUWzS0WmS+0egtDDutg==
 var DefaultBaseURL = "https://streams.canonical.com/juju/tools"
 
 const (
+	// Legacy release directory for Juju < 1.21.
+	LegacyReleaseDirectory = "releases"
+
 	// Used to specify the released tools metadata.
 	ReleasedStream = "released"
+
+	// Used to specify metadata for testing tools.
+	TestingStream = "testing"
+
+	// Used to specify the proposed tools metadata.
+	ProposedStream = "proposed"
+
+	// Used to specify the devel tools metadata.
+	DevelStream = "devel"
 )
 
 // ToolsConstraint defines criteria used to find a tools metadata record.
@@ -112,7 +129,6 @@ type ToolsConstraint struct {
 	Version      version.Number
 	MajorVersion int
 	MinorVersion int
-	Released     bool
 }
 
 // NewVersionedToolsConstraint returns a ToolsConstraint for a tools with a specific version.
@@ -121,16 +137,24 @@ func NewVersionedToolsConstraint(vers version.Number, params simplestreams.Looku
 }
 
 // NewGeneralToolsConstraint returns a ToolsConstraint for tools with matching major/minor version numbers.
-func NewGeneralToolsConstraint(majorVersion, minorVersion int, released bool, params simplestreams.LookupParams) *ToolsConstraint {
+func NewGeneralToolsConstraint(majorVersion, minorVersion int, params simplestreams.LookupParams) *ToolsConstraint {
 	return &ToolsConstraint{LookupParams: params, Version: version.Zero,
-		MajorVersion: majorVersion, MinorVersion: minorVersion, Released: released}
+		MajorVersion: majorVersion, MinorVersion: minorVersion}
 }
 
-// Ids generates a string array representing product ids formed similarly to an ISCSI qualified name (IQN).
-func (tc *ToolsConstraint) Ids() ([]string, error) {
+// IndexIds generates a string array representing product ids formed similarly to an ISCSI qualified name (IQN).
+func (tc *ToolsConstraint) IndexIds() []string {
+	if tc.Stream == "" {
+		return nil
+	}
+	return []string{ToolsContentId(tc.Stream)}
+}
+
+// ProductIds generates a string array representing product ids formed similarly to an ISCSI qualified name (IQN).
+func (tc *ToolsConstraint) ProductIds() ([]string, error) {
 	var allIds []string
 	for _, series := range tc.Series {
-		version, err := ubuntu.SeriesVersion(series)
+		version, err := version.SeriesVersion(series)
 		if err != nil {
 			return nil, err
 		}
@@ -166,11 +190,12 @@ func (t *ToolsMetadata) binary() version.Binary {
 		Number: version.MustParse(t.Version),
 		Series: t.Release,
 		Arch:   t.Arch,
+		OS:     version.MustOSFromSeries(t.Release),
 	}
 }
 
 func (t *ToolsMetadata) productId() (string, error) {
-	seriesVersion, err := ubuntu.SeriesVersion(t.Release)
+	seriesVersion, err := version.SeriesVersion(t.Release)
 	if err != nil {
 		return "", err
 	}
@@ -185,14 +210,19 @@ func Fetch(
 	sources []simplestreams.DataSource, cons *ToolsConstraint,
 	onlySigned bool) ([]*ToolsMetadata, *simplestreams.ResolveInfo, error) {
 
-	params := simplestreams.ValueParams{
-		DataType:        ContentDownload,
-		FilterFunc:      appendMatchingTools,
-		MirrorContentId: ToolsContentId,
-		ValueTemplate:   ToolsMetadata{},
-		PublicKey:       simplestreamsToolsPublicKey,
+	params := simplestreams.GetMetadataParams{
+		StreamsVersion:   currentStreamsVersion,
+		OnlySigned:       onlySigned,
+		LookupConstraint: cons,
+		ValueParams: simplestreams.ValueParams{
+			DataType:        ContentDownload,
+			FilterFunc:      appendMatchingTools,
+			MirrorContentId: ToolsContentId(cons.Stream),
+			ValueTemplate:   ToolsMetadata{},
+			PublicKey:       simplestreamsToolsPublicKey,
+		},
 	}
-	items, resolveInfo, err := simplestreams.GetMetadata(sources, currentStreamsVersion, cons, onlySigned, params)
+	items, resolveInfo, err := simplestreams.GetMetadata(sources, params)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -236,9 +266,6 @@ func appendMatchingTools(source simplestreams.DataSource, matchingTools []interf
 		if toolsConstraint, ok := cons.(*ToolsConstraint); ok {
 			tmNumber := version.MustParse(tm.Version)
 			if toolsConstraint.Version == version.Zero {
-				if toolsConstraint.Released && tmNumber.IsDev() {
-					continue
-				}
 				if toolsConstraint.MajorVersion >= 0 && toolsConstraint.MajorVersion != tmNumber.Major {
 					continue
 				}
@@ -267,10 +294,10 @@ type MetadataFile struct {
 // MetadataFromTools returns a tools metadata list derived from the
 // given tools list. The size and sha256 will not be computed if
 // missing.
-func MetadataFromTools(toolsList coretools.List) []*ToolsMetadata {
+func MetadataFromTools(toolsList coretools.List, toolsDir string) []*ToolsMetadata {
 	metadata := make([]*ToolsMetadata, len(toolsList))
 	for i, t := range toolsList {
-		path := fmt.Sprintf("releases/juju-%s-%s-%s.tgz", t.Version.Number, t.Version.Series, t.Version.Arch)
+		path := fmt.Sprintf("%s/juju-%s-%s-%s.tgz", toolsDir, t.Version.Number, t.Version.Series, t.Version.Arch)
 		metadata[i] = &ToolsMetadata{
 			Release:  t.Version.Series,
 			Version:  t.Version.Number.String(),
@@ -287,14 +314,22 @@ func MetadataFromTools(toolsList coretools.List) []*ToolsMetadata {
 // ResolveMetadata resolves incomplete metadata
 // by fetching the tools from storage and computing
 // the size and hash locally.
-func ResolveMetadata(stor storage.StorageReader, metadata []*ToolsMetadata) error {
+func ResolveMetadata(stor storage.StorageReader, toolsDir string, metadata []*ToolsMetadata) error {
 	for _, md := range metadata {
 		if md.Size != 0 {
 			continue
 		}
 		binary := md.binary()
-		logger.Infof("Fetching tools to generate hash: %v", binary)
-		size, sha256hash, err := fetchToolsHash(stor, binary)
+		logger.Infof("Fetching tools from dir %q to generate hash: %v", toolsDir, binary)
+		size, sha256hash, err := fetchToolsHash(stor, toolsDir, binary)
+		// Older versions of Juju only know about ppc64, not ppc64el,
+		// so if there's no metadata for ppc64, dd metadata for that arch.
+		if errors.IsNotFound(err) && binary.Arch == arch.LEGACY_PPC64 {
+			ppc64elBinary := binary
+			ppc64elBinary.Arch = arch.PPC64EL
+			md.Path = strings.Replace(md.Path, binary.Arch, ppc64elBinary.Arch, -1)
+			size, sha256hash, err = fetchToolsHash(stor, toolsDir, ppc64elBinary)
+		}
 		if err != nil {
 			return err
 		}
@@ -341,10 +376,10 @@ func MergeMetadata(tmlist1, tmlist2 []*ToolsMetadata) ([]*ToolsMetadata, error)
 	return list, nil
 }
 
-// ReadMetadata returns the tools metadata from the given storage.
-func ReadMetadata(store storage.StorageReader) ([]*ToolsMetadata, error) {
+// ReadMetadata returns the tools metadata from the given storage for the specified stream.
+func ReadMetadata(store storage.StorageReader, stream string) ([]*ToolsMetadata, error) {
 	dataSource := storage.NewStorageSimpleStreamsDataSource("existing metadata", store, storage.BaseToolsPath)
-	toolsConstraint, err := makeToolsConstraint(simplestreams.CloudSpec{}, -1, -1, coretools.Filter{})
+	toolsConstraint, err := makeToolsConstraint(simplestreams.CloudSpec{}, stream, -1, -1, coretools.Filter{})
 	if err != nil {
 		return nil, err
 	}
@@ -356,40 +391,132 @@ func ReadMetadata(store storage.StorageReader) ([]*ToolsMetadata, error) {
 	return metadata, nil
 }
 
-var PublicMirrorsInfo = `{
- "mirrors": {
-  "com.ubuntu.juju:released:tools": [
-     {
-      "datatype": "content-download",
-      "path": "streams/v1/cpc-mirrors.json",
-      "updated": "{{updated}}",
-      "format": "mirrors:1.0"
-     }
-  ]
- }
-}
-`
+// AllMetadataStreams is the set of streams for which there will be simplestreams tools metadata.
+var AllMetadataStreams = []string{ReleasedStream, ProposedStream, TestingStream, DevelStream}
 
-// WriteMetadata writes the given tools metadata to the given storage.
-func WriteMetadata(stor storage.Storage, metadata []*ToolsMetadata, writeMirrors ShouldWriteMirrors) error {
+// ReadAllMetadata returns the tools metadata from the given storage for all streams.
+// The result is a map of metadata slices, keyed on stream.
+func ReadAllMetadata(store storage.StorageReader) (map[string][]*ToolsMetadata, error) {
+	streamMetadata := make(map[string][]*ToolsMetadata)
+	for _, stream := range AllMetadataStreams {
+		metadata, err := ReadMetadata(store, stream)
+		if err != nil {
+			return nil, err
+		}
+		if len(metadata) == 0 {
+			continue
+		}
+		streamMetadata[stream] = metadata
+	}
+	return streamMetadata, nil
+}
+
+// removeMetadataUpdated unmarshalls simplestreams metadata, clears the
+// updated attribute, and then marshalls back to a string.
+func removeMetadataUpdated(metadataBytes []byte) (string, error) {
+	var metadata map[string]interface{}
+	err := json.Unmarshal(metadataBytes, &metadata)
+	if err != nil {
+		return "", err
+	}
+	delete(metadata, "updated")
+
+	metadataJson, err := json.Marshal(metadata)
+	if err != nil {
+		return "", err
+	}
+	return string(metadataJson), nil
+}
+
+// metadataUnchanged returns true if the content of metadata for stream in stor is the same
+// as generatedMetadata, ignoring the "updated" attribute.
+func metadataUnchanged(stor storage.Storage, stream string, generatedMetadata []byte) (bool, error) {
+	mdPath := ProductMetadataPath(stream)
+	filePath := path.Join(storage.BaseToolsPath, mdPath)
+	existingDataReader, err := stor.Get(filePath)
+	// If the file can't be retrieved, consider it has changed.
+	if err != nil {
+		return false, nil
+	}
+	defer existingDataReader.Close()
+	existingData, err := ioutil.ReadAll(existingDataReader)
+	if err != nil {
+		return false, err
+	}
+
+	// To do the comparison, we unmarshall the metadata, clear the
+	// updated value, and marshall back to a string.
+	existingMetadata, err := removeMetadataUpdated(existingData)
+	if err != nil {
+		return false, err
+	}
+	newMetadata, err := removeMetadataUpdated(generatedMetadata)
+	if err != nil {
+		return false, err
+	}
+	return existingMetadata == newMetadata, nil
+}
+
+// WriteMetadata writes the given tools metadata for the specified streams to the given storage.
+// streamMetadata contains all known metadata so that the correct index files can be written.
+// Only product files for the specified streams are written.
+func WriteMetadata(stor storage.Storage, streamMetadata map[string][]*ToolsMetadata, streams []string, writeMirrors ShouldWriteMirrors) error {
 	updated := time.Now()
-	index, products, err := MarshalToolsMetadataJSON(metadata, updated)
+	index, legacyIndex, products, err := MarshalToolsMetadataJSON(streamMetadata, updated)
 	if err != nil {
 		return err
 	}
 	metadataInfo := []MetadataFile{
-		{simplestreams.UnsignedIndex(currentStreamsVersion), index},
-		{ProductMetadataPath, products},
+		{simplestreams.UnsignedIndex(currentStreamsVersion, IndexFileVersion), index},
+	}
+	if legacyIndex != nil {
+		metadataInfo = append(metadataInfo, MetadataFile{
+			simplestreams.UnsignedIndex(currentStreamsVersion, 1), legacyIndex,
+		})
+	}
+	for _, stream := range streams {
+		if metadata, ok := products[stream]; ok {
+			// If metadata hasn't changed, do not overwrite.
+			unchanged, err := metadataUnchanged(stor, stream, metadata)
+			if err != nil {
+				return err
+			}
+			if unchanged {
+				logger.Infof("Metadata for stream %q unchanged", stream)
+				continue
+			}
+			// Metadata is different, so include it.
+			metadataInfo = append(metadataInfo, MetadataFile{ProductMetadataPath(stream), metadata})
+		}
 	}
 	if writeMirrors {
-		mirrorsUpdated := updated.Format("20060102") // YYYYMMDD
-		mirrorsInfo := strings.Replace(PublicMirrorsInfo, "{{updated}}", mirrorsUpdated, -1)
+		streamsMirrorsMetadata := make(map[string][]simplestreams.MirrorReference)
+		for stream, _ := range streamMetadata {
+			streamsMirrorsMetadata[ToolsContentId(stream)] = []simplestreams.MirrorReference{{
+				Updated:  updated.Format("20060102"), // YYYYMMDD
+				DataType: ContentDownload,
+				Format:   simplestreams.MirrorFormat,
+				Path:     simplestreams.MirrorFile,
+			}}
+		}
+		mirrorsMetadata := map[string]map[string][]simplestreams.MirrorReference{
+			"mirrors": streamsMirrorsMetadata,
+		}
+		mirrorsInfo, err := json.MarshalIndent(&mirrorsMetadata, "", "    ")
+		if err != nil {
+			return err
+		}
 		metadataInfo = append(
-			metadataInfo, MetadataFile{simplestreams.UnsignedMirror(currentStreamsVersion), []byte(mirrorsInfo)})
+			metadataInfo, MetadataFile{simplestreams.UnsignedMirror(currentStreamsVersion), mirrorsInfo})
 	}
+	return writeMetadataFiles(stor, metadataInfo)
+}
+
+var writeMetadataFiles = func(stor storage.Storage, metadataInfo []MetadataFile) error {
 	for _, md := range metadataInfo {
-		logger.Infof("Writing %s", "tools/"+md.Path)
-		err = stor.Put(path.Join(storage.BaseToolsPath, md.Path), bytes.NewReader(md.Data), int64(len(md.Data)))
+		filePath := path.Join(storage.BaseToolsPath, md.Path)
+		logger.Infof("Writing %s", filePath)
+		err := stor.Put(filePath, bytes.NewReader(md.Data), int64(len(md.Data)))
 		if err != nil {
 			return err
 		}
@@ -407,22 +534,23 @@ const (
 // MergeAndWriteMetadata reads the existing metadata from storage (if any),
 // and merges it with metadata generated from the given tools list. The
 // resulting metadata is written to storage.
-func MergeAndWriteMetadata(stor storage.Storage, tools coretools.List, writeMirrors ShouldWriteMirrors) error {
-	existing, err := ReadMetadata(stor)
+func MergeAndWriteMetadata(stor storage.Storage, toolsDir, stream string, tools coretools.List, writeMirrors ShouldWriteMirrors) error {
+	existing, err := ReadAllMetadata(stor)
 	if err != nil {
 		return err
 	}
-	metadata := MetadataFromTools(tools)
-	if metadata, err = MergeMetadata(metadata, existing); err != nil {
+	metadata := MetadataFromTools(tools, toolsDir)
+	if metadata, err = MergeMetadata(metadata, existing[stream]); err != nil {
 		return err
 	}
-	return WriteMetadata(stor, metadata, writeMirrors)
+	existing[stream] = metadata
+	return WriteMetadata(stor, existing, []string{stream}, writeMirrors)
 }
 
 // fetchToolsHash fetches the tools from storage and calculates
 // its size in bytes and computes a SHA256 hash of its contents.
-func fetchToolsHash(stor storage.StorageReader, ver version.Binary) (size int64, sha256hash hash.Hash, err error) {
-	r, err := storage.Get(stor, StorageName(ver))
+func fetchToolsHash(stor storage.StorageReader, stream string, ver version.Binary) (size int64, sha256hash hash.Hash, err error) {
+	r, err := storage.Get(stor, StorageName(ver, stream))
 	if err != nil {
 		return 0, nil, err
 	}
