@@ -5,17 +5,22 @@ package tools_test
 
 import (
 	"bytes"
+	"encoding/json"
 	"flag"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"net/http"
+	"os"
+	"path/filepath"
 	"reflect"
 	"strings"
 	"testing"
 
+	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
+	gc "gopkg.in/check.v1"
 	"launchpad.net/goamz/aws"
-	gc "launchpad.net/gocheck"
 
 	"github.com/juju/juju/environs/filestorage"
 	"github.com/juju/juju/environs/jujutest"
@@ -67,6 +72,7 @@ func setupSimpleStreamsTests(t *testing.T) {
 				CloudSpec: testData.validCloudSpec,
 				Series:    []string{version.Current.Series},
 				Arches:    []string{"amd64"},
+				Stream:    "released",
 			}), testData.requireSigned)
 	}
 	registerSimpleStreamsTests()
@@ -86,6 +92,7 @@ func registerSimpleStreamsTests() {
 				},
 				Series: []string{"precise"},
 				Arches: []string{"amd64", "arm"},
+				Stream: "released",
 			}),
 		},
 	})
@@ -118,14 +125,14 @@ func (s *simplestreamsSuite) TearDownSuite(c *gc.C) {
 }
 
 var fetchTests = []struct {
-	region   string
-	series   string
-	version  string
-	major    int
-	minor    int
-	released bool
-	arches   []string
-	tools    []*tools.ToolsMetadata
+	region  string
+	series  string
+	version string
+	stream  string
+	major   int
+	minor   int
+	arches  []string
+	tools   []*tools.ToolsMetadata
 }{{
 	series:  "precise",
 	arches:  []string{"amd64", "arm"},
@@ -136,7 +143,7 @@ var fetchTests = []struct {
 			Version:  "1.13.0",
 			Arch:     "amd64",
 			Size:     2973595,
-			Path:     "tools/releases/20130806/juju-1.13.0-precise-amd64.tgz",
+			Path:     "tools/released/20130806/juju-1.13.0-precise-amd64.tgz",
 			FileType: "tar.gz",
 			SHA256:   "447aeb6a934a5eaec4f703eda4ef2dde",
 		},
@@ -151,7 +158,7 @@ var fetchTests = []struct {
 			Version:  "1.13.0",
 			Arch:     "amd64",
 			Size:     2973173,
-			Path:     "tools/releases/20130806/juju-1.13.0-raring-amd64.tgz",
+			Path:     "tools/released/20130806/juju-1.13.0-raring-amd64.tgz",
 			FileType: "tar.gz",
 			SHA256:   "df07ac5e1fb4232d4e9aa2effa57918a",
 		},
@@ -166,7 +173,7 @@ var fetchTests = []struct {
 			Version:  "1.11.4",
 			Arch:     "arm",
 			Size:     1950327,
-			Path:     "tools/releases/20130806/juju-1.11.4-raring-arm.tgz",
+			Path:     "tools/released/20130806/juju-1.11.4-raring-arm.tgz",
 			FileType: "tar.gz",
 			SHA256:   "6472014e3255e3fe7fbd3550ef3f0a11",
 		},
@@ -181,7 +188,7 @@ var fetchTests = []struct {
 			Version:  "2.0.1",
 			Arch:     "arm",
 			Size:     1951096,
-			Path:     "tools/releases/20130806/juju-2.0.1-precise-arm.tgz",
+			Path:     "tools/released/20130806/juju-2.0.1-precise-arm.tgz",
 			FileType: "tar.gz",
 			SHA256:   "f65a92b3b41311bdf398663ee1c5cd0c",
 		},
@@ -197,7 +204,7 @@ var fetchTests = []struct {
 			Version:  "1.11.4",
 			Arch:     "arm",
 			Size:     1951096,
-			Path:     "tools/releases/20130806/juju-1.11.4-precise-arm.tgz",
+			Path:     "tools/released/20130806/juju-1.11.4-precise-arm.tgz",
 			FileType: "tar.gz",
 			SHA256:   "f65a92b3b41311bdf398663ee1c5cd0c",
 		},
@@ -206,26 +213,25 @@ var fetchTests = []struct {
 			Version:  "1.11.5",
 			Arch:     "arm",
 			Size:     2031281,
-			Path:     "tools/releases/20130803/juju-1.11.5-precise-arm.tgz",
+			Path:     "tools/released/20130803/juju-1.11.5-precise-arm.tgz",
 			FileType: "tar.gz",
 			SHA256:   "df07ac5e1fb4232d4e9aa2effa57918a",
 		},
 	},
 }, {
-	series:   "raring",
-	arches:   []string{"amd64", "arm"},
-	major:    1,
-	minor:    -1,
-	released: true,
+	series:  "trusty",
+	arches:  []string{"amd64"},
+	version: "1.16.0",
+	stream:  "testing",
 	tools: []*tools.ToolsMetadata{
 		{
-			Release:  "raring",
-			Version:  "1.14.0",
+			Release:  "trusty",
+			Version:  "1.16.0",
 			Arch:     "amd64",
-			Size:     2973173,
-			Path:     "tools/releases/20130806/juju-1.14.0-raring-amd64.tgz",
+			Size:     2973512,
+			Path:     "tools/testing/20130806/juju-1.16.0-trusty-amd64.tgz",
 			FileType: "tar.gz",
-			SHA256:   "df07ac5e1fb4232d4e9aa2effa57918a",
+			SHA256:   "447aeb6a934a5eaec4f703eda4ef2dac",
 		},
 	},
 }}
@@ -233,12 +239,16 @@ var fetchTests = []struct {
 func (s *simplestreamsSuite) TestFetch(c *gc.C) {
 	for i, t := range fetchTests {
 		c.Logf("test %d", i)
+		if t.stream == "" {
+			t.stream = "released"
+		}
 		var toolsConstraint *tools.ToolsConstraint
 		if t.version == "" {
-			toolsConstraint = tools.NewGeneralToolsConstraint(t.major, t.minor, t.released, simplestreams.LookupParams{
+			toolsConstraint = tools.NewGeneralToolsConstraint(t.major, t.minor, simplestreams.LookupParams{
 				CloudSpec: simplestreams.CloudSpec{"us-east-1", "https://ec2.us-east-1.amazonaws.com"},
 				Series:    []string{t.series},
 				Arches:    t.arches,
+				Stream:    t.stream,
 			})
 		} else {
 			toolsConstraint = tools.NewVersionedToolsConstraint(version.MustParse(t.version),
@@ -246,18 +256,19 @@ func (s *simplestreamsSuite) TestFetch(c *gc.C) {
 					CloudSpec: simplestreams.CloudSpec{"us-east-1", "https://ec2.us-east-1.amazonaws.com"},
 					Series:    []string{t.series},
 					Arches:    t.arches,
+					Stream:    t.stream,
 				})
 		}
 		// Add invalid datasource and check later that resolveInfo is correct.
 		invalidSource := simplestreams.NewURLDataSource("invalid", "file://invalid", utils.VerifySSLHostnames)
 		tools, resolveInfo, err := tools.Fetch(
 			[]simplestreams.DataSource{invalidSource, s.Source}, toolsConstraint, s.RequireSigned)
-		if !c.Check(err, gc.IsNil) {
+		if !c.Check(err, jc.ErrorIsNil) {
 			continue
 		}
 		for _, tm := range t.tools {
 			tm.FullPath, err = s.Source.URL(tm.Path)
-			c.Assert(err, gc.IsNil)
+			c.Assert(err, jc.ErrorIsNil)
 		}
 		c.Check(tools, gc.DeepEquals, t.tools)
 		c.Check(resolveInfo, gc.DeepEquals, &simplestreams.ResolveInfo{
@@ -269,15 +280,28 @@ func (s *simplestreamsSuite) TestFetch(c *gc.C) {
 	}
 }
 
+func (s *simplestreamsSuite) TestFetchNoMatchingStream(c *gc.C) {
+	toolsConstraint := tools.NewGeneralToolsConstraint(2, -1, simplestreams.LookupParams{
+		CloudSpec: simplestreams.CloudSpec{"us-east-1", "https://ec2.us-east-1.amazonaws.com"},
+		Series:    []string{"precise"},
+		Arches:    []string{},
+		Stream:    "proposed",
+	})
+	_, _, err := tools.Fetch(
+		[]simplestreams.DataSource{s.Source}, toolsConstraint, s.RequireSigned)
+	c.Assert(err, gc.ErrorMatches, `index file missing "content-download" data not found`)
+}
+
 func (s *simplestreamsSuite) TestFetchWithMirror(c *gc.C) {
-	toolsConstraint := tools.NewGeneralToolsConstraint(1, 13, false, simplestreams.LookupParams{
+	toolsConstraint := tools.NewGeneralToolsConstraint(1, 13, simplestreams.LookupParams{
 		CloudSpec: simplestreams.CloudSpec{"us-west-2", "https://ec2.us-west-2.amazonaws.com"},
 		Series:    []string{"precise"},
 		Arches:    []string{"amd64"},
+		Stream:    "released",
 	})
 	toolsMetadata, resolveInfo, err := tools.Fetch(
 		[]simplestreams.DataSource{s.Source}, toolsConstraint, s.RequireSigned)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(len(toolsMetadata), gc.Equals, 1)
 
 	expectedMetadata := &tools.ToolsMetadata{
@@ -290,7 +314,7 @@ func (s *simplestreamsSuite) TestFetchWithMirror(c *gc.C) {
 		FileType: "tar.gz",
 		SHA256:   "447aeb6a934a5eaec4f703eda4ef2dde",
 	}
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(toolsMetadata[0], gc.DeepEquals, expectedMetadata)
 	c.Assert(resolveInfo, gc.DeepEquals, &simplestreams.ResolveInfo{
 		Source:    "test",
@@ -300,7 +324,7 @@ func (s *simplestreamsSuite) TestFetchWithMirror(c *gc.C) {
 	})
 }
 
-func assertMetadataMatches(c *gc.C, storageDir string, toolList coretools.List, metadata []*tools.ToolsMetadata) {
+func assertMetadataMatches(c *gc.C, storageDir string, stream string, toolList coretools.List, metadata []*tools.ToolsMetadata) {
 	var expectedMetadata []*tools.ToolsMetadata = make([]*tools.ToolsMetadata, len(toolList))
 	for i, tool := range toolList {
 		expectedMetadata[i] = &tools.ToolsMetadata{
@@ -308,7 +332,7 @@ func assertMetadataMatches(c *gc.C, storageDir string, toolList coretools.List,
 			Version:  tool.Version.Number.String(),
 			Arch:     tool.Version.Arch,
 			Size:     tool.Size,
-			Path:     fmt.Sprintf("releases/juju-%s.tgz", tool.Version.String()),
+			Path:     fmt.Sprintf("%s/juju-%s.tgz", stream, tool.Version.String()),
 			FileType: "tar.gz",
 			SHA256:   tool.SHA256,
 		}
@@ -330,11 +354,11 @@ func (s *simplestreamsSuite) TestWriteMetadataNoFetch(c *gc.C) {
 	}
 	dir := c.MkDir()
 	writer, err := filestorage.NewFileStorageWriter(dir)
-	c.Assert(err, gc.IsNil)
-	err = tools.MergeAndWriteMetadata(writer, toolsList, tools.DoNotWriteMirrors)
-	c.Assert(err, gc.IsNil)
-	metadata := toolstesting.ParseMetadataFromDir(c, dir, false)
-	assertMetadataMatches(c, dir, toolsList, metadata)
+	c.Assert(err, jc.ErrorIsNil)
+	err = tools.MergeAndWriteMetadata(writer, "proposed", "proposed", toolsList, tools.DoNotWriteMirrors)
+	c.Assert(err, jc.ErrorIsNil)
+	metadata := toolstesting.ParseMetadataFromDir(c, dir, "proposed", false)
+	assertMetadataMatches(c, dir, "proposed", toolsList, metadata)
 }
 
 func (s *simplestreamsSuite) assertWriteMetadata(c *gc.C, withMirrors bool) {
@@ -343,7 +367,7 @@ func (s *simplestreamsSuite) assertWriteMetadata(c *gc.C, withMirrors bool) {
 		"2.0.1-raring-amd64",
 	}
 	dir := c.MkDir()
-	toolstesting.MakeTools(c, dir, "releases", versionStrings)
+	toolstesting.MakeTools(c, dir, "proposed", versionStrings)
 
 	toolsList := coretools.List{
 		{
@@ -358,15 +382,19 @@ func (s *simplestreamsSuite) assertWriteMetadata(c *gc.C, withMirrors bool) {
 		},
 	}
 	writer, err := filestorage.NewFileStorageWriter(dir)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	writeMirrors := tools.DoNotWriteMirrors
 	if withMirrors {
 		writeMirrors = tools.WriteMirrors
 	}
-	err = tools.MergeAndWriteMetadata(writer, toolsList, writeMirrors)
-	c.Assert(err, gc.IsNil)
-	metadata := toolstesting.ParseMetadataFromDir(c, dir, withMirrors)
-	assertMetadataMatches(c, dir, toolsList, metadata)
+	err = tools.MergeAndWriteMetadata(writer, "proposed", "proposed", toolsList, writeMirrors)
+	c.Assert(err, jc.ErrorIsNil)
+	metadata := toolstesting.ParseMetadataFromDir(c, dir, "proposed", withMirrors)
+	assertMetadataMatches(c, dir, "proposed", toolsList, metadata)
+
+	// No release stream generated so there will not be a legacy index file created.
+	_, err = writer.Get("tools/streams/v1/index.json")
+	c.Assert(err, gc.NotNil)
 }
 
 func (s *simplestreamsSuite) TestWriteMetadata(c *gc.C) {
@@ -391,9 +419,9 @@ func (s *simplestreamsSuite) TestWriteMetadataMergeWithExisting(c *gc.C) {
 		},
 	}
 	writer, err := filestorage.NewFileStorageWriter(dir)
-	c.Assert(err, gc.IsNil)
-	err = tools.MergeAndWriteMetadata(writer, existingToolsList, tools.DoNotWriteMirrors)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
+	err = tools.MergeAndWriteMetadata(writer, "testing", "testing", existingToolsList, tools.WriteMirrors)
+	c.Assert(err, jc.ErrorIsNil)
 	newToolsList := coretools.List{
 		existingToolsList[0],
 		{
@@ -402,24 +430,50 @@ func (s *simplestreamsSuite) TestWriteMetadataMergeWithExisting(c *gc.C) {
 			SHA256:  "def",
 		},
 	}
-	err = tools.MergeAndWriteMetadata(writer, newToolsList, tools.DoNotWriteMirrors)
-	c.Assert(err, gc.IsNil)
+	err = tools.MergeAndWriteMetadata(writer, "testing", "testing", newToolsList, tools.WriteMirrors)
+	c.Assert(err, jc.ErrorIsNil)
 	requiredToolsList := append(existingToolsList, newToolsList[1])
-	metadata := toolstesting.ParseMetadataFromDir(c, dir, false)
-	assertMetadataMatches(c, dir, requiredToolsList, metadata)
+	metadata := toolstesting.ParseMetadataFromDir(c, dir, "testing", true)
+	assertMetadataMatches(c, dir, "testing", requiredToolsList, metadata)
+
+	err = tools.MergeAndWriteMetadata(writer, "devel", "devel", newToolsList, tools.WriteMirrors)
+	c.Assert(err, jc.ErrorIsNil)
+	metadata = toolstesting.ParseMetadataFromDir(c, dir, "testing", true)
+	assertMetadataMatches(c, dir, "testing", requiredToolsList, metadata)
+	metadata = toolstesting.ParseMetadataFromDir(c, dir, "devel", true)
+	assertMetadataMatches(c, dir, "devel", newToolsList, metadata)
 }
 
 type productSpecSuite struct{}
 
 var _ = gc.Suite(&productSpecSuite{})
 
-func (s *productSpecSuite) TestId(c *gc.C) {
+func (s *productSpecSuite) TestIndexIdNoStream(c *gc.C) {
 	toolsConstraint := tools.NewVersionedToolsConstraint(version.MustParse("1.13.0"), simplestreams.LookupParams{
 		Series: []string{"precise"},
 		Arches: []string{"amd64"},
 	})
-	ids, err := toolsConstraint.Ids()
-	c.Assert(err, gc.IsNil)
+	ids := toolsConstraint.IndexIds()
+	c.Assert(ids, gc.HasLen, 0)
+}
+
+func (s *productSpecSuite) TestIndexId(c *gc.C) {
+	toolsConstraint := tools.NewVersionedToolsConstraint(version.MustParse("1.13.0"), simplestreams.LookupParams{
+		Series: []string{"precise"},
+		Arches: []string{"amd64"},
+		Stream: "proposed",
+	})
+	ids := toolsConstraint.IndexIds()
+	c.Assert(ids, gc.DeepEquals, []string{"com.ubuntu.juju:proposed:tools"})
+}
+
+func (s *productSpecSuite) TestProductId(c *gc.C) {
+	toolsConstraint := tools.NewVersionedToolsConstraint(version.MustParse("1.13.0"), simplestreams.LookupParams{
+		Series: []string{"precise"},
+		Arches: []string{"amd64"},
+	})
+	ids, err := toolsConstraint.ProductIds()
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ids, gc.DeepEquals, []string{"com.ubuntu.juju:12.04:amd64"})
 }
 
@@ -428,8 +482,8 @@ func (s *productSpecSuite) TestIdMultiArch(c *gc.C) {
 		Series: []string{"precise"},
 		Arches: []string{"amd64", "arm"},
 	})
-	ids, err := toolsConstraint.Ids()
-	c.Assert(err, gc.IsNil)
+	ids, err := toolsConstraint.ProductIds()
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ids, gc.DeepEquals, []string{
 		"com.ubuntu.juju:12.04:amd64",
 		"com.ubuntu.juju:12.04:arm"})
@@ -439,31 +493,34 @@ func (s *productSpecSuite) TestIdMultiSeries(c *gc.C) {
 	toolsConstraint := tools.NewVersionedToolsConstraint(version.MustParse("1.11.3"), simplestreams.LookupParams{
 		Series: []string{"precise", "raring"},
 		Arches: []string{"amd64"},
+		Stream: "released",
 	})
-	ids, err := toolsConstraint.Ids()
-	c.Assert(err, gc.IsNil)
+	ids, err := toolsConstraint.ProductIds()
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ids, gc.DeepEquals, []string{
 		"com.ubuntu.juju:12.04:amd64",
 		"com.ubuntu.juju:13.04:amd64"})
 }
 
 func (s *productSpecSuite) TestIdWithMajorVersionOnly(c *gc.C) {
-	toolsConstraint := tools.NewGeneralToolsConstraint(1, -1, false, simplestreams.LookupParams{
+	toolsConstraint := tools.NewGeneralToolsConstraint(1, -1, simplestreams.LookupParams{
 		Series: []string{"precise"},
 		Arches: []string{"amd64"},
+		Stream: "released",
 	})
-	ids, err := toolsConstraint.Ids()
-	c.Assert(err, gc.IsNil)
+	ids, err := toolsConstraint.ProductIds()
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ids, gc.DeepEquals, []string{`com.ubuntu.juju:12.04:amd64`})
 }
 
 func (s *productSpecSuite) TestIdWithMajorMinorVersion(c *gc.C) {
-	toolsConstraint := tools.NewGeneralToolsConstraint(1, 2, false, simplestreams.LookupParams{
+	toolsConstraint := tools.NewGeneralToolsConstraint(1, 2, simplestreams.LookupParams{
 		Series: []string{"precise"},
 		Arches: []string{"amd64"},
+		Stream: "released",
 	})
-	ids, err := toolsConstraint.Ids()
-	c.Assert(err, gc.IsNil)
+	ids, err := toolsConstraint.ProductIds()
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(ids, gc.DeepEquals, []string{`com.ubuntu.juju:12.04:amd64`})
 }
 
@@ -494,7 +551,7 @@ func (s *productSpecSuite) TestLargeNumber(c *gc.C) {
         }
     }`
 	cloudMetadata, err := simplestreams.ParseCloudMetadata([]byte(json), "products:1.0", "", tools.ToolsMetadata{})
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cloudMetadata.Products, gc.HasLen, 1)
 	product := cloudMetadata.Products["com.ubuntu.juju:1.10.0:amd64"]
 	c.Assert(product, gc.NotNil)
@@ -515,7 +572,7 @@ type metadataHelperSuite struct {
 var _ = gc.Suite(&metadataHelperSuite{})
 
 func (*metadataHelperSuite) TestMetadataFromTools(c *gc.C) {
-	metadata := tools.MetadataFromTools(nil)
+	metadata := tools.MetadataFromTools(nil, "proposed")
 	c.Assert(metadata, gc.HasLen, 0)
 
 	toolsList := coretools.List{{
@@ -524,11 +581,11 @@ func (*metadataHelperSuite) TestMetadataFromTools(c *gc.C) {
 		SHA256:  "abc",
 	}, {
 		Version: version.MustParseBinary("2.0.1-raring-amd64"),
-		URL:     "file:///tmp/releases/juju-2.0.1-raring-amd64.tgz",
+		URL:     "file:///tmp/proposed/juju-2.0.1-raring-amd64.tgz",
 		Size:    456,
 		SHA256:  "xyz",
 	}}
-	metadata = tools.MetadataFromTools(toolsList)
+	metadata = tools.MetadataFromTools(toolsList, "proposed")
 	c.Assert(metadata, gc.HasLen, len(toolsList))
 	for i, t := range toolsList {
 		md := metadata[i]
@@ -538,7 +595,7 @@ func (*metadataHelperSuite) TestMetadataFromTools(c *gc.C) {
 		// FullPath is only filled out when reading tools using simplestreams.
 		// It's not needed elsewhere and requires a URL() call.
 		c.Assert(md.FullPath, gc.Equals, "")
-		c.Assert(md.Path, gc.Equals, tools.StorageName(t.Version)[len("tools/"):])
+		c.Assert(md.Path, gc.Equals, tools.StorageName(t.Version, "proposed")[len("tools/"):])
 		c.Assert(md.FileType, gc.Equals, "tar.gz")
 		c.Assert(md.Size, gc.Equals, t.Size)
 		c.Assert(md.SHA256, gc.Equals, t.SHA256)
@@ -558,7 +615,7 @@ func (c *countingStorage) Get(name string) (io.ReadCloser, error) {
 func (*metadataHelperSuite) TestResolveMetadata(c *gc.C) {
 	var versionStrings = []string{"1.2.3-precise-amd64"}
 	dir := c.MkDir()
-	toolstesting.MakeTools(c, dir, "releases", versionStrings)
+	toolstesting.MakeTools(c, dir, "released", versionStrings)
 	toolsList := coretools.List{{
 		Version: version.MustParseBinary(versionStrings[0]),
 		Size:    123,
@@ -566,28 +623,78 @@ func (*metadataHelperSuite) TestResolveMetadata(c *gc.C) {
 	}}
 
 	stor, err := filestorage.NewFileStorageReader(dir)
-	c.Assert(err, gc.IsNil)
-	err = tools.ResolveMetadata(stor, nil)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
+	err = tools.ResolveMetadata(stor, "released", nil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	// We already have size/sha256, so ensure that storage isn't consulted.
 	countingStorage := &countingStorage{StorageReader: stor}
-	metadata := tools.MetadataFromTools(toolsList)
-	err = tools.ResolveMetadata(countingStorage, metadata)
-	c.Assert(err, gc.IsNil)
+	metadata := tools.MetadataFromTools(toolsList, "released")
+	err = tools.ResolveMetadata(countingStorage, "released", metadata)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(countingStorage.counter, gc.Equals, 0)
 
 	// Now clear size/sha256, and check that it is called, and
 	// the size/sha256 sum are updated.
 	metadata[0].Size = 0
 	metadata[0].SHA256 = ""
-	err = tools.ResolveMetadata(countingStorage, metadata)
-	c.Assert(err, gc.IsNil)
+	err = tools.ResolveMetadata(countingStorage, "released", metadata)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(countingStorage.counter, gc.Equals, 1)
 	c.Assert(metadata[0].Size, gc.Not(gc.Equals), 0)
 	c.Assert(metadata[0].SHA256, gc.Not(gc.Equals), "")
 }
 
+func (*metadataHelperSuite) TestResolveMetadataLegacyPPC64(c *gc.C) {
+	var versionStrings = []string{"1.2.3-precise-amd64", "1.2.3-precise-ppc64el"}
+	dir := c.MkDir()
+	toolstesting.MakeTools(c, dir, "released", versionStrings)
+
+	toolsList := coretools.List{
+		{
+			Version: version.MustParseBinary(versionStrings[0]),
+		}, {
+			Version: version.MustParseBinary(versionStrings[1]),
+		}, {
+			Version: version.MustParseBinary("1.2.3-precise-ppc64"),
+		},
+	}
+	toolsMetadata := tools.MetadataFromTools(toolsList, dir)
+	stor, err := filestorage.NewFileStorageReader(dir)
+	c.Assert(err, jc.ErrorIsNil)
+	err = tools.ResolveMetadata(stor, "released", toolsMetadata)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(toolsMetadata, gc.DeepEquals, []*tools.ToolsMetadata{
+		{
+			Release:  "precise",
+			Version:  "1.2.3",
+			Arch:     "amd64",
+			Size:     19,
+			FileType: "tar.gz",
+			SHA256:   "dcdd65b962b804a3d63b108d670290ee95a867a97fe9b9f99b2b77b5c7173e59",
+			Path:     fmt.Sprintf("%s/juju-1.2.3-precise-amd64.tgz", dir),
+		},
+		{
+			Release:  "precise",
+			Version:  "1.2.3",
+			Arch:     "ppc64el",
+			Size:     21,
+			FileType: "tar.gz",
+			SHA256:   "a3460ed45eb07a69adfcd541413a495f988c5842d715c6a40353075c3ad47af2",
+			Path:     fmt.Sprintf("%s/juju-1.2.3-precise-ppc64el.tgz", dir),
+		},
+		{
+			Release:  "precise",
+			Version:  "1.2.3",
+			Arch:     "ppc64",
+			Size:     21,
+			FileType: "tar.gz",
+			SHA256:   "a3460ed45eb07a69adfcd541413a495f988c5842d715c6a40353075c3ad47af2",
+			Path:     fmt.Sprintf("%s/juju-1.2.3-precise-ppc64el.tgz", dir),
+		},
+	})
+}
+
 func (*metadataHelperSuite) TestMergeMetadata(c *gc.C) {
 	md1 := &tools.ToolsMetadata{
 		Release: "precise",
@@ -685,7 +792,7 @@ func (*metadataHelperSuite) TestMergeMetadata(c *gc.C) {
 		c.Logf("test %d: %s", i, test.name)
 		merged, err := tools.MergeMetadata(test.lhs, test.rhs)
 		if test.err == "" {
-			c.Assert(err, gc.IsNil)
+			c.Assert(err, jc.ErrorIsNil)
 			c.Assert(merged, gc.DeepEquals, test.merged)
 		} else {
 			c.Assert(err, gc.ErrorMatches, test.err)
@@ -694,33 +801,191 @@ func (*metadataHelperSuite) TestMergeMetadata(c *gc.C) {
 	}
 }
 
-func (*metadataHelperSuite) TestReadWriteMetadata(c *gc.C) {
-	metadata := []*tools.ToolsMetadata{{
-		Release: "precise",
-		Version: "1.2.3",
-		Arch:    "amd64",
-		Path:    "path1",
-	}, {
-		Release: "raring",
-		Version: "1.2.3",
-		Arch:    "amd64",
-		Path:    "path2",
-	}}
+func (*metadataHelperSuite) TestReadWriteMetadataSingleStream(c *gc.C) {
+	metadata := map[string][]*tools.ToolsMetadata{
+		"released": []*tools.ToolsMetadata{{
+			Release: "precise",
+			Version: "1.2.3",
+			Arch:    "amd64",
+			Path:    "path1",
+		}, {
+			Release: "raring",
+			Version: "1.2.3",
+			Arch:    "amd64",
+			Path:    "path2",
+		}},
+	}
 
 	stor, err := filestorage.NewFileStorageWriter(c.MkDir())
-	c.Assert(err, gc.IsNil)
-	out, err := tools.ReadMetadata(stor)
+	c.Assert(err, jc.ErrorIsNil)
+	out, err := tools.ReadAllMetadata(stor)
+	c.Assert(err, jc.ErrorIsNil) // non-existence is not an error
 	c.Assert(out, gc.HasLen, 0)
-	c.Assert(err, gc.IsNil) // non-existence is not an error
-	err = tools.WriteMetadata(stor, metadata, tools.DoNotWriteMirrors)
-	c.Assert(err, gc.IsNil)
-	out, err = tools.ReadMetadata(stor)
-	for _, md := range out {
-		// FullPath is set by ReadMetadata.
-		c.Assert(md.FullPath, gc.Not(gc.Equals), "")
-		md.FullPath = ""
+	err = tools.WriteMetadata(stor, metadata, []string{"released"}, tools.DoNotWriteMirrors)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Read back what was just written.
+	out, err = tools.ReadAllMetadata(stor)
+	for _, outMetadata := range out {
+		for _, md := range outMetadata {
+			// FullPath is set by ReadAllMetadata.
+			c.Assert(md.FullPath, gc.Not(gc.Equals), "")
+			md.FullPath = ""
+		}
 	}
-	c.Assert(out, gc.DeepEquals, metadata)
+	c.Assert(out, jc.DeepEquals, metadata)
+}
+
+func (*metadataHelperSuite) writeMetadataMultipleStream(c *gc.C) (storage.StorageReader, map[string][]*tools.ToolsMetadata) {
+	metadata := map[string][]*tools.ToolsMetadata{
+		"released": []*tools.ToolsMetadata{{
+			Release: "precise",
+			Version: "1.2.3",
+			Arch:    "amd64",
+			Path:    "path1",
+		}},
+		"proposed": []*tools.ToolsMetadata{{
+			Release: "raring",
+			Version: "1.2.3",
+			Arch:    "amd64",
+			Path:    "path2",
+		}},
+	}
+
+	stor, err := filestorage.NewFileStorageWriter(c.MkDir())
+	c.Assert(err, jc.ErrorIsNil)
+	out, err := tools.ReadAllMetadata(stor)
+	c.Assert(out, gc.HasLen, 0)
+	c.Assert(err, jc.ErrorIsNil) // non-existence is not an error
+	err = tools.WriteMetadata(stor, metadata, []string{"released", "proposed"}, tools.DoNotWriteMirrors)
+	c.Assert(err, jc.ErrorIsNil)
+	return stor, metadata
+}
+
+func (s *metadataHelperSuite) TestReadWriteMetadataMultipleStream(c *gc.C) {
+	stor, metadata := s.writeMetadataMultipleStream(c)
+	// Read back what was just written.
+	out, err := tools.ReadAllMetadata(stor)
+	c.Assert(err, jc.ErrorIsNil)
+	for _, outMetadata := range out {
+		for _, md := range outMetadata {
+			// FullPath is set by ReadAllMetadata.
+			c.Assert(md.FullPath, gc.Not(gc.Equals), "")
+			md.FullPath = ""
+		}
+	}
+	c.Assert(out, jc.DeepEquals, metadata)
+}
+
+func (s *metadataHelperSuite) TestWriteMetadataLegacyIndex(c *gc.C) {
+	stor, _ := s.writeMetadataMultipleStream(c)
+	// Read back the legacy index
+	rdr, err := stor.Get("tools/streams/v1/index.json")
+	c.Assert(err, jc.ErrorIsNil)
+	data, err := ioutil.ReadAll(rdr)
+	c.Assert(err, jc.ErrorIsNil)
+	var indices simplestreams.Indices
+	err = json.Unmarshal(data, &indices)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(indices.Indexes, gc.HasLen, 1)
+	indices.Updated = ""
+	c.Assert(indices.Indexes["com.ubuntu.juju:released:tools"], gc.NotNil)
+	indices.Indexes["com.ubuntu.juju:released:tools"].Updated = ""
+	expected := simplestreams.Indices{
+		Format: "index:1.0",
+		Indexes: map[string]*simplestreams.IndexMetadata{
+			"com.ubuntu.juju:released:tools": &simplestreams.IndexMetadata{
+				Format:           "products:1.0",
+				DataType:         "content-download",
+				ProductsFilePath: "streams/v1/com.ubuntu.juju-released-tools.json",
+				ProductIds:       []string{"com.ubuntu.juju:12.04:amd64"},
+			},
+		},
+	}
+	c.Assert(indices, jc.DeepEquals, expected)
+}
+
+func (s *metadataHelperSuite) TestReadWriteMetadataUnchanged(c *gc.C) {
+	metadata := map[string][]*tools.ToolsMetadata{
+		"released": []*tools.ToolsMetadata{{
+			Release: "precise",
+			Version: "1.2.3",
+			Arch:    "amd64",
+			Path:    "path1",
+		}, {
+			Release: "raring",
+			Version: "1.2.3",
+			Arch:    "amd64",
+			Path:    "path2",
+		}},
+	}
+
+	stor, err := filestorage.NewFileStorageWriter(c.MkDir())
+	c.Assert(err, jc.ErrorIsNil)
+	err = tools.WriteMetadata(stor, metadata, []string{"released"}, tools.DoNotWriteMirrors)
+	c.Assert(err, jc.ErrorIsNil)
+
+	s.PatchValue(tools.WriteMetadataFiles, func(stor storage.Storage, metadataInfo []tools.MetadataFile) error {
+		// The product data is the same, we only write the indices.
+		c.Assert(metadataInfo, gc.HasLen, 2)
+		c.Assert(metadataInfo[0].Path, gc.Equals, "streams/v1/index2.json")
+		c.Assert(metadataInfo[1].Path, gc.Equals, "streams/v1/index.json")
+		return nil
+	})
+	err = tools.WriteMetadata(stor, metadata, []string{"released"}, tools.DoNotWriteMirrors)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
+func (*metadataHelperSuite) TestReadMetadataPrefersNewIndex(c *gc.C) {
+	metadataDir := c.MkDir()
+
+	// Generate metadata and rename index to index.json
+	metadata := map[string][]*tools.ToolsMetadata{
+		"proposed": []*tools.ToolsMetadata{{
+			Release: "precise",
+			Version: "1.2.3",
+			Arch:    "amd64",
+			Path:    "path1",
+		}},
+		"released": []*tools.ToolsMetadata{{
+			Release: "trusty",
+			Version: "1.2.3",
+			Arch:    "amd64",
+			Path:    "path1",
+		}},
+	}
+	stor, err := filestorage.NewFileStorageWriter(metadataDir)
+	c.Assert(err, jc.ErrorIsNil)
+	err = tools.WriteMetadata(stor, metadata, []string{"proposed", "released"}, tools.DoNotWriteMirrors)
+	c.Assert(err, jc.ErrorIsNil)
+	err = os.Rename(
+		filepath.Join(metadataDir, "tools", "streams", "v1", "index2.json"),
+		filepath.Join(metadataDir, "tools", "streams", "v1", "index.json"),
+	)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Generate different metadata with index2.json
+	metadata = map[string][]*tools.ToolsMetadata{
+		"released": []*tools.ToolsMetadata{{
+			Release: "precise",
+			Version: "1.2.3",
+			Arch:    "amd64",
+			Path:    "path1",
+		}},
+	}
+	err = tools.WriteMetadata(stor, metadata, []string{"released"}, tools.DoNotWriteMirrors)
+	c.Assert(err, jc.ErrorIsNil)
+
+	// Read back all metadata, expecting to find metadata in index2.json.
+	out, err := tools.ReadAllMetadata(stor)
+	for _, outMetadata := range out {
+		for _, md := range outMetadata {
+			// FullPath is set by ReadAllMetadata.
+			c.Assert(md.FullPath, gc.Not(gc.Equals), "")
+			md.FullPath = ""
+		}
+	}
+	c.Assert(out, jc.DeepEquals, metadata)
 }
 
 type signedSuite struct {
@@ -747,7 +1012,7 @@ func (s *signedSuite) SetUpSuite(c *gc.C) {
 	r := bytes.NewReader([]byte(rawUnsignedIndex))
 	signedData, err := simplestreams.Encode(
 		r, sstesting.SignedMetadataPrivateKey, sstesting.PrivateKeyPassphrase)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	imageData["/signed/streams/v1/index.sjson"] = string(signedData)
 
 	// Replace the tools path in the unsigned data with a different one so we can test that the right
@@ -757,7 +1022,7 @@ func (s *signedSuite) SetUpSuite(c *gc.C) {
 	r = bytes.NewReader([]byte(rawUnsignedProduct))
 	signedData, err = simplestreams.Encode(
 		r, sstesting.SignedMetadataPrivateKey, sstesting.PrivateKeyPassphrase)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	imageData["/signed/streams/v1/tools_metadata.sjson"] = string(signedData)
 	testRoundTripper.Sub = jujutest.NewCannedRoundTripper(
 		imageData, map[string]int{"signedtest://unauth": http.StatusUnauthorized})
@@ -775,10 +1040,11 @@ func (s *signedSuite) TestSignedToolsMetadata(c *gc.C) {
 		CloudSpec: simplestreams.CloudSpec{"us-east-1", "https://ec2.us-east-1.amazonaws.com"},
 		Series:    []string{"precise"},
 		Arches:    []string{"amd64"},
+		Stream:    "released",
 	})
 	toolsMetadata, resolveInfo, err := tools.Fetch(
 		[]simplestreams.DataSource{signedSource}, toolsConstraint, true)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(len(toolsMetadata), gc.Equals, 1)
 	c.Assert(toolsMetadata[0].Path, gc.Equals, "tools/releases/20130806/juju-1.13.1-precise-amd64.tgz")
 	c.Assert(resolveInfo, gc.DeepEquals, &simplestreams.ResolveInfo{
