@@ -1,14 +1,17 @@
 package juju_test
 
 import (
+	"github.com/juju/names"
+
+	"github.com/juju/juju/api"
 	"github.com/juju/juju/juju"
 	"github.com/juju/juju/network"
-	"github.com/juju/juju/state/api"
 )
 
 type mockAPIState struct {
 	close func(juju.APIState) error
 
+	addr         string
 	apiHostPorts [][]network.HostPort
 	environTag   string
 }
@@ -20,12 +23,16 @@ func (s *mockAPIState) Close() error {
 	return nil
 }
 
+func (s *mockAPIState) Addr() string {
+	return s.addr
+}
+
 func (s *mockAPIState) APIHostPorts() [][]network.HostPort {
 	return s.apiHostPorts
 }
 
-func (s *mockAPIState) EnvironTag() string {
-	return s.environTag
+func (s *mockAPIState) EnvironTag() (names.EnvironTag, error) {
+	return names.ParseEnvironTag(s.environTag)
 }
 
 func panicAPIOpen(apiInfo *api.Info, opts api.DialOpts) (juju.APIState, error) {
