@@ -6,7 +6,7 @@ package osenv_test
 import (
 	"path/filepath"
 
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/juju/osenv"
 )
