@@ -1,13 +1,14 @@
-// Copyright 2013 Canonical Ltd.
+// Copyright 2014 Canonical Ltd.
+// Copyright 2014 Cloudbase Solutions SRL
 // Licensed under the AGPLv3, see LICENCE file for details.
 
 package osenv_test
 
 import (
-	"path/filepath"
 	"runtime"
 
-	gc "launchpad.net/gocheck"
+	jc "github.com/juju/testing/checkers"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/juju/osenv"
 	"github.com/juju/juju/testing"
@@ -19,18 +20,6 @@ type varsSuite struct {
 
 var _ = gc.Suite(&varsSuite{})
 
-func (s *varsSuite) TestJujuHomeWin(c *gc.C) {
-	path := `P:\FooBar\AppData`
-	s.PatchEnvironment("APPDATA", path)
-	c.Assert(osenv.JujuHomeWin(), gc.Equals, filepath.Join(path, "Juju"))
-}
-
-func (s *varsSuite) TestJujuHomeLinux(c *gc.C) {
-	path := `/foo/bar/baz/`
-	s.PatchEnvironment("HOME", path)
-	c.Assert(osenv.JujuHomeLinux(), gc.Equals, filepath.Join(path, ".juju"))
-}
-
 func (s *varsSuite) TestJujuHomeEnvVar(c *gc.C) {
 	path := "/foo/bar/baz"
 	s.PatchEnvironment(osenv.JujuHomeEnvKey, path)
@@ -53,3 +42,22 @@ func (s *varsSuite) TestBlankJujuHomeEnvVar(c *gc.C) {
 		c.Assert(osenv.JujuHomeDir(), gc.Equals, osenv.JujuHomeLinux())
 	}
 }
+
+func (s *varsSuite) TestMergeEnvironment(c *gc.C) {
+	c.Check(osenv.MergeEnvironment(nil, nil), gc.HasLen, 0)
+	initial := map[string]string{"a": "foo", "b": "bar"}
+	newValues := map[string]string{"a": "baz", "c": "omg"}
+	expected := map[string]string{"a": "baz", "c": "omg"}
+
+	created := osenv.MergeEnvironment(nil, newValues)
+	c.Check(created, jc.DeepEquals, expected)
+	// Show that the map returned isn't the one passed in.
+	newValues["d"] = "another"
+	c.Check(created, jc.DeepEquals, expected)
+
+	created = osenv.MergeEnvironment(initial, newValues)
+	expected = map[string]string{"a": "baz", "b": "bar", "c": "omg", "d": "another"}
+	// The returned value is the inital map.
+	c.Check(created, jc.DeepEquals, expected)
+	c.Check(initial, jc.DeepEquals, expected)
+}
