@@ -13,8 +13,8 @@ var (
 	SSLKeyPath       = sslKeyPath
 
 	UpstartConfInstall          = &upstartConfInstall
-	UpstartConfExists           = &upstartConfExists
 	UpstartService              = upstartService
+	UpstartServiceExists        = &upstartServiceExists
 	UpstartServiceRunning       = &upstartServiceRunning
 	UpstartServiceStopAndRemove = &upstartServiceStopAndRemove
 	UpstartServiceStop          = &upstartServiceStop
