@@ -17,13 +17,13 @@ import (
 	"github.com/juju/testing"
 	jc "github.com/juju/testing/checkers"
 	"github.com/juju/utils"
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
-	"github.com/juju/juju/state/api/params"
+	"github.com/juju/juju/service/common"
+	"github.com/juju/juju/service/upstart"
 	coretesting "github.com/juju/juju/testing"
-	"github.com/juju/juju/upstart"
 	"github.com/juju/juju/version"
 )
 
@@ -35,7 +35,7 @@ type MongoSuite struct {
 	mongodPath       string
 
 	installError error
-	installed    []upstart.Conf
+	installed    []upstart.Service
 
 	removeError error
 	removed     []upstart.Service
@@ -43,7 +43,12 @@ type MongoSuite struct {
 
 var _ = gc.Suite(&MongoSuite{})
 
-var testInfo = params.StateServingInfo{
+var testInfo = struct {
+	StatePort    int
+	Cert         string
+	PrivateKey   string
+	SharedSecret string
+}{
 	StatePort:    25252,
 	Cert:         "foobar-cert",
 	PrivateKey:   "foobar-privkey",
@@ -52,9 +57,13 @@ var testInfo = params.StateServingInfo{
 
 func makeEnsureServerParams(dataDir, namespace string) mongo.EnsureServerParams {
 	return mongo.EnsureServerParams{
-		StateServingInfo: testInfo,
-		DataDir:          dataDir,
-		Namespace:        namespace,
+		StatePort:    testInfo.StatePort,
+		Cert:         testInfo.Cert,
+		PrivateKey:   testInfo.PrivateKey,
+		SharedSecret: testInfo.SharedSecret,
+
+		DataDir:   dataDir,
+		Namespace: namespace,
 	}
 }
 
@@ -65,7 +74,7 @@ func (s *MongoSuite) SetUpTest(c *gc.C) {
 
 	s.mongodPath = filepath.Join(c.MkDir(), "mongod")
 	err := ioutil.WriteFile(s.mongodPath, []byte("#!/bin/bash\n\nprintf %s 'db version v2.4.9'\n"), 0755)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	s.PatchValue(&mongo.JujuMongodPath, s.mongodPath)
 
 	// Patch "df" such that it always reports there's 1MB free.
@@ -86,7 +95,7 @@ func (s *MongoSuite) SetUpTest(c *gc.C) {
 	s.mongodConfigPath = filepath.Join(testPath, "mongodConfig")
 	s.PatchValue(mongo.MongoConfigPath, s.mongodConfigPath)
 
-	s.PatchValue(mongo.UpstartConfInstall, func(conf *upstart.Conf) error {
+	s.PatchValue(mongo.UpstartConfInstall, func(conf *upstart.Service) error {
 		s.installed = append(s.installed, *conf)
 		return s.installError
 	})
@@ -103,7 +112,7 @@ func (s *MongoSuite) SetUpTest(c *gc.C) {
 
 func (s *MongoSuite) TestJujuMongodPath(c *gc.C) {
 	obtained, err := mongo.Path()
-	c.Check(err, gc.IsNil)
+	c.Check(err, jc.ErrorIsNil)
 	c.Check(obtained, gc.Equals, s.mongodPath)
 }
 
@@ -112,14 +121,14 @@ func (s *MongoSuite) TestDefaultMongodPath(c *gc.C) {
 	s.PatchEnvPathPrepend(filepath.Dir(s.mongodPath))
 
 	obtained, err := mongo.Path()
-	c.Check(err, gc.IsNil)
+	c.Check(err, jc.ErrorIsNil)
 	c.Check(obtained, gc.Equals, s.mongodPath)
 }
 
 func (s *MongoSuite) TestMakeJournalDirs(c *gc.C) {
 	dir := c.MkDir()
 	err := mongo.MakeJournalDirs(dir)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	testJournalDirs(dir, c)
 }
@@ -129,53 +138,32 @@ func testJournalDirs(dir string, c *gc.C) {
 
 	c.Assert(journalDir, jc.IsDirectory)
 	info, err := os.Stat(filepath.Join(journalDir, "prealloc.0"))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	size := int64(1024 * 1024)
 
 	c.Assert(info.Size(), gc.Equals, size)
 	info, err = os.Stat(filepath.Join(journalDir, "prealloc.1"))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(info.Size(), gc.Equals, size)
 	info, err = os.Stat(filepath.Join(journalDir, "prealloc.2"))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(info.Size(), gc.Equals, size)
 }
 
 func (s *MongoSuite) TestEnsureServer(c *gc.C) {
-	dataDir := c.MkDir()
-	dbDir := filepath.Join(dataDir, "db")
-	namespace := "namespace"
-
-	mockShellCommand(c, &s.CleanupSuite, "apt-get")
-
-	err := mongo.EnsureServer(makeEnsureServerParams(dataDir, namespace))
-	c.Assert(err, gc.IsNil)
-
-	testJournalDirs(dbDir, c)
-
-	assertInstalled := func() {
-		c.Assert(s.installed, gc.HasLen, 1)
-		conf := s.installed[0]
-		c.Assert(conf.Name, gc.Equals, "juju-db-namespace")
-		c.Assert(conf.InitDir, gc.Equals, "/etc/init")
-		c.Assert(conf.Desc, gc.Equals, "juju state database")
-		c.Assert(conf.Cmd, gc.Matches, regexp.QuoteMeta(s.mongodPath)+".*")
-		// TODO(nate) set Out so that mongod output goes somewhere useful?
-		c.Assert(conf.Out, gc.Equals, "")
-	}
-	assertInstalled()
+	dataDir := s.testEnsureServerNumaCtl(c, false)
 
 	contents, err := ioutil.ReadFile(s.mongodConfigPath)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(contents, jc.DeepEquals, []byte("ENABLE_MONGODB=no"))
 
 	contents, err = ioutil.ReadFile(mongo.SSLKeyPath(dataDir))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(string(contents), gc.Equals, testInfo.Cert+"\n"+testInfo.PrivateKey)
 
 	contents, err = ioutil.ReadFile(mongo.SharedSecretPath(dataDir))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(string(contents), gc.Equals, testInfo.SharedSecret)
 
 	// make sure that we log the version of mongodb as we get ready to
@@ -190,9 +178,10 @@ func (s *MongoSuite) TestEnsureServer(c *gc.C) {
 func (s *MongoSuite) TestEnsureServerServerExistsAndRunning(c *gc.C) {
 	dataDir := c.MkDir()
 	namespace := "namespace"
+
 	mockShellCommand(c, &s.CleanupSuite, "apt-get")
 
-	s.PatchValue(mongo.UpstartConfExists, func(svc *upstart.Conf) bool {
+	s.PatchValue(mongo.UpstartServiceExists, func(svc *upstart.Service) bool {
 		return true
 	})
 	s.PatchValue(mongo.UpstartServiceRunning, func(svc *upstart.Service) bool {
@@ -203,16 +192,17 @@ func (s *MongoSuite) TestEnsureServerServerExistsAndRunning(c *gc.C) {
 	})
 
 	err := mongo.EnsureServer(makeEnsureServerParams(dataDir, namespace))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(s.installed, gc.HasLen, 0)
 }
 
 func (s *MongoSuite) TestEnsureServerServerExistsNotRunningIsStarted(c *gc.C) {
 	dataDir := c.MkDir()
 	namespace := "namespace"
+
 	mockShellCommand(c, &s.CleanupSuite, "apt-get")
 
-	s.PatchValue(mongo.UpstartConfExists, func(svc *upstart.Conf) bool {
+	s.PatchValue(mongo.UpstartServiceExists, func(svc *upstart.Service) bool {
 		return true
 	})
 	s.PatchValue(mongo.UpstartServiceRunning, func(svc *upstart.Service) bool {
@@ -225,7 +215,7 @@ func (s *MongoSuite) TestEnsureServerServerExistsNotRunningIsStarted(c *gc.C) {
 	})
 
 	err := mongo.EnsureServer(makeEnsureServerParams(dataDir, namespace))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(s.installed, gc.HasLen, 0)
 	c.Assert(started, jc.IsTrue)
 }
@@ -233,9 +223,10 @@ func (s *MongoSuite) TestEnsureServerServerExistsNotRunningIsStarted(c *gc.C) {
 func (s *MongoSuite) TestEnsureServerServerExistsNotRunningStartError(c *gc.C) {
 	dataDir := c.MkDir()
 	namespace := "namespace"
+
 	mockShellCommand(c, &s.CleanupSuite, "apt-get")
 
-	s.PatchValue(mongo.UpstartConfExists, func(svc *upstart.Conf) bool {
+	s.PatchValue(mongo.UpstartServiceExists, func(svc *upstart.Service) bool {
 		return true
 	})
 	s.PatchValue(mongo.UpstartServiceRunning, func(svc *upstart.Service) bool {
@@ -250,6 +241,44 @@ func (s *MongoSuite) TestEnsureServerServerExistsNotRunningStartError(c *gc.C) {
 	c.Assert(s.installed, gc.HasLen, 0)
 }
 
+func (s *MongoSuite) TestEnsureServerNumaCtl(c *gc.C) {
+	s.testEnsureServerNumaCtl(c, true)
+}
+
+func (s *MongoSuite) testEnsureServerNumaCtl(c *gc.C, setNumaPolicy bool) string {
+	dataDir := c.MkDir()
+	dbDir := filepath.Join(dataDir, "db")
+	namespace := "namespace"
+
+	mockShellCommand(c, &s.CleanupSuite, "apt-get")
+
+	testParams := makeEnsureServerParams(dataDir, namespace)
+	testParams.SetNumaControlPolicy = setNumaPolicy
+	err := mongo.EnsureServer(testParams)
+	c.Assert(err, jc.ErrorIsNil)
+
+	testJournalDirs(dbDir, c)
+
+	assertInstalled := func() {
+		c.Assert(s.installed, gc.HasLen, 1)
+		service := s.installed[0]
+		c.Assert(service.Name, gc.Equals, "juju-db-namespace")
+		c.Assert(service.Conf.InitDir, gc.Equals, "/etc/init")
+		c.Assert(service.Conf.Desc, gc.Equals, "juju state database")
+		if setNumaPolicy {
+			stripped := strings.Replace(service.Conf.ExtraScript, "\n", "", -1)
+			c.Assert(stripped, gc.Matches, `.* sysctl .*`)
+		} else {
+			c.Assert(service.Conf.ExtraScript, gc.Equals, "")
+		}
+		c.Assert(service.Conf.Cmd, gc.Matches, ".*"+regexp.QuoteMeta(s.mongodPath)+".*")
+		// TODO(nate) set Out so that mongod output goes somewhere useful?
+		c.Assert(service.Conf.Out, gc.Equals, "")
+	}
+	assertInstalled()
+	return dataDir
+}
+
 func (s *MongoSuite) TestInstallMongod(c *gc.C) {
 	type installs struct {
 		series string
@@ -274,7 +303,7 @@ func (s *MongoSuite) TestInstallMongod(c *gc.C) {
 		s.PatchValue(&version.Current.Series, test.series)
 
 		err := mongo.EnsureServer(makeEnsureServerParams(dataDir, namespace))
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 
 		cmds := getMockShellCalls(c, output)
 
@@ -296,7 +325,7 @@ func (s *MongoSuite) TestInstallMongodServiceExists(c *gc.C) {
 	dataDir := c.MkDir()
 	namespace := "namespace"
 
-	s.PatchValue(mongo.UpstartConfExists, func(svc *upstart.Conf) bool {
+	s.PatchValue(mongo.UpstartServiceExists, func(svc *upstart.Service) bool {
 		return true
 	})
 	s.PatchValue(mongo.UpstartServiceRunning, func(svc *upstart.Service) bool {
@@ -307,7 +336,7 @@ func (s *MongoSuite) TestInstallMongodServiceExists(c *gc.C) {
 	})
 
 	err := mongo.EnsureServer(makeEnsureServerParams(dataDir, namespace))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(s.installed, gc.HasLen, 0)
 
 	// We still attempt to install mongodb, despite the service existing.
@@ -318,17 +347,33 @@ func (s *MongoSuite) TestInstallMongodServiceExists(c *gc.C) {
 func (s *MongoSuite) TestUpstartServiceWithReplSet(c *gc.C) {
 	dataDir := c.MkDir()
 
-	svc, err := mongo.UpstartService("", dataDir, dataDir, mongo.JujuMongodPath, 1234, 1024)
-	c.Assert(err, gc.IsNil)
-	c.Assert(strings.Contains(svc.Cmd, "--replSet"), jc.IsTrue)
+	svc, err := mongo.UpstartService("", dataDir, dataDir, mongo.JujuMongodPath, 1234, 1024, false)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(strings.Contains(svc.Conf.Cmd, "--replSet"), jc.IsTrue)
+}
+
+func (s *MongoSuite) TestUpstartServiceWithNumCtl(c *gc.C) {
+	dataDir := c.MkDir()
+
+	svc, err := mongo.UpstartService("", dataDir, dataDir, mongo.JujuMongodPath, 1234, 1024, true)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(svc.Conf.ExtraScript, gc.Not(gc.Matches), "")
+}
+
+func (s *MongoSuite) TestUpstartServiceIPv6(c *gc.C) {
+	dataDir := c.MkDir()
+
+	svc, err := mongo.UpstartService("", dataDir, dataDir, mongo.JujuMongodPath, 1234, 1024, false)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(strings.Contains(svc.Conf.Cmd, "--ipv6"), jc.IsTrue)
 }
 
 func (s *MongoSuite) TestUpstartServiceWithJournal(c *gc.C) {
 	dataDir := c.MkDir()
 
-	svc, err := mongo.UpstartService("", dataDir, dataDir, mongo.JujuMongodPath, 1234, 1024)
-	c.Assert(err, gc.IsNil)
-	journalPresent := strings.Contains(svc.Cmd, " --journal ") || strings.HasSuffix(svc.Cmd, " --journal")
+	svc, err := mongo.UpstartService("", dataDir, dataDir, mongo.JujuMongodPath, 1234, 1024, false)
+	c.Assert(err, jc.ErrorIsNil)
+	journalPresent := strings.Contains(svc.Conf.Cmd, " --journal ") || strings.HasSuffix(svc.Conf.Cmd, " --journal")
 	c.Assert(journalPresent, jc.IsTrue)
 }
 
@@ -336,7 +381,7 @@ func (s *MongoSuite) TestNoAuthCommandWithJournal(c *gc.C) {
 	dataDir := c.MkDir()
 
 	cmd, err := mongo.NoauthCommand(dataDir, 1234)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	var isJournalPresent bool
 	for _, value := range cmd.Args {
 		if value == "--journal" {
@@ -348,10 +393,10 @@ func (s *MongoSuite) TestNoAuthCommandWithJournal(c *gc.C) {
 
 func (s *MongoSuite) TestRemoveService(c *gc.C) {
 	err := mongo.RemoveService("namespace")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(s.removed, jc.DeepEquals, []upstart.Service{{
-		Name:    "juju-db-namespace",
-		InitDir: upstart.InitDir,
+		Name: "juju-db-namespace",
+		Conf: common.Conf{InitDir: upstart.InitDir},
 	}})
 }
 
@@ -370,7 +415,7 @@ func (s *MongoSuite) TestQuantalAptAddRepo(c *gc.C) {
 	s.PatchValue(&version.Current.Series, "trusty")
 	failCmd(filepath.Join(dir, "mongod"))
 	err = mongo.EnsureServer(makeEnsureServerParams(dir, ""))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *MongoSuite) TestNoMongoDir(c *gc.C) {
@@ -379,10 +424,10 @@ func (s *MongoSuite) TestNoMongoDir(c *gc.C) {
 	mockShellCommand(c, &s.CleanupSuite, "apt-get")
 	dataDir := filepath.Join(c.MkDir(), "dir", "data")
 	err := mongo.EnsureServer(makeEnsureServerParams(dataDir, ""))
-	c.Check(err, gc.IsNil)
+	c.Check(err, jc.ErrorIsNil)
 
 	_, err = os.Stat(filepath.Join(dataDir, "db"))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *MongoSuite) TestServiceName(c *gc.C) {
@@ -431,10 +476,10 @@ func (s *MongoSuite) TestSelectPeerHostPort(c *gc.C) {
 
 func (s *MongoSuite) TestGenerateSharedSecret(c *gc.C) {
 	secret, err := mongo.GenerateSharedSecret()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(secret, gc.HasLen, 1024)
 	_, err = base64.StdEncoding.DecodeString(secret)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *MongoSuite) TestAddPPAInQuantal(c *gc.C) {
@@ -445,7 +490,7 @@ func (s *MongoSuite) TestAddPPAInQuantal(c *gc.C) {
 
 	dataDir := c.MkDir()
 	err := mongo.EnsureServer(makeEnsureServerParams(dataDir, ""))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	c.Assert(getMockShellCalls(c, addAptRepoOut), gc.DeepEquals, [][]string{{
 		"-y",
@@ -482,7 +527,7 @@ func mockShellCommand(c *gc.C, s *testing.CleanupSuite, name string) string {
 } >> ` + utils.ShQuote(outputFile) + `
 `
 	err := ioutil.WriteFile(filepath.Join(dir, name), []byte(contents), 0755)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	return outputFile
 }
 
@@ -495,7 +540,7 @@ func getMockShellCalls(c *gc.C, file string) [][]string {
 	if os.IsNotExist(err) {
 		return nil
 	}
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	s := string(data)
 	parts := strings.Split(s, "\n-\n")
 	c.Assert(parts[len(parts)-1], gc.Equals, "")
