@@ -24,13 +24,11 @@ import (
 	"github.com/juju/juju/environs/instances"
 	"github.com/juju/juju/environs/simplestreams"
 	"github.com/juju/juju/environs/storage"
-	envtools "github.com/juju/juju/environs/tools"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/common"
 	"github.com/juju/juju/state"
-	"github.com/juju/juju/state/api"
-	"github.com/juju/juju/state/api/params"
+	"github.com/juju/juju/state/multiwatcher"
 )
 
 const (
@@ -65,9 +63,6 @@ type azureEnviron struct {
 	// only be accessed using a lock or a snapshot.
 	sync.Mutex
 
-	// name is immutable; it does not need locking.
-	name string
-
 	// archMutex gates access to supportedArchitectures
 	archMutex sync.Mutex
 	// supportedArchitectures caches the architectures
@@ -84,18 +79,26 @@ type azureEnviron struct {
 	// private storage.  This is automatically queried from Azure on
 	// startup.
 	storageAccountKey string
+
+	// api is a management API for Microsoft Azure.
+	api *gwacl.ManagementAPI
+
+	// vnet describes the configured virtual network.
+	vnet *gwacl.VirtualNetworkSite
+
+	// availableRoleSizes is the role sizes available in the configured
+	// location. This will be reset whenever the location configuration changes.
+	availableRoleSizes set.Strings
 }
 
 // azureEnviron implements Environ and HasRegion.
 var _ environs.Environ = (*azureEnviron)(nil)
 var _ simplestreams.HasRegion = (*azureEnviron)(nil)
-var _ imagemetadata.SupportsCustomSources = (*azureEnviron)(nil)
-var _ envtools.SupportsCustomSources = (*azureEnviron)(nil)
 var _ state.Prechecker = (*azureEnviron)(nil)
 
 // NewEnviron creates a new azureEnviron.
 func NewEnviron(cfg *config.Config) (*azureEnviron, error) {
-	env := azureEnviron{name: cfg.Name()}
+	var env azureEnviron
 	err := env.SetConfig(cfg)
 	if err != nil {
 		return nil, err
@@ -119,31 +122,22 @@ func extractStorageKey(keys *gwacl.StorageAccountKeys) string {
 
 // queryStorageAccountKey retrieves the storage account's key from Azure.
 func (env *azureEnviron) queryStorageAccountKey() (string, error) {
-	azure, err := env.getManagementAPI()
-	if err != nil {
-		return "", err
-	}
-	defer env.releaseManagementAPI(azure)
+	snap := env.getSnapshot()
 
-	accountName := env.getSnapshot().ecfg.storageAccountName()
-	keys, err := azure.GetStorageAccountKeys(accountName)
+	accountName := snap.ecfg.storageAccountName()
+	keys, err := snap.api.GetStorageAccountKeys(accountName)
 	if err != nil {
-		return "", fmt.Errorf("cannot obtain storage account keys: %v", err)
+		return "", errors.Annotate(err, "cannot obtain storage account keys")
 	}
 
 	key := extractStorageKey(keys)
 	if key == "" {
-		return "", fmt.Errorf("no keys available for storage account")
+		return "", errors.New("no keys available for storage account")
 	}
 
 	return key, nil
 }
 
-// Name is specified in the Environ interface.
-func (env *azureEnviron) Name() string {
-	return env.name
-}
-
 // getSnapshot produces an atomic shallow copy of the environment object.
 // Whenever you need to access the environment object's fields without
 // modifying them, get a snapshot and read its fields instead.  You will
@@ -164,62 +158,124 @@ func (env *azureEnviron) getSnapshot() *azureEnviron {
 }
 
 // getAffinityGroupName returns the name of the affinity group used by all
-// the Services in this environment.
+// the Services in this environment. The affinity group name is immutable,
+// so there is no need to use a configuration snapshot.
 func (env *azureEnviron) getAffinityGroupName() string {
 	return env.getEnvPrefix() + "ag"
 }
 
-func (env *azureEnviron) createAffinityGroup() error {
-	affinityGroupName := env.getAffinityGroupName()
-	azure, err := env.getManagementAPI()
-	if err != nil {
-		return err
-	}
-	defer env.releaseManagementAPI(azure)
+// getLocation gets the configured Location for the environment.
+func (env *azureEnviron) getLocation() string {
 	snap := env.getSnapshot()
+	return snap.ecfg.location()
+}
+
+func (env *azureEnviron) createAffinityGroup() error {
+	snap := env.getSnapshot()
+	affinityGroupName := env.getAffinityGroupName()
 	location := snap.ecfg.location()
 	cag := gwacl.NewCreateAffinityGroup(affinityGroupName, affinityGroupName, affinityGroupName, location)
-	return azure.CreateAffinityGroup(&gwacl.CreateAffinityGroupRequest{
-		CreateAffinityGroup: cag})
+	return snap.api.CreateAffinityGroup(&gwacl.CreateAffinityGroupRequest{
+		CreateAffinityGroup: cag,
+	})
 }
 
 func (env *azureEnviron) deleteAffinityGroup() error {
+	snap := env.getSnapshot()
 	affinityGroupName := env.getAffinityGroupName()
-	azure, err := env.getManagementAPI()
-	if err != nil {
-		return err
+	return snap.api.DeleteAffinityGroup(&gwacl.DeleteAffinityGroupRequest{
+		Name: affinityGroupName,
+	})
+}
+
+// getAvailableRoleSizes returns the role sizes available for the configured
+// location.
+func (env *azureEnviron) getAvailableRoleSizes() (_ set.Strings, err error) {
+	defer errors.DeferredAnnotatef(&err, "cannot get available role sizes")
+
+	snap := env.getSnapshot()
+	if snap.availableRoleSizes != nil {
+		return snap.availableRoleSizes, nil
 	}
-	defer env.releaseManagementAPI(azure)
-	return azure.DeleteAffinityGroup(&gwacl.DeleteAffinityGroupRequest{
-		Name: affinityGroupName})
+	locations, err := snap.api.ListLocations()
+	if err != nil {
+		return nil, errors.Annotate(err, "cannot list locations")
+	}
+	var available set.Strings
+	for _, location := range locations {
+		if location.Name != snap.ecfg.location() {
+			continue
+		}
+		if location.ComputeCapabilities == nil {
+			return nil, errors.Annotate(err, "cannot determine compute capabilities")
+		}
+		available = set.NewStrings(location.ComputeCapabilities.VirtualMachineRoleSizes...)
+		break
+	}
+	if available == nil {
+		return nil, errors.NotFoundf("location %q", snap.ecfg.location())
+	}
+	env.Lock()
+	env.availableRoleSizes = available
+	env.Unlock()
+	return available, nil
 }
 
 // getVirtualNetworkName returns the name of the virtual network used by all
-// the VMs in this environment.
+// the VMs in this environment. The virtual network name is immutable,
+// so there is no need to use a configuration snapshot.
 func (env *azureEnviron) getVirtualNetworkName() string {
 	return env.getEnvPrefix() + "vnet"
 }
 
-func (env *azureEnviron) createVirtualNetwork() error {
-	// Note: the Azure documentation recommends to use
-	// Location when creating virtual network sites.
-	// We have historically used Affinity Group, and
-	// have observed intermittent issues when switching.
-	// http://msdn.microsoft.com/en-us/library/azure/jj157100.aspx
-	vnetName := env.getVirtualNetworkName()
-	azure, err := env.getManagementAPI()
-	if err != nil {
-		return err
+// getVirtualNetwork returns the virtual network used by all the VMs in this
+// environment.
+func (env *azureEnviron) getVirtualNetwork() (*gwacl.VirtualNetworkSite, error) {
+	snap := env.getSnapshot()
+	if snap.vnet != nil {
+		return snap.vnet, nil
 	}
-	defer env.releaseManagementAPI(azure)
+	cfg, err := env.api.GetNetworkConfiguration()
+	if err != nil {
+		return nil, errors.Annotate(err, "error getting network configuration")
+	}
+	var vnet *gwacl.VirtualNetworkSite
+	vnetName := env.getVirtualNetworkName()
+	if cfg != nil && cfg.VirtualNetworkSites != nil {
+		for _, site := range *cfg.VirtualNetworkSites {
+			if site.Name == vnetName {
+				vnet = &site
+				break
+			}
+		}
+	}
+	if vnet == nil {
+		return nil, errors.NotFoundf("virtual network %q", vnetName)
+	}
+	env.Lock()
+	env.vnet = vnet
+	env.Unlock()
+	return vnet, nil
+}
+
+// createVirtualNetwork creates a virtual network for the environment.
+func (env *azureEnviron) createVirtualNetwork() error {
+	snap := env.getSnapshot()
+	vnetName := env.getVirtualNetworkName()
 	virtualNetwork := gwacl.VirtualNetworkSite{
-		Name:          vnetName,
-		AffinityGroup: env.getAffinityGroupName(),
+		Name:     vnetName,
+		Location: snap.ecfg.location(),
 		AddressSpacePrefixes: []string{
 			networkDefinition,
 		},
 	}
-	return azure.AddVirtualNetworkSite(&virtualNetwork)
+	if err := snap.api.AddVirtualNetworkSite(&virtualNetwork); err != nil {
+		return errors.Trace(err)
+	}
+	env.Lock()
+	env.vnet = &virtualNetwork
+	env.Unlock()
+	return nil
 }
 
 // deleteVnetAttempt is an AttemptyStrategy for use
@@ -235,14 +291,11 @@ var deleteVnetAttempt = utils.AttemptStrategy{
 var networkInUse = regexp.MustCompile(".*The virtual network .* is currently in use.*")
 
 func (env *azureEnviron) deleteVirtualNetwork() error {
-	azure, err := env.getManagementAPI()
-	if err != nil {
-		return err
-	}
-	defer env.releaseManagementAPI(azure)
+	snap := env.getSnapshot()
+	vnetName := env.getVirtualNetworkName()
+	var err error
 	for a := deleteVnetAttempt.Start(); a.Next(); {
-		vnetName := env.getVirtualNetworkName()
-		err = azure.RemoveVirtualNetworkSite(vnetName)
+		err = snap.api.RemoveVirtualNetworkSite(vnetName)
 		if err == nil {
 			return nil
 		}
@@ -259,7 +312,8 @@ func (env *azureEnviron) deleteVirtualNetwork() error {
 }
 
 // getContainerName returns the name of the private storage account container
-// that this environment is using.
+// that this environment is using. The container name is immutable,
+// so there is no need to use a configuration snapshot.
 func (env *azureEnviron) getContainerName() string {
 	return env.getEnvPrefix() + "private"
 }
@@ -281,11 +335,11 @@ func isVirtualNetworkExist(err error) bool {
 }
 
 // Bootstrap is specified in the Environ interface.
-func (env *azureEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.BootstrapParams) (err error) {
+func (env *azureEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.BootstrapParams) (arch, series string, _ environs.BootstrapFinalizer, err error) {
 	// The creation of the affinity group and the virtual network is specific to the Azure provider.
 	err = env.createAffinityGroup()
 	if err != nil && !isHTTPConflict(err) {
-		return err
+		return "", "", nil, err
 	}
 	// If we fail after this point, clean up the affinity group.
 	defer func() {
@@ -296,7 +350,7 @@ func (env *azureEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.
 
 	err = env.createVirtualNetwork()
 	if err != nil && !isVirtualNetworkExist(err) {
-		return err
+		return "", "", nil, err
 	}
 	// If we fail after this point, clean up the virtual network.
 	defer func() {
@@ -307,9 +361,62 @@ func (env *azureEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.
 	return common.Bootstrap(ctx, env, args)
 }
 
-// StateInfo is specified in the Environ interface.
-func (env *azureEnviron) StateInfo() (*state.Info, *api.Info, error) {
-	return common.StateInfo(env)
+// isLegacyInstance reports whether the instance is a
+// legacy instance (i.e. one-to-one cloud service to instance).
+func isLegacyInstance(inst *azureInstance) (bool, error) {
+	snap := inst.environ.getSnapshot()
+	serviceName := inst.hostedService.ServiceName
+	service, err := snap.api.GetHostedServiceProperties(serviceName, true)
+	if err != nil {
+		return false, err
+	} else if len(service.Deployments) != 1 {
+		return false, nil
+	}
+	deploymentName := service.Deployments[0].Name
+	return deploymentName == deploymentNameV1(serviceName), nil
+}
+
+// StateServerInstances is specified in the Environ interface.
+func (env *azureEnviron) StateServerInstances() ([]instance.Id, error) {
+	// Locate the state-server cloud service, and get its addresses.
+	instances, err := env.AllInstances()
+	if err != nil {
+		return nil, err
+	}
+	var stateServerInstanceIds []instance.Id
+	var loadStateFile bool
+	for _, inst := range instances {
+		azureInstance := inst.(*azureInstance)
+		label := azureInstance.hostedService.Label
+		if decoded, err := base64.StdEncoding.DecodeString(label); err == nil {
+			if string(decoded) == stateServerLabel {
+				stateServerInstanceIds = append(stateServerInstanceIds, inst.Id())
+				continue
+			}
+		}
+		if !loadStateFile {
+			_, roleName := env.splitInstanceId(azureInstance.Id())
+			if roleName == "" {
+				loadStateFile = true
+			}
+		}
+	}
+	if loadStateFile {
+		// Some legacy instances were found, so we must load provider-state
+		// to find which instance was the original state server. If we find
+		// a legacy environment, then stateServerInstanceIds will not contain
+		// the original bootstrap instance, which is the only one that will
+		// be in provider-state.
+		instanceIds, err := common.ProviderStateInstances(env, env.Storage())
+		if err != nil {
+			return nil, err
+		}
+		stateServerInstanceIds = append(stateServerInstanceIds, instanceIds...)
+	}
+	if len(stateServerInstanceIds) == 0 {
+		return nil, environs.ErrNoInstances
+	}
+	return stateServerInstanceIds, nil
 }
 
 // Config is specified in the Environ interface.
@@ -320,6 +427,11 @@ func (env *azureEnviron) Config() *config.Config {
 
 // SetConfig is specified in the Environ interface.
 func (env *azureEnviron) SetConfig(cfg *config.Config) error {
+	var oldLocation string
+	if snap := env.getSnapshot(); snap.ecfg != nil {
+		oldLocation = snap.ecfg.location()
+	}
+
 	ecfg, err := azureEnvironProvider{}.newConfig(cfg)
 	if err != nil {
 		return err
@@ -341,6 +453,20 @@ func (env *azureEnviron) SetConfig(cfg *config.Config) error {
 	// be appropriate for the new config.
 	env.storageAccountKey = ""
 
+	subscription := ecfg.managementSubscriptionId()
+	certKeyPEM := []byte(ecfg.managementCertificate())
+	location := ecfg.location()
+	mgtAPI, err := gwacl.NewManagementAPICertDataWithRetryPolicy(subscription, certKeyPEM, certKeyPEM, location, retryPolicy)
+	if err != nil {
+		return errors.Annotate(err, "cannot acquire management API")
+	}
+	env.api = mgtAPI
+
+	// If the location changed, reset the available role sizes.
+	if location != oldLocation {
+		env.availableRoleSizes = nil
+	}
+
 	return nil
 }
 
@@ -377,7 +503,7 @@ func newHostedService(azure *gwacl.ManagementAPI, prefix, affinityGroupName, lab
 		createdService, err = attemptCreateService(azure, prefix, affinityGroupName, label)
 	}
 	if err != nil {
-		return nil, fmt.Errorf("could not create hosted service: %v", err)
+		return nil, errors.Annotate(err, "could not create hosted service")
 	}
 	if createdService == nil {
 		return nil, fmt.Errorf("could not come up with a unique hosted service name - is your randomizer initialized?")
@@ -413,6 +539,11 @@ func (env *azureEnviron) SupportNetworks() bool {
 	return false
 }
 
+// SupportAddressAllocation is specified on the EnvironCapability interface.
+func (e *azureEnviron) SupportAddressAllocation(netId network.Id) (bool, error) {
+	return false, nil
+}
+
 // selectInstanceTypeAndImage returns the appropriate instances.InstanceType and
 // the OS image name for launching a virtual machine with the given parameters.
 func (env *azureEnviron) selectInstanceTypeAndImage(constraint *instances.InstanceConstraint) (*instances.InstanceType, string, error) {
@@ -550,7 +681,7 @@ func (env *azureEnviron) createInstance(azure *gwacl.ManagementAPI, role *gwacl.
 			env.getVirtualNetworkName(),
 		)
 		if err := azure.AddDeployment(deployment, service.ServiceName); err != nil {
-			return nil, err
+			return nil, errors.Annotate(err, "error creating VM deployment")
 		}
 		service.Deployments = append(service.Deployments, *deployment)
 	} else {
@@ -581,14 +712,14 @@ func deploymentNameV2(serviceName string) string {
 }
 
 // StartInstance is specified in the InstanceBroker interface.
-func (env *azureEnviron) StartInstance(args environs.StartInstanceParams) (_ instance.Instance, _ *instance.HardwareCharacteristics, _ []network.Info, err error) {
+func (env *azureEnviron) StartInstance(args environs.StartInstanceParams) (*environs.StartInstanceResult, error) {
 	if args.MachineConfig.HasNetworks() {
-		return nil, nil, nil, fmt.Errorf("starting instances with networks is not supported yet.")
+		return nil, errors.New("starting instances with networks is not supported yet")
 	}
 
-	err = environs.FinishMachineConfig(args.MachineConfig, env.Config(), args.Constraints)
+	err := environs.FinishMachineConfig(args.MachineConfig, env.Config())
 	if err != nil {
-		return nil, nil, nil, err
+		return nil, err
 	}
 
 	// Pick envtools.  Needed for the custom data (which is what we normally
@@ -599,15 +730,9 @@ func (env *azureEnviron) StartInstance(args environs.StartInstanceParams) (_ ins
 	// Compose userdata.
 	userData, err := makeCustomData(args.MachineConfig)
 	if err != nil {
-		return nil, nil, nil, fmt.Errorf("custom data: %v", err)
+		return nil, errors.Annotate(err, "cannot compose user data")
 	}
 
-	azure, err := env.getManagementAPI()
-	if err != nil {
-		return nil, nil, nil, err
-	}
-	defer env.releaseManagementAPI(azure)
-
 	snapshot := env.getSnapshot()
 	location := snapshot.ecfg.location()
 	instanceType, sourceImageName, err := env.selectInstanceTypeAndImage(&instances.InstanceConstraint{
@@ -617,7 +742,7 @@ func (env *azureEnviron) StartInstance(args environs.StartInstanceParams) (_ ins
 		Constraints: args.Constraints,
 	})
 	if err != nil {
-		return nil, nil, nil, err
+		return nil, err
 	}
 
 	// We use the cloud service label as a way to group instances with
@@ -627,7 +752,7 @@ func (env *azureEnviron) StartInstance(args environs.StartInstanceParams) (_ ins
 	if args.DistributionGroup != nil && snapshot.ecfg.availabilitySetsEnabled() {
 		instanceIds, err := args.DistributionGroup()
 		if err != nil {
-			return nil, nil, nil, err
+			return nil, err
 		}
 		for _, id := range instanceIds {
 			cloudServiceName, _ = env.splitInstanceId(id)
@@ -640,17 +765,11 @@ func (env *azureEnviron) StartInstance(args environs.StartInstanceParams) (_ ins
 	vhd := env.newOSDisk(sourceImageName)
 	// If we're creating machine-0, we'll want to expose port 22.
 	// All other machines get an auto-generated public port for SSH.
-	stateServer := false
-	for _, job := range args.MachineConfig.Jobs {
-		if job == params.JobManageEnviron {
-			stateServer = true
-			break
-		}
-	}
+	stateServer := multiwatcher.AnyJobNeedsState(args.MachineConfig.Jobs...)
 	role := env.newRole(instanceType.Id, vhd, userData, stateServer)
-	inst, err := createInstance(env, azure.ManagementAPI, role, cloudServiceName, stateServer)
+	inst, err := createInstance(env, snapshot.api, role, cloudServiceName, stateServer)
 	if err != nil {
-		return nil, nil, nil, err
+		return nil, err
 	}
 	hc := &instance.HardwareCharacteristics{
 		Mem:      &instanceType.Mem,
@@ -660,7 +779,10 @@ func (env *azureEnviron) StartInstance(args environs.StartInstanceParams) (_ ins
 	if len(instanceType.Arches) == 1 {
 		hc.Arch = &instanceType.Arches[0]
 	}
-	return inst, hc, nil, nil
+	return &environs.StartInstanceResult{
+		Instance: inst,
+		Hardware: hc,
+	}, nil
 }
 
 // getInstance returns an up-to-date version of the instance with the given
@@ -730,10 +852,6 @@ func (env *azureEnviron) newOSDisk(sourceImageName string) *gwacl.OSVirtualHardD
 // getInitialEndpoints returns a slice of the endpoints every instance should have open
 // (ssh port, etc).
 func (env *azureEnviron) getInitialEndpoints(stateServer bool) []gwacl.InputEndpoint {
-	// TODO(axw) either proxy ssh traffic through one of the
-	// randomly chosen VMs to the internal address, or otherwise
-	// don't load balance SSH and provide a way of getting the
-	// local port.
 	cfg := env.Config()
 	endpoints := []gwacl.InputEndpoint{{
 		LocalPort: 22,
@@ -743,11 +861,6 @@ func (env *azureEnviron) getInitialEndpoints(stateServer bool) []gwacl.InputEndp
 	}}
 	if stateServer {
 		endpoints = append(endpoints, []gwacl.InputEndpoint{{
-			LocalPort: cfg.StatePort(),
-			Port:      cfg.StatePort(),
-			Protocol:  "tcp",
-			Name:      "stateport",
-		}, {
 			LocalPort: cfg.APIPort(),
 			Port:      cfg.APIPort(),
 			Protocol:  "tcp",
@@ -797,11 +910,7 @@ func (env *azureEnviron) newRole(roleSize string, vhd *gwacl.OSVirtualHardDisk,
 
 // StopInstances is specified in the InstanceBroker interface.
 func (env *azureEnviron) StopInstances(ids ...instance.Id) error {
-	context, err := env.getManagementAPI()
-	if err != nil {
-		return err
-	}
-	defer env.releaseManagementAPI(context)
+	snap := env.getSnapshot()
 
 	// Map services to role names we want to delete.
 	serviceInstances := make(map[string]map[string]bool)
@@ -829,14 +938,14 @@ func (env *azureEnviron) StopInstances(ids ...instance.Id) error {
 	// found to cause conflict responses, so we do everything serially.
 	for _, serviceName := range serviceNames {
 		deleteRoleNames := serviceInstances[serviceName]
-		service, err := context.GetHostedServiceProperties(serviceName, true)
+		service, err := snap.api.GetHostedServiceProperties(serviceName, true)
 		if err != nil {
 			return err
 		} else if len(service.Deployments) != 1 {
 			continue
 		}
 		// Filter the instances that have no corresponding role.
-		var roleNames set.Strings
+		roleNames := make(set.Strings)
 		for _, role := range service.Deployments[0].RoleList {
 			roleNames.Add(role.RoleName)
 		}
@@ -849,12 +958,12 @@ func (env *azureEnviron) StopInstances(ids ...instance.Id) error {
 		// entire cloud service or we'll get an error. deleteRoleNames
 		// is nil if we're dealing with a legacy deployment.
 		if deleteRoleNames == nil || len(deleteRoleNames) == roleNames.Size() {
-			if err := context.DeleteHostedService(serviceName); err != nil {
+			if err := snap.api.DeleteHostedService(serviceName); err != nil {
 				return err
 			}
 		} else {
 			for roleName := range deleteRoleNames {
-				if err := context.DeleteRole(&gwacl.DeleteRoleRequest{
+				if err := snap.api.DeleteRole(&gwacl.DeleteRoleRequest{
 					ServiceName:    serviceName,
 					DeploymentName: service.Deployments[0].Name,
 					RoleName:       roleName,
@@ -868,23 +977,43 @@ func (env *azureEnviron) StopInstances(ids ...instance.Id) error {
 	return nil
 }
 
+// hostedServices returns all services for this environment.
+func (env *azureEnviron) hostedServices() ([]gwacl.HostedServiceDescriptor, error) {
+	snap := env.getSnapshot()
+	services, err := snap.api.ListHostedServices()
+	if err != nil {
+		return nil, err
+	}
+
+	var filteredServices []gwacl.HostedServiceDescriptor
+	// Service names are prefixed with the environment name, followed by "-".
+	// We must be careful not to include services where the environment name
+	// is a substring of another name. ie we mustn't allow "azure" to match "azure-1".
+	envPrefix := env.getEnvPrefix()
+	// Just in case.
+	filterPrefix := regexp.QuoteMeta(envPrefix)
+
+	// Now filter the services.
+	prefixMatch := regexp.MustCompile("^" + filterPrefix + "[^-]*$")
+	for _, service := range services {
+		if prefixMatch.Match([]byte(service.ServiceName)) {
+			filteredServices = append(filteredServices, service)
+		}
+	}
+	return filteredServices, nil
+}
+
 // destroyAllServices destroys all Cloud Services and deployments contained.
 // This is needed to clean up broken environments, in which there are cloud
 // services with no deployments.
 func (env *azureEnviron) destroyAllServices() error {
-	context, err := env.getManagementAPI()
-	if err != nil {
-		return err
-	}
-	defer env.releaseManagementAPI(context)
-
-	request := &gwacl.ListPrefixedHostedServicesRequest{ServiceNamePrefix: env.getEnvPrefix()}
-	services, err := context.ListPrefixedHostedServices(request)
+	services, err := env.hostedServices()
 	if err != nil {
 		return err
 	}
+	snap := env.getSnapshot()
 	for _, service := range services {
-		if err := context.DeleteHostedService(service.ServiceName); err != nil {
+		if err := snap.api.DeleteHostedService(service.ServiceName); err != nil {
 			return err
 		}
 	}
@@ -910,18 +1039,14 @@ func (env *azureEnviron) splitInstanceId(id instance.Id) (service, role string)
 
 // Instances is specified in the Environ interface.
 func (env *azureEnviron) Instances(ids []instance.Id) ([]instance.Instance, error) {
-	context, err := env.getManagementAPI()
-	if err != nil {
-		return nil, err
-	}
-	defer env.releaseManagementAPI(context)
+	snap := env.getSnapshot()
 
 	type instanceId struct {
 		serviceName, roleName string
 	}
 
 	instancesIds := make([]instanceId, len(ids))
-	var serviceNames set.Strings
+	serviceNames := make(set.Strings)
 	for i, id := range ids {
 		serviceName, roleName := env.splitInstanceId(id)
 		if serviceName == "" {
@@ -935,7 +1060,7 @@ func (env *azureEnviron) Instances(ids []instance.Id) ([]instance.Instance, erro
 	}
 
 	// Map service names to gwacl.HostedServices.
-	services, err := context.ListSpecificHostedServices(&gwacl.ListSpecificHostedServicesRequest{
+	services, err := snap.api.ListSpecificHostedServices(&gwacl.ListSpecificHostedServicesRequest{
 		ServiceNames: serviceNames.Values(),
 	})
 	if err != nil {
@@ -946,14 +1071,14 @@ func (env *azureEnviron) Instances(ids []instance.Id) ([]instance.Instance, erro
 	}
 	hostedServices := make(map[string]*gwacl.HostedService)
 	for _, s := range services {
-		hostedService, err := context.GetHostedServiceProperties(s.ServiceName, true)
+		hostedService, err := snap.api.GetHostedServiceProperties(s.ServiceName, true)
 		if err != nil {
 			return nil, err
 		}
 		hostedServices[s.ServiceName] = hostedService
 	}
 
-	err = nil
+	var validInstances int
 	instances := make([]instance.Instance, len(ids))
 	for i, id := range instancesIds {
 		if id.serviceName == "" {
@@ -961,34 +1086,46 @@ func (env *azureEnviron) Instances(ids []instance.Id) ([]instance.Instance, erro
 			continue
 		}
 		hostedService := hostedServices[id.serviceName]
-		instance, err := env.getInstance(hostedService, id.roleName)
+		instance, err := snap.getInstance(hostedService, id.roleName)
 		if err == nil {
 			instances[i] = instance
+			validInstances++
 		} else {
 			logger.Debugf("failed to get instance for role %q in service %q: %v", id.roleName, hostedService.ServiceName, err)
 		}
 	}
-	for _, instance := range instances {
-		if instance == nil {
-			err = environs.ErrPartialInstances
-		}
+
+	switch validInstances {
+	case len(instances):
+		err = nil
+	case 0:
+		instances = nil
+		err = environs.ErrNoInstances
+	default:
+		err = environs.ErrPartialInstances
 	}
 	return instances, err
 }
 
-// AllocateAddress requests a new address to be allocated for the
+// AllocateAddress requests an address to be allocated for the
 // given instance on the given network. This is not implemented on the
 // Azure provider yet.
-func (*azureEnviron) AllocateAddress(_ instance.Id, _ network.Id) (network.Address, error) {
-	return network.Address{}, errors.NotImplementedf("AllocateAddress")
+func (*azureEnviron) AllocateAddress(_ instance.Id, _ network.Id, _ network.Address) error {
+	return errors.NotImplementedf("AllocateAddress")
 }
 
-// ListNetworks returns basic information about all networks known
+// ReleaseAddress releases a specific address previously allocated with
+// AllocateAddress.
+func (*azureEnviron) ReleaseAddress(_ instance.Id, _ network.Id, _ network.Address) error {
+	return errors.NotImplementedf("ReleaseAddress")
+}
+
+// Subnets returns basic information about all subnets known
 // by the provider for the environment. They may be unknown to juju
 // yet (i.e. when called initially or when a new network was created).
 // This is not implemented by the Azure provider yet.
-func (*azureEnviron) ListNetworks() ([]network.BasicInfo, error) {
-	return nil, errors.NotImplementedf("ListNetworks")
+func (*azureEnviron) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
+	return nil, errors.NotImplementedf("Subnets")
 }
 
 // AllInstances is specified in the InstanceBroker interface.
@@ -996,21 +1133,16 @@ func (env *azureEnviron) AllInstances() ([]instance.Instance, error) {
 	// The instance list is built using the list of all the Azure
 	// Services (instance==service).
 	// Acquire management API object.
-	context, err := env.getManagementAPI()
-	if err != nil {
-		return nil, err
-	}
-	defer env.releaseManagementAPI(context)
+	snap := env.getSnapshot()
 
-	request := &gwacl.ListPrefixedHostedServicesRequest{ServiceNamePrefix: env.getEnvPrefix()}
-	serviceDescriptors, err := context.ListPrefixedHostedServices(request)
+	serviceDescriptors, err := env.hostedServices()
 	if err != nil {
 		return nil, err
 	}
 
 	var instances []instance.Instance
 	for _, sd := range serviceDescriptors {
-		hostedService, err := context.GetHostedServiceProperties(sd.ServiceName, true)
+		hostedService, err := snap.api.GetHostedServiceProperties(sd.ServiceName, true)
 		if err != nil {
 			return nil, err
 		} else if len(hostedService.Deployments) != 1 {
@@ -1018,7 +1150,7 @@ func (env *azureEnviron) AllInstances() ([]instance.Instance, error) {
 		}
 		deployment := &hostedService.Deployments[0]
 		for _, role := range deployment.RoleList {
-			instance, err := env.getInstance(hostedService, role.RoleName)
+			instance, err := snap.getInstance(hostedService, role.RoleName)
 			if err != nil {
 				return nil, err
 			}
@@ -1029,9 +1161,10 @@ func (env *azureEnviron) AllInstances() ([]instance.Instance, error) {
 }
 
 // getEnvPrefix returns the prefix used to name the objects specific to this
-// environment.
+// environment. The environment prefix name is immutable, so there is no need
+// to use a configuration snapshot.
 func (env *azureEnviron) getEnvPrefix() string {
-	return fmt.Sprintf("juju-%s-", env.Name())
+	return fmt.Sprintf("juju-%s-", env.Config().Name())
 }
 
 // Storage is specified in the Environ interface.
@@ -1041,7 +1174,7 @@ func (env *azureEnviron) Storage() storage.Storage {
 
 // Destroy is specified in the Environ interface.
 func (env *azureEnviron) Destroy() error {
-	logger.Debugf("destroying environment %q", env.name)
+	logger.Debugf("destroying environment %q", env.Config().Name())
 
 	// Stop all instances.
 	if err := env.destroyAllServices(); err != nil {
@@ -1073,20 +1206,20 @@ func (env *azureEnviron) Destroy() error {
 
 // OpenPorts is specified in the Environ interface. However, Azure does not
 // support the global firewall mode.
-func (env *azureEnviron) OpenPorts(ports []network.Port) error {
+func (env *azureEnviron) OpenPorts(ports []network.PortRange) error {
 	return nil
 }
 
 // ClosePorts is specified in the Environ interface. However, Azure does not
 // support the global firewall mode.
-func (env *azureEnviron) ClosePorts(ports []network.Port) error {
+func (env *azureEnviron) ClosePorts(ports []network.PortRange) error {
 	return nil
 }
 
 // Ports is specified in the Environ interface.
-func (env *azureEnviron) Ports() ([]network.Port, error) {
+func (env *azureEnviron) Ports() ([]network.PortRange, error) {
 	// TODO: implement this.
-	return []network.Port{}, nil
+	return []network.PortRange{}, nil
 }
 
 // Provider is specified in the Environ interface.
@@ -1094,18 +1227,6 @@ func (env *azureEnviron) Provider() environs.EnvironProvider {
 	return azureEnvironProvider{}
 }
 
-// azureManagementContext wraps two things: a gwacl.ManagementAPI (effectively
-// a session on the Azure management API) and a tempCertFile, which keeps track
-// of the temporary certificate file that needs to be deleted once we're done
-// with this particular session.
-// Since it embeds *gwacl.ManagementAPI, you can use it much as if it were a
-// pointer to a ManagementAPI object.  Just don't forget to release it after
-// use.
-type azureManagementContext struct {
-	*gwacl.ManagementAPI
-	certFile *tempCertFile
-}
-
 var (
 	retryPolicy = gwacl.RetryPolicy{
 		NbRetries: 6,
@@ -1118,48 +1239,6 @@ var (
 		Delay: 10 * time.Second}
 )
 
-// getManagementAPI obtains a context object for interfacing with Azure's
-// management API.
-// For now, each invocation just returns a separate object.  This is probably
-// wasteful (each context gets its own SSL connection) and may need optimizing
-// later.
-func (env *azureEnviron) getManagementAPI() (*azureManagementContext, error) {
-	snap := env.getSnapshot()
-	subscription := snap.ecfg.managementSubscriptionId()
-	certData := snap.ecfg.managementCertificate()
-	certFile, err := newTempCertFile([]byte(certData))
-	if err != nil {
-		return nil, err
-	}
-	// After this point, if we need to leave prematurely, we should clean
-	// up that certificate file.
-	location := snap.ecfg.location()
-	mgtAPI, err := gwacl.NewManagementAPIWithRetryPolicy(subscription, certFile.Path(), location, retryPolicy)
-	if err != nil {
-		certFile.Delete()
-		return nil, err
-	}
-	context := azureManagementContext{
-		ManagementAPI: mgtAPI,
-		certFile:      certFile,
-	}
-	return &context, nil
-}
-
-// releaseManagementAPI frees up a context object obtained through
-// getManagementAPI.
-func (env *azureEnviron) releaseManagementAPI(context *azureManagementContext) {
-	// Be tolerant to incomplete context objects, in case we ever get
-	// called during cleanup of a failed attempt to create one.
-	if context == nil || context.certFile == nil {
-		return
-	}
-	// For now, all that needs doing is to delete the temporary certificate
-	// file.  We may do cleverer things later, such as connection pooling
-	// where this method returns a context to the pool.
-	context.certFile.Delete()
-}
-
 // updateStorageAccountKey queries the storage account key, and updates the
 // version cached in env.storageAccountKey.
 //
@@ -1229,37 +1308,8 @@ func (env *azureEnviron) getStorageContext() (*gwacl.StorageContext, error) {
 	return &context, nil
 }
 
-// baseURLs specifies an Azure specific location where we look for simplestreams information.
-// It contains the central databases for the released and daily streams, but this may
-// become more configurable.  This variable is here as a placeholder, but also
-// as an injection point for tests.
-var baseURLs = []string{}
-
-// GetImageSources returns a list of sources which are used to search for simplestreams image metadata.
-func (env *azureEnviron) GetImageSources() ([]simplestreams.DataSource, error) {
-	sources := make([]simplestreams.DataSource, 1+len(baseURLs))
-	sources[0] = storage.NewStorageSimpleStreamsDataSource("cloud storage", env.Storage(), storage.BaseImagesPath)
-	for i, url := range baseURLs {
-		sources[i+1] = simplestreams.NewURLDataSource("Azure base URL", url, utils.VerifySSLHostnames)
-	}
-	return sources, nil
-}
-
-// GetToolsSources returns a list of sources which are used to search for simplestreams tools metadata.
-func (env *azureEnviron) GetToolsSources() ([]simplestreams.DataSource, error) {
-	// Add the simplestreams source off the control bucket.
-	sources := []simplestreams.DataSource{
-		storage.NewStorageSimpleStreamsDataSource("cloud storage", env.Storage(), storage.BaseToolsPath)}
-	return sources, nil
-}
-
-// getImageMetadataSigningRequired returns whether this environment requires
-// image metadata from Simplestreams to be signed.
-func (env *azureEnviron) getImageMetadataSigningRequired() bool {
-	// Hard-coded to true for now.  Once we support custom base URLs,
-	// this may have to change.
-	return true
-}
+// TODO(ericsnow) lp-1398055
+// Implement the ZonedEnviron interface.
 
 // Region is specified in the HasRegion interface.
 func (env *azureEnviron) Region() (simplestreams.CloudSpec, error) {
