@@ -7,7 +7,7 @@
 // The configuration YAML for the testing environment
 // must specify a "state-server" property with a boolean
 // value. If this is true, a state server will be started
-// the first time StateInfo is called on a newly reset environment.
+// when the environment is bootstrapped.
 //
 // The configuration data also accepts a "broken" property
 // of type boolean. If this is non-empty, any operation
@@ -23,7 +23,6 @@
 package dummy
 
 import (
-	"errors"
 	"fmt"
 	"net"
 	"net/http"
@@ -33,19 +32,19 @@ import (
 	"sync"
 	"time"
 
+	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	"github.com/juju/names"
 	"github.com/juju/schema"
 	gitjujutesting "github.com/juju/testing"
 
+	"github.com/juju/juju/agent"
+	"github.com/juju/juju/api"
+	"github.com/juju/juju/apiserver"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
-	"github.com/juju/juju/environs/bootstrap"
+	"github.com/juju/juju/environs/cloudinit"
 	"github.com/juju/juju/environs/config"
-	"github.com/juju/juju/environs/imagemetadata"
-	"github.com/juju/juju/environs/simplestreams"
-	"github.com/juju/juju/environs/storage"
-	"github.com/juju/juju/environs/tools"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/arch"
 	"github.com/juju/juju/mongo"
@@ -53,20 +52,26 @@ import (
 	"github.com/juju/juju/provider"
 	"github.com/juju/juju/provider/common"
 	"github.com/juju/juju/state"
-	"github.com/juju/juju/state/api"
-	"github.com/juju/juju/state/api/params"
-	"github.com/juju/juju/state/apiserver"
+	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/testing"
+	coretools "github.com/juju/juju/tools"
 )
 
 var logger = loggo.GetLogger("juju.provider.dummy")
 
+var transientErrorInjection chan error
+
+const (
+	BootstrapInstanceId = instance.Id("localhost")
+)
+
 // SampleConfig() returns an environment configuration with all required
 // attributes set.
 func SampleConfig() testing.Attrs {
 	return testing.Attrs{
 		"type":                      "dummy",
 		"name":                      "only",
+		"uuid":                      "90168e4c-2f10-4e9c-83c2-feedfacee5a9",
 		"authorized-keys":           testing.FakeAuthKeys,
 		"firewall-mode":             config.FwInstance,
 		"admin-secret":              testing.DefaultMongoPassword,
@@ -81,18 +86,49 @@ func SampleConfig() testing.Attrs {
 
 		"secret":       "pork",
 		"state-server": true,
+		"prefer-ipv6":  true,
 	}
 }
 
+// PatchTransientErrorInjectionChannel sets the transientInjectionError
+// channel which can be used to inject errors into StartInstance for
+// testing purposes
+// The injected errors will use the string received on the channel
+// and the instance's state will eventually go to error, while the
+// received string will appear in the info field of the machine's status
+func PatchTransientErrorInjectionChannel(c chan error) func() {
+	return gitjujutesting.PatchValue(&transientErrorInjection, c)
+}
+
+// AdminUserTag returns the user tag used to bootstrap the dummy environment.
+// The dummy bootstrapping is handled slightly differently, and the user is
+// created as part of the bootstrap process.  This method is used to provide
+// tests a way to get to the user name that was used to initialise the
+// database, and as such, is the owner of the initial environment.
+func AdminUserTag() names.UserTag {
+	return names.NewLocalUserTag("dummy-admin")
+}
+
 // stateInfo returns a *state.Info which allows clients to connect to the
-// shared dummy state, if it exists.
-func stateInfo() *state.Info {
+// shared dummy state, if it exists. If preferIPv6 is true, an IPv6 endpoint
+// will be added as primary.
+func stateInfo(preferIPv6 bool) *mongo.MongoInfo {
 	if gitjujutesting.MgoServer.Addr() == "" {
 		panic("dummy environ state tests must be run with MgoTestPackage")
 	}
-	return &state.Info{
+	mongoPort := strconv.Itoa(gitjujutesting.MgoServer.Port())
+	var addrs []string
+	if preferIPv6 {
+		addrs = []string{
+			net.JoinHostPort("::1", mongoPort),
+			net.JoinHostPort("localhost", mongoPort),
+		}
+	} else {
+		addrs = []string{net.JoinHostPort("localhost", mongoPort)}
+	}
+	return &mongo.MongoInfo{
 		Info: mongo.Info{
-			Addrs:  []string{gitjujutesting.MgoServer.Addr()},
+			Addrs:  addrs,
 			CACert: testing.CACert,
 		},
 	}
@@ -107,6 +143,12 @@ type OpBootstrap struct {
 	Args    environs.BootstrapParams
 }
 
+type OpFinalizeBootstrap struct {
+	Context       environs.BootstrapContext
+	Env           string
+	MachineConfig *cloudinit.MachineConfig
+}
+
 type OpDestroy struct {
 	Env   string
 	Error error
@@ -119,23 +161,32 @@ type OpAllocateAddress struct {
 	Address    network.Address
 }
 
+type OpReleaseAddress struct {
+	Env        string
+	InstanceId instance.Id
+	NetworkId  network.Id
+	Address    network.Address
+}
+
 type OpListNetworks struct {
 	Env  string
-	Info []network.BasicInfo
+	Info []network.SubnetInfo
 }
 
 type OpStartInstance struct {
-	Env          string
-	MachineId    string
-	MachineNonce string
-	Instance     instance.Instance
-	Constraints  constraints.Value
-	Networks     []string
-	NetworkInfo  []network.Info
-	Info         *state.Info
-	Jobs         []params.MachineJob
-	APIInfo      *api.Info
-	Secret       string
+	Env              string
+	MachineId        string
+	MachineNonce     string
+	PossibleTools    coretools.List
+	Instance         instance.Instance
+	Constraints      constraints.Value
+	Networks         []string
+	NetworkInfo      []network.Info
+	Info             *mongo.MongoInfo
+	Jobs             []multiwatcher.MachineJob
+	APIInfo          *api.Info
+	Secret           string
+	AgentEnvironment map[string]string
 }
 
 type OpStopInstances struct {
@@ -147,14 +198,14 @@ type OpOpenPorts struct {
 	Env        string
 	MachineId  string
 	InstanceId instance.Id
-	Ports      []network.Port
+	Ports      []network.PortRange
 }
 
 type OpClosePorts struct {
 	Env        string
 	MachineId  string
 	InstanceId instance.Id
-	Ports      []network.Port
+	Ports      []network.PortRange
 }
 
 type OpPutFile struct {
@@ -189,13 +240,15 @@ type environState struct {
 	maxId        int // maximum instance id allocated so far.
 	maxAddr      int // maximum allocated address last byte
 	insts        map[instance.Id]*dummyInstance
-	globalPorts  map[network.Port]bool
+	globalPorts  map[network.PortRange]bool
 	bootstrapped bool
 	storageDelay time.Duration
 	storage      *storageServer
+	apiListener  net.Listener
 	httpListener net.Listener
 	apiServer    *apiserver.Server
 	apiState     *state.State
+	preferIPv6   bool
 }
 
 // environ represents a client's connection to a given environment's
@@ -208,8 +261,6 @@ type environ struct {
 	ecfgUnlocked *environConfig
 }
 
-var _ imagemetadata.SupportsCustomSources = (*environ)(nil)
-var _ tools.SupportsCustomSources = (*environ)(nil)
 var _ environs.Environ = (*environ)(nil)
 
 // discardOperations discards all Operations written to it.
@@ -243,6 +294,9 @@ func Reset() {
 	providerInstance.ops = discardOperations
 	for _, s := range p.state {
 		s.httpListener.Close()
+		if s.apiListener != nil {
+			s.apiListener.Close()
+		}
 		s.destroy()
 	}
 	providerInstance.state = make(map[int]*environState)
@@ -301,17 +355,17 @@ func newState(name string, ops chan<- Operation, policy state.Policy) *environSt
 		ops:         ops,
 		statePolicy: policy,
 		insts:       make(map[instance.Id]*dummyInstance),
-		globalPorts: make(map[network.Port]bool),
+		globalPorts: make(map[network.PortRange]bool),
 	}
 	s.storage = newStorageServer(s, "/"+name+"/private")
-	s.listen()
+	s.listenStorage()
 	return s
 }
 
-// listen starts a network listener listening for http
+// listenStorage starts a network listener listening for http
 // requests to retrieve files in the state's storage.
-func (s *environState) listen() {
-	l, err := net.Listen("tcp", "127.0.0.1:0")
+func (s *environState) listenStorage() {
+	l, err := net.Listen("tcp", ":0")
 	if err != nil {
 		panic(fmt.Errorf("cannot start listener: %v", err))
 	}
@@ -321,6 +375,17 @@ func (s *environState) listen() {
 	go http.Serve(l, mux)
 }
 
+// listenAPI starts a network listener listening for API
+// connections and proxies them to the API server port.
+func (s *environState) listenAPI() int {
+	l, err := net.Listen("tcp", ":0")
+	if err != nil {
+		panic(fmt.Errorf("cannot start listener: %v", err))
+	}
+	s.apiListener = l
+	return l.Addr().(*net.TCPAddr).Port
+}
+
 // SetStatePolicy sets the state.Policy to use when a
 // state server is initialised by dummy.
 func SetStatePolicy(policy state.Policy) {
@@ -486,22 +551,23 @@ func (p *environProvider) prepare(cfg *config.Config) (*config.Config, error) {
 	if ecfg.stateId() != noStateId {
 		return cfg, nil
 	}
-	// The environment has not been prepared,
-	// so create it and set its state identifier accordingly.
 	if ecfg.stateServer() && len(p.state) != 0 {
 		for _, old := range p.state {
 			panic(fmt.Errorf("cannot share a state between two dummy environs; old %q; new %q", old.name, name))
 		}
 	}
+	// The environment has not been prepared,
+	// so create it and set its state identifier accordingly.
 	state := newState(name, p.ops, p.statePolicy)
 	p.maxStateId++
 	state.id = p.maxStateId
 	p.state[state.id] = state
-	// Add the state id to the configuration we use to
-	// in the returned environment.
-	return cfg.Apply(map[string]interface{}{
-		"state-id": fmt.Sprint(state.id),
-	})
+
+	attrs := map[string]interface{}{"state-id": fmt.Sprint(state.id)}
+	if ecfg.stateServer() {
+		attrs["api-port"] = state.listenAPI()
+	}
+	return cfg.Apply(attrs)
 }
 
 func (*environProvider) SecretAttrs(cfg *config.Config) (map[string]string, error) {
@@ -545,13 +611,9 @@ func (e *environ) checkBroken(method string) error {
 	return nil
 }
 
-func (e *environ) Name() string {
-	return e.name
-}
-
 // SupportedArchitectures is specified on the EnvironCapability interface.
 func (*environ) SupportedArchitectures() ([]string, error) {
-	return []string{arch.AMD64, arch.I386, arch.PPC64}, nil
+	return []string{arch.AMD64, arch.I386, arch.PPC64EL}, nil
 }
 
 // SupportNetworks is specified on the EnvironCapability interface.
@@ -559,6 +621,11 @@ func (*environ) SupportNetworks() bool {
 	return true
 }
 
+// SupportAddressAllocation is specified on the EnvironCapability interface.
+func (e *environ) SupportAddressAllocation(netId network.Id) (bool, error) {
+	return false, nil
+}
+
 // PrecheckInstance is specified in the state.Prechecker interface.
 func (*environ) PrecheckInstance(series string, cons constraints.Value, placement string) error {
 	if placement != "" && placement != "valid" {
@@ -567,68 +634,70 @@ func (*environ) PrecheckInstance(series string, cons constraints.Value, placemen
 	return nil
 }
 
-// GetImageSources returns a list of sources which are used to search for simplestreams image metadata.
-func (e *environ) GetImageSources() ([]simplestreams.DataSource, error) {
-	return []simplestreams.DataSource{
-		storage.NewStorageSimpleStreamsDataSource("cloud storage", e.Storage(), storage.BaseImagesPath)}, nil
-}
-
-// GetToolsSources returns a list of sources which are used to search for simplestreams tools metadata.
-func (e *environ) GetToolsSources() ([]simplestreams.DataSource, error) {
-	return []simplestreams.DataSource{
-		storage.NewStorageSimpleStreamsDataSource("cloud storage", e.Storage(), storage.BaseToolsPath)}, nil
-}
-
-func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.BootstrapParams) error {
-	selectedTools, err := common.EnsureBootstrapTools(ctx, e, config.PreferredSeries(e.Config()), args.Constraints.Arch)
+func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.BootstrapParams) (arch, series string, _ environs.BootstrapFinalizer, _ error) {
+	series = config.PreferredSeries(e.Config())
+	availableTools, err := args.AvailableTools.Match(coretools.Filter{Series: series})
 	if err != nil {
-		return err
+		return "", "", nil, err
 	}
+	arch = availableTools.Arches()[0]
 
 	defer delay()
 	if err := e.checkBroken("Bootstrap"); err != nil {
-		return err
+		return "", "", nil, err
 	}
+	network.InitializeFromConfig(e.Config())
 	password := e.Config().AdminSecret()
 	if password == "" {
-		return fmt.Errorf("admin-secret is required for bootstrap")
+		return "", "", nil, fmt.Errorf("admin-secret is required for bootstrap")
 	}
 	if _, ok := e.Config().CACert(); !ok {
-		return fmt.Errorf("no CA certificate in environment configuration")
+		return "", "", nil, fmt.Errorf("no CA certificate in environment configuration")
 	}
 
-	logger.Infof("would pick tools from %s", selectedTools)
+	logger.Infof("would pick tools from %s", availableTools)
 	cfg, err := environs.BootstrapConfig(e.Config())
 	if err != nil {
-		return fmt.Errorf("cannot make bootstrap config: %v", err)
+		return "", "", nil, fmt.Errorf("cannot make bootstrap config: %v", err)
 	}
 
 	estate, err := e.state()
 	if err != nil {
-		return err
+		return "", "", nil, err
 	}
 	estate.mu.Lock()
 	defer estate.mu.Unlock()
 	if estate.bootstrapped {
-		return fmt.Errorf("environment is already bootstrapped")
+		return "", "", nil, fmt.Errorf("environment is already bootstrapped")
 	}
-	// Write the bootstrap file just like a normal provider. However
-	// we need to release the mutex for the save state to work, so regain
-	// it after the call.
-	estate.mu.Unlock()
-	if err := bootstrap.SaveState(e.Storage(), &bootstrap.BootstrapState{StateInstances: []instance.Id{"localhost"}}); err != nil {
-		logger.Errorf("failed to save state instances: %v", err)
-		estate.mu.Lock() // otherwise defered unlock will fail
-		return err
+	estate.preferIPv6 = e.Config().PreferIPv6()
+
+	// Create an instance for the bootstrap node.
+	logger.Infof("creating bootstrap instance")
+	i := &dummyInstance{
+		id:           BootstrapInstanceId,
+		addresses:    network.NewAddresses("localhost"),
+		ports:        make(map[network.PortRange]bool),
+		machineId:    agent.BootstrapMachineId,
+		series:       series,
+		firewallMode: e.Config().FirewallMode(),
+		state:        estate,
+		stateServer:  true,
 	}
-	estate.mu.Lock() // back at it
+	estate.insts[i.id] = i
 
 	if e.ecfg().stateServer() {
 		// TODO(rog) factor out relevant code from cmd/jujud/bootstrap.go
 		// so that we can call it here.
 
-		info := stateInfo()
-		st, err := state.Initialize(info, cfg, mongo.DefaultDialOpts(), estate.statePolicy)
+		info := stateInfo(estate.preferIPv6)
+		// Since the admin user isn't setup until after here,
+		// the password in the info structure is empty, so the admin
+		// user is constructed with an empty password here.
+		// It is set just below.
+		st, err := state.Initialize(
+			AdminUserTag(), info, cfg,
+			mongo.DefaultDialOpts(), estate.statePolicy)
 		if err != nil {
 			panic(err)
 		}
@@ -638,14 +707,27 @@ func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.Bootstr
 		if err := st.SetAdminMongoPassword(password); err != nil {
 			panic(err)
 		}
-		_, err = st.AddAdminUser(password)
+		if err := st.MongoSession().DB("admin").Login("admin", password); err != nil {
+			panic(err)
+		}
+		env, err := st.Environment()
 		if err != nil {
 			panic(err)
 		}
-		estate.apiServer, err = apiserver.NewServer(st, apiserver.ServerConfig{
-			Addr:    "localhost:0",
+		owner, err := st.User(env.Owner())
+		if err != nil {
+			panic(err)
+		}
+		// We log this out for test purposes only. No one in real life can use
+		// a dummy provider for anything other than testing, so logging the password
+		// here is fine.
+		logger.Debugf("setting password for %q to %q", owner.Name(), password)
+		owner.SetPassword(password)
+
+		estate.apiServer, err = apiserver.NewServer(st, estate.apiListener, apiserver.ServerConfig{
 			Cert:    []byte(testing.ServerCert),
 			Key:     []byte(testing.ServerKey),
+			Tag:     names.NewMachineTag("0"),
 			DataDir: DataDir,
 			LogDir:  LogDir,
 		})
@@ -656,29 +738,33 @@ func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.Bootstr
 	}
 	estate.bootstrapped = true
 	estate.ops <- OpBootstrap{Context: ctx, Env: e.name, Args: args}
-	return nil
+	finalize := func(ctx environs.BootstrapContext, mcfg *cloudinit.MachineConfig) error {
+		estate.ops <- OpFinalizeBootstrap{Context: ctx, Env: e.name, MachineConfig: mcfg}
+		return nil
+	}
+	return arch, series, finalize, nil
 }
 
-func (e *environ) StateInfo() (*state.Info, *api.Info, error) {
+func (e *environ) StateServerInstances() ([]instance.Id, error) {
 	estate, err := e.state()
 	if err != nil {
-		return nil, nil, err
+		return nil, err
 	}
 	estate.mu.Lock()
 	defer estate.mu.Unlock()
-	if err := e.checkBroken("StateInfo"); err != nil {
-		return nil, nil, err
-	}
-	if !e.ecfg().stateServer() {
-		return nil, nil, errors.New("dummy environment has no state configured")
+	if err := e.checkBroken("StateServerInstances"); err != nil {
+		return nil, err
 	}
 	if !estate.bootstrapped {
-		return nil, nil, environs.ErrNotBootstrapped
+		return nil, environs.ErrNotBootstrapped
 	}
-	return stateInfo(), &api.Info{
-		Addrs:  []string{estate.apiServer.Addr()},
-		CACert: testing.CACert,
-	}, nil
+	var stateServerInstances []instance.Id
+	for _, v := range estate.insts {
+		if v.stateServer {
+			stateServerInstances = append(stateServerInstances, v.Id())
+		}
+	}
+	return stateServerInstances, nil
 }
 
 func (e *environ) Config() *config.Config {
@@ -732,40 +818,53 @@ func (e *environ) ConstraintsValidator() (constraints.Validator, error) {
 }
 
 // StartInstance is specified in the InstanceBroker interface.
-func (e *environ) StartInstance(args environs.StartInstanceParams) (instance.Instance, *instance.HardwareCharacteristics, []network.Info, error) {
+func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.StartInstanceResult, error) {
 
 	defer delay()
 	machineId := args.MachineConfig.MachineId
 	logger.Infof("dummy startinstance, machine %s", machineId)
 	if err := e.checkBroken("StartInstance"); err != nil {
-		return nil, nil, nil, err
+		return nil, err
 	}
 	estate, err := e.state()
 	if err != nil {
-		return nil, nil, nil, err
+		return nil, err
 	}
 	estate.mu.Lock()
 	defer estate.mu.Unlock()
+
+	// check if an error has been injected on the transientErrorInjection channel (testing purposes)
+	select {
+	case injectedError := <-transientErrorInjection:
+		return nil, injectedError
+	default:
+	}
+
 	if args.MachineConfig.MachineNonce == "" {
-		return nil, nil, nil, fmt.Errorf("cannot start instance: missing machine nonce")
+		return nil, errors.New("cannot start instance: missing machine nonce")
 	}
 	if _, ok := e.Config().CACert(); !ok {
-		return nil, nil, nil, fmt.Errorf("no CA certificate in environment configuration")
+		return nil, errors.New("no CA certificate in environment configuration")
 	}
-	if args.MachineConfig.StateInfo.Tag != names.NewMachineTag(machineId).String() {
-		return nil, nil, nil, fmt.Errorf("entity tag must match started machine")
+	if args.MachineConfig.MongoInfo.Tag != names.NewMachineTag(machineId) {
+		return nil, errors.New("entity tag must match started machine")
 	}
-	if args.MachineConfig.APIInfo.Tag != names.NewMachineTag(machineId).String() {
-		return nil, nil, nil, fmt.Errorf("entity tag must match started machine")
+	if args.MachineConfig.APIInfo.Tag != names.NewMachineTag(machineId) {
+		return nil, errors.New("entity tag must match started machine")
 	}
 	logger.Infof("would pick tools from %s", args.Tools)
 	series := args.Tools.OneSeries()
 
 	idString := fmt.Sprintf("%s-%d", e.name, estate.maxId)
+	addrs := network.NewAddresses(idString+".dns", "127.0.0.1")
+	if estate.preferIPv6 {
+		addrs = append(addrs, network.NewAddress(fmt.Sprintf("fc00::%x", estate.maxId+1), network.ScopeUnknown))
+	}
+	logger.Debugf("StartInstance addresses: %v", addrs)
 	i := &dummyInstance{
 		id:           instance.Id(idString),
-		addresses:    network.NewAddresses(idString + ".dns"),
-		ports:        make(map[network.Port]bool),
+		addresses:    addrs,
+		ports:        make(map[network.PortRange]bool),
 		machineId:    machineId,
 		series:       series,
 		firewallMode: e.Config().FirewallMode(),
@@ -829,19 +928,25 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (instance.Ins
 	estate.insts[i.id] = i
 	estate.maxId++
 	estate.ops <- OpStartInstance{
-		Env:          e.name,
-		MachineId:    machineId,
-		MachineNonce: args.MachineConfig.MachineNonce,
-		Constraints:  args.Constraints,
-		Networks:     args.MachineConfig.Networks,
-		NetworkInfo:  networkInfo,
-		Jobs:         args.MachineConfig.Jobs,
-		Instance:     i,
-		Info:         args.MachineConfig.StateInfo,
-		APIInfo:      args.MachineConfig.APIInfo,
-		Secret:       e.ecfg().secret(),
+		Env:              e.name,
+		MachineId:        machineId,
+		MachineNonce:     args.MachineConfig.MachineNonce,
+		PossibleTools:    args.Tools,
+		Constraints:      args.Constraints,
+		Networks:         args.MachineConfig.Networks,
+		NetworkInfo:      networkInfo,
+		Instance:         i,
+		Jobs:             args.MachineConfig.Jobs,
+		Info:             args.MachineConfig.MongoInfo,
+		APIInfo:          args.MachineConfig.APIInfo,
+		AgentEnvironment: args.MachineConfig.AgentEnvironment,
+		Secret:           e.ecfg().secret(),
 	}
-	return i, hc, networkInfo, nil
+	return &environs.StartInstanceResult{
+		Instance:    i,
+		Hardware:    hc,
+		NetworkInfo: networkInfo,
+	}, nil
 }
 
 func (e *environ) StopInstances(ids ...instance.Id) error {
@@ -896,40 +1001,55 @@ func (e *environ) Instances(ids []instance.Id) (insts []instance.Instance, err e
 	return
 }
 
-// AllocateAddress requests a new address to be allocated for the
+// AllocateAddress requests an address to be allocated for the
 // given instance on the given network.
-func (env *environ) AllocateAddress(instId instance.Id, netId network.Id) (network.Address, error) {
+func (env *environ) AllocateAddress(instId instance.Id, netId network.Id, addr network.Address) error {
 	if err := env.checkBroken("AllocateAddress"); err != nil {
-		return network.Address{}, err
+		return err
 	}
 
 	estate, err := env.state()
 	if err != nil {
-		return network.Address{}, err
+		return err
 	}
 	estate.mu.Lock()
 	defer estate.mu.Unlock()
 	estate.maxAddr++
-	// TODO(dimitern) Once we have integrated networks
-	// and addresses, make sure we return a valid address
-	// for the given network, and we also have the network
-	// already registered.
-	newAddress := network.NewAddress(
-		fmt.Sprintf("0.1.2.%d", estate.maxAddr),
-		network.ScopeCloudLocal,
-	)
 	estate.ops <- OpAllocateAddress{
 		Env:        env.name,
 		InstanceId: instId,
 		NetworkId:  netId,
-		Address:    newAddress,
+		Address:    addr,
 	}
-	return newAddress, nil
+	return nil
 }
 
-// ListNetworks implements environs.Environ.ListNetworks.
-func (env *environ) ListNetworks() ([]network.BasicInfo, error) {
-	if err := env.checkBroken("ListNetworks"); err != nil {
+// ReleaseAddress releases a specific address previously allocated with
+// AllocateAddress.
+func (env *environ) ReleaseAddress(instId instance.Id, netId network.Id, addr network.Address) error {
+	if err := env.checkBroken("ReleaseAddress"); err != nil {
+		return err
+	}
+
+	estate, err := env.state()
+	if err != nil {
+		return err
+	}
+	estate.mu.Lock()
+	defer estate.mu.Unlock()
+	estate.maxAddr++
+	estate.ops <- OpReleaseAddress{
+		Env:        env.name,
+		InstanceId: instId,
+		NetworkId:  netId,
+		Address:    addr,
+	}
+	return nil
+}
+
+// Subnets implements environs.Environ.Subnets.
+func (env *environ) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
+	if err := env.checkBroken("Subnets"); err != nil {
 		return nil, err
 	}
 
@@ -940,7 +1060,7 @@ func (env *environ) ListNetworks() ([]network.BasicInfo, error) {
 	estate.mu.Lock()
 	defer estate.mu.Unlock()
 
-	netInfo := []network.BasicInfo{
+	netInfo := []network.SubnetInfo{
 		{CIDR: "0.10.0.0/8", ProviderId: "dummy-private"},
 		{CIDR: "0.20.0.0/24", ProviderId: "dummy-public"},
 	}
@@ -969,7 +1089,7 @@ func (e *environ) AllInstances() ([]instance.Instance, error) {
 	return insts, nil
 }
 
-func (e *environ) OpenPorts(ports []network.Port) error {
+func (e *environ) OpenPorts(ports []network.PortRange) error {
 	if mode := e.ecfg().FirewallMode(); mode != config.FwGlobal {
 		return fmt.Errorf("invalid firewall mode %q for opening ports on environment", mode)
 	}
@@ -985,7 +1105,7 @@ func (e *environ) OpenPorts(ports []network.Port) error {
 	return nil
 }
 
-func (e *environ) ClosePorts(ports []network.Port) error {
+func (e *environ) ClosePorts(ports []network.PortRange) error {
 	if mode := e.ecfg().FirewallMode(); mode != config.FwGlobal {
 		return fmt.Errorf("invalid firewall mode %q for closing ports on environment", mode)
 	}
@@ -1001,7 +1121,7 @@ func (e *environ) ClosePorts(ports []network.Port) error {
 	return nil
 }
 
-func (e *environ) Ports() (ports []network.Port, err error) {
+func (e *environ) Ports() (ports []network.PortRange, err error) {
 	if mode := e.ecfg().FirewallMode(); mode != config.FwGlobal {
 		return nil, fmt.Errorf("invalid firewall mode %q for retrieving ports from environment", mode)
 	}
@@ -1014,7 +1134,7 @@ func (e *environ) Ports() (ports []network.Port, err error) {
 	for p := range estate.globalPorts {
 		ports = append(ports, p)
 	}
-	network.SortPorts(ports)
+	network.SortPortRanges(ports)
 	return
 }
 
@@ -1024,12 +1144,13 @@ func (*environ) Provider() environs.EnvironProvider {
 
 type dummyInstance struct {
 	state        *environState
-	ports        map[network.Port]bool
+	ports        map[network.PortRange]bool
 	id           instance.Id
 	status       string
 	machineId    string
 	series       string
 	firewallMode string
+	stateServer  bool
 
 	mu        sync.Mutex
 	addresses []network.Address
@@ -1049,6 +1170,7 @@ func SetInstanceAddresses(inst instance.Instance, addrs []network.Address) {
 	inst0 := inst.(*dummyInstance)
 	inst0.mu.Lock()
 	inst0.addresses = append(inst0.addresses[:0], addrs...)
+	logger.Debugf("setting instance %q addresses to %v", inst0.Id(), addrs)
 	inst0.mu.Unlock()
 }
 
@@ -1071,7 +1193,7 @@ func (inst *dummyInstance) Addresses() ([]network.Address, error) {
 	return append([]network.Address{}, inst.addresses...), nil
 }
 
-func (inst *dummyInstance) OpenPorts(machineId string, ports []network.Port) error {
+func (inst *dummyInstance) OpenPorts(machineId string, ports []network.PortRange) error {
 	defer delay()
 	logger.Infof("openPorts %s, %#v", machineId, ports)
 	if inst.firewallMode != config.FwInstance {
@@ -1095,7 +1217,7 @@ func (inst *dummyInstance) OpenPorts(machineId string, ports []network.Port) err
 	return nil
 }
 
-func (inst *dummyInstance) ClosePorts(machineId string, ports []network.Port) error {
+func (inst *dummyInstance) ClosePorts(machineId string, ports []network.PortRange) error {
 	defer delay()
 	if inst.firewallMode != config.FwInstance {
 		return fmt.Errorf("invalid firewall mode %q for closing ports on instance",
@@ -1118,7 +1240,7 @@ func (inst *dummyInstance) ClosePorts(machineId string, ports []network.Port) er
 	return nil
 }
 
-func (inst *dummyInstance) Ports(machineId string) (ports []network.Port, err error) {
+func (inst *dummyInstance) Ports(machineId string) (ports []network.PortRange, err error) {
 	defer delay()
 	if inst.firewallMode != config.FwInstance {
 		return nil, fmt.Errorf("invalid firewall mode %q for retrieving ports from instance",
@@ -1132,7 +1254,7 @@ func (inst *dummyInstance) Ports(machineId string) (ports []network.Port, err er
 	for p := range inst.ports {
 		ports = append(ports, p)
 	}
-	network.SortPorts(ports)
+	network.SortPortRanges(ports)
 	return
 }
 
