@@ -10,15 +10,14 @@ import (
 	"strings"
 
 	jc "github.com/juju/testing/checkers"
+	gc "gopkg.in/check.v1"
 	amzec2 "launchpad.net/goamz/ec2"
-	gc "launchpad.net/gocheck"
 
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/environs/jujutest"
 	"github.com/juju/juju/environs/storage"
-	envtesting "github.com/juju/juju/environs/testing"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/arch"
 	"github.com/juju/juju/juju/testing"
@@ -75,23 +74,14 @@ type LiveTests struct {
 }
 
 func (t *LiveTests) SetUpSuite(c *gc.C) {
+	// Upload arches that ec2 supports; add to this
+	// as ec2 coverage expands.
+	t.UploadArches = []string{arch.AMD64, arch.I386}
 	t.BaseSuite.SetUpSuite(c)
 	t.LiveTests.SetUpSuite(c)
-	// TODO: Share code from jujutest.LiveTests for creating environment
-	e, err := environs.NewFromAttrs(t.TestConfig)
-	c.Assert(err, gc.IsNil)
-
-	// Put some fake tools in place so that tests that are simply
-	// starting instances without any need to check if those instances
-	// are running will find them in the public bucket.
-	envtesting.UploadFakeTools(c, e.Storage())
 }
 
 func (t *LiveTests) TearDownSuite(c *gc.C) {
-	if t.Env == nil {
-		// This can happen if SetUpSuite fails.
-		return
-	}
 	t.LiveTests.TearDownSuite(c)
 	t.BaseSuite.TearDownSuite(c)
 }
@@ -99,12 +89,11 @@ func (t *LiveTests) TearDownSuite(c *gc.C) {
 func (t *LiveTests) SetUpTest(c *gc.C) {
 	t.BaseSuite.SetUpTest(c)
 	t.LiveTests.SetUpTest(c)
-	t.PatchValue(&version.Current, version.Binary{
+	t.BaseSuite.PatchValue(&version.Current, version.Binary{
 		Number: version.Current.Number,
 		Series: coretesting.FakeDefaultSeries,
 		Arch:   arch.AMD64,
 	})
-
 }
 
 func (t *LiveTests) TearDownTest(c *gc.C) {
@@ -115,6 +104,7 @@ func (t *LiveTests) TearDownTest(c *gc.C) {
 // TODO(niemeyer): Looks like many of those tests should be moved to jujutest.LiveTests.
 
 func (t *LiveTests) TestInstanceAttributes(c *gc.C) {
+	t.PrepareOnce(c)
 	inst, hc := testing.AssertStartInstance(c, t.Env, "30")
 	defer t.Env.StopInstances(inst.Id())
 	// Sanity check for hardware characteristics.
@@ -125,19 +115,20 @@ func (t *LiveTests) TestInstanceAttributes(c *gc.C) {
 	c.Assert(hc.CpuPower, gc.NotNil)
 	addresses, err := jujutesting.WaitInstanceAddresses(t.Env, inst.Id())
 	// TODO(niemeyer): This assert sometimes fails with "no instances found"
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(addresses, gc.Not(gc.HasLen), 0)
 
 	insts, err := t.Env.Instances([]instance.Id{inst.Id()})
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(len(insts), gc.Equals, 1)
 
 	ec2inst := ec2.InstanceEC2(insts[0])
-	c.Assert(ec2inst.DNSName, gc.Equals, addresses[0].Value)
+	c.Assert(ec2inst.IPAddress, gc.Equals, addresses[0].Value)
 	c.Assert(ec2inst.InstanceType, gc.Equals, "m1.small")
 }
 
 func (t *LiveTests) TestStartInstanceConstraints(c *gc.C) {
+	t.PrepareOnce(c)
 	cons := constraints.MustParse("mem=2G")
 	inst, hc := testing.AssertStartInstanceWithConstraints(c, t.Env, "30", cons)
 	defer t.Env.StopInstances(inst.Id())
@@ -151,7 +142,12 @@ func (t *LiveTests) TestStartInstanceConstraints(c *gc.C) {
 }
 
 func (t *LiveTests) TestInstanceGroups(c *gc.C) {
-	t.PrepareOnce(c)
+	t.BootstrapOnce(c)
+	allInsts, err := t.Env.AllInstances()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(allInsts, gc.HasLen, 1) // bootstrap instance
+	bootstrapInstId := allInsts[0].Id()
+
 	ec2conn := ec2.EnvironEC2(t.Env)
 
 	groups := amzec2.SecurityGroupNames(
@@ -169,7 +165,7 @@ func (t *LiveTests) TestInstanceGroups(c *gc.C) {
 	// Add two permissions: one is required and should be left alone;
 	// the other is not and should be deleted.
 	// N.B. this is unfortunately sensitive to the actual set of permissions used.
-	_, err := ec2conn.AuthorizeSecurityGroup(oldJujuGroup,
+	_, err = ec2conn.AuthorizeSecurityGroup(oldJujuGroup,
 		[]amzec2.IPPerm{
 			{
 				Protocol:  "tcp",
@@ -184,7 +180,7 @@ func (t *LiveTests) TestInstanceGroups(c *gc.C) {
 				SourceIPs: []string{"3.4.5.6/32"},
 			},
 		})
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	inst0, _ := testing.AssertStartInstance(c, t.Env, "98")
 	defer t.Env.StopInstances(inst0.Id())
@@ -197,7 +193,7 @@ func (t *LiveTests) TestInstanceGroups(c *gc.C) {
 	defer t.Env.StopInstances(inst1.Id())
 
 	groupsResp, err := ec2conn.SecurityGroups(groups, nil)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(groupsResp.Groups, gc.HasLen, len(groups))
 
 	// For each group, check that it exists and record its id.
@@ -224,9 +220,8 @@ func (t *LiveTests) TestInstanceGroups(c *gc.C) {
 	// that the unneeded permission that we added earlier
 	// has been deleted).
 	perms := info[0].IPPerms
-	c.Assert(perms, gc.HasLen, 6)
+	c.Assert(perms, gc.HasLen, 5)
 	checkPortAllowed(c, perms, 22) // SSH
-	checkPortAllowed(c, perms, coretesting.FakeConfig()["state-port"].(int))
 	checkPortAllowed(c, perms, coretesting.FakeConfig()["api-port"].(int))
 	checkSecurityGroupAllowed(c, perms, groups[0])
 
@@ -235,7 +230,7 @@ func (t *LiveTests) TestInstanceGroups(c *gc.C) {
 
 	// Check that each instance is part of the correct groups.
 	resp, err := ec2conn.Instances([]string{string(inst0.Id()), string(inst1.Id())}, nil)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(resp.Reservations, gc.HasLen, 2)
 	for _, r := range resp.Reservations {
 		c.Assert(r.Instances, gc.HasLen, 1)
@@ -264,25 +259,36 @@ func (t *LiveTests) TestInstanceGroups(c *gc.C) {
 		return ids
 	}
 	insts, err := t.Env.Instances(instIds)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(instIds, jc.SameContents, idsFromInsts(insts))
-	allInsts, err := t.Env.AllInstances()
-	c.Assert(err, gc.IsNil)
+	allInsts, err = t.Env.AllInstances()
+	c.Assert(err, jc.ErrorIsNil)
+	// ignore the bootstrap instance
+	for i, inst := range allInsts {
+		if inst.Id() == bootstrapInstId {
+			if i+1 < len(allInsts) {
+				copy(allInsts[i:], allInsts[i+1:])
+			}
+			allInsts = allInsts[:len(allInsts)-1]
+			break
+		}
+	}
 	c.Assert(instIds, jc.SameContents, idsFromInsts(allInsts))
 }
 
 func (t *LiveTests) TestDestroy(c *gc.C) {
-	s := t.Env.Storage()
+	t.PrepareOnce(c)
+	s := t.Env.(environs.EnvironStorage).Storage()
 	err := s.Put("foo", strings.NewReader("foo"), 3)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	err = s.Put("bar", strings.NewReader("bar"), 3)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	// Check that the bucket exists, so we can be sure
 	// we have checked correctly that it's been destroyed.
 	names, err := storage.List(s, "")
-	c.Assert(err, gc.IsNil)
-	c.Assert(len(names) >= 2, gc.Equals, true)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(len(names) >= 2, jc.IsTrue)
 
 	t.Destroy(c)
 	for a := ec2.ShortAttempt.Start(); a.Next(); {
@@ -336,6 +342,7 @@ func checkSecurityGroupAllowed(c *gc.C, perms []amzec2.IPPerm, g amzec2.Security
 }
 
 func (t *LiveTests) TestStopInstances(c *gc.C) {
+	t.PrepareOnce(c)
 	// It would be nice if this test was in jujutest, but
 	// there's no way for jujutest to fabricate a valid-looking
 	// instance id.
@@ -344,7 +351,7 @@ func (t *LiveTests) TestStopInstances(c *gc.C) {
 	inst2, _ := testing.AssertStartInstance(c, t.Env, "41")
 
 	err := t.Env.StopInstances(inst0.Id(), inst1.Id(), inst2.Id())
-	c.Check(err, gc.IsNil)
+	c.Check(err, jc.ErrorIsNil)
 
 	var insts []instance.Instance
 
@@ -370,6 +377,7 @@ func (t *LiveTests) TestStopInstances(c *gc.C) {
 }
 
 func (t *LiveTests) TestPutBucketOnlyOnce(c *gc.C) {
+	t.PrepareOnce(c)
 	s3inst := ec2.EnvironS3(t.Env)
 	b := s3inst.Bucket("test-once-" + uniqueName)
 	s := ec2.BucketStorage(b)
@@ -381,13 +389,13 @@ func (t *LiveTests) TestPutBucketOnlyOnce(c *gc.C) {
 	// the PutBucket again.
 
 	err := s.Put("test-object", strings.NewReader("test"), 4)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	err = s.Remove("test-object")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	err = ec2.DeleteBucket(s)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	err = s.Put("test-object", strings.NewReader("test"), 4)
 	c.Assert(err, gc.ErrorMatches, ".*The specified bucket does not exist")
@@ -406,12 +414,12 @@ func createGroup(c *gc.C, ec2conn *amzec2.EC2, name, descr string) amzec2.Securi
 
 	// Found duplicate group, so revoke its permissions and return it.
 	gresp, err := ec2conn.SecurityGroups(amzec2.SecurityGroupNames(name), nil)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	gi := gresp.Groups[0]
 	if len(gi.IPPerms) > 0 {
 		_, err = ec2conn.RevokeSecurityGroup(gi.SecurityGroup, gi.IPPerms)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 	}
 	return gi.SecurityGroup
 }
