@@ -17,6 +17,7 @@ import (
 	"github.com/juju/errors"
 	"github.com/juju/utils/proxy"
 	"github.com/juju/utils/shell"
+	"github.com/juju/utils/symlink"
 
 	"github.com/juju/juju/agent"
 	coreCloudinit "github.com/juju/juju/cloudinit"
@@ -25,24 +26,21 @@ import (
 	"github.com/juju/juju/container"
 	"github.com/juju/juju/container/factory"
 	"github.com/juju/juju/environs"
-	"github.com/juju/juju/environs/bootstrap"
 	"github.com/juju/juju/environs/cloudinit"
 	"github.com/juju/juju/environs/config"
 	"github.com/juju/juju/environs/filestorage"
 	"github.com/juju/juju/environs/httpstorage"
-	"github.com/juju/juju/environs/simplestreams"
 	"github.com/juju/juju/environs/storage"
-	envtools "github.com/juju/juju/environs/tools"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/arch"
 	"github.com/juju/juju/juju/osenv"
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/common"
-	"github.com/juju/juju/state"
-	"github.com/juju/juju/state/api"
-	"github.com/juju/juju/state/api/params"
-	"github.com/juju/juju/upstart"
+	servicecommon "github.com/juju/juju/service/common"
+	"github.com/juju/juju/service/upstart"
+	"github.com/juju/juju/state/multiwatcher"
+	"github.com/juju/juju/tools"
 	"github.com/juju/juju/version"
 	"github.com/juju/juju/worker/terminationworker"
 )
@@ -54,9 +52,6 @@ const bootstrapInstanceId instance.Id = "localhost"
 // localEnviron implements Environ.
 var _ environs.Environ = (*localEnviron)(nil)
 
-// localEnviron implements SupportsCustomSources.
-var _ envtools.SupportsCustomSources = (*localEnviron)(nil)
-
 type localEnviron struct {
 	common.SupportsUnitPlacementPolicy
 
@@ -69,13 +64,6 @@ type localEnviron struct {
 	containerManager container.Manager
 }
 
-// GetToolsSources returns a list of sources which are used to search for simplestreams tools metadata.
-func (e *localEnviron) GetToolsSources() ([]simplestreams.DataSource, error) {
-	// Add the simplestreams source off the control bucket.
-	return []simplestreams.DataSource{
-		storage.NewStorageSimpleStreamsDataSource("cloud storage", e.Storage(), storage.BaseToolsPath)}, nil
-}
-
 // SupportedArchitectures is specified on the EnvironCapability interface.
 func (*localEnviron) SupportedArchitectures() ([]string, error) {
 	localArch := arch.HostArch()
@@ -87,6 +75,11 @@ func (*localEnviron) SupportNetworks() bool {
 	return false
 }
 
+// SupportAddressAllocation is specified on the EnvironCapability interface.
+func (e *localEnviron) SupportAddressAllocation(netId network.Id) (bool, error) {
+	return false, nil
+}
+
 func (*localEnviron) PrecheckInstance(series string, cons constraints.Value, placement string) error {
 	if placement != "" {
 		return fmt.Errorf("unknown placement directive: %s", placement)
@@ -94,11 +87,6 @@ func (*localEnviron) PrecheckInstance(series string, cons constraints.Value, pla
 	return nil
 }
 
-// Name is specified in the Environ interface.
-func (env *localEnviron) Name() string {
-	return env.name
-}
-
 func (env *localEnviron) machineAgentServiceName() string {
 	return "juju-agent-" + env.config.namespace()
 }
@@ -111,32 +99,25 @@ func ensureNotRoot() error {
 }
 
 // Bootstrap is specified in the Environ interface.
-func (env *localEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.BootstrapParams) error {
+func (env *localEnviron) Bootstrap(
+	ctx environs.BootstrapContext,
+	args environs.BootstrapParams,
+) (arch, series string, _ environs.BootstrapFinalizer, _ error) {
 	if err := ensureNotRoot(); err != nil {
-		return err
-	}
-	privateKey, err := common.GenerateSystemSSHKey(env)
-	if err != nil {
-		return err
+		return "", "", nil, err
 	}
 
-	// Before we write the agent config file, we need to make sure the
-	// instance is saved in the StateInfo.
-	if err := bootstrap.SaveState(env.Storage(), &bootstrap.BootstrapState{
-		StateInstances: []instance.Id{bootstrapInstanceId},
+	// Make sure there are tools available for the
+	// host's architecture and series.
+	if _, err := args.AvailableTools.Match(tools.Filter{
+		Arch:   version.Current.Arch,
+		Series: version.Current.Series,
 	}); err != nil {
-		logger.Errorf("failed to save state instances: %v", err)
-		return err
+		return "", "", nil, err
 	}
 
-	vers := version.Current
-	selectedTools, err := common.EnsureBootstrapTools(ctx, env, vers.Series, &vers.Arch)
-	if err != nil {
-		return err
-	}
-
-	// Record the bootstrap IP, so the containers know where to go for storage.
 	cfg, err := env.Config().Apply(map[string]interface{}{
+		// Record the bootstrap IP, so the containers know where to go for storage.
 		"bootstrap-ip": env.bridgeAddress,
 	})
 	if err == nil {
@@ -144,35 +125,65 @@ func (env *localEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.
 	}
 	if err != nil {
 		logger.Errorf("failed to apply bootstrap-ip to config: %v", err)
-		return err
+		return "", "", nil, err
 	}
+	return version.Current.Arch, version.Current.Series, env.finishBootstrap, nil
+}
 
-	mcfg := environs.NewBootstrapMachineConfig(privateKey)
+// finishBootstrap converts the machine config to cloud-config,
+// converts that to a script, and then executes it locally.
+func (env *localEnviron) finishBootstrap(ctx environs.BootstrapContext, mcfg *cloudinit.MachineConfig) error {
 	mcfg.InstanceId = bootstrapInstanceId
-	mcfg.Tools = selectedTools[0]
 	mcfg.DataDir = env.config.rootDir()
 	mcfg.LogDir = fmt.Sprintf("/var/log/juju-%s", env.config.namespace())
-	mcfg.Jobs = []params.MachineJob{params.JobManageEnviron}
 	mcfg.CloudInitOutputLog = filepath.Join(mcfg.DataDir, "cloud-init-output.log")
-	mcfg.DisablePackageCommands = true
+
+	// No JobManageNetworking added in order not to change the network
+	// configuration of the user's machine.
+	mcfg.Jobs = []multiwatcher.MachineJob{multiwatcher.JobManageEnviron}
+
 	mcfg.MachineAgentServiceName = env.machineAgentServiceName()
 	mcfg.AgentEnvironment = map[string]string{
 		agent.Namespace:   env.config.namespace(),
 		agent.StorageDir:  env.config.storageDir(),
 		agent.StorageAddr: env.config.storageAddr(),
+		agent.LxcBridge:   env.config.networkBridge(),
 
 		// The local provider only supports a single state server,
 		// so we make the oplog size to a small value. This makes
 		// the preallocation faster with no disadvantage.
 		agent.MongoOplogSize: "1", // 1MB
 	}
-	if err := environs.FinishMachineConfig(mcfg, cfg, args.Constraints); err != nil {
+
+	if err := environs.FinishMachineConfig(mcfg, env.Config()); err != nil {
 		return err
 	}
-	// don't write proxy settings for local machine
+
+	// Since Juju's state machine is currently the host machine
+	// for local providers, don't stomp on it.
+	cfgAttrs := env.config.AllAttrs()
+	if val, ok := cfgAttrs["enable-os-refresh-update"].(bool); !ok {
+		logger.Infof("local provider; disabling refreshing OS updates.")
+		mcfg.EnableOSRefreshUpdate = false
+	} else {
+		mcfg.EnableOSRefreshUpdate = val
+	}
+	if val, ok := cfgAttrs["enable-os-upgrade"].(bool); !ok {
+		logger.Infof("local provider; disabling OS upgrades.")
+		mcfg.EnableOSUpgrade = false
+	} else {
+		mcfg.EnableOSUpgrade = val
+	}
+
+	// don't write proxy or mirror settings for local machine
 	mcfg.AptProxySettings = proxy.Settings{}
 	mcfg.ProxySettings = proxy.Settings{}
+	mcfg.AptMirror = ""
+
 	cloudcfg := coreCloudinit.New()
+	cloudcfg.SetAptUpdate(mcfg.EnableOSRefreshUpdate)
+	cloudcfg.SetAptUpgrade(mcfg.EnableOSUpgrade)
+
 	// Since rsyslogd is restricted by apparmor to only write to /var/log/**
 	// we now provide a symlink to the written file in the local log dir.
 	// Also, we leave the old all-machines.log file in
@@ -182,7 +193,7 @@ func (env *localEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.
 	if err := os.RemoveAll(localLogDir); err != nil {
 		return err
 	}
-	if err := os.Symlink(mcfg.LogDir, localLogDir); err != nil {
+	if err := symlink.New(mcfg.LogDir, localLogDir); err != nil {
 		return err
 	}
 	if err := os.Remove(mcfg.CloudInitOutputLog); err != nil && !os.IsNotExist(err) {
@@ -192,17 +203,18 @@ func (env *localEnviron) Bootstrap(ctx environs.BootstrapContext, args environs.
 		fmt.Sprintf("rm -fr %s", mcfg.LogDir),
 		fmt.Sprintf("rm -f /var/spool/rsyslog/machine-0-%s", env.config.namespace()),
 	)
-	if err := cloudinit.ConfigureJuju(mcfg, cloudcfg); err != nil {
+	udata, err := cloudinit.NewUserdataConfig(mcfg, cloudcfg)
+	if err != nil {
 		return err
 	}
-	return finishBootstrap(mcfg, cloudcfg, ctx)
+	if err := udata.ConfigureJuju(); err != nil {
+		return err
+	}
+	return executeCloudConfig(ctx, mcfg, cloudcfg)
 }
 
-// finishBootstrap converts the machine config to cloud-config,
-// converts that to a script, and then executes it locally.
-//
-// mcfg is supplied for testing purposes.
-var finishBootstrap = func(mcfg *cloudinit.MachineConfig, cloudcfg *coreCloudinit.Config, ctx environs.BootstrapContext) error {
+var executeCloudConfig = func(ctx environs.BootstrapContext, mcfg *cloudinit.MachineConfig, cloudcfg *coreCloudinit.Config) error {
+	// Finally, convert cloud-config to a script and execute it.
 	configScript, err := sshinit.ConfigureScript(cloudcfg)
 	if err != nil {
 		return nil
@@ -215,9 +227,17 @@ var finishBootstrap = func(mcfg *cloudinit.MachineConfig, cloudcfg *coreCloudini
 	return cmd.Run()
 }
 
-// StateInfo is specified in the Environ interface.
-func (env *localEnviron) StateInfo() (*state.Info, *api.Info, error) {
-	return common.StateInfo(env)
+// StateServerInstances is specified in the Environ interface.
+func (env *localEnviron) StateServerInstances() ([]instance.Id, error) {
+	agentsDir := filepath.Join(env.config.rootDir(), "agents")
+	_, err := os.Stat(agentsDir)
+	if os.IsNotExist(err) {
+		return nil, environs.ErrNotBootstrapped
+	}
+	if err != nil {
+		return nil, err
+	}
+	return []instance.Id{bootstrapInstanceId}, nil
 }
 
 // Config is specified in the Environ interface.
@@ -239,15 +259,11 @@ func (env *localEnviron) SetConfig(cfg *config.Config) error {
 	env.config = ecfg
 	env.name = ecfg.Name()
 	containerType := ecfg.container()
-	toolsDir := filepath.Join(env.config.storageDir(), "tools", "releases")
-	if _, err = os.Stat(toolsDir); err != nil {
-		toolsDir = ""
-	}
 	managerConfig := container.ManagerConfig{
-		container.ConfigName:     env.config.namespace(),
-		container.ConfigLogDir:   env.config.logDir(),
-		container.ConfigToolsDir: toolsDir,
+		container.ConfigName:   env.config.namespace(),
+		container.ConfigLogDir: env.config.logDir(),
 	}
+	var imageURLGetter container.ImageURLGetter
 	if containerType == instance.LXC {
 		if useLxcClone, ok := cfg.LXCUseClone(); ok {
 			managerConfig["use-clone"] = fmt.Sprint(useLxcClone)
@@ -255,9 +271,18 @@ func (env *localEnviron) SetConfig(cfg *config.Config) error {
 		if useLxcCloneAufs, ok := cfg.LXCUseCloneAUFS(); ok {
 			managerConfig["use-aufs"] = fmt.Sprint(useLxcCloneAufs)
 		}
+		// For lxc containers, we cache image tarballs in the environment storage, so here
+		// we construct a URL getter.
+		if uuid, ok := ecfg.UUID(); ok {
+			var caCert []byte = nil
+			if cert, ok := cfg.CACert(); ok {
+				caCert = []byte(cert)
+			}
+			imageURLGetter = container.NewImageURLGetter(ecfg.stateServerAddr(), uuid, caCert)
+		}
 	}
 	env.containerManager, err = factory.NewContainerManager(
-		containerType, managerConfig)
+		containerType, managerConfig, imageURLGetter)
 	if err != nil {
 		return err
 	}
@@ -341,30 +366,18 @@ func (env *localEnviron) ConstraintsValidator() (constraints.Validator, error) {
 }
 
 // StartInstance is specified in the InstanceBroker interface.
-func (env *localEnviron) StartInstance(args environs.StartInstanceParams) (instance.Instance, *instance.HardwareCharacteristics, []network.Info, error) {
+func (env *localEnviron) StartInstance(args environs.StartInstanceParams) (*environs.StartInstanceResult, error) {
 	if args.MachineConfig.HasNetworks() {
-		return nil, nil, nil, fmt.Errorf("starting instances with networks is not supported yet.")
+		return nil, fmt.Errorf("starting instances with networks is not supported yet.")
 	}
 	series := args.Tools.OneSeries()
 	logger.Debugf("StartInstance: %q, %s", args.MachineConfig.MachineId, series)
 	args.MachineConfig.Tools = args.Tools[0]
 
-	// For LXC containers we update the tools URL to point to a mounted directory
-	// and the container then copies the tools from there. This is in response
-	// to bug 1357552.
-	if env.config.container() == instance.LXC {
-		storageReleasesDir, err := env.Storage().URL("tools/releases")
-		if err != nil {
-			return nil, nil, nil, err
-		}
-		args.MachineConfig.Tools.URL = strings.Replace(
-			args.MachineConfig.Tools.URL, storageReleasesDir, "file:///tmp/juju/tools", -1)
-	}
-
 	args.MachineConfig.MachineContainerType = env.config.container()
 	logger.Debugf("tools: %#v", args.MachineConfig.Tools)
-	if err := environs.FinishMachineConfig(args.MachineConfig, env.config.Config, args.Constraints); err != nil {
-		return nil, nil, nil, err
+	if err := environs.FinishMachineConfig(args.MachineConfig, env.config.Config); err != nil {
+		return nil, err
 	}
 	// TODO: evaluate the impact of setting the contstraints on the
 	// machineConfig for all machines rather than just state server nodes.
@@ -373,9 +386,12 @@ func (env *localEnviron) StartInstance(args environs.StartInstanceParams) (insta
 	args.MachineConfig.AgentEnvironment[agent.Namespace] = env.config.namespace()
 	inst, hardware, err := createContainer(env, args)
 	if err != nil {
-		return nil, nil, nil, err
+		return nil, err
 	}
-	return inst, hardware, nil, nil
+	return &environs.StartInstanceResult{
+		Instance: inst,
+		Hardware: hardware,
+	}, nil
 }
 
 // Override for testing.
@@ -433,19 +449,25 @@ func (env *localEnviron) Instances(ids []instance.Id) ([]instance.Instance, erro
 	return insts, err
 }
 
-// AllocateAddress requests a new address to be allocated for the
+// AllocateAddress requests an address to be allocated for the
 // given instance on the given network. This is not supported on the
 // local provider.
-func (*localEnviron) AllocateAddress(_ instance.Id, _ network.Id) (network.Address, error) {
-	return network.Address{}, errors.NotSupportedf("AllocateAddress")
+func (*localEnviron) AllocateAddress(_ instance.Id, _ network.Id, _ network.Address) error {
+	return errors.NotSupportedf("AllocateAddress")
 }
 
-// ListNetworks returns basic information about all networks known
+// ReleaseAddress releases a specific address previously allocated with
+// AllocateAddress.
+func (*localEnviron) ReleaseAddress(_ instance.Id, _ network.Id, _ network.Address) error {
+	return errors.NotSupportedf("ReleaseAddress")
+}
+
+// Subnets returns basic information about all subnets known
 // by the provider for the environment. They may be unknown to juju
 // yet (i.e. when called initially or when a new network was created).
 // This is not implemented by the local provider yet.
-func (*localEnviron) ListNetworks() ([]network.BasicInfo, error) {
-	return nil, errors.NotImplementedf("ListNetworks")
+func (*localEnviron) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
+	return nil, errors.NotSupportedf("Subnets")
 }
 
 // AllInstances is specified in the InstanceBroker interface.
@@ -492,7 +514,7 @@ func (env *localEnviron) Destroy() error {
 		}
 		args := []string{
 			"env", osenv.JujuHomeEnvKey + "=" + osenv.JujuHome(),
-			juju, "destroy-environment", "-y", "--force", env.Name(),
+			juju, "destroy-environment", "-y", "--force", env.Config().Name(),
 		}
 		cmd := exec.Command("sudo", args...)
 		cmd.Stdout = os.Stdout
@@ -527,7 +549,7 @@ func (env *localEnviron) Destroy() error {
 	// Stop the mongo database and machine agent. It's possible that the
 	// service doesn't exist or is not running, so don't check the error.
 	mongo.RemoveService(env.config.namespace())
-	upstart.NewService(env.machineAgentServiceName()).StopAndRemove()
+	upstart.NewService(env.machineAgentServiceName(), servicecommon.Conf{}).StopAndRemove()
 
 	// Finally, remove the data-dir.
 	if err := os.RemoveAll(env.config.rootDir()); err != nil && !os.IsNotExist(err) {
@@ -545,17 +567,17 @@ func (env *localEnviron) Destroy() error {
 }
 
 // OpenPorts is specified in the Environ interface.
-func (env *localEnviron) OpenPorts(ports []network.Port) error {
+func (env *localEnviron) OpenPorts(ports []network.PortRange) error {
 	return fmt.Errorf("open ports not implemented")
 }
 
 // ClosePorts is specified in the Environ interface.
-func (env *localEnviron) ClosePorts(ports []network.Port) error {
+func (env *localEnviron) ClosePorts(ports []network.PortRange) error {
 	return fmt.Errorf("close ports not implemented")
 }
 
 // Ports is specified in the Environ interface.
-func (env *localEnviron) Ports() ([]network.Port, error) {
+func (env *localEnviron) Ports() ([]network.PortRange, error) {
 	return nil, nil
 }
 
