@@ -12,12 +12,12 @@ import (
 	"time"
 
 	jc "github.com/juju/testing/checkers"
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
+	"github.com/juju/juju/agent"
 	coreCloudinit "github.com/juju/juju/cloudinit"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/container"
-	"github.com/juju/juju/container/kvm"
 	"github.com/juju/juju/container/lxc"
 	containertesting "github.com/juju/juju/container/testing"
 	"github.com/juju/juju/environs"
@@ -29,12 +29,12 @@ import (
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/arch"
 	"github.com/juju/juju/juju/osenv"
-	"github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/provider/local"
-	"github.com/juju/juju/state/api/params"
-	coretesting "github.com/juju/juju/testing"
-	"github.com/juju/juju/upstart"
+	"github.com/juju/juju/service/common"
+	"github.com/juju/juju/service/upstart"
+	"github.com/juju/juju/state/multiwatcher"
+	coretools "github.com/juju/juju/tools"
 	"github.com/juju/juju/version"
 )
 
@@ -62,39 +62,35 @@ func (*environSuite) TestOpenFailsWithProtectedDirectories(c *gc.C) {
 	testConfig, err := testConfig.Apply(map[string]interface{}{
 		"root-dir": "/usr/lib/juju",
 	})
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	environ, err := local.Provider.Open(testConfig)
 	c.Assert(err, gc.ErrorMatches, "failure setting config: mkdir .* permission denied")
 	c.Assert(environ, gc.IsNil)
 }
 
-func (s *environSuite) TestNameAndStorage(c *gc.C) {
+func (s *environSuite) TestName(c *gc.C) {
 	testConfig := minimalConfig(c)
 	environ, err := local.Provider.Open(testConfig)
-	c.Assert(err, gc.IsNil)
-	c.Assert(environ.Name(), gc.Equals, "test")
-	c.Assert(environ.Storage(), gc.NotNil)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(environ.Config().Name(), gc.Equals, "test")
 }
 
 func (s *environSuite) TestGetToolsMetadataSources(c *gc.C) {
 	testConfig := minimalConfig(c)
 	environ, err := local.Provider.Open(testConfig)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	sources, err := tools.GetMetadataSources(environ)
-	c.Assert(err, gc.IsNil)
-	c.Assert(len(sources), gc.Equals, 1)
-	url, err := sources[0].URL("")
-	c.Assert(err, gc.IsNil)
-	c.Assert(strings.Contains(url, "/tools"), jc.IsTrue)
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(sources, gc.HasLen, 0)
 }
 
 func (*environSuite) TestSupportedArchitectures(c *gc.C) {
 	testConfig := minimalConfig(c)
 	environ, err := local.Provider.Open(testConfig)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	arches, err := environ.SupportedArchitectures()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	for _, a := range arches {
 		c.Assert(arch.IsSupportedArch(a), jc.IsTrue)
 	}
@@ -103,10 +99,20 @@ func (*environSuite) TestSupportedArchitectures(c *gc.C) {
 func (*environSuite) TestSupportNetworks(c *gc.C) {
 	testConfig := minimalConfig(c)
 	environ, err := local.Provider.Open(testConfig)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(environ.SupportNetworks(), jc.IsFalse)
 }
 
+func (*environSuite) TestSupportAddressAllocation(c *gc.C) {
+	testConfig := minimalConfig(c)
+	env, err := local.Provider.Open(testConfig)
+	c.Assert(err, jc.ErrorIsNil)
+
+	result, err := env.SupportAddressAllocation("")
+	c.Assert(result, jc.IsFalse)
+	c.Assert(err, jc.ErrorIsNil)
+}
+
 type localJujuTestSuite struct {
 	baseProviderSuite
 	jujutest.Tests
@@ -119,7 +125,7 @@ func (s *localJujuTestSuite) SetUpTest(c *gc.C) {
 	s.baseProviderSuite.SetUpTest(c)
 	// Construct the directories first.
 	err := local.CreateDirs(c, minimalConfig(c))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	s.testPath = c.MkDir()
 	s.fakesudo = filepath.Join(s.testPath, "sudo")
 	s.PatchEnvPathPrepend(s.testPath)
@@ -128,14 +134,14 @@ func (s *localJujuTestSuite) SetUpTest(c *gc.C) {
 
 	// Write a fake "sudo" which records its args to sudo.args.
 	err = ioutil.WriteFile(s.fakesudo, []byte(echoCommandScript), 0755)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	// Add in an admin secret
 	s.Tests.TestConfig["admin-secret"] = "sekrit"
 	s.PatchValue(local.CheckIfRoot, func() bool { return false })
 	s.Tests.SetUpTest(c)
 
-	s.PatchValue(local.FinishBootstrap, func(mcfg *cloudinit.MachineConfig, cloudcfg *coreCloudinit.Config, ctx environs.BootstrapContext) error {
+	s.PatchValue(local.ExecuteCloudConfig, func(environs.BootstrapContext, *cloudinit.MachineConfig, *coreCloudinit.Config) error {
 		return nil
 	})
 }
@@ -149,7 +155,7 @@ func (s *localJujuTestSuite) MakeTool(c *gc.C, name, script string) {
 	path := filepath.Join(s.testPath, name)
 	script = "#!/bin/bash\n" + script
 	err := ioutil.WriteFile(path, []byte(script), 0755)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *localJujuTestSuite) StoppedStatus(c *gc.C) {
@@ -170,28 +176,66 @@ func (s *localJujuTestSuite) TestStartStop(c *gc.C) {
 	c.Skip("StartInstance not implemented yet.")
 }
 
-func (s *localJujuTestSuite) testBootstrap(c *gc.C, cfg *config.Config) (env environs.Environ) {
-	ctx := coretesting.Context(c)
+func (s *localJujuTestSuite) testBootstrap(c *gc.C, cfg *config.Config) environs.Environ {
+	ctx := envtesting.BootstrapContext(c)
 	environ, err := local.Provider.Prepare(ctx, cfg)
-	c.Assert(err, gc.IsNil)
-	envtesting.UploadFakeTools(c, environ.Storage())
-	defer environ.Storage().RemoveAll()
-	err = environ.Bootstrap(ctx, environs.BootstrapParams{})
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
+	availableTools := coretools.List{&coretools.Tools{
+		Version: version.Current,
+		URL:     "http://testing.invalid/tools.tar.gz",
+	}}
+	_, _, finalizer, err := environ.Bootstrap(ctx, environs.BootstrapParams{
+		AvailableTools: availableTools,
+	})
+	c.Assert(err, jc.ErrorIsNil)
+	mcfg, err := environs.NewBootstrapMachineConfig(constraints.Value{}, "quantal")
+	c.Assert(err, jc.ErrorIsNil)
+	mcfg.Tools = availableTools[0]
+	err = finalizer(ctx, mcfg)
+	c.Assert(err, jc.ErrorIsNil)
 	return environ
 }
 
 func (s *localJujuTestSuite) TestBootstrap(c *gc.C) {
-	s.PatchValue(local.FinishBootstrap, func(mcfg *cloudinit.MachineConfig, cloudcfg *coreCloudinit.Config, ctx environs.BootstrapContext) error {
-		c.Assert(cloudcfg.AptUpdate(), jc.IsFalse)
-		c.Assert(cloudcfg.AptUpgrade(), jc.IsFalse)
-		c.Assert(cloudcfg.Packages(), gc.HasLen, 0)
+
+	minCfg := minimalConfig(c)
+
+	mockFinish := func(ctx environs.BootstrapContext, mcfg *cloudinit.MachineConfig, cloudcfg *coreCloudinit.Config) error {
+
+		envCfgAttrs := minCfg.AllAttrs()
+		if val, ok := envCfgAttrs["enable-os-refresh-update"]; !ok {
+			c.Check(cloudcfg.AptUpdate(), jc.IsFalse)
+		} else {
+			c.Check(cloudcfg.AptUpdate(), gc.Equals, val)
+		}
+
+		if val, ok := envCfgAttrs["enable-os-upgrade"]; !ok {
+			c.Check(cloudcfg.AptUpgrade(), jc.IsFalse)
+		} else {
+			c.Check(cloudcfg.AptUpgrade(), gc.Equals, val)
+		}
+
+		if !mcfg.EnableOSRefreshUpdate {
+			c.Assert(cloudcfg.Packages(), gc.HasLen, 0)
+		}
 		c.Assert(mcfg.AgentEnvironment, gc.Not(gc.IsNil))
+		c.Assert(mcfg.AgentEnvironment[agent.LxcBridge], gc.Not(gc.Equals), "")
 		// local does not allow machine-0 to host units
-		c.Assert(mcfg.Jobs, gc.DeepEquals, []params.MachineJob{params.JobManageEnviron})
+		c.Assert(mcfg.Jobs, gc.DeepEquals, []multiwatcher.MachineJob{multiwatcher.JobManageEnviron})
 		return nil
+	}
+	s.PatchValue(local.ExecuteCloudConfig, mockFinish)
+
+	// Test that defaults are correct.
+	s.testBootstrap(c, minCfg)
+
+	// Test that overrides work.
+	minCfg, err := minCfg.Apply(map[string]interface{}{
+		"enable-os-refresh-update": true,
+		"enable-os-upgrade":        true,
 	})
-	s.testBootstrap(c, minimalConfig(c))
+	c.Assert(err, jc.ErrorIsNil)
+	s.testBootstrap(c, minCfg)
 }
 
 func (s *localJujuTestSuite) TestDestroy(c *gc.C) {
@@ -200,7 +244,7 @@ func (s *localJujuTestSuite) TestDestroy(c *gc.C) {
 	// Succeeds because there's no "agents" directory,
 	// so destroy will just return without attempting
 	// sudo or anything.
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(s.fakesudo+".args", jc.DoesNotExist)
 }
 
@@ -208,16 +252,16 @@ func (s *localJujuTestSuite) makeAgentsDir(c *gc.C, env environs.Environ) {
 	rootDir := env.Config().AllAttrs()["root-dir"].(string)
 	agentsDir := filepath.Join(rootDir, "agents")
 	err := os.Mkdir(agentsDir, 0755)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *localJujuTestSuite) TestDestroyCallSudo(c *gc.C) {
 	env := s.testBootstrap(c, minimalConfig(c))
 	s.makeAgentsDir(c, env)
 	err := env.Destroy()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	data, err := ioutil.ReadFile(s.fakesudo + ".args")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	expected := []string{
 		s.fakesudo,
 		"env",
@@ -238,24 +282,23 @@ func (s *localJujuTestSuite) makeFakeUpstartScripts(c *gc.C, env environs.Enviro
 	s.MakeTool(c, "start", `echo "some-service start/running, process 123"`)
 
 	namespace := env.Config().AllAttrs()["namespace"].(string)
-	mongoService = upstart.NewService(mongo.ServiceName(namespace))
-	mongoConf := upstart.Conf{
-		Service: *mongoService,
-		Desc:    "fake mongo",
-		Cmd:     "echo FAKE",
+	mongoConf := common.Conf{
+		Desc: "fake mongo",
+		Cmd:  "echo FAKE",
 	}
-	err := mongoConf.Install()
-	c.Assert(err, gc.IsNil)
+	mongoService = upstart.NewService(mongo.ServiceName(namespace), mongoConf)
+	err := mongoService.Install()
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(mongoService.Installed(), jc.IsTrue)
 
-	machineAgent = upstart.NewService(fmt.Sprintf("juju-agent-%s", namespace))
-	agentConf := upstart.Conf{
-		Service: *machineAgent,
-		Desc:    "fake agent",
-		Cmd:     "echo FAKE",
+	agentConf := common.Conf{
+		Desc: "fake agent",
+		Cmd:  "echo FAKE",
 	}
-	err = agentConf.Install()
-	c.Assert(err, gc.IsNil)
+	machineAgent = upstart.NewService(fmt.Sprintf("juju-agent-%s", namespace), agentConf)
+
+	err = machineAgent.Install()
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(machineAgent.Installed(), jc.IsTrue)
 
 	return mongoService, machineAgent
@@ -268,7 +311,7 @@ func (s *localJujuTestSuite) TestDestroyRemovesUpstartServices(c *gc.C) {
 	s.PatchValue(local.CheckIfRoot, func() bool { return true })
 
 	err := env.Destroy()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	c.Assert(mongo.Installed(), jc.IsFalse)
 	c.Assert(machineAgent.Installed(), jc.IsFalse)
@@ -284,13 +327,13 @@ func (s *localJujuTestSuite) TestDestroyRemovesContainers(c *gc.C) {
 		container.ConfigName:   namespace,
 		container.ConfigLogDir: "logdir",
 		"use-clone":            "false",
-	})
-	c.Assert(err, gc.IsNil)
+	}, nil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	machine1 := containertesting.CreateContainer(c, manager, "1")
 
 	err = env.Destroy()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	container := s.ContainerFactory.New(string(machine1.Id()))
 	c.Assert(container.IsConstructed(), jc.IsFalse)
@@ -303,13 +346,13 @@ func (s *localJujuTestSuite) TestBootstrapRemoveLeftovers(c *gc.C) {
 	// Create a dir inside local/log that should be removed by Bootstrap.
 	logThings := filepath.Join(rootDir, "log", "things")
 	err := os.MkdirAll(logThings, 0755)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	// Create a cloud-init-output.log in root-dir that should be
 	// removed/truncated by Bootstrap.
 	cloudInitOutputLog := filepath.Join(rootDir, "cloud-init-output.log")
 	err = ioutil.WriteFile(cloudInitOutputLog, []byte("ohai"), 0644)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	s.testBootstrap(c, cfg)
 	c.Assert(logThings, jc.DoesNotExist)
@@ -317,91 +360,23 @@ func (s *localJujuTestSuite) TestBootstrapRemoveLeftovers(c *gc.C) {
 	c.Assert(filepath.Join(rootDir, "log"), jc.IsSymlink)
 }
 
-func (s *localJujuTestSuite) TestToolsURLPatchedForLxc(c *gc.C) {
-	dir := c.MkDir()
-	toolsDir := filepath.Join(dir, "storage", "tools", "releases")
-	err := os.MkdirAll(toolsDir, 0755)
-	c.Assert(err, gc.IsNil)
-	config := localConfig(c, map[string]interface{}{
-		"root-dir":  dir,
-		"container": "lxc",
-	})
-	ctx := coretesting.Context(c)
-	env, err := local.Provider.Prepare(ctx, config)
-	c.Assert(err, gc.IsNil)
-
-	machineId := "1"
-	stateInfo := testing.FakeStateInfo(machineId)
-	apiInfo := testing.FakeAPIInfo(machineId)
-	machineConfig := environs.NewMachineConfig(machineId, "", nil, stateInfo, apiInfo)
-	possibleTools := envtesting.AssertUploadFakeToolsVersions(
-		c, env.Storage(), version.MustParseBinary("5.4.5-precise-amd64"))
-	params := environs.StartInstanceParams{
-		Tools:         possibleTools,
-		MachineConfig: machineConfig,
-	}
-
-	local.PatchCreateContainer(&s.CleanupSuite, c, "file:///tmp/juju/tools/juju-5.4.5-precise-amd64.tgz")
-	inst, _, _, err := env.StartInstance(params)
-	c.Assert(err, gc.IsNil)
-	c.Assert(inst.Id(), gc.Equals, instance.Id("mock"))
-}
-
-func (s *localJujuTestSuite) TestToolsURLNotPatchedForKvm(c *gc.C) {
-	s.PatchValue(&kvm.IsKVMSupported, func() (bool, error) {
-		return true, nil
-	})
-	s.PatchValue(&local.VerifyPrerequisites, func(_ instance.ContainerType) error {
-		return nil
-	})
-	dir := c.MkDir()
-	toolsDir := filepath.Join(dir, "storage", "tools", "releases")
-	err := os.MkdirAll(toolsDir, 0755)
-	c.Assert(err, gc.IsNil)
-	config := localConfig(c, map[string]interface{}{
-		"root-dir":  dir,
-		"container": "kvm",
-	})
-	ctx := coretesting.Context(c)
-	env, err := local.Provider.Prepare(ctx, config)
-	c.Assert(err, gc.IsNil)
-
-	machineId := "1"
-	stateInfo := testing.FakeStateInfo(machineId)
-	apiInfo := testing.FakeAPIInfo(machineId)
-	machineConfig := environs.NewMachineConfig(machineId, "", nil, stateInfo, apiInfo)
-	possibleTools := envtesting.AssertUploadFakeToolsVersions(
-		c, env.Storage(), version.MustParseBinary("5.4.5-precise-amd64"))
-	params := environs.StartInstanceParams{
-		Tools:         possibleTools,
-		MachineConfig: machineConfig,
-	}
-
-	url, err := env.Storage().URL("tools/releases/juju-5.4.5-precise-amd64.tgz")
-	c.Assert(err, gc.IsNil)
-	local.PatchCreateContainer(&s.CleanupSuite, c, url)
-	inst, _, _, err := env.StartInstance(params)
-	c.Assert(err, gc.IsNil)
-	c.Assert(inst.Id(), gc.Equals, instance.Id("mock"))
-}
-
 func (s *localJujuTestSuite) TestConstraintsValidator(c *gc.C) {
-	ctx := coretesting.Context(c)
+	ctx := envtesting.BootstrapContext(c)
 	env, err := local.Provider.Prepare(ctx, minimalConfig(c))
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	validator, err := env.ConstraintsValidator()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	hostArch := arch.HostArch()
 	cons := constraints.MustParse(fmt.Sprintf("arch=%s instance-type=foo tags=bar cpu-power=10 cpu-cores=2", hostArch))
 	unsupported, err := validator.Validate(cons)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(unsupported, jc.SameContents, []string{"cpu-cores", "cpu-power", "instance-type", "tags"})
 }
 
 func (s *localJujuTestSuite) TestConstraintsValidatorVocab(c *gc.C) {
 	env := s.Prepare(c)
 	validator, err := env.ConstraintsValidator()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	hostArch := arch.HostArch()
 	var invalidArch string
@@ -415,3 +390,16 @@ func (s *localJujuTestSuite) TestConstraintsValidatorVocab(c *gc.C) {
 	_, err = validator.Validate(cons)
 	c.Assert(err, gc.ErrorMatches, "invalid constraint value: arch="+invalidArch+"\nvalid values are:.*")
 }
+
+func (s *localJujuTestSuite) TestStateServerInstances(c *gc.C) {
+	env := s.testBootstrap(c, minimalConfig(c))
+
+	instances, err := env.StateServerInstances()
+	c.Assert(err, gc.Equals, environs.ErrNotBootstrapped)
+	c.Assert(instances, gc.HasLen, 0)
+
+	s.makeAgentsDir(c, env)
+	instances, err = env.StateServerInstances()
+	c.Assert(err, jc.ErrorIsNil)
+	c.Assert(instances, gc.DeepEquals, []instance.Id{"localhost"})
+}
