@@ -133,6 +133,11 @@ func (mi *maasInstance) architecture() (arch, subarch string, err error) {
 	return arch, subarch, nil
 }
 
+func (mi *maasInstance) zone() string {
+	zone, _ := mi.getMaasObject().GetField("zone")
+	return zone
+}
+
 func (mi *maasInstance) cpuCount() (uint64, error) {
 	count, err := mi.getMaasObject().GetMap()["cpu_count"].GetFloat64()
 	if err != nil {
@@ -182,10 +187,12 @@ func (mi *maasInstance) hardwareCharacteristics() (*instance.HardwareCharacteris
 	if err != nil {
 		return nil, errors.Annotate(err, "error determining available memory")
 	}
+	zone := mi.zone()
 	hc := &instance.HardwareCharacteristics{
-		Arch:     &nodeArch,
-		CpuCores: &nodeCpuCount,
-		Mem:      &nodeMemoryMB,
+		Arch:             &nodeArch,
+		CpuCores:         &nodeCpuCount,
+		Mem:              &nodeMemoryMB,
+		AvailabilityZone: &zone,
 	}
 	nodeTags, err := mi.tagNames()
 	if err != nil && !errors.IsNotFound(err) {
@@ -203,17 +210,17 @@ func (mi *maasInstance) hostname() (string, error) {
 }
 
 // MAAS does not do firewalling so these port methods do nothing.
-func (mi *maasInstance) OpenPorts(machineId string, ports []network.Port) error {
+func (mi *maasInstance) OpenPorts(machineId string, ports []network.PortRange) error {
 	logger.Debugf("unimplemented OpenPorts() called")
 	return nil
 }
 
-func (mi *maasInstance) ClosePorts(machineId string, ports []network.Port) error {
+func (mi *maasInstance) ClosePorts(machineId string, ports []network.PortRange) error {
 	logger.Debugf("unimplemented ClosePorts() called")
 	return nil
 }
 
-func (mi *maasInstance) Ports(machineId string) ([]network.Port, error) {
+func (mi *maasInstance) Ports(machineId string) ([]network.PortRange, error) {
 	logger.Debugf("unimplemented Ports() called")
-	return []network.Port{}, nil
+	return nil, nil
 }
