@@ -24,6 +24,7 @@ import (
 	"launchpad.net/goose/nova"
 	"launchpad.net/goose/swift"
 
+	"github.com/juju/errors"
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/environs/config"
@@ -37,7 +38,7 @@ import (
 	"github.com/juju/juju/network"
 	"github.com/juju/juju/provider/common"
 	"github.com/juju/juju/state"
-	"github.com/juju/juju/state/api"
+	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/tools"
 )
 
@@ -49,6 +50,8 @@ var _ environs.EnvironProvider = (*environProvider)(nil)
 
 var providerInstance environProvider
 
+var makeServiceURL = client.AuthenticatingClient.MakeServiceURL
+
 // Use shortAttempt to poll for short-term events.
 // TODO: This was kept to a long timeout because Nova needs more time than EC2.
 // For example, HP Cloud takes around 9.1 seconds (10 samples) to return a
@@ -61,6 +64,8 @@ var shortAttempt = utils.AttemptStrategy{
 
 func init() {
 	environs.RegisterProvider("openstack", environProvider{})
+	environs.RegisterImageDataSourceFunc("keystone catalog", getKeystoneImageSource)
+	envtools.RegisterToolsDataSourceFunc("keystone catalog", getKeystoneToolsSource)
 }
 
 func (p environProvider) BoilerplateConfig() string {
@@ -87,11 +92,11 @@ openstack:
     #
     # network: <your network label or uuid>
 
-    # tools-metadata-url specifies the location of the Juju tools and
+    # agent-metadata-url specifies the location of the Juju tools and
     # metadata. It defaults to the global public tools metadata
     # location https://streams.canonical.com/tools.
     #
-    # tools-metadata-url:  https://your-tools-metadata-url
+    # agent-metadata-url:  https://your-agent-metadata-url
 
     # image-metadata-url specifies the location of Ubuntu cloud image
     # metadata. It defaults to the global public image metadata
@@ -99,12 +104,18 @@ openstack:
     #
     # image-metadata-url:  https://your-image-metadata-url
 
-    # image-stream chooses a simplestreams stream to select OS images
-    # from, for example daily or released images (or any other stream
+    # image-stream chooses a simplestreams stream from which to select
+    # OS images, for example daily or released images (or any other stream
     # available on simplestreams).
     #
     # image-stream: "released"
 
+    # agent-stream chooses a simplestreams stream from which to select tools,
+    # for example released or proposed tools (or any other stream available
+    # on simplestreams).
+    #
+    # agent-stream: "released"
+
     # auth-url defaults to the value of the environment variable
     # OS_AUTH_URL, but can be specified here.
     #
@@ -141,6 +152,20 @@ openstack:
     # access-key: <secret>
     # secret-key: <secret>
 
+    # Whether or not to refresh the list of available updates for an
+    # OS. The default option of true is recommended for use in
+    # production systems, but disabling this can speed up local
+    # deployments for development or testing.
+    #
+    # enable-os-refresh-update: true
+
+    # Whether or not to perform OS upgrades when machines are
+    # provisioned. The default option of true is recommended for use
+    # in production systems, but disabling this can speed up local
+    # deployments for development or testing.
+    #
+    # enable-os-upgrade: true
+
 # https://juju.ubuntu.com/docs/config-hpcloud.html
 hpcloud:
     type: openstack
@@ -163,12 +188,18 @@ hpcloud:
     #
     # tenant-name: <your tenant name>
 
-    # image-stream chooses a simplestreams stream to select OS images
-    # from, for example daily or released images (or any other stream
+    # image-stream chooses a simplestreams stream from which to select
+    # OS images, for example daily or released images (or any other stream
     # available on simplestreams).
     #
     # image-stream: "released"
 
+    # agent-stream chooses a simplestreams stream from which to select tools,
+    # for example released or proposed tools (or any other stream available
+    # on simplestreams).
+    #
+    # agent-stream: "released"
+
     # auth-url holds the keystone url for authentication. It defaults
     # to the value of the environment variable OS_AUTH_URL.
     #
@@ -197,6 +228,20 @@ hpcloud:
     # access-key: <secret>
     # secret-key: <secret>
 
+    # Whether or not to refresh the list of available updates for an
+    # OS. The default option of true is recommended for use in
+    # production systems, but disabling this can speed up local
+    # deployments for development or testing.
+    #
+    # enable-os-refresh-update: true
+
+    # Whether or not to perform OS upgrades when machines are
+    # provisioned. The default option of true is recommended for use
+    # in production systems, but disabling this can speed up local
+    # deployments for development or testing.
+    #
+    # enable-os-upgrade: true
+
 `[1:]
 }
 
@@ -224,7 +269,15 @@ func (p environProvider) Prepare(ctx environs.BootstrapContext, cfg *config.Conf
 	if err != nil {
 		return nil, err
 	}
-	return p.Open(cfg)
+	e, err := p.Open(cfg)
+	if err != nil {
+		return nil, err
+	}
+	// Verify credentials.
+	if err := authenticateClient(e.(*environ)); err != nil {
+		return nil, err
+	}
+	return e, nil
 }
 
 // MetadataLookupParams returns parameters which are used to query image metadata to
@@ -288,26 +341,24 @@ type environ struct {
 	supportedArchitectures []string
 
 	ecfgMutex       sync.Mutex
-	imageBaseMutex  sync.Mutex
-	toolsBaseMutex  sync.Mutex
 	ecfgUnlocked    *environConfig
 	client          client.AuthenticatingClient
 	novaUnlocked    *nova.Client
 	storageUnlocked storage.Storage
-	// An ordered list of sources in which to find the simplestreams index files used to
-	// look up image ids.
-	imageSources []simplestreams.DataSource
-	// An ordered list of paths in which to find the simplestreams index files used to
-	// look up tools ids.
-	toolsSources []simplestreams.DataSource
+
+	// keystoneImageDataSource caches the result of getKeystoneImageSource.
+	keystoneImageDataSourceMutex sync.Mutex
+	keystoneImageDataSource      simplestreams.DataSource
+
+	// keystoneToolsDataSource caches the result of getKeystoneToolsSource.
+	keystoneToolsDataSourceMutex sync.Mutex
+	keystoneToolsDataSource      simplestreams.DataSource
 
 	availabilityZonesMutex sync.Mutex
 	availabilityZones      []common.AvailabilityZone
 }
 
 var _ environs.Environ = (*environ)(nil)
-var _ imagemetadata.SupportsCustomSources = (*environ)(nil)
-var _ envtools.SupportsCustomSources = (*environ)(nil)
 var _ simplestreams.HasRegion = (*environ)(nil)
 var _ state.Prechecker = (*environ)(nil)
 var _ state.InstanceDistributor = (*environ)(nil)
@@ -372,6 +423,7 @@ func (inst *openstackInstance) hardwareCharacteristics() *instance.HardwareChara
 		hc.CpuPower = inst.instType.CpuPower
 		// tags not currently supported on openstack
 	}
+	hc.AvailabilityZone = &inst.serverDetail.AvailabilityZone
 	return hc
 }
 
@@ -397,10 +449,10 @@ func (inst *openstackInstance) Addresses() ([]network.Address, error) {
 		return nil, err
 	}
 	var floatingIP string
-	if inst.floatingIP != nil {
+	if inst.floatingIP != nil && inst.floatingIP.IP != "" {
 		floatingIP = inst.floatingIP.IP
+		logger.Debugf("instance %v has floating IP address: %v", inst.Id(), floatingIP)
 	}
-	logger.Infof("instance %v has floating IP address: %v", inst.Id(), floatingIP)
 	return convertNovaAddresses(floatingIP, addresses), nil
 }
 
@@ -443,7 +495,7 @@ func convertNovaAddresses(publicIP string, addresses map[string][]nova.IPAddress
 
 // TODO: following 30 lines nearly verbatim from environs/ec2
 
-func (inst *openstackInstance) OpenPorts(machineId string, ports []network.Port) error {
+func (inst *openstackInstance) OpenPorts(machineId string, ports []network.PortRange) error {
 	if inst.e.Config().FirewallMode() != config.FwInstance {
 		return fmt.Errorf("invalid firewall mode %q for opening ports on instance",
 			inst.e.Config().FirewallMode())
@@ -456,7 +508,7 @@ func (inst *openstackInstance) OpenPorts(machineId string, ports []network.Port)
 	return nil
 }
 
-func (inst *openstackInstance) ClosePorts(machineId string, ports []network.Port) error {
+func (inst *openstackInstance) ClosePorts(machineId string, ports []network.PortRange) error {
 	if inst.e.Config().FirewallMode() != config.FwInstance {
 		return fmt.Errorf("invalid firewall mode %q for closing ports on instance",
 			inst.e.Config().FirewallMode())
@@ -469,13 +521,17 @@ func (inst *openstackInstance) ClosePorts(machineId string, ports []network.Port
 	return nil
 }
 
-func (inst *openstackInstance) Ports(machineId string) ([]network.Port, error) {
+func (inst *openstackInstance) Ports(machineId string) ([]network.PortRange, error) {
 	if inst.e.Config().FirewallMode() != config.FwInstance {
 		return nil, fmt.Errorf("invalid firewall mode %q for retrieving ports from instance",
 			inst.e.Config().FirewallMode())
 	}
 	name := inst.e.machineGroupName(machineId)
-	return inst.e.portsInGroup(name)
+	portRanges, err := inst.e.portsInGroup(name)
+	if err != nil {
+		return nil, err
+	}
+	return portRanges, nil
 }
 
 func (e *environ) ecfg() *environConfig {
@@ -492,10 +548,6 @@ func (e *environ) nova() *nova.Client {
 	return nova
 }
 
-func (e *environ) Name() string {
-	return e.name
-}
-
 // SupportedArchitectures is specified on the EnvironCapability interface.
 func (e *environ) SupportedArchitectures() ([]string, error) {
 	e.archMutex.Lock()
@@ -523,6 +575,11 @@ func (e *environ) SupportNetworks() bool {
 	return false
 }
 
+// SupportAddressAllocation is specified on the EnvironCapability interface.
+func (e *environ) SupportAddressAllocation(netId network.Id) (bool, error) {
+	return false, nil
+}
+
 var unsupportedConstraints = []string{
 	constraints.Tags,
 	constraints.CpuPower,
@@ -663,19 +720,18 @@ func (e *environ) Storage() storage.Storage {
 	return stor
 }
 
-func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.BootstrapParams) error {
+func (e *environ) Bootstrap(ctx environs.BootstrapContext, args environs.BootstrapParams) (arch, series string, _ environs.BootstrapFinalizer, _ error) {
 	// The client's authentication may have been reset when finding tools if the agent-version
 	// attribute was updated so we need to re-authenticate. This will be a no-op if already authenticated.
 	// An authenticated client is needed for the URL() call below.
-	err := e.client.Authenticate()
-	if err != nil {
-		return err
+	if err := authenticateClient(e); err != nil {
+		return "", "", nil, err
 	}
 	return common.Bootstrap(ctx, e, args)
 }
 
-func (e *environ) StateInfo() (*state.Info, *api.Info, error) {
-	return common.StateInfo(e)
+func (e *environ) StateServerInstances() ([]instance.Id, error) {
+	return common.ProviderStateInstances(e, e.Storage())
 }
 
 func (e *environ) Config() *config.Config {
@@ -709,6 +765,22 @@ func (e *environ) authClient(ecfg *environConfig, authModeCfg AuthMode) client.A
 	return newClient(cred, authMode, nil)
 }
 
+var authenticateClient = func(e *environ) error {
+	err := e.client.Authenticate()
+	if err != nil {
+		// Log the error in case there are any useful hints,
+		// but provide a readable and helpful error message
+		// to the user.
+		logger.Debugf("authentication failed: %v", err)
+		return jujuerrors.New(`authentication failed.
+
+Please ensure the credentials are correct. A common mistake is
+to specify the wrong tenant. Use the OpenStack "project" name
+for tenant-name in your environment configuration.`)
+	}
+	return nil
+}
+
 func (e *environ) SetConfig(cfg *config.Config) error {
 	ecfg, err := providerInstance.newConfig(cfg)
 	if err != nil {
@@ -723,6 +795,7 @@ func (e *environ) SetConfig(cfg *config.Config) error {
 	e.ecfgUnlocked = ecfg
 
 	e.client = e.authClient(ecfg, authModeCfg)
+
 	e.novaUnlocked = nova.New(e.client)
 
 	// create new control storage instance, existing instances continue
@@ -738,64 +811,48 @@ func (e *environ) SetConfig(cfg *config.Config) error {
 	return nil
 }
 
-// GetImageSources returns a list of sources which are used to search for simplestreams image metadata.
-func (e *environ) GetImageSources() ([]simplestreams.DataSource, error) {
-	e.imageBaseMutex.Lock()
-	defer e.imageBaseMutex.Unlock()
-
-	if e.imageSources != nil {
-		return e.imageSources, nil
+// getKeystoneImageSource is an imagemetadata.ImageDataSourceFunc that
+// returns a DataSource using the "product-streams" keystone URL.
+func getKeystoneImageSource(env environs.Environ) (simplestreams.DataSource, error) {
+	e, ok := env.(*environ)
+	if !ok {
+		return nil, jujuerrors.NotSupportedf("non-openstack environment")
 	}
-	if !e.client.IsAuthenticated() {
-		err := e.client.Authenticate()
-		if err != nil {
-			return nil, err
-		}
-	}
-	// Add the simplestreams source off the control bucket.
-	e.imageSources = append(e.imageSources, storage.NewStorageSimpleStreamsDataSource(
-		"cloud storage", e.Storage(), storage.BaseImagesPath))
-	// Add the simplestreams base URL from keystone if it is defined.
-	productStreamsURL, err := e.client.MakeServiceURL("product-streams", nil)
-	if err == nil {
-		verify := utils.VerifySSLHostnames
-		if !e.Config().SSLHostnameVerification() {
-			verify = utils.NoVerifySSLHostnames
-		}
-		source := simplestreams.NewURLDataSource("keystone catalog", productStreamsURL, verify)
-		e.imageSources = append(e.imageSources, source)
-	}
-	return e.imageSources, nil
+	return e.getKeystoneDataSource(&e.keystoneImageDataSourceMutex, &e.keystoneImageDataSource, "product-streams")
 }
 
-// GetToolsSources returns a list of sources which are used to search for simplestreams tools metadata.
-func (e *environ) GetToolsSources() ([]simplestreams.DataSource, error) {
-	e.toolsBaseMutex.Lock()
-	defer e.toolsBaseMutex.Unlock()
+// getKeystoneToolsSource is a tools.ToolsDataSourceFunc that
+// returns a DataSource using the "juju-tools" keystone URL.
+func getKeystoneToolsSource(env environs.Environ) (simplestreams.DataSource, error) {
+	e, ok := env.(*environ)
+	if !ok {
+		return nil, jujuerrors.NotSupportedf("non-openstack environment")
+	}
+	return e.getKeystoneDataSource(&e.keystoneToolsDataSourceMutex, &e.keystoneToolsDataSource, "juju-tools")
+}
 
-	if e.toolsSources != nil {
-		return e.toolsSources, nil
+func (e *environ) getKeystoneDataSource(mu *sync.Mutex, datasource *simplestreams.DataSource, keystoneName string) (simplestreams.DataSource, error) {
+	mu.Lock()
+	defer mu.Unlock()
+	if *datasource != nil {
+		return *datasource, nil
 	}
 	if !e.client.IsAuthenticated() {
-		err := e.client.Authenticate()
-		if err != nil {
+		if err := authenticateClient(e); err != nil {
 			return nil, err
 		}
 	}
+
+	url, err := makeServiceURL(e.client, keystoneName, nil)
+	if err != nil {
+		return nil, jujuerrors.NewNotSupported(err, fmt.Sprintf("cannot make service URL: %v", err))
+	}
 	verify := utils.VerifySSLHostnames
 	if !e.Config().SSLHostnameVerification() {
 		verify = utils.NoVerifySSLHostnames
 	}
-	// Add the simplestreams source off the control bucket.
-	e.toolsSources = append(e.toolsSources, storage.NewStorageSimpleStreamsDataSource(
-		"cloud storage", e.Storage(), storage.BaseToolsPath))
-	// Add the simplestreams base URL from keystone if it is defined.
-	toolsURL, err := e.client.MakeServiceURL("juju-tools", nil)
-	if err == nil {
-		source := simplestreams.NewURLDataSource("keystone catalog", toolsURL, verify)
-		e.toolsSources = append(e.toolsSources, source)
-	}
-	return e.toolsSources, nil
+	*datasource = simplestreams.NewURLDataSource("keystone catalog", url, verify)
+	return *datasource, nil
 }
 
 // TODO(gz): Move this somewhere more reusable
@@ -855,7 +912,7 @@ func (e *environ) allocatePublicIP() (*nova.FloatingIP, error) {
 		if err != nil {
 			return nil, err
 		}
-		logger.Debugf("allocated new public ip: %v", newfip.IP)
+		logger.Debugf("allocated new public IP: %v", newfip.IP)
 	}
 	return newfip, nil
 }
@@ -889,15 +946,15 @@ func (e *environ) DistributeInstances(candidates, distributionGroup []instance.I
 var availabilityZoneAllocations = common.AvailabilityZoneAllocations
 
 // StartInstance is specified in the InstanceBroker interface.
-func (e *environ) StartInstance(args environs.StartInstanceParams) (instance.Instance, *instance.HardwareCharacteristics, []network.Info, error) {
+func (e *environ) StartInstance(args environs.StartInstanceParams) (*environs.StartInstanceResult, error) {
 	var availabilityZones []string
 	if args.Placement != "" {
 		placement, err := e.parsePlacement(args.Placement)
 		if err != nil {
-			return nil, nil, nil, err
+			return nil, err
 		}
 		if !placement.availabilityZone.State.Available {
-			return nil, nil, nil, fmt.Errorf("availability zone %q is unavailable", placement.availabilityZone.Name)
+			return nil, fmt.Errorf("availability zone %q is unavailable", placement.availabilityZone.Name)
 		}
 		availabilityZones = append(availabilityZones, placement.availabilityZone.Name)
 	}
@@ -911,7 +968,7 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (instance.Ins
 		if args.DistributionGroup != nil {
 			group, err = args.DistributionGroup()
 			if err != nil {
-				return nil, nil, nil, err
+				return nil, err
 			}
 		}
 		zoneInstances, err := availabilityZoneAllocations(e, group)
@@ -919,7 +976,7 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (instance.Ins
 			// Availability zones are an extension, so we may get a
 			// not implemented error; ignore these.
 		} else if err != nil {
-			return nil, nil, nil, err
+			return nil, err
 		} else {
 			for _, zone := range zoneInstances {
 				availabilityZones = append(availabilityZones, zone.ZoneName)
@@ -932,7 +989,7 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (instance.Ins
 	}
 
 	if args.MachineConfig.HasNetworks() {
-		return nil, nil, nil, fmt.Errorf("starting instances with networks is not supported yet.")
+		return nil, fmt.Errorf("starting instances with networks is not supported yet.")
 	}
 
 	series := args.Tools.OneSeries()
@@ -944,21 +1001,21 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (instance.Ins
 		Constraints: args.Constraints,
 	})
 	if err != nil {
-		return nil, nil, nil, err
+		return nil, err
 	}
 	tools, err := args.Tools.Match(tools.Filter{Arch: spec.Image.Arch})
 	if err != nil {
-		return nil, nil, nil, fmt.Errorf("chosen architecture %v not present in %v", spec.Image.Arch, arches)
+		return nil, fmt.Errorf("chosen architecture %v not present in %v", spec.Image.Arch, arches)
 	}
 
 	args.MachineConfig.Tools = tools[0]
 
-	if err := environs.FinishMachineConfig(args.MachineConfig, e.Config(), args.Constraints); err != nil {
-		return nil, nil, nil, err
+	if err := environs.FinishMachineConfig(args.MachineConfig, e.Config()); err != nil {
+		return nil, err
 	}
 	userData, err := environs.ComposeUserData(args.MachineConfig, nil)
 	if err != nil {
-		return nil, nil, nil, fmt.Errorf("cannot make user data: %v", err)
+		return nil, fmt.Errorf("cannot make user data: %v", err)
 	}
 	logger.Debugf("openstack user data; %d bytes", len(userData))
 	var networks = []nova.ServerNetworks{}
@@ -966,7 +1023,7 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (instance.Ins
 	if usingNetwork != "" {
 		networkId, err := e.resolveNetwork(usingNetwork)
 		if err != nil {
-			return nil, nil, nil, err
+			return nil, err
 		}
 		logger.Debugf("using network id %q", networkId)
 		networks = append(networks, nova.ServerNetworks{NetworkId: networkId})
@@ -976,16 +1033,16 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (instance.Ins
 	if withPublicIP {
 		logger.Debugf("allocating public IP address for openstack node")
 		if fip, err := e.allocatePublicIP(); err != nil {
-			return nil, nil, nil, fmt.Errorf("cannot allocate a public IP as needed: %v", err)
+			return nil, fmt.Errorf("cannot allocate a public IP as needed: %v", err)
 		} else {
 			publicIP = fip
 			logger.Infof("allocated public IP %s", publicIP.IP)
 		}
 	}
 	cfg := e.Config()
-	groups, err := e.setUpGroups(args.MachineConfig.MachineId, cfg.StatePort(), cfg.APIPort())
+	groups, err := e.setUpGroups(args.MachineConfig.MachineId, cfg.APIPort())
 	if err != nil {
-		return nil, nil, nil, fmt.Errorf("cannot set up groups: %v", err)
+		return nil, fmt.Errorf("cannot set up groups: %v", err)
 	}
 	var groupNames = make([]nova.SecurityGroupName, len(groups))
 	for i, g := range groups {
@@ -1015,11 +1072,11 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (instance.Ins
 		}
 	}
 	if err != nil {
-		return nil, nil, nil, fmt.Errorf("cannot run instance: %v", err)
+		return nil, fmt.Errorf("cannot run instance: %v", err)
 	}
 	detail, err := e.nova().GetServer(server.Id)
 	if err != nil {
-		return nil, nil, nil, fmt.Errorf("cannot get started instance: %v", err)
+		return nil, fmt.Errorf("cannot get started instance: %v", err)
 	}
 	inst := &openstackInstance{
 		e:            e,
@@ -1034,12 +1091,20 @@ func (e *environ) StartInstance(args environs.StartInstanceParams) (instance.Ins
 				// ignore the failure at this stage, just log it
 				logger.Debugf("failed to terminate instance %q: %v", inst.Id(), err)
 			}
-			return nil, nil, nil, fmt.Errorf("cannot assign public address %s to instance %q: %v", publicIP.IP, inst.Id(), err)
+			return nil, fmt.Errorf("cannot assign public address %s to instance %q: %v", publicIP.IP, inst.Id(), err)
 		}
 		inst.floatingIP = publicIP
 		logger.Infof("assigned public IP %s to %q", publicIP.IP, inst.Id())
 	}
-	return inst, inst.hardwareCharacteristics(), nil, nil
+	if multiwatcher.AnyJobNeedsState(args.MachineConfig.Jobs...) {
+		if err := common.AddStateInstance(e.Storage(), inst.Id()); err != nil {
+			logger.Errorf("could not record instance in provider-state: %v", err)
+		}
+	}
+	return &environs.StartInstanceResult{
+		Instance: inst,
+		Hardware: inst.hardwareCharacteristics(),
+	}, nil
 }
 
 func isNoValidHostsError(err error) bool {
@@ -1076,7 +1141,7 @@ func (e *environ) StopInstances(ids ...instance.Id) error {
 	if securityGroupNames != nil {
 		return e.deleteSecurityGroups(securityGroupNames)
 	}
-	return nil
+	return common.RemoveStateInstances(e.Storage(), ids...)
 }
 
 // collectInstances tries to get information on each instance id in ids.
@@ -1107,7 +1172,7 @@ func (e *environ) collectInstances(ids []instance.Id, out map[string]instance.In
 		if server, found := serversById[string(id)]; found {
 			// HPCloud uses "BUILD(spawning)" as an intermediate BUILD states once networking is available.
 			switch server.Status {
-			case nova.StatusActive, nova.StatusBuild, nova.StatusBuildSpawning:
+			case nova.StatusActive, nova.StatusBuild, nova.StatusBuildSpawning, nova.StatusShutoff, nova.StatusSuspended:
 				// TODO(wallyworld): lookup the flavor details to fill in the instance type data
 				out[string(id)] = &openstackInstance{e: e, serverDetail: &server}
 				continue
@@ -1174,19 +1239,25 @@ func (e *environ) Instances(ids []instance.Id) ([]instance.Instance, error) {
 	return insts, err
 }
 
-// AllocateAddress requests a new address to be allocated for the
+// AllocateAddress requests an address to be allocated for the
 // given instance on the given network. This is not implemented on the
 // OpenStack provider yet.
-func (*environ) AllocateAddress(_ instance.Id, _ network.Id) (network.Address, error) {
-	return network.Address{}, jujuerrors.NotImplementedf("AllocateAddress")
+func (*environ) AllocateAddress(_ instance.Id, _ network.Id, _ network.Address) error {
+	return jujuerrors.NotImplementedf("AllocateAddress")
 }
 
-// ListNetworks returns basic information about all networks known
+// ReleaseAddress releases a specific address previously allocated with
+// AllocateAddress.
+func (*environ) ReleaseAddress(_ instance.Id, _ network.Id, _ network.Address) error {
+	return errors.NotImplementedf("ReleaseAddress")
+}
+
+// Subnets returns basic information about all subnets known
 // by the provider for the environment. They may be unknown to juju
 // yet (i.e. when called initially or when a new network was created).
 // This is not implemented by the OpenStack provider yet.
-func (*environ) ListNetworks() ([]network.BasicInfo, error) {
-	return nil, jujuerrors.NotImplementedf("ListNetworks")
+func (*environ) Subnets(_ instance.Id) ([]network.SubnetInfo, error) {
+	return nil, jujuerrors.NotImplementedf("Subnets")
 }
 
 func (e *environ) AllInstances() (insts []instance.Instance, err error) {
@@ -1218,16 +1289,19 @@ func (e *environ) AllInstances() (insts []instance.Instance, err error) {
 func (e *environ) Destroy() error {
 	err := common.Destroy(e)
 	if err != nil {
-		return err
+		return jujuerrors.Trace(err)
+	}
+	if err := e.Storage().RemoveAll(); err != nil {
+		return jujuerrors.Trace(err)
 	}
 	novaClient := e.nova()
 	securityGroups, err := novaClient.ListSecurityGroups()
 	if err != nil {
-		return err
+		return jujuerrors.Annotate(err, "cannot list security groups")
 	}
 	re, err := regexp.Compile(fmt.Sprintf("^%s(-\\d+)?$", e.jujuGroupName()))
 	if err != nil {
-		return err
+		return jujuerrors.Trace(err)
 	}
 	globalGroupName := e.globalGroupName()
 	for _, group := range securityGroups {
@@ -1254,30 +1328,40 @@ func (e *environ) jujuGroupName() string {
 }
 
 func (e *environ) machineFullName(machineId string) string {
-	return fmt.Sprintf("juju-%s-%s", e.Name(), names.NewMachineTag(machineId))
+	return fmt.Sprintf("juju-%s-%s", e.Config().Name(), names.NewMachineTag(machineId))
 }
 
 // machinesFilter returns a nova.Filter matching all machines in the environment.
 func (e *environ) machinesFilter() *nova.Filter {
 	filter := nova.NewFilter()
-	filter.Set(nova.FilterServer, fmt.Sprintf("juju-%s-machine-\\d*", e.Name()))
+	filter.Set(nova.FilterServer, fmt.Sprintf("juju-%s-machine-\\d*", e.Config().Name()))
 	return filter
 }
 
-func (e *environ) openPortsInGroup(name string, ports []network.Port) error {
+// portsToRuleInfo maps port ranges to nova rules
+func portsToRuleInfo(groupId string, ports []network.PortRange) []nova.RuleInfo {
+	rules := make([]nova.RuleInfo, len(ports))
+	for i, portRange := range ports {
+		rules[i] = nova.RuleInfo{
+			ParentGroupId: groupId,
+			FromPort:      portRange.FromPort,
+			ToPort:        portRange.ToPort,
+			IPProtocol:    portRange.Protocol,
+			Cidr:          "0.0.0.0/0",
+		}
+	}
+	return rules
+}
+
+func (e *environ) openPortsInGroup(name string, portRanges []network.PortRange) error {
 	novaclient := e.nova()
 	group, err := novaclient.SecurityGroupByName(name)
 	if err != nil {
 		return err
 	}
-	for _, port := range ports {
-		_, err := novaclient.CreateSecurityGroupRule(nova.RuleInfo{
-			ParentGroupId: group.Id,
-			FromPort:      port.Number,
-			ToPort:        port.Number,
-			IPProtocol:    port.Protocol,
-			Cidr:          "0.0.0.0/0",
-		})
+	rules := portsToRuleInfo(group.Id, portRanges)
+	for _, rule := range rules {
+		_, err := novaclient.CreateSecurityGroupRule(rule)
 		if err != nil {
 			// TODO: if err is not rule already exists, raise?
 			logger.Debugf("error creating security group rule: %v", err.Error())
@@ -1286,8 +1370,18 @@ func (e *environ) openPortsInGroup(name string, ports []network.Port) error {
 	return nil
 }
 
-func (e *environ) closePortsInGroup(name string, ports []network.Port) error {
-	if len(ports) == 0 {
+// ruleMatchesPortRange checks if supplied nova security group rule matches the port range
+func ruleMatchesPortRange(rule nova.SecurityGroupRule, portRange network.PortRange) bool {
+	if rule.IPProtocol == nil || rule.FromPort == nil || rule.ToPort == nil {
+		return false
+	}
+	return *rule.IPProtocol == portRange.Protocol &&
+		*rule.FromPort == portRange.FromPort &&
+		*rule.ToPort == portRange.ToPort
+}
+
+func (e *environ) closePortsInGroup(name string, portRanges []network.PortRange) error {
+	if len(portRanges) == 0 {
 		return nil
 	}
 	novaclient := e.nova()
@@ -1296,11 +1390,9 @@ func (e *environ) closePortsInGroup(name string, ports []network.Port) error {
 		return err
 	}
 	// TODO: Hey look ma, it's quadratic
-	for _, port := range ports {
+	for _, portRange := range portRanges {
 		for _, p := range (*group).Rules {
-			if p.IPProtocol == nil || *p.IPProtocol != port.Protocol ||
-				p.FromPort == nil || *p.FromPort != port.Number ||
-				p.ToPort == nil || *p.ToPort != port.Number {
+			if !ruleMatchesPortRange(p, portRange) {
 				continue
 			}
 			err := novaclient.DeleteSecurityGroupRule(p.Id)
@@ -1313,26 +1405,25 @@ func (e *environ) closePortsInGroup(name string, ports []network.Port) error {
 	return nil
 }
 
-func (e *environ) portsInGroup(name string) (ports []network.Port, err error) {
+func (e *environ) portsInGroup(name string) (portRanges []network.PortRange, err error) {
 	group, err := e.nova().SecurityGroupByName(name)
 	if err != nil {
 		return nil, err
 	}
 	for _, p := range (*group).Rules {
-		for i := *p.FromPort; i <= *p.ToPort; i++ {
-			ports = append(ports, network.Port{
-				Protocol: *p.IPProtocol,
-				Number:   i,
-			})
-		}
+		portRanges = append(portRanges, network.PortRange{
+			Protocol: *p.IPProtocol,
+			FromPort: *p.FromPort,
+			ToPort:   *p.ToPort,
+		})
 	}
-	network.SortPorts(ports)
-	return ports, nil
+	network.SortPortRanges(portRanges)
+	return portRanges, nil
 }
 
 // TODO: following 30 lines nearly verbatim from environs/ec2
 
-func (e *environ) OpenPorts(ports []network.Port) error {
+func (e *environ) OpenPorts(ports []network.PortRange) error {
 	if e.Config().FirewallMode() != config.FwGlobal {
 		return fmt.Errorf("invalid firewall mode %q for opening ports on environment",
 			e.Config().FirewallMode())
@@ -1344,7 +1435,7 @@ func (e *environ) OpenPorts(ports []network.Port) error {
 	return nil
 }
 
-func (e *environ) ClosePorts(ports []network.Port) error {
+func (e *environ) ClosePorts(ports []network.PortRange) error {
 	if e.Config().FirewallMode() != config.FwGlobal {
 		return fmt.Errorf("invalid firewall mode %q for closing ports on environment",
 			e.Config().FirewallMode())
@@ -1356,7 +1447,7 @@ func (e *environ) ClosePorts(ports []network.Port) error {
 	return nil
 }
 
-func (e *environ) Ports() ([]network.Port, error) {
+func (e *environ) Ports() ([]network.PortRange, error) {
 	if e.Config().FirewallMode() != config.FwGlobal {
 		return nil, fmt.Errorf("invalid firewall mode %q for retrieving ports from environment",
 			e.Config().FirewallMode())
@@ -1368,7 +1459,7 @@ func (e *environ) Provider() environs.EnvironProvider {
 	return &providerInstance
 }
 
-func (e *environ) setUpGlobalGroup(groupName string, statePort, apiPort int) (nova.SecurityGroup, error) {
+func (e *environ) setUpGlobalGroup(groupName string, apiPort int) (nova.SecurityGroup, error) {
 	return e.ensureGroup(groupName,
 		[]nova.RuleInfo{
 			{
@@ -1377,12 +1468,6 @@ func (e *environ) setUpGlobalGroup(groupName string, statePort, apiPort int) (no
 				ToPort:     22,
 				Cidr:       "0.0.0.0/0",
 			},
-			{
-				IPProtocol: "tcp",
-				FromPort:   statePort,
-				ToPort:     statePort,
-				Cidr:       "0.0.0.0/0",
-			},
 			{
 				IPProtocol: "tcp",
 				FromPort:   apiPort,
@@ -1418,8 +1503,8 @@ func (e *environ) setUpGlobalGroup(groupName string, statePort, apiPort int) (no
 // Note: ideally we'd have a better way to determine group membership so that 2
 // people that happen to share an openstack account and name their environment
 // "openstack" don't end up destroying each other's machines.
-func (e *environ) setUpGroups(machineId string, statePort, apiPort int) ([]nova.SecurityGroup, error) {
-	jujuGroup, err := e.setUpGlobalGroup(e.jujuGroupName(), statePort, apiPort)
+func (e *environ) setUpGroups(machineId string, apiPort int) ([]nova.SecurityGroup, error) {
+	jujuGroup, err := e.setUpGlobalGroup(e.jujuGroupName(), apiPort)
 	if err != nil {
 		return nil, err
 	}
