@@ -20,8 +20,9 @@ import (
 
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/instance"
+	"github.com/juju/juju/mongo"
 	"github.com/juju/juju/network"
-	"github.com/juju/juju/state/api/params"
+	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/state/presence"
 	"github.com/juju/juju/tools"
 	"github.com/juju/juju/version"
@@ -43,59 +44,61 @@ const (
 	_ MachineJob = iota
 	JobHostUnits
 	JobManageEnviron
+	JobManageNetworking
 
 	// Deprecated in 1.18.
 	JobManageStateDeprecated
 )
 
-var jobNames = map[MachineJob]params.MachineJob{
-	JobHostUnits:     params.JobHostUnits,
-	JobManageEnviron: params.JobManageEnviron,
+var jobNames = map[MachineJob]multiwatcher.MachineJob{
+	JobHostUnits:        multiwatcher.JobHostUnits,
+	JobManageEnviron:    multiwatcher.JobManageEnviron,
+	JobManageNetworking: multiwatcher.JobManageNetworking,
 
 	// Deprecated in 1.18.
-	JobManageStateDeprecated: params.JobManageStateDeprecated,
+	JobManageStateDeprecated: multiwatcher.JobManageStateDeprecated,
 }
 
 // AllJobs returns all supported machine jobs.
 func AllJobs() []MachineJob {
-	return []MachineJob{JobHostUnits, JobManageEnviron}
-}
-
-// ToParams returns the job as params.MachineJob.
-func (job MachineJob) ToParams() params.MachineJob {
-	if paramsJob, ok := jobNames[job]; ok {
-		return paramsJob
+	return []MachineJob{
+		JobHostUnits,
+		JobManageEnviron,
+		JobManageNetworking,
 	}
-	return params.MachineJob(fmt.Sprintf("<unknown job %d>", int(job)))
 }
 
-// MachineJobFromParams returns the job corresponding to params.MachineJob.
-func MachineJobFromParams(job params.MachineJob) (MachineJob, error) {
-	for machineJob, paramJob := range jobNames {
-		if paramJob == job {
-			return machineJob, nil
-		}
+// ToParams returns the job as multiwatcher.MachineJob.
+func (job MachineJob) ToParams() multiwatcher.MachineJob {
+	if jujuJob, ok := jobNames[job]; ok {
+		return jujuJob
 	}
-	return -1, fmt.Errorf("invalid machine job %q", job)
+	return multiwatcher.MachineJob(fmt.Sprintf("<unknown job %d>", int(job)))
 }
 
-// paramsJobsFromJobs converts state jobs to params jobs.
-func paramsJobsFromJobs(jobs []MachineJob) []params.MachineJob {
-	paramsJobs := make([]params.MachineJob, len(jobs))
+// params.JobsFromJobs converts state jobs to juju jobs.
+func paramsJobsFromJobs(jobs []MachineJob) []multiwatcher.MachineJob {
+	jujuJobs := make([]multiwatcher.MachineJob, len(jobs))
 	for i, machineJob := range jobs {
-		paramsJobs[i] = machineJob.ToParams()
+		jujuJobs[i] = machineJob.ToParams()
 	}
-	return paramsJobs
+	return jujuJobs
 }
 
 func (job MachineJob) String() string {
 	return string(job.ToParams())
 }
 
+// manualMachinePrefix signals as prefix of Nonce that a machine is
+// manually provisioned.
+const manualMachinePrefix = "manual:"
+
 // machineDoc represents the internal state of a machine in MongoDB.
-// Note the correspondence with MachineInfo in state/api/params.
+// Note the correspondence with MachineInfo in apiserver/juju.
 type machineDoc struct {
-	Id            string `bson:"_id"`
+	DocID         string `bson:"_id"`
+	Id            string `bson:"machineid"`
+	EnvUUID       string `bson:"env-uuid"`
 	Nonce         string
 	Series        string
 	ContainerType string
@@ -120,11 +123,6 @@ type machineDoc struct {
 	// Placement is the placement directive that should be used when provisioning
 	// an instance for the machine.
 	Placement string `bson:",omitempty"`
-	// Deprecated. InstanceId, now lives on instanceData.
-	// This attribute is retained so that data from existing machines can be read.
-	// SCHEMACHANGE
-	// TODO(wallyworld): remove this attribute when schema upgrades are possible.
-	InstanceId instance.Id
 }
 
 func newMachine(st *State, doc *machineDoc) *Machine {
@@ -134,12 +132,16 @@ func newMachine(st *State, doc *machineDoc) *Machine {
 	}
 	machine.annotator = annotator{
 		globalKey: machine.globalKey(),
-		tag:       machine.Tag().String(),
+		tag:       machine.Tag(),
 		st:        st,
 	}
 	return machine
 }
 
+func wantsVote(jobs []MachineJob, noVote bool) bool {
+	return hasJob(jobs, JobManageEnviron) && !noVote
+}
+
 // Id returns the machine id.
 func (m *Machine) Id() string {
 	return m.doc.Id
@@ -167,8 +169,10 @@ func (m *Machine) globalKey() string {
 
 // instanceData holds attributes relevant to a provisioned machine.
 type instanceData struct {
-	Id         string      `bson:"_id"`
+	DocID      string      `bson:"_id"`
+	MachineId  string      `bson:"machineid"`
 	InstanceId instance.Id `bson:"instanceid"`
+	EnvUUID    string      `bson:"env-uuid"`
 	Status     string      `bson:"status,omitempty"`
 	Arch       *string     `bson:"arch,omitempty"`
 	Mem        *uint64     `bson:"mem,omitempty"`
@@ -176,16 +180,18 @@ type instanceData struct {
 	CpuCores   *uint64     `bson:"cpucores,omitempty"`
 	CpuPower   *uint64     `bson:"cpupower,omitempty"`
 	Tags       *[]string   `bson:"tags,omitempty"`
+	AvailZone  *string     `bson:"availzone,omitempty"`
 }
 
 func hardwareCharacteristics(instData instanceData) *instance.HardwareCharacteristics {
 	return &instance.HardwareCharacteristics{
-		Arch:     instData.Arch,
-		Mem:      instData.Mem,
-		RootDisk: instData.RootDisk,
-		CpuCores: instData.CpuCores,
-		CpuPower: instData.CpuPower,
-		Tags:     instData.Tags,
+		Arch:             instData.Arch,
+		Mem:              instData.Mem,
+		RootDisk:         instData.RootDisk,
+		CpuCores:         instData.CpuCores,
+		CpuPower:         instData.CpuPower,
+		Tags:             instData.Tags,
+		AvailabilityZone: instData.AvailZone,
 	}
 }
 
@@ -234,7 +240,7 @@ func (m *Machine) Jobs() []MachineJob {
 // WantsVote reports whether the machine is a state server
 // that wants to take part in peer voting.
 func (m *Machine) WantsVote() bool {
-	return hasJob(m.doc.Jobs, JobManageEnviron) && !m.doc.NoVote
+	return wantsVote(m.doc.Jobs, m.doc.NoVote)
 }
 
 // HasVote reports whether that machine is currently a voting
@@ -249,7 +255,7 @@ func (m *Machine) HasVote() bool {
 func (m *Machine) SetHasVote(hasVote bool) error {
 	ops := []txn.Op{{
 		C:      machinesC,
-		Id:     m.doc.Id,
+		Id:     m.doc.DocID,
 		Assert: notDeadDoc,
 		Update: bson.D{{"$set", bson.D{{"hasvote", hasVote}}}},
 	}}
@@ -270,7 +276,7 @@ func (m *Machine) IsManual() (bool, error) {
 	// Apart from the bootstrap machine, manually provisioned
 	// machines have a nonce prefixed with "manual:". This is
 	// unique to manual provisioning.
-	if strings.HasPrefix(m.doc.Nonce, "manual:") {
+	if strings.HasPrefix(m.doc.Nonce, manualMachinePrefix) {
 		return true, nil
 	}
 	// The bootstrap machine uses BootstrapNonce, so in that
@@ -310,18 +316,21 @@ func checkVersionValidity(v version.Binary) error {
 // SetAgentVersion sets the version of juju that the agent is
 // currently running.
 func (m *Machine) SetAgentVersion(v version.Binary) (err error) {
-	defer errors.Maskf(&err, "cannot set agent version for machine %v", m)
+	defer errors.DeferredAnnotatef(&err, "cannot set agent version for machine %v", m)
 	if err = checkVersionValidity(v); err != nil {
 		return err
 	}
 	tools := &tools.Tools{Version: v}
 	ops := []txn.Op{{
 		C:      machinesC,
-		Id:     m.doc.Id,
+		Id:     m.doc.DocID,
 		Assert: notDeadDoc,
 		Update: bson.D{{"$set", bson.D{{"tools", tools}}}},
 	}}
-	if err := m.st.runTransaction(ops); err != nil {
+	// A "raw" transaction is needed here because this function gets
+	// called before database migraions have run so we don't
+	// necessarily want the env UUID added to the id.
+	if err := m.st.runRawTransaction(ops); err != nil {
 		return onAbort(err, ErrDead)
 	}
 	m.doc.Tools = tools
@@ -332,7 +341,10 @@ func (m *Machine) SetAgentVersion(v version.Binary) (err error) {
 // should use to communicate with the state servers.  Previous passwords
 // are invalidated.
 func (m *Machine) SetMongoPassword(password string) error {
-	return m.st.setMongoPassword(m.Tag().String(), password)
+	if !m.IsManager() {
+		return errors.NotSupportedf("setting mongo password for non-state server machine %v", m)
+	}
+	return mongo.SetAdminMongoPassword(m.st.db.Session, m.Tag().String(), password)
 }
 
 // SetPassword sets the password for the machine's agent.
@@ -349,7 +361,7 @@ func (m *Machine) SetPassword(password string) error {
 func (m *Machine) setPasswordHash(passwordHash string) error {
 	ops := []txn.Op{{
 		C:      machinesC,
-		Id:     m.doc.Id,
+		Id:     m.doc.DocID,
 		Assert: notDeadDoc,
 		Update: bson.D{{"$set", bson.D{{"passwordhash", passwordHash}}}},
 	}}
@@ -403,7 +415,7 @@ func (m *Machine) ForceDestroy() error {
 	if !m.IsManager() {
 		ops := []txn.Op{{
 			C:      machinesC,
-			Id:     m.doc.Id,
+			Id:     m.doc.DocID,
 			Assert: bson.D{{"jobs", bson.D{{"$nin", []MachineJob{JobManageEnviron}}}}},
 		}, m.st.newCleanupOp(cleanupForceDestroyedMachine, m.doc.Id)}
 		if err := m.st.runTransaction(ops); err != txn.ErrAborted {
@@ -443,7 +455,7 @@ func (m *Machine) Containers() ([]string, error) {
 	defer closer()
 
 	var mc machineContainers
-	err := containerRefs.FindId(m.Id()).One(&mc)
+	err := containerRefs.FindId(m.doc.DocID).One(&mc)
 	if err == nil {
 		return mc.Children, nil
 	}
@@ -506,7 +518,7 @@ func (original *Machine) advanceLifecycle(life Life) (err error) {
 	// op and
 	op := txn.Op{
 		C:      machinesC,
-		Id:     m.doc.Id,
+		Id:     m.doc.DocID,
 		Update: bson.D{{"$set", bson.D{{"life", life}}}},
 	}
 	advanceAsserts := bson.D{
@@ -574,13 +586,28 @@ func (original *Machine) advanceLifecycle(life Life) (err error) {
 	return err
 }
 
+func (m *Machine) removePortsOps() ([]txn.Op, error) {
+	if m.doc.Life != Dead {
+		return nil, errors.Errorf("machine is not dead")
+	}
+	ports, err := m.AllPorts()
+	if err != nil {
+		return nil, err
+	}
+	var ops []txn.Op
+	for _, p := range ports {
+		ops = append(ops, p.removeOps()...)
+	}
+	return ops, nil
+}
+
 func (m *Machine) removeNetworkInterfacesOps() ([]txn.Op, error) {
 	if m.doc.Life != Dead {
 		return nil, errors.Errorf("machine is not dead")
 	}
 	ops := []txn.Op{{
 		C:      machinesC,
-		Id:     m.doc.Id,
+		Id:     m.doc.DocID,
 		Assert: isDeadDoc,
 	}}
 	sel := bson.D{{"machineid", m.doc.Id}}
@@ -602,37 +629,48 @@ func (m *Machine) removeNetworkInterfacesOps() ([]txn.Op, error) {
 // Remove removes the machine from state. It will fail if the machine
 // is not Dead.
 func (m *Machine) Remove() (err error) {
-	defer errors.Maskf(&err, "cannot remove machine %s", m.doc.Id)
+	defer errors.DeferredAnnotatef(&err, "cannot remove machine %s", m.doc.Id)
 	if m.doc.Life != Dead {
 		return fmt.Errorf("machine is not dead")
 	}
 	ops := []txn.Op{
 		{
 			C:      machinesC,
-			Id:     m.doc.Id,
+			Id:     m.doc.DocID,
 			Assert: txn.DocExists,
 			Remove: true,
 		},
 		{
 			C:      machinesC,
-			Id:     m.doc.Id,
+			Id:     m.doc.DocID,
 			Assert: isDeadDoc,
 		},
 		{
 			C:      instanceDataC,
-			Id:     m.doc.Id,
+			Id:     m.doc.DocID,
 			Remove: true,
 		},
 		removeStatusOp(m.st, m.globalKey()),
 		removeConstraintsOp(m.st, m.globalKey()),
 		removeRequestedNetworksOp(m.st, m.globalKey()),
 		annotationRemoveOp(m.st, m.globalKey()),
+		removeRebootDocOp(m.st, m.globalKey()),
 	}
 	ifacesOps, err := m.removeNetworkInterfacesOps()
 	if err != nil {
 		return err
 	}
+	portsOps, err := m.removePortsOps()
+	if err != nil {
+		return err
+	}
+	blockDeviceOps, err := removeMachineBlockDevicesOps(m.st, m.Id())
+	if err != nil {
+		return err
+	}
 	ops = append(ops, ifacesOps...)
+	ops = append(ops, portsOps...)
+	ops = append(ops, blockDeviceOps...)
 	ops = append(ops, removeContainerRefOps(m.st, m.Id())...)
 	// The only abort conditions in play indicate that the machine has already
 	// been removed.
@@ -646,13 +684,13 @@ func (m *Machine) Refresh() error {
 	machines, closer := m.st.getCollection(machinesC)
 	defer closer()
 
-	doc := machineDoc{}
-	err := machines.FindId(m.doc.Id).One(&doc)
+	var doc machineDoc
+	err := machines.FindId(m.doc.DocID).One(&doc)
 	if err == mgo.ErrNotFound {
 		return errors.NotFoundf("machine %v", m)
 	}
 	if err != nil {
-		return fmt.Errorf("cannot refresh machine %v: %v", m, err)
+		return errors.Annotatef(err, "cannot refresh machine %v", m)
 	}
 	m.doc = doc
 	return nil
@@ -666,7 +704,7 @@ func (m *Machine) AgentPresence() (bool, error) {
 
 // WaitAgentPresence blocks until the respective agent is alive.
 func (m *Machine) WaitAgentPresence(timeout time.Duration) (err error) {
-	defer errors.Maskf(&err, "waiting for agent of machine %v", m)
+	defer errors.DeferredAnnotatef(&err, "waiting for agent of machine %v", m)
 	ch := make(chan presence.Change)
 	m.st.pwatcher.Watch(m.globalKey(), ch)
 	defer m.st.pwatcher.Unwatch(m.globalKey(), ch)
@@ -710,54 +748,37 @@ func (m *Machine) SetAgentPresence() (*presence.Pinger, error) {
 // InstanceId returns the provider specific instance id for this
 // machine, or a NotProvisionedError, if not set.
 func (m *Machine) InstanceId() (instance.Id, error) {
-	// SCHEMACHANGE
-	// TODO(wallyworld) - remove this backward compatibility code when schema upgrades are possible
-	// (we first check for InstanceId stored on the machineDoc)
-	if m.doc.InstanceId != "" {
-		return m.doc.InstanceId, nil
-	}
 	instData, err := getInstanceData(m.st, m.Id())
-	if (err == nil && instData.InstanceId == "") || errors.IsNotFound(err) {
-		err = NotProvisionedError(m.Id())
+	if errors.IsNotFound(err) {
+		err = errors.NotProvisionedf("machine %v", m.Id())
 	}
 	if err != nil {
 		return "", err
 	}
-	return instData.InstanceId, nil
+	return instData.InstanceId, err
 }
 
 // InstanceStatus returns the provider specific instance status for this machine,
 // or a NotProvisionedError if instance is not yet provisioned.
 func (m *Machine) InstanceStatus() (string, error) {
-	// SCHEMACHANGE
-	// InstanceId may not be stored in the instanceData doc, so we
-	// get it using an API on machine which knows to look in the old
-	// place if necessary.
-	instId, err := m.InstanceId()
-	if err != nil {
-		return "", err
-	}
 	instData, err := getInstanceData(m.st, m.Id())
-	if (err == nil && instId == "") || errors.IsNotFound(err) {
-		err = NotProvisionedError(m.Id())
+	if errors.IsNotFound(err) {
+		err = errors.NotProvisionedf("machine %v", m.Id())
 	}
 	if err != nil {
 		return "", err
 	}
-	return instData.Status, nil
+	return instData.Status, err
 }
 
 // SetInstanceStatus sets the provider specific instance status for a machine.
 func (m *Machine) SetInstanceStatus(status string) (err error) {
-	defer errors.Maskf(&err, "cannot set instance status for machine %q", m)
+	defer errors.DeferredAnnotatef(&err, "cannot set instance status for machine %q", m)
 
-	// SCHEMACHANGE - we can't do this yet until the schema is updated
-	// so just do a txn.DocExists for now.
-	// provisioned := bson.D{{"instanceid", bson.D{{"$ne", ""}}}}
 	ops := []txn.Op{
 		{
 			C:      instanceDataC,
-			Id:     m.doc.Id,
+			Id:     m.doc.DocID,
 			Assert: txn.DocExists,
 			Update: bson.D{{"$set", bson.D{{"status", status}}}},
 		},
@@ -768,12 +789,29 @@ func (m *Machine) SetInstanceStatus(status string) (err error) {
 	} else if err != txn.ErrAborted {
 		return err
 	}
-	return NotProvisionedError(m.Id())
+	return errors.NotProvisionedf("machine %v", m.Id())
+}
+
+// AvailabilityZone returns the provier-specific instance availability
+// zone in which the machine was provisioned.
+func (m *Machine) AvailabilityZone() (string, error) {
+	instData, err := getInstanceData(m.st, m.Id())
+	if errors.IsNotFound(err) {
+		return "", errors.Trace(errors.NotProvisionedf("machine %v", m.Id()))
+	}
+	if err != nil {
+		return "", errors.Trace(err)
+	}
+	var zone string
+	if instData.AvailZone != nil {
+		zone = *instData.AvailZone
+	}
+	return zone, nil
 }
 
 // Units returns all the units that have been assigned to the machine.
 func (m *Machine) Units() (units []*Unit, err error) {
-	defer errors.Maskf(&err, "cannot get units assigned to machine %v", m)
+	defer errors.DeferredAnnotatef(&err, "cannot get units assigned to machine %v", m)
 	unitsCollection, closer := m.st.getCollection(unitsC)
 	defer closer()
 
@@ -805,7 +843,7 @@ func (m *Machine) Units() (units []*Unit, err error) {
 // lost) after starting the instance, we can be sure that only a single
 // instance will be able to act for that machine.
 func (m *Machine) SetProvisioned(id instance.Id, nonce string, characteristics *instance.HardwareCharacteristics) (err error) {
-	defer errors.Maskf(&err, "cannot set instance data for machine %q", m)
+	defer errors.DeferredAnnotatef(&err, "cannot set instance data for machine %q", m)
 
 	if id == "" || nonce == "" {
 		return fmt.Errorf("instance id and nonce cannot be empty")
@@ -815,27 +853,28 @@ func (m *Machine) SetProvisioned(id instance.Id, nonce string, characteristics *
 		characteristics = &instance.HardwareCharacteristics{}
 	}
 	instData := &instanceData{
-		Id:         m.doc.Id,
+		DocID:      m.doc.DocID,
+		MachineId:  m.doc.Id,
 		InstanceId: id,
+		EnvUUID:    m.doc.EnvUUID,
 		Arch:       characteristics.Arch,
 		Mem:        characteristics.Mem,
 		RootDisk:   characteristics.RootDisk,
 		CpuCores:   characteristics.CpuCores,
 		CpuPower:   characteristics.CpuPower,
 		Tags:       characteristics.Tags,
+		AvailZone:  characteristics.AvailabilityZone,
 	}
-	// SCHEMACHANGE
-	// TODO(wallyworld) - do not check instanceId on machineDoc after schema is upgraded
-	notSetYet := bson.D{{"instanceid", ""}, {"nonce", ""}}
+
 	ops := []txn.Op{
 		{
 			C:      machinesC,
-			Id:     m.doc.Id,
-			Assert: append(isAliveDoc, notSetYet...),
-			Update: bson.D{{"$set", bson.D{{"instanceid", id}, {"nonce", nonce}}}},
+			Id:     m.doc.DocID,
+			Assert: append(isAliveDoc, bson.DocElem{"nonce", ""}),
+			Update: bson.D{{"$set", bson.D{{"nonce", nonce}}}},
 		}, {
 			C:      instanceDataC,
-			Id:     m.doc.Id,
+			Id:     m.doc.DocID,
 			Assert: txn.DocMissing,
 			Insert: instData,
 		},
@@ -843,14 +882,10 @@ func (m *Machine) SetProvisioned(id instance.Id, nonce string, characteristics *
 
 	if err = m.st.runTransaction(ops); err == nil {
 		m.doc.Nonce = nonce
-		// SCHEMACHANGE
-		// TODO(wallyworld) - remove this backward compatibility code when schema upgrades are possible
-		// (InstanceId is stored on the instanceData document but we duplicate the value on the machineDoc.
-		m.doc.InstanceId = id
 		return nil
 	} else if err != txn.ErrAborted {
 		return err
-	} else if alive, err := isAlive(m.st.db, machinesC, m.doc.Id); err != nil {
+	} else if alive, err := isAlive(m.st, machinesC, m.doc.DocID); err != nil {
 		return err
 	} else if !alive {
 		return errNotAlive
@@ -878,7 +913,7 @@ func (m *Machine) SetInstanceInfo(
 			// Ignore already existing networks.
 			continue
 		} else if err != nil {
-			return err
+			return errors.Trace(err)
 		}
 	}
 	for _, iface := range interfaces {
@@ -887,37 +922,18 @@ func (m *Machine) SetInstanceInfo(
 			// Ignore already existing network interfaces.
 			continue
 		} else if err != nil {
-			return err
+			return errors.Trace(err)
 		}
 	}
 	return m.SetProvisioned(id, nonce, characteristics)
 }
 
-// notProvisionedError records an error when a machine is not provisioned.
-type notProvisionedError struct {
-	machineId string
-}
-
-func NotProvisionedError(machineId string) error {
-	return &notProvisionedError{machineId}
-}
-
-func (e *notProvisionedError) Error() string {
-	return fmt.Sprintf("machine %v is not provisioned", e.machineId)
-}
-
-// IsNotProvisionedError returns true if err is a notProvisionedError.
-func IsNotProvisionedError(err error) bool {
-	_, ok := err.(*notProvisionedError)
-	return ok
-}
-
 func mergedAddresses(machineAddresses, providerAddresses []address) []network.Address {
 	merged := make([]network.Address, 0, len(providerAddresses)+len(machineAddresses))
-	var providerValues set.Strings
+	providerValues := set.NewStrings()
 	for _, address := range providerAddresses {
 		// Older versions of Juju may have stored an empty address so ignore it here.
-		if address.Value == "" {
+		if address.Value == "" || providerValues.Contains(address.Value) {
 			continue
 		}
 		providerValues.Add(address.Value)
@@ -935,8 +951,9 @@ func mergedAddresses(machineAddresses, providerAddresses []address) []network.Ad
 // determined both by the machine itself, and by asking the provider.
 //
 // The addresses returned by the provider shadow any of the addresses
-// that the machine reported with the same address value. Provider-reported
-// addresses always come before machine-reported addresses.
+// that the machine reported with the same address value.
+// Provider-reported addresses always come before machine-reported
+// addresses. Duplicates are removed.
 func (m *Machine) Addresses() (addresses []network.Address) {
 	return mergedAddresses(m.doc.MachineAddresses, m.doc.Addresses)
 }
@@ -972,6 +989,11 @@ func (m *Machine) SetMachineAddresses(addresses ...network.Address) (err error)
 // MachineAddresses, depending on the field argument).
 func (m *Machine) setAddresses(addresses []network.Address, field *[]address, fieldName string) error {
 	var changed bool
+	envConfig, err := m.st.EnvironConfig()
+	if err != nil {
+		return err
+	}
+	network.SortAddresses(addresses, envConfig.PreferIPv6())
 	stateAddresses := instanceAddressesToAddresses(addresses)
 	buildTxn := func(attempt int) ([]txn.Op, error) {
 		changed = false
@@ -985,7 +1007,7 @@ func (m *Machine) setAddresses(addresses []network.Address, field *[]address, fi
 		}
 		op := txn.Op{
 			C:      machinesC,
-			Id:     m.doc.Id,
+			Id:     m.doc.DocID,
 			Assert: append(bson.D{{fieldName, *field}}, notDeadDoc...),
 		}
 		if !addressesEqual(addresses, addressesToInstanceAddresses(*field)) {
@@ -1027,7 +1049,7 @@ func (m *Machine) Networks() ([]*Network, error) {
 	networksCollection, closer := m.st.getCollection(networksC)
 	defer closer()
 
-	sel := bson.D{{"_id", bson.D{{"$in", requestedNetworks}}}}
+	sel := bson.D{{"name", bson.D{{"$in", requestedNetworks}}}}
 	err = networksCollection.Find(sel).All(&docs)
 	if err != nil {
 		return nil, err
@@ -1064,7 +1086,7 @@ func (m *Machine) NetworkInterfaces() ([]*NetworkInterface, error) {
 // this to succeed. If a network interface already exists, the
 // returned error satisfies errors.IsAlreadyExists.
 func (m *Machine) AddNetworkInterface(args NetworkInterfaceInfo) (iface *NetworkInterface, err error) {
-	defer errors.Contextf(&err, "cannot add network interface %q to machine %q", args.InterfaceName, m.doc.Id)
+	defer errors.DeferredAnnotatef(&err, "cannot add network interface %q to machine %q", args.InterfaceName, m.doc.Id)
 
 	if args.MACAddress == "" {
 		return nil, fmt.Errorf("MAC address must be not empty")
@@ -1075,18 +1097,15 @@ func (m *Machine) AddNetworkInterface(args NetworkInterfaceInfo) (iface *Network
 	if args.InterfaceName == "" {
 		return nil, fmt.Errorf("interface name must be not empty")
 	}
-	aliveAndNotProvisioned := append(isAliveDoc, bson.D{{"nonce", ""}}...)
-	doc := newNetworkInterfaceDoc(args)
-	doc.MachineId = m.doc.Id
-	doc.Id = bson.NewObjectId()
+	doc := newNetworkInterfaceDoc(m.doc.Id, m.st.EnvironUUID(), args)
 	ops := []txn.Op{{
 		C:      networksC,
-		Id:     args.NetworkName,
+		Id:     m.st.docID(args.NetworkName),
 		Assert: txn.DocExists,
 	}, {
 		C:      machinesC,
-		Id:     m.doc.Id,
-		Assert: aliveAndNotProvisioned,
+		Id:     m.doc.DocID,
+		Assert: isAliveDoc,
 	}, {
 		C:      networkInterfacesC,
 		Id:     doc.Id,
@@ -1104,9 +1123,6 @@ func (m *Machine) AddNetworkInterface(args NetworkInterfaceInfo) (iface *Network
 			return nil, err
 		} else if m.doc.Life != Alive {
 			return nil, fmt.Errorf("machine is not alive")
-		} else if m.doc.Nonce != "" {
-			msg := "machine already provisioned: dynamic network interfaces not currently supported"
-			return nil, fmt.Errorf(msg)
 		}
 		// Should never happen.
 		logger.Errorf("unhandled assert while adding network interface doc %#v", doc)
@@ -1165,7 +1181,7 @@ func (m *Machine) Constraints() (constraints.Value, error) {
 // instance for the machine. It will fail if the machine is Dead, or if it
 // is already provisioned.
 func (m *Machine) SetConstraints(cons constraints.Value) (err error) {
-	defer errors.Maskf(&err, "cannot set constraints")
+	defer errors.DeferredAnnotatef(&err, "cannot set constraints")
 	unsupported, err := m.st.validateConstraints(cons)
 	if len(unsupported) > 0 {
 		logger.Warningf(
@@ -1177,7 +1193,7 @@ func (m *Machine) SetConstraints(cons constraints.Value) (err error) {
 	ops := []txn.Op{
 		{
 			C:      machinesC,
-			Id:     m.doc.Id,
+			Id:     m.doc.DocID,
 			Assert: append(isAliveDoc, notSetYet...),
 		},
 		setConstraintsOp(m.st, m.globalKey(), cons),
@@ -1198,7 +1214,7 @@ func (m *Machine) SetConstraints(cons constraints.Value) (err error) {
 		}
 		if _, err := m.InstanceId(); err == nil {
 			return nil, fmt.Errorf("machine is already provisioned")
-		} else if !IsNotProvisionedError(err) {
+		} else if !errors.IsNotProvisioned(err) {
 			return nil, err
 		}
 		return ops, nil
@@ -1207,7 +1223,7 @@ func (m *Machine) SetConstraints(cons constraints.Value) (err error) {
 }
 
 // Status returns the status of the machine.
-func (m *Machine) Status() (status params.Status, info string, data params.StatusData, err error) {
+func (m *Machine) Status() (status Status, info string, data map[string]interface{}, err error) {
 	doc, err := getStatus(m.st, m.globalKey())
 	if err != nil {
 		return "", "", nil, err
@@ -1219,27 +1235,23 @@ func (m *Machine) Status() (status params.Status, info string, data params.Statu
 }
 
 // SetStatus sets the status of the machine.
-func (m *Machine) SetStatus(status params.Status, info string, data params.StatusData) error {
-	doc := statusDoc{
-		Status:     status,
-		StatusInfo: info,
-		StatusData: data,
-	}
+func (m *Machine) SetStatus(status Status, info string, data map[string]interface{}) error {
 	// If a machine is not yet provisioned, we allow its status
 	// to be set back to pending (when a retry is to occur).
 	_, err := m.InstanceId()
-	allowPending := IsNotProvisionedError(err)
-	if err := doc.validateSet(allowPending); err != nil {
+	allowPending := errors.IsNotProvisioned(err)
+	doc, err := newMachineStatusDoc(status, info, data, allowPending)
+	if err != nil {
 		return err
 	}
 	ops := []txn.Op{{
 		C:      machinesC,
-		Id:     m.doc.Id,
+		Id:     m.doc.DocID,
 		Assert: notDeadDoc,
 	},
-		updateStatusOp(m.st, m.globalKey(), doc),
+		updateStatusOp(m.st, m.globalKey(), doc.statusDoc),
 	}
-	if err := m.st.runTransaction(ops); err != nil {
+	if err = m.st.runTransaction(ops); err != nil {
 		return fmt.Errorf("cannot set status of machine %q: %v", m, onAbort(err, errNotAlive))
 	}
 	return nil
@@ -1294,7 +1306,7 @@ func (m *Machine) updateSupportedContainers(supportedContainers []instance.Conta
 	ops := []txn.Op{
 		{
 			C:      machinesC,
-			Id:     m.doc.Id,
+			Id:     m.doc.DocID,
 			Assert: notDeadDoc,
 			Update: bson.D{
 				{"$set", bson.D{
@@ -1334,10 +1346,10 @@ func (m *Machine) markInvalidContainers() error {
 				logger.Errorf("finding status of container %v to mark as invalid: %v", containerId, err)
 				continue
 			}
-			if status == params.StatusPending {
+			if status == StatusPending {
 				containerType := ContainerTypeFromId(containerId)
 				container.SetStatus(
-					params.StatusError, "unsupported container", params.StatusData{"type": containerType})
+					StatusError, "unsupported container", map[string]interface{}{"type": containerType})
 			} else {
 				logger.Errorf("unsupported container %v has unexpected status %v", containerId, status)
 			}
@@ -1345,3 +1357,22 @@ func (m *Machine) markInvalidContainers() error {
 	}
 	return nil
 }
+
+// SetMachineBlockDevices sets the block devices visible on the machine.
+func (m *Machine) SetMachineBlockDevices(info ...BlockDeviceInfo) error {
+	return setMachineBlockDevices(m.st, m.Id(), info)
+}
+
+// BlockDevices gets the aggregated list of block devices attached to the
+// machine.
+func (m *Machine) BlockDevices() ([]BlockDevice, error) {
+	devices, err := getMachineBlockDevices(m.st, m.Id())
+	if err != nil {
+		return nil, err
+	}
+	result := make([]BlockDevice, len(devices))
+	for i, dev := range devices {
+		result[i] = dev
+	}
+	return result, nil
+}
