@@ -11,7 +11,7 @@ import (
 	"github.com/juju/errors"
 	"gopkg.in/mgo.v2"
 
-	"github.com/juju/juju/state/api/params"
+	"github.com/juju/juju/network"
 	"github.com/juju/juju/state/multiwatcher"
 	"github.com/juju/juju/state/watcher"
 )
@@ -26,15 +26,17 @@ type allWatcherStateBacking struct {
 
 type backingMachine machineDoc
 
-func (m *backingMachine) updated(st *State, store *multiwatcher.Store, id interface{}) error {
-	info := &params.MachineInfo{
+func (m *backingMachine) updated(st *State, store *multiwatcherStore, id interface{}) error {
+	info := &multiwatcher.MachineInfo{
 		Id:                       m.Id,
-		Life:                     params.Life(m.Life.String()),
+		Life:                     multiwatcher.Life(m.Life.String()),
 		Series:                   m.Series,
 		Jobs:                     paramsJobsFromJobs(m.Jobs),
 		Addresses:                mergedAddresses(m.MachineAddresses, m.Addresses),
 		SupportedContainers:      m.SupportedContainers,
 		SupportedContainersKnown: m.SupportedContainersKnown,
+		HasVote:                  m.HasVote,
+		WantsVote:                wantsVote(m.Jobs, m.NoVote),
 	}
 
 	oldInfo := store.Get(info.EntityId())
@@ -45,12 +47,12 @@ func (m *backingMachine) updated(st *State, store *multiwatcher.Store, id interf
 		if err != nil {
 			return err
 		}
-		info.Status = sdoc.Status
+		info.Status = multiwatcher.Status(sdoc.Status)
 		info.StatusInfo = sdoc.StatusInfo
 	} else {
 		// The entry already exists, so preserve the current status and
 		// instance data.
-		oldInfo := oldInfo.(*params.MachineInfo)
+		oldInfo := oldInfo.(*multiwatcher.MachineInfo)
 		info.Status = oldInfo.Status
 		info.StatusInfo = oldInfo.StatusInfo
 		info.InstanceId = oldInfo.InstanceId
@@ -71,27 +73,83 @@ func (m *backingMachine) updated(st *State, store *multiwatcher.Store, id interf
 	return nil
 }
 
-func (svc *backingMachine) removed(st *State, store *multiwatcher.Store, id interface{}) error {
-	store.Remove(params.EntityId{
+func (m *backingMachine) removed(st *State, store *multiwatcherStore, id interface{}) {
+	// TODO(mjs) This isn't correct - the store should be using
+	// environment UUID prefixed ids but we can't fix it properly
+	// until davecheney smashes the allwatcher to apiserver/params
+	// dependency.
+	store.Remove(multiwatcher.EntityId{
 		Kind: "machine",
-		Id:   id,
+		Id:   st.localID(id.(string)),
 	})
-	return nil
 }
 
 func (m *backingMachine) mongoId() interface{} {
-	return m.Id
+	return m.DocID
 }
 
 type backingUnit unitDoc
 
-func (u *backingUnit) updated(st *State, store *multiwatcher.Store, id interface{}) error {
-	info := &params.UnitInfo{
-		Name:      u.Name,
-		Service:   u.Service,
-		Series:    u.Series,
-		MachineId: u.MachineId,
-		Ports:     u.Ports,
+// translateLegacyUnitAgentStatus returns the status value the current GUI expects to see.
+// This is a short term requirement until the GUI is updated to be able to handle
+// the new values.
+// We use string literals here to avoid import loops and lots of messy refactoring.
+func translateLegacyUnitAgentStatus(in multiwatcher.Status) multiwatcher.Status {
+	switch in {
+	case multiwatcher.Status("failed"):
+		return multiwatcher.Status("down")
+	case multiwatcher.Status("active"):
+		return multiwatcher.Status("started")
+	}
+	return in
+}
+
+func getUnitPortRangesAndPorts(st *State, unitName string) ([]network.PortRange, []network.Port, error) {
+	// Get opened port ranges for the unit and convert them to ports,
+	// as older clients/servers do not know about ranges). See bug
+	// http://pad.lv/1418344 for more info.
+	unit, err := st.Unit(unitName)
+	if errors.IsNotFound(err) {
+		// Empty slices ensure backwards compatibility with older clients.
+		// See Bug #1425435.
+		return []network.PortRange{}, []network.Port{}, nil
+	} else if err != nil {
+		return nil, nil, errors.Annotatef(err, "failed to get unit %q", unitName)
+	}
+	portRanges, err := unit.OpenedPorts()
+	// Since the port ranges are associated with the unit's machine,
+	// we need to check for NotAssignedError.
+	if IsNotAssigned(err) {
+		// Not assigned, so there won't be any ports opened.
+		// Empty slices ensure backwards compatibility with older clients.
+		// See Bug #1425435.
+		return []network.PortRange{}, []network.Port{}, nil
+	} else if err != nil {
+		return nil, nil, errors.Annotate(err, "failed to get unit port ranges")
+	}
+	// For backward compatibility, if there are no ports opened, return an
+	// empty slice rather than a nil slice. Use a len(portRanges) capacity to
+	// avoid unnecessary allocations, since most of the times only specific
+	// ports are opened by charms.
+	compatiblePorts := make([]network.Port, 0, len(portRanges))
+	for _, portRange := range portRanges {
+		for j := portRange.FromPort; j <= portRange.ToPort; j++ {
+			compatiblePorts = append(compatiblePorts, network.Port{
+				Number:   j,
+				Protocol: portRange.Protocol,
+			})
+		}
+	}
+	return portRanges, compatiblePorts, nil
+}
+
+func (u *backingUnit) updated(st *State, store *multiwatcherStore, id interface{}) error {
+	info := &multiwatcher.UnitInfo{
+		Name:        u.Name,
+		Service:     u.Service,
+		Series:      u.Series,
+		MachineId:   u.MachineId,
+		Subordinate: u.Principal != "",
 	}
 	if u.CharmURL != nil {
 		info.CharmURL = u.CharmURL.String()
@@ -99,18 +157,28 @@ func (u *backingUnit) updated(st *State, store *multiwatcher.Store, id interface
 	oldInfo := store.Get(info.EntityId())
 	if oldInfo == nil {
 		// We're adding the entry for the first time,
-		// so fetch the associated unit status.
+		// so fetch the associated unit status and opened ports.
 		sdoc, err := getStatus(st, unitGlobalKey(u.Name))
 		if err != nil {
 			return err
 		}
-		info.Status = sdoc.Status
+		info.Status = multiwatcher.Status(sdoc.Status)
+		info.Status = translateLegacyUnitAgentStatus(info.Status)
 		info.StatusInfo = sdoc.StatusInfo
+		portRanges, compatiblePorts, err := getUnitPortRangesAndPorts(st, u.Name)
+		if err != nil {
+			return errors.Trace(err)
+		}
+		info.PortRanges = portRanges
+		info.Ports = compatiblePorts
+
 	} else {
-		// The entry already exists, so preserve the current status.
-		oldInfo := oldInfo.(*params.UnitInfo)
+		// The entry already exists, so preserve the current status and ports.
+		oldInfo := oldInfo.(*multiwatcher.UnitInfo)
 		info.Status = oldInfo.Status
 		info.StatusInfo = oldInfo.StatusInfo
+		info.Ports = oldInfo.Ports
+		info.PortRanges = oldInfo.PortRanges
 	}
 	publicAddress, privateAddress, err := getUnitAddresses(st, u.Name)
 	if err != nil {
@@ -127,7 +195,10 @@ func (u *backingUnit) updated(st *State, store *multiwatcher.Store, id interface
 // this approach for backwards compatibility.
 func getUnitAddresses(st *State, unitName string) (publicAddress, privateAddress string, err error) {
 	u, err := st.Unit(unitName)
-	if err != nil {
+	if errors.IsNotFound(err) {
+		// Not found, so there won't be any addresses.
+		return "", "", nil
+	} else if err != nil {
 		return "", "", err
 	}
 	publicAddress, _ = u.PublicAddress()
@@ -135,29 +206,36 @@ func getUnitAddresses(st *State, unitName string) (publicAddress, privateAddress
 	return publicAddress, privateAddress, nil
 }
 
-func (svc *backingUnit) removed(st *State, store *multiwatcher.Store, id interface{}) error {
-	store.Remove(params.EntityId{
+func (u *backingUnit) removed(st *State, store *multiwatcherStore, id interface{}) {
+	// TODO(mjs) as per backingMachine.removed()
+	store.Remove(multiwatcher.EntityId{
 		Kind: "unit",
-		Id:   id,
+		Id:   st.localID(id.(string)),
 	})
-	return nil
 }
 
-func (m *backingUnit) mongoId() interface{} {
-	return m.Name
+func (u *backingUnit) mongoId() interface{} {
+	return u.DocID
 }
 
 type backingService serviceDoc
 
-func (svc *backingService) updated(st *State, store *multiwatcher.Store, id interface{}) error {
-
-	info := &params.ServiceInfo{
-		Name:     svc.Name,
-		Exposed:  svc.Exposed,
-		CharmURL: svc.CharmURL.String(),
-		OwnerTag: svc.fixOwnerTag(),
-		Life:     params.Life(svc.Life.String()),
-		MinUnits: svc.MinUnits,
+func (svc *backingService) updated(st *State, store *multiwatcherStore, id interface{}) error {
+	if svc.CharmURL == nil {
+		return errors.Errorf("charm url is nil")
+	}
+	env, err := st.Environment()
+	if err != nil {
+		return errors.Trace(err)
+	}
+	info := &multiwatcher.ServiceInfo{
+		Name:        svc.Name,
+		Exposed:     svc.Exposed,
+		CharmURL:    svc.CharmURL.String(),
+		OwnerTag:    svc.fixOwnerTag(env),
+		Life:        multiwatcher.Life(svc.Life.String()),
+		MinUnits:    svc.MinUnits,
+		Subordinate: svc.Subordinate,
 	}
 	oldInfo := store.Get(info.EntityId())
 	needConfig := false
@@ -172,7 +250,7 @@ func (svc *backingService) updated(st *State, store *multiwatcher.Store, id inte
 		needConfig = true
 	} else {
 		// The entry already exists, so preserve the current status.
-		oldInfo := oldInfo.(*params.ServiceInfo)
+		oldInfo := oldInfo.(*multiwatcher.ServiceInfo)
 		info.Constraints = oldInfo.Constraints
 		if info.CharmURL == oldInfo.CharmURL {
 			// The charm URL remains the same - we can continue to
@@ -195,38 +273,38 @@ func (svc *backingService) updated(st *State, store *multiwatcher.Store, id inte
 	return nil
 }
 
-func (svc *backingService) removed(st *State, store *multiwatcher.Store, id interface{}) error {
-	store.Remove(params.EntityId{
+func (svc *backingService) removed(st *State, store *multiwatcherStore, id interface{}) {
+	// TODO(mjs) as per backingMachine.removed()
+	store.Remove(multiwatcher.EntityId{
 		Kind: "service",
-		Id:   id,
+		Id:   st.localID(id.(string)),
 	})
-	return nil
 }
 
 // SCHEMACHANGE
 // TODO(mattyw) remove when schema upgrades are possible
-func (svc *backingService) fixOwnerTag() string {
+func (svc *backingService) fixOwnerTag(env *Environment) string {
 	if svc.OwnerTag != "" {
 		return svc.OwnerTag
 	}
-	return "user-admin"
+	return env.Owner().String()
 }
 
-func (m *backingService) mongoId() interface{} {
-	return m.Name
+func (svc *backingService) mongoId() interface{} {
+	return svc.DocID
 }
 
 type backingRelation relationDoc
 
-func (r *backingRelation) updated(st *State, store *multiwatcher.Store, id interface{}) error {
-	eps := make([]params.Endpoint, len(r.Endpoints))
+func (r *backingRelation) updated(st *State, store *multiwatcherStore, id interface{}) error {
+	eps := make([]multiwatcher.Endpoint, len(r.Endpoints))
 	for i, ep := range r.Endpoints {
-		eps[i] = params.Endpoint{
+		eps[i] = multiwatcher.Endpoint{
 			ServiceName: ep.ServiceName,
 			Relation:    ep.Relation,
 		}
 	}
-	info := &params.RelationInfo{
+	info := &multiwatcher.RelationInfo{
 		Key:       r.Key,
 		Id:        r.Id,
 		Endpoints: eps,
@@ -235,22 +313,22 @@ func (r *backingRelation) updated(st *State, store *multiwatcher.Store, id inter
 	return nil
 }
 
-func (svc *backingRelation) removed(st *State, store *multiwatcher.Store, id interface{}) error {
-	store.Remove(params.EntityId{
+func (r *backingRelation) removed(st *State, store *multiwatcherStore, id interface{}) {
+	// TODO(mjs) as per backingMachine.removed()
+	store.Remove(multiwatcher.EntityId{
 		Kind: "relation",
-		Id:   id,
+		Id:   st.localID(id.(string)),
 	})
-	return nil
 }
 
-func (m *backingRelation) mongoId() interface{} {
-	return m.Key
+func (r *backingRelation) mongoId() interface{} {
+	return r.DocID
 }
 
 type backingAnnotation annotatorDoc
 
-func (a *backingAnnotation) updated(st *State, store *multiwatcher.Store, id interface{}) error {
-	info := &params.AnnotationInfo{
+func (a *backingAnnotation) updated(st *State, store *multiwatcherStore, id interface{}) error {
+	info := &multiwatcher.AnnotationInfo{
 		Tag:         a.Tag,
 		Annotations: a.Annotations,
 	}
@@ -258,16 +336,16 @@ func (a *backingAnnotation) updated(st *State, store *multiwatcher.Store, id int
 	return nil
 }
 
-func (svc *backingAnnotation) removed(st *State, store *multiwatcher.Store, id interface{}) error {
-	tag, ok := tagForGlobalKey(id.(string))
+func (a *backingAnnotation) removed(st *State, store *multiwatcherStore, id interface{}) {
+	localID := st.localID(id.(string))
+	tag, ok := tagForGlobalKey(localID)
 	if !ok {
-		panic(fmt.Errorf("unknown global key %q in state", id))
+		panic(fmt.Errorf("unknown global key %q in state", localID))
 	}
-	store.Remove(params.EntityId{
+	store.Remove(multiwatcher.EntityId{
 		Kind: "annotation",
 		Id:   tag,
 	})
-	return nil
 }
 
 func (a *backingAnnotation) mongoId() interface{} {
@@ -276,8 +354,8 @@ func (a *backingAnnotation) mongoId() interface{} {
 
 type backingStatus statusDoc
 
-func (s *backingStatus) updated(st *State, store *multiwatcher.Store, id interface{}) error {
-	parentId, ok := backingEntityIdForGlobalKey(id.(string))
+func (s *backingStatus) updated(st *State, store *multiwatcherStore, id interface{}) error {
+	parentId, ok := backingEntityIdForGlobalKey(st.localID(id.(string)))
 	if !ok {
 		return nil
 	}
@@ -286,15 +364,16 @@ func (s *backingStatus) updated(st *State, store *multiwatcher.Store, id interfa
 	case nil:
 		// The parent info doesn't exist. Ignore the status until it does.
 		return nil
-	case *params.UnitInfo:
+	case *multiwatcher.UnitInfo:
 		newInfo := *info
-		newInfo.Status = s.Status
+		newInfo.Status = multiwatcher.Status(s.Status)
+		newInfo.Status = translateLegacyUnitAgentStatus(newInfo.Status)
 		newInfo.StatusInfo = s.StatusInfo
 		newInfo.StatusData = s.StatusData
 		info0 = &newInfo
-	case *params.MachineInfo:
+	case *multiwatcher.MachineInfo:
 		newInfo := *info
-		newInfo.Status = s.Status
+		newInfo.Status = multiwatcher.Status(s.Status)
 		newInfo.StatusInfo = s.StatusInfo
 		newInfo.StatusData = s.StatusData
 		info0 = &newInfo
@@ -305,20 +384,20 @@ func (s *backingStatus) updated(st *State, store *multiwatcher.Store, id interfa
 	return nil
 }
 
-func (s *backingStatus) removed(st *State, store *multiwatcher.Store, id interface{}) error {
+func (s *backingStatus) removed(st *State, store *multiwatcherStore, id interface{}) {
 	// If the status is removed, the parent will follow not long after,
 	// so do nothing.
-	return nil
 }
 
-func (a *backingStatus) mongoId() interface{} {
+func (s *backingStatus) mongoId() interface{} {
 	panic("cannot find mongo id from status document")
 }
 
 type backingConstraints constraintsDoc
 
-func (s *backingConstraints) updated(st *State, store *multiwatcher.Store, id interface{}) error {
-	parentId, ok := backingEntityIdForGlobalKey(id.(string))
+func (c *backingConstraints) updated(st *State, store *multiwatcherStore, id interface{}) error {
+	localID := st.localID(id.(string))
+	parentId, ok := backingEntityIdForGlobalKey(localID)
 	if !ok {
 		return nil
 	}
@@ -327,32 +406,31 @@ func (s *backingConstraints) updated(st *State, store *multiwatcher.Store, id in
 	case nil:
 		// The parent info doesn't exist. Ignore the status until it does.
 		return nil
-	case *params.UnitInfo, *params.MachineInfo:
+	case *multiwatcher.UnitInfo, *multiwatcher.MachineInfo:
 		// We don't (yet) publish unit or machine constraints.
 		return nil
-	case *params.ServiceInfo:
+	case *multiwatcher.ServiceInfo:
 		newInfo := *info
-		newInfo.Constraints = constraintsDoc(*s).value()
+		newInfo.Constraints = constraintsDoc(*c).value()
 		info0 = &newInfo
 	default:
-		panic(fmt.Errorf("status for unexpected entity with id %q; type %T", id, info))
+		panic(fmt.Errorf("status for unexpected entity with id %q; type %T", localID, info))
 	}
 	store.Update(info0)
 	return nil
 }
 
-func (s *backingConstraints) removed(st *State, store *multiwatcher.Store, id interface{}) error {
-	return nil
-}
+func (c *backingConstraints) removed(st *State, store *multiwatcherStore, id interface{}) {}
 
-func (a *backingConstraints) mongoId() interface{} {
+func (c *backingConstraints) mongoId() interface{} {
 	panic("cannot find mongo id from constraints document")
 }
 
 type backingSettings map[string]interface{}
 
-func (s *backingSettings) updated(st *State, store *multiwatcher.Store, id interface{}) error {
-	parentId, url, ok := backingEntityIdForSettingsKey(id.(string))
+func (s *backingSettings) updated(st *State, store *multiwatcherStore, id interface{}) error {
+	localID := st.localID(id.(string))
+	parentId, url, ok := backingEntityIdForSettingsKey(localID)
 	if !ok {
 		return nil
 	}
@@ -361,7 +439,7 @@ func (s *backingSettings) updated(st *State, store *multiwatcher.Store, id inter
 	case nil:
 		// The parent info doesn't exist. Ignore the status until it does.
 		return nil
-	case *params.ServiceInfo:
+	case *multiwatcher.ServiceInfo:
 		// If we're seeing settings for the service with a different
 		// charm URL, we ignore them - we will fetch
 		// them again when the service charm changes.
@@ -381,18 +459,16 @@ func (s *backingSettings) updated(st *State, store *multiwatcher.Store, id inter
 	return nil
 }
 
-func (s *backingSettings) removed(st *State, store *multiwatcher.Store, id interface{}) error {
-	return nil
-}
+func (s *backingSettings) removed(st *State, store *multiwatcherStore, id interface{}) {}
 
-func (a *backingSettings) mongoId() interface{} {
+func (s *backingSettings) mongoId() interface{} {
 	panic("cannot find mongo id from settings document")
 }
 
 // backingEntityIdForSettingsKey returns the entity id for the given
 // settings key. Any extra information in the key is returned in
 // extra.
-func backingEntityIdForSettingsKey(key string) (eid params.EntityId, extra string, ok bool) {
+func backingEntityIdForSettingsKey(key string) (eid multiwatcher.EntityId, extra string, ok bool) {
 	if !strings.HasPrefix(key, "s#") {
 		eid, ok = backingEntityIdForGlobalKey(key)
 		return
@@ -400,30 +476,104 @@ func backingEntityIdForSettingsKey(key string) (eid params.EntityId, extra strin
 	key = key[2:]
 	i := strings.Index(key, "#")
 	if i == -1 {
-		return params.EntityId{}, "", false
+		return multiwatcher.EntityId{}, "", false
 	}
-	eid = (&params.ServiceInfo{Name: key[0:i]}).EntityId()
+	eid = (&multiwatcher.ServiceInfo{Name: key[0:i]}).EntityId()
 	extra = key[i+1:]
 	ok = true
 	return
 }
 
+type backingOpenedPorts map[string]interface{}
+
+func (p *backingOpenedPorts) updated(st *State, store *multiwatcherStore, id interface{}) error {
+	localID := st.localID(id.(string))
+	parentId, ok := backingEntityIdForOpenedPortsKey(localID)
+	if !ok {
+		return nil
+	}
+	switch info := store.Get(parentId).(type) {
+	case nil:
+		// The parent info doesn't exist. This is unexpected because the port
+		// always refers to a machine. Anyway, ignore the ports for now.
+		return nil
+	case *multiwatcher.MachineInfo:
+		// Retrieve the units placed in the machine.
+		units, err := st.UnitsFor(info.Id)
+		if err != nil {
+			return errors.Trace(err)
+		}
+		// Update the ports on all units assigned to the machine.
+		for _, u := range units {
+			if err := updateUnitPorts(st, store, u); err != nil {
+				return errors.Trace(err)
+			}
+		}
+	}
+	return nil
+}
+
+func (p *backingOpenedPorts) removed(st *State, store *multiwatcherStore, id interface{}) {}
+
+func (p *backingOpenedPorts) mongoId() interface{} {
+	panic("cannot find mongo id from openedPorts document")
+}
+
+// updateUnitPorts updates the Ports and PortRanges info of the given unit.
+func updateUnitPorts(st *State, store *multiwatcherStore, u *Unit) error {
+	eid, ok := backingEntityIdForGlobalKey(u.globalKey())
+	if !ok {
+		// This should never happen.
+		return errors.New("cannot retrieve entity id for unit")
+	}
+	switch oldInfo := store.Get(eid).(type) {
+	case nil:
+		// The unit info doesn't exist. This is unlikely to happen, but ignore
+		// the status until a unitInfo is included in the store.
+		return nil
+	case *multiwatcher.UnitInfo:
+		portRanges, compatiblePorts, err := getUnitPortRangesAndPorts(st, oldInfo.Name)
+		if err != nil {
+			return errors.Trace(err)
+		}
+		unitInfo := *oldInfo
+		unitInfo.PortRanges = portRanges
+		unitInfo.Ports = compatiblePorts
+		store.Update(&unitInfo)
+	default:
+		return nil
+	}
+	return nil
+}
+
+// backingEntityIdForOpenedPortsKey returns the entity id for the given
+// openedPorts key. Any extra information in the key is discarded.
+func backingEntityIdForOpenedPortsKey(key string) (multiwatcher.EntityId, bool) {
+	parts, err := extractPortsIdParts(key)
+	if err != nil {
+		logger.Debugf("cannot parse ports key %q: %v", key, err)
+		return multiwatcher.EntityId{}, false
+	}
+	return backingEntityIdForGlobalKey(machineGlobalKey(parts[1]))
+}
+
 // backingEntityIdForGlobalKey returns the entity id for the given global key.
 // It returns false if the key is not recognized.
-func backingEntityIdForGlobalKey(key string) (params.EntityId, bool) {
+func backingEntityIdForGlobalKey(key string) (multiwatcher.EntityId, bool) {
 	if len(key) < 3 || key[1] != '#' {
-		return params.EntityId{}, false
+		return multiwatcher.EntityId{}, false
 	}
 	id := key[2:]
 	switch key[0] {
 	case 'm':
-		return (&params.MachineInfo{Id: id}).EntityId(), true
+		return (&multiwatcher.MachineInfo{Id: id}).EntityId(), true
 	case 'u':
-		return (&params.UnitInfo{Name: id}).EntityId(), true
+		return (&multiwatcher.UnitInfo{Name: id}).EntityId(), true
 	case 's':
-		return (&params.ServiceInfo{Name: id}).EntityId(), true
+		return (&multiwatcher.ServiceInfo{Name: id}).EntityId(), true
+	default:
+		return multiwatcher.EntityId{}, false
 	}
-	return params.EntityId{}, false
 }
 
 // backingEntityDoc is implemented by the documents in
@@ -431,29 +581,18 @@ func backingEntityIdForGlobalKey(key string) (params.EntityId, bool) {
 type backingEntityDoc interface {
 	// updated is called when the document has changed.
 	// The mongo _id value of the document is provided in id.
-	updated(st *State, store *multiwatcher.Store, id interface{}) error
+	updated(st *State, store *multiwatcherStore, id interface{}) error
 
 	// removed is called when the document has changed.
 	// The receiving instance will not contain any data.
 	// The mongo _id value of the document is provided in id.
-	removed(st *State, store *multiwatcher.Store, id interface{}) error
+	removed(st *State, store *multiwatcherStore, id interface{})
 
 	// mongoId returns the mongo _id field of the document.
 	// It is currently never called for subsidiary documents.
 	mongoId() interface{}
 }
 
-var (
-	_ backingEntityDoc = (*backingMachine)(nil)
-	_ backingEntityDoc = (*backingUnit)(nil)
-	_ backingEntityDoc = (*backingService)(nil)
-	_ backingEntityDoc = (*backingRelation)(nil)
-	_ backingEntityDoc = (*backingAnnotation)(nil)
-	_ backingEntityDoc = (*backingStatus)(nil)
-	_ backingEntityDoc = (*backingConstraints)(nil)
-	_ backingEntityDoc = (*backingSettings)(nil)
-)
-
 // allWatcherStateCollection holds information about a
 // collection watched by an allWatcher and the
 // type of value we use to store entity information
@@ -469,7 +608,7 @@ type allWatcherStateCollection struct {
 	subsidiary bool
 }
 
-func newAllWatcherStateBacking(st *State) multiwatcher.Backing {
+func newAllWatcherStateBacking(st *State) Backing {
 	collectionByType := make(map[reflect.Type]allWatcherStateCollection)
 	b := &allWatcherStateBacking{
 		st:               st,
@@ -503,6 +642,10 @@ func newAllWatcherStateBacking(st *State) multiwatcher.Backing {
 		Collection: st.db.C(settingsC),
 		infoType:   reflect.TypeOf(backingSettings{}),
 		subsidiary: true,
+	}, {
+		Collection: st.db.C(openedPortsC),
+		infoType:   reflect.TypeOf(backingOpenedPorts{}),
+		subsidiary: true,
 	}}
 	// Populate the collection maps from the above set of collections.
 	for _, c := range collections {
@@ -534,7 +677,7 @@ func (b *allWatcherStateBacking) Unwatch(in chan<- watcher.Change) {
 }
 
 // GetAll fetches all items that we want to watch from the state.
-func (b *allWatcherStateBacking) GetAll(all *multiwatcher.Store) error {
+func (b *allWatcherStateBacking) GetAll(all *multiwatcherStore) error {
 	db, closer := b.st.newDB()
 	defer closer()
 
@@ -559,7 +702,7 @@ func (b *allWatcherStateBacking) GetAll(all *multiwatcher.Store) error {
 
 // Changed updates the allWatcher's idea of the current state
 // in response to the given change.
-func (b *allWatcherStateBacking) Changed(all *multiwatcher.Store, change watcher.Change) error {
+func (b *allWatcherStateBacking) Changed(all *multiwatcherStore, change watcher.Change) error {
 	db, closer := b.st.newDB()
 	defer closer()
 
@@ -569,13 +712,15 @@ func (b *allWatcherStateBacking) Changed(all *multiwatcher.Store, change watcher
 	}
 	col := db.C(c.Name)
 	doc := reflect.New(c.infoType).Interface().(backingEntityDoc)
+
 	// TODO(rog) investigate ways that this can be made more efficient
 	// than simply fetching each entity in turn.
 	// TODO(rog) avoid fetching documents that we have no interest
 	// in, such as settings changes to entities we don't care about.
 	err := col.FindId(change.Id).One(doc)
 	if err == mgo.ErrNotFound {
-		return doc.removed(b.st, all, change.Id)
+		doc.removed(b.st, all, change.Id)
+		return nil
 	}
 	if err != nil {
 		return err
