@@ -4,8 +4,9 @@
 package state
 
 import (
-	"github.com/juju/charm"
-	gc "launchpad.net/gocheck"
+	jc "github.com/juju/testing/checkers"
+	gc "gopkg.in/check.v1"
+	"gopkg.in/juju/charm.v4"
 )
 
 type RelationSuite struct{}
@@ -34,6 +35,6 @@ func (s *RelationSuite) TestRelatedEndpoints(c *gc.C) {
 	}}
 	r := &Relation{nil, relationDoc{Endpoints: eps}}
 	relatedEps, err := r.RelatedEndpoints("mike")
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(relatedEps, gc.DeepEquals, eps)
 }
