@@ -10,16 +10,15 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/juju/charm"
 	"github.com/juju/errors"
 	"github.com/juju/names"
 	jujutxn "github.com/juju/txn"
+	"gopkg.in/juju/charm.v4"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
 
 	"github.com/juju/juju/constraints"
-	"github.com/juju/juju/state/api/params"
 )
 
 // Service represents the state of a service.
@@ -30,21 +29,24 @@ type Service struct {
 }
 
 // serviceDoc represents the internal state of a service in MongoDB.
-// Note the correspondence with ServiceInfo in state/api/params.
+// Note the correspondence with ServiceInfo in apiserver/params.
 type serviceDoc struct {
-	Name          string `bson:"_id"`
-	Series        string
-	Subordinate   bool
-	CharmURL      *charm.URL
-	ForceCharm    bool
-	Life          Life
-	UnitSeq       int
-	UnitCount     int
-	RelationCount int
-	Exposed       bool
-	MinUnits      int
-	OwnerTag      string
-	TxnRevno      int64 `bson:"txn-revno"`
+	DocID             string     `bson:"_id"`
+	Name              string     `bson:"name"`
+	EnvUUID           string     `bson:"env-uuid"`
+	Series            string     `bson:"series"`
+	Subordinate       bool       `bson:"subordinate"`
+	CharmURL          *charm.URL `bson:"charmurl"`
+	ForceCharm        bool       `bson:forcecharm"`
+	Life              Life       `bson:"life"`
+	UnitSeq           int        `bson:"unitseq"`
+	UnitCount         int        `bson:"unitcount"`
+	RelationCount     int        `bson:"relationcount"`
+	Exposed           bool       `bson:"exposed"`
+	MinUnits          int        `bson:"minunits"`
+	OwnerTag          string     `bson:"ownertag"`
+	TxnRevno          int64      `bson:"txn-revno"`
+	MetricCredentials []byte     `bson:"metric-credentials"`
 }
 
 func newService(st *State, doc *serviceDoc) *Service {
@@ -54,7 +56,7 @@ func newService(st *State, doc *serviceDoc) *Service {
 	}
 	svc.annotator = annotator{
 		globalKey: svc.globalKey(),
-		tag:       svc.Tag().String(),
+		tag:       svc.Tag(),
 		st:        st,
 	}
 	return svc
@@ -104,7 +106,7 @@ var errRefresh = stderrors.New("state seems inconsistent, refresh and try again"
 // some point; if the service has no units, and no relation involving the
 // service has any units in scope, they are all removed immediately.
 func (s *Service) Destroy() (err error) {
-	defer errors.Maskf(&err, "cannot destroy service %q", s)
+	defer errors.DeferredAnnotatef(&err, "cannot destroy service %q", s)
 	defer func() {
 		if err == nil {
 			// This is a white lie; the document might actually be removed.
@@ -158,7 +160,7 @@ func (s *Service) destroyOps() ([]txn.Op, error) {
 		if err == errAlreadyDying {
 			relOps = []txn.Op{{
 				C:      relationsC,
-				Id:     rel.doc.Key,
+				Id:     rel.doc.DocID,
 				Assert: bson.D{{"life", Dying}},
 			}}
 		} else if err != nil {
@@ -191,7 +193,7 @@ func (s *Service) destroyOps() ([]txn.Op, error) {
 	// about is that *some* unit is, or is not, keeping the service from
 	// being removed: the difference between 1 unit and 1000 is irrelevant.
 	if s.doc.UnitCount > 0 {
-		ops = append(ops, s.st.newCleanupOp(cleanupUnitsForDyingService, s.doc.Name+"/"))
+		ops = append(ops, s.st.newCleanupOp(cleanupUnitsForDyingService, s.doc.Name))
 		notLastRefs = append(notLastRefs, bson.D{{"unitcount", bson.D{{"$gt", 0}}}}...)
 	} else {
 		notLastRefs = append(notLastRefs, bson.D{{"unitcount", 0}}...)
@@ -203,7 +205,7 @@ func (s *Service) destroyOps() ([]txn.Op, error) {
 	}
 	return append(ops, txn.Op{
 		C:      servicesC,
-		Id:     s.doc.Name,
+		Id:     s.doc.DocID,
 		Assert: notLastRefs,
 		Update: update,
 	}), nil
@@ -212,18 +214,19 @@ func (s *Service) destroyOps() ([]txn.Op, error) {
 // removeOps returns the operations required to remove the service. Supplied
 // asserts will be included in the operation on the service document.
 func (s *Service) removeOps(asserts bson.D) []txn.Op {
+	settingsDocID := s.st.docID(s.settingsKey())
 	ops := []txn.Op{{
 		C:      servicesC,
-		Id:     s.doc.Name,
+		Id:     s.doc.DocID,
 		Assert: asserts,
 		Remove: true,
 	}, {
 		C:      settingsrefsC,
-		Id:     s.settingsKey(),
+		Id:     settingsDocID,
 		Remove: true,
 	}, {
 		C:      settingsC,
-		Id:     s.settingsKey(),
+		Id:     settingsDocID,
 		Remove: true,
 	}}
 	ops = append(ops, removeRequestedNetworksOp(s.st, s.globalKey()))
@@ -253,7 +256,7 @@ func (s *Service) ClearExposed() error {
 func (s *Service) setExposed(exposed bool) (err error) {
 	ops := []txn.Op{{
 		C:      servicesC,
-		Id:     s.doc.Name,
+		Id:     s.doc.DocID,
 		Assert: isAliveDoc,
 		Update: bson.D{{"$set", bson.D{{"exposed", exposed}}}},
 	}}
@@ -363,7 +366,7 @@ func (s *Service) checkRelationsOps(ch *Charm, relations []*Relation) ([]txn.Op,
 		}
 		asserts = append(asserts, txn.Op{
 			C:      relationsC,
-			Id:     rel.doc.Key,
+			Id:     rel.doc.DocID,
 			Assert: txn.DocExists,
 		})
 	}
@@ -373,72 +376,80 @@ func (s *Service) checkRelationsOps(ch *Charm, relations []*Relation) ([]txn.Op,
 // changeCharmOps returns the operations necessary to set a service's
 // charm URL to a new value.
 func (s *Service) changeCharmOps(ch *Charm, force bool) ([]txn.Op, error) {
-	settings, closer := s.st.getCollection(settingsC)
-	defer closer()
-
 	// Build the new service config from what can be used of the old one.
+	var newSettings charm.Settings
 	oldSettings, err := readSettings(s.st, s.settingsKey())
-	if err != nil {
-		return nil, err
+	if err == nil {
+		// Filter the old settings through to get the new settings.
+		newSettings = ch.Config().FilterSettings(oldSettings.Map())
+	} else if errors.IsNotFound(err) {
+		// No old settings, start with empty new settings.
+		newSettings = make(charm.Settings)
+	} else {
+		return nil, errors.Trace(err)
 	}
-	newSettings := ch.Config().FilterSettings(oldSettings.Map())
 
 	// Create or replace service settings.
 	var settingsOp txn.Op
 	newKey := serviceSettingsKey(s.doc.Name, ch.URL())
-	if count, err := settings.FindId(newKey).Count(); err != nil {
-		return nil, err
-	} else if count == 0 {
+	if _, err := readSettings(s.st, newKey); errors.IsNotFound(err) {
 		// No settings for this key yet, create it.
 		settingsOp = createSettingsOp(s.st, newKey, newSettings)
+	} else if err != nil {
+		return nil, errors.Trace(err)
 	} else {
 		// Settings exist, just replace them with the new ones.
-		var err error
 		settingsOp, _, err = replaceSettingsOp(s.st, newKey, newSettings)
 		if err != nil {
-			return nil, err
+			return nil, errors.Trace(err)
 		}
 	}
 
 	// Add or create a reference to the new settings doc.
 	incOp, err := settingsIncRefOp(s.st, s.doc.Name, ch.URL(), true)
 	if err != nil {
-		return nil, err
+		return nil, errors.Trace(err)
 	}
-	// Drop the reference to the old settings doc.
-	decOps, err := settingsDecRefOps(s.st, s.doc.Name, s.doc.CharmURL) // current charm
-	if err != nil {
-		return nil, err
+	var decOps []txn.Op
+	// Drop the reference to the old settings doc (if they exist).
+	if oldSettings != nil {
+		decOps, err = settingsDecRefOps(s.st, s.doc.Name, s.doc.CharmURL) // current charm
+		if err != nil {
+			return nil, errors.Trace(err)
+		}
 	}
 
 	// Build the transaction.
+	var ops []txn.Op
 	differentCharm := bson.D{{"charmurl", bson.D{{"$ne", ch.URL()}}}}
-	ops := []txn.Op{
-		// Old settings shouldn't change
-		oldSettings.assertUnchangedOp(),
-		// Create/replace with new settings.
+	if oldSettings != nil {
+		// Old settings shouldn't change (when they exist).
+		ops = append(ops, oldSettings.assertUnchangedOp())
+	}
+	ops = append(ops, []txn.Op{
+		// Create or replace new settings.
 		settingsOp,
 		// Increment the ref count.
 		incOp,
 		// Update the charm URL and force flag (if relevant).
 		{
 			C:      servicesC,
-			Id:     s.doc.Name,
-			Assert: append(isAliveDoc, differentCharm...),
+			Id:     s.doc.DocID,
+			Assert: append(notDeadDoc, differentCharm...),
 			Update: bson.D{{"$set", bson.D{{"charmurl", ch.URL()}, {"forcecharm", force}}}},
 		},
-	}
+	}...)
 	// Add any extra peer relations that need creation.
 	newPeers := s.extraPeerRelations(ch.Meta())
 	peerOps, err := s.st.addPeerRelationsOps(s.doc.Name, newPeers)
 	if err != nil {
-		return nil, err
+		return nil, errors.Trace(err)
 	}
 
 	// Get all relations - we need to check them later.
 	relations, err := s.Relations()
 	if err != nil {
-		return nil, err
+		return nil, errors.Trace(err)
 	}
 	// Make sure the relation count does not change.
 	sameRelCount := bson.D{{"relationcount", len(relations)}}
@@ -447,14 +458,14 @@ func (s *Service) changeCharmOps(ch *Charm, force bool) ([]txn.Op, error) {
 	// Update the relation count as well.
 	ops = append(ops, txn.Op{
 		C:      servicesC,
-		Id:     s.doc.Name,
-		Assert: append(isAliveDoc, sameRelCount...),
+		Id:     s.doc.DocID,
+		Assert: append(notDeadDoc, sameRelCount...),
 		Update: bson.D{{"$inc", bson.D{{"relationcount", len(newPeers)}}}},
 	})
 	// Check relations to ensure no active relations are removed.
 	relOps, err := s.checkRelationsOps(ch, relations)
 	if err != nil {
-		return nil, err
+		return nil, errors.Trace(err)
 	}
 	ops = append(ops, relOps...)
 
@@ -465,54 +476,57 @@ func (s *Service) changeCharmOps(ch *Charm, force bool) ([]txn.Op, error) {
 // SetCharm changes the charm for the service. New units will be started with
 // this charm, and existing units will be upgraded to use it. If force is true,
 // units will be upgraded even if they are in an error state.
-func (s *Service) SetCharm(ch *Charm, force bool) (err error) {
-	services, closer := s.st.getCollection(servicesC)
-	defer closer()
-	settings := services.Database.C(settingsC)
-
+func (s *Service) SetCharm(ch *Charm, force bool) error {
 	if ch.Meta().Subordinate != s.doc.Subordinate {
-		return fmt.Errorf("cannot change a service's subordinacy")
+		return errors.Errorf("cannot change a service's subordinacy")
 	}
 	if ch.URL().Series != s.doc.Series {
-		return fmt.Errorf("cannot change a service's series")
+		return errors.Errorf("cannot change a service's series")
 	}
+
+	services, closer := s.st.getCollection(servicesC)
+	defer closer()
+
 	buildTxn := func(attempt int) ([]txn.Op, error) {
 		if attempt > 0 {
-			// If the service is not alive, fail out immediately; otherwise,
-			// data changed underneath us, so retry.
-			if alive, err := isAliveWithSession(settings, s.doc.Name); err != nil {
-				return nil, err
-			} else if !alive {
-				return nil, fmt.Errorf("service %q is not alive", s.doc.Name)
+			// NOTE: We're explicitly allowing SetCharm to succeed
+			// when the service is Dying, because service/charm
+			// upgrades should still be allowed to apply to dying
+			// services and units, so that bugs in departed/broken
+			// hooks can be addressed at runtime.
+			if notDead, err := isNotDeadWithSession(services, s.doc.DocID); err != nil {
+				return nil, errors.Trace(err)
+			} else if !notDead {
+				return nil, ErrDead
 			}
 		}
 		// Make sure the service doesn't have this charm already.
-		sel := bson.D{{"_id", s.doc.Name}, {"charmurl", ch.URL()}}
+		sel := bson.D{{"_id", s.doc.DocID}, {"charmurl", ch.URL()}}
 		var ops []txn.Op
 		if count, err := services.Find(sel).Count(); err != nil {
-			return nil, err
+			return nil, errors.Trace(err)
 		} else if count == 1 {
 			// Charm URL already set; just update the force flag.
 			sameCharm := bson.D{{"charmurl", ch.URL()}}
 			ops = []txn.Op{{
 				C:      servicesC,
-				Id:     s.doc.Name,
-				Assert: append(isAliveDoc, sameCharm...),
+				Id:     s.doc.DocID,
+				Assert: append(notDeadDoc, sameCharm...),
 				Update: bson.D{{"$set", bson.D{{"forcecharm", force}}}},
 			}}
 		} else {
 			// Change the charm URL.
 			ops, err = s.changeCharmOps(ch, force)
 			if err != nil {
-				return nil, err
+				return nil, errors.Trace(err)
 			}
 		}
 		return ops, nil
 	}
-	if err = s.st.run(buildTxn); err == nil {
+	err := s.st.run(buildTxn)
+	if err == nil {
 		s.doc.CharmURL = ch.URL()
 		s.doc.ForceCharm = force
-		return nil
 	}
 	return err
 }
@@ -529,7 +543,7 @@ func (s *Service) Refresh() error {
 	services, closer := s.st.getCollection(servicesC)
 	defer closer()
 
-	err := services.FindId(s.doc.Name).One(&s.doc)
+	err := services.FindId(s.doc.DocID).One(&s.doc)
 	if err == mgo.ErrNotFound {
 		return errors.NotFoundf("service %q", s)
 	}
@@ -546,7 +560,7 @@ func (s *Service) newUnitName() (string, error) {
 
 	change := mgo.Change{Update: bson.D{{"$inc", bson.D{{"unitseq", 1}}}}}
 	result := serviceDoc{}
-	if _, err := services.Find(bson.D{{"_id", s.doc.Name}}).Apply(change, &result); err == mgo.ErrNotFound {
+	if _, err := services.Find(bson.D{{"_id", s.doc.DocID}}).Apply(change, &result); err == mgo.ErrNotFound {
 		return "", errors.NotFoundf("service %q", s)
 	} else if err != nil {
 		return "", fmt.Errorf("cannot increment unit sequence: %v", err)
@@ -570,35 +584,40 @@ func (s *Service) addUnitOps(principalName string, asserts bson.D) (string, []tx
 	if err != nil {
 		return "", nil, err
 	}
+	docID := s.st.docID(name)
 	globalKey := unitGlobalKey(name)
 	udoc := &unitDoc{
+		DocID:     docID,
 		Name:      name,
+		EnvUUID:   s.doc.EnvUUID,
 		Service:   s.doc.Name,
 		Series:    s.doc.Series,
 		Life:      Alive,
 		Principal: principalName,
 	}
 	sdoc := statusDoc{
-		Status: params.StatusPending,
+		Status:  StatusAllocating,
+		EnvUUID: s.st.EnvironUUID(),
 	}
 	ops := []txn.Op{
 		{
 			C:      unitsC,
-			Id:     name,
+			Id:     docID,
 			Assert: txn.DocMissing,
 			Insert: udoc,
 		},
 		createStatusOp(s.st, globalKey, sdoc),
+		createMeterStatusOp(s.st, globalKey, &meterStatusDoc{Code: MeterNotSet}),
 		{
 			C:      servicesC,
-			Id:     s.doc.Name,
+			Id:     s.doc.DocID,
 			Assert: append(isAliveDoc, asserts...),
 			Update: bson.D{{"$inc", bson.D{{"unitcount", 1}}}},
 		}}
 	if s.doc.Subordinate {
 		ops = append(ops, txn.Op{
 			C:  unitsC,
-			Id: principalName,
+			Id: s.st.docID(principalName),
 			Assert: append(isAliveDoc, bson.DocElem{
 				"subordinates", bson.D{{"$not", bson.RegEx{Pattern: "^" + s.doc.Name + "/"}}},
 			}),
@@ -618,31 +637,27 @@ func (s *Service) addUnitOps(principalName string, asserts bson.D) (string, []tx
 	return name, ops, nil
 }
 
-// GetOwnerTag returns the owner of this service
-// SCHEMACHANGE
-// TODO(mattyw) remove when schema upgrades are possible
-func (s *serviceDoc) GetOwnerTag() string {
-	if s.OwnerTag != "" {
-		return s.OwnerTag
-	}
-	return "user-admin"
-}
-
 // SCHEMACHANGE
 // TODO(mattyw) remove when schema upgrades are possible
 func (s *Service) GetOwnerTag() string {
-	return s.doc.GetOwnerTag()
+	owner := s.doc.OwnerTag
+	if owner == "" {
+		// We know that if there was no owner, it was created with an early
+		// version of juju, and that admin was the only user.
+		owner = names.NewUserTag("admin").String()
+	}
+	return owner
 }
 
 // AddUnit adds a new principal unit to the service.
 func (s *Service) AddUnit() (unit *Unit, err error) {
-	defer errors.Maskf(&err, "cannot add unit to service %q", s)
+	defer errors.DeferredAnnotatef(&err, "cannot add unit to service %q", s)
 	name, ops, err := s.addUnitOps("", nil)
 	if err != nil {
 		return nil, err
 	}
 	if err := s.st.runTransaction(ops); err == txn.ErrAborted {
-		if alive, err := isAlive(s.st.db, servicesC, s.doc.Name); err != nil {
+		if alive, err := isAlive(s.st, servicesC, s.doc.DocID); err != nil {
 			return nil, err
 		} else if !alive {
 			return nil, fmt.Errorf("service is not alive")
@@ -651,43 +666,38 @@ func (s *Service) AddUnit() (unit *Unit, err error) {
 	} else if err != nil {
 		return nil, err
 	}
-	return s.Unit(name)
+	return s.st.Unit(name)
 }
 
 // removeUnitOps returns the operations necessary to remove the supplied unit,
 // assuming the supplied asserts apply to the unit document.
 func (s *Service) removeUnitOps(u *Unit, asserts bson.D) ([]txn.Op, error) {
-	var ops []txn.Op
-	if s.doc.Subordinate {
-		ops = append(ops, txn.Op{
-			C:      unitsC,
-			Id:     u.doc.Principal,
-			Assert: txn.DocExists,
-			Update: bson.D{{"$pull", bson.D{{"subordinates", u.doc.Name}}}},
-		})
-	} else if u.doc.MachineId != "" {
-		ops = append(ops, txn.Op{
-			C:      machinesC,
-			Id:     u.doc.MachineId,
-			Assert: txn.DocExists,
-			Update: bson.D{{"$pull", bson.D{{"principals", u.doc.Name}}}},
-		})
+	ops, err := u.destroyHostOps(s)
+	if err != nil {
+		return nil, err
 	}
+	portsOps, err := removePortsForUnitOps(s.st, u)
+	if err != nil {
+		return nil, err
+	}
+
 	observedFieldsMatch := bson.D{
 		{"charmurl", u.doc.CharmURL},
 		{"machineid", u.doc.MachineId},
 	}
 	ops = append(ops, txn.Op{
 		C:      unitsC,
-		Id:     u.doc.Name,
+		Id:     u.doc.DocID,
 		Assert: append(observedFieldsMatch, asserts...),
 		Remove: true,
 	},
 		removeConstraintsOp(s.st, u.globalKey()),
 		removeStatusOp(s.st, u.globalKey()),
+		removeMeterStatusOp(s.st, u.globalKey()),
 		annotationRemoveOp(s.st, u.globalKey()),
 		s.st.newCleanupOp(cleanupRemovedUnit, u.doc.Name),
 	)
+	ops = append(ops, portsOps...)
 	if u.doc.CharmURL != nil {
 		decOps, err := settingsDecRefOps(s.st, s.doc.Name, u.doc.CharmURL)
 		if errors.IsNotFound(err) {
@@ -703,7 +713,7 @@ func (s *Service) removeUnitOps(u *Unit, asserts bson.D) ([]txn.Op, error) {
 	}
 	svcOp := txn.Op{
 		C:      servicesC,
-		Id:     s.doc.Name,
+		Id:     s.doc.DocID,
 		Update: bson.D{{"$inc", bson.D{{"unitcount", -1}}}},
 	}
 	if s.doc.Life == Alive {
@@ -717,24 +727,9 @@ func (s *Service) removeUnitOps(u *Unit, asserts bson.D) ([]txn.Op, error) {
 			}},
 		}
 	}
-	return append(ops, svcOp), nil
-}
+	ops = append(ops, svcOp)
 
-// Unit returns the service's unit with name.
-func (s *Service) Unit(name string) (*Unit, error) {
-	if !names.IsUnit(name) {
-		return nil, fmt.Errorf("%q is not a valid unit name", name)
-	}
-
-	units, closer := s.st.getCollection(unitsC)
-	defer closer()
-
-	udoc := &unitDoc{}
-	sel := bson.D{{"_id", name}, {"service", s.doc.Name}}
-	if err := units.Find(sel).One(udoc); err != nil {
-		return nil, fmt.Errorf("cannot get unit %q from service %q: %v", name, s.doc.Name, err)
-	}
-	return newUnit(s.st, udoc), nil
+	return ops, nil
 }
 
 // AllUnits returns all units of the service.
@@ -763,7 +758,7 @@ func (s *Service) Relations() (relations []*Relation, err error) {
 }
 
 func serviceRelations(st *State, name string) (relations []*Relation, err error) {
-	defer errors.Maskf(&err, "can't get relations for service %q", name)
+	defer errors.DeferredAnnotatef(&err, "can't get relations for service %q", name)
 	relationsCollection, closer := st.getCollection(relationsC)
 	defer closer()
 
@@ -840,14 +835,14 @@ func (s *Service) SetConstraints(cons constraints.Value) (err error) {
 	if s.doc.Subordinate {
 		return ErrSubordinateConstraints
 	}
-	defer errors.Maskf(&err, "cannot set constraints")
+	defer errors.DeferredAnnotatef(&err, "cannot set constraints")
 	if s.doc.Life != Alive {
 		return errNotAlive
 	}
 	ops := []txn.Op{
 		{
 			C:      servicesC,
-			Id:     s.doc.Name,
+			Id:     s.doc.DocID,
 			Assert: isAliveDoc,
 		},
 		setConstraintsOp(s.st, s.globalKey(), cons),
@@ -862,6 +857,42 @@ func (s *Service) Networks() ([]string, error) {
 	return readRequestedNetworks(s.st, s.globalKey())
 }
 
+// MetricCredentials returns any metric credentials associated with this service.
+func (s *Service) MetricCredentials() []byte {
+	return s.doc.MetricCredentials
+}
+
+// SetMetricCredentials updates the metric credentials associated with this service.
+func (s *Service) SetMetricCredentials(b []byte) error {
+	buildTxn := func(attempt int) ([]txn.Op, error) {
+		if attempt > 0 {
+			alive, err := isAlive(s.st, servicesC, s.doc.DocID)
+			if err != nil {
+				return nil, errors.Trace(err)
+			} else if !alive {
+				return nil, errNotAlive
+			}
+		}
+		ops := []txn.Op{
+			{
+				C:      servicesC,
+				Id:     s.doc.DocID,
+				Assert: isAliveDoc,
+				Update: bson.M{"$set": bson.M{"metric-credentials": b}},
+			},
+		}
+		return ops, nil
+	}
+	if err := s.st.run(buildTxn); err != nil {
+		if err == errNotAlive {
+			return errors.New("cannot update metric credentials: service " + err.Error())
+		}
+		return errors.Annotatef(err, "cannot update metric credentials")
+	}
+	s.doc.MetricCredentials = b
+	return nil
+}
+
 // settingsIncRefOp returns an operation that increments the ref count
 // of the service settings identified by serviceName and curl. If
 // canCreate is false, a missing document will be treated as an error;
@@ -875,18 +906,20 @@ func settingsIncRefOp(st *State, serviceName string, curl *charm.URL, canCreate
 		return txn.Op{}, err
 	} else if count == 0 {
 		if !canCreate {
-			return txn.Op{}, errors.NotFoundf("service settings")
+			return txn.Op{}, errors.NotFoundf("service %q settings for charm %q", serviceName, curl)
 		}
 		return txn.Op{
 			C:      settingsrefsC,
-			Id:     key,
+			Id:     st.docID(key),
 			Assert: txn.DocMissing,
-			Insert: settingsRefsDoc{1},
+			Insert: settingsRefsDoc{
+				RefCount: 1,
+				EnvUUID:  st.EnvironUUID()},
 		}, nil
 	}
 	return txn.Op{
 		C:      settingsrefsC,
-		Id:     key,
+		Id:     st.docID(key),
 		Assert: txn.DocExists,
 		Update: bson.D{{"$inc", bson.D{{"refcount", 1}}}},
 	}, nil
@@ -907,21 +940,22 @@ func settingsDecRefOps(st *State, serviceName string, curl *charm.URL) ([]txn.Op
 	} else if err != nil {
 		return nil, err
 	}
+	docID := st.docID(key)
 	if doc.RefCount == 1 {
 		return []txn.Op{{
 			C:      settingsrefsC,
-			Id:     key,
+			Id:     docID,
 			Assert: bson.D{{"refcount", 1}},
 			Remove: true,
 		}, {
 			C:      settingsC,
-			Id:     key,
+			Id:     docID,
 			Remove: true,
 		}}, nil
 	}
 	return []txn.Op{{
 		C:      settingsrefsC,
-		Id:     key,
+		Id:     docID,
 		Assert: bson.D{{"refcount", bson.D{{"$gt", 1}}}},
 		Update: bson.D{{"$inc", bson.D{{"refcount", -1}}}},
 	}}, nil
@@ -944,4 +978,5 @@ func settingsDecRefOps(st *State, serviceName string, curl *charm.URL) ([]txn.Op
 // always the same as the settingsDoc's id.
 type settingsRefsDoc struct {
 	RefCount int
+	EnvUUID  string `bson:"env-uuid"`
 }
