@@ -134,8 +134,8 @@ func cacheKeys(caches ...map[string]interface{}) map[string]bool {
 // overwriting unrelated changes made to the node since it was last read.
 func (c *Settings) Write() ([]ItemChange, error) {
 	changes := []ItemChange{}
-	updates := map[string]interface{}{}
-	deletions := map[string]int{}
+	updates := bson.M{}
+	deletions := bson.M{}
 	for key := range cacheKeys(c.disk, c.core) {
 		old, ondisk := c.disk[key]
 		new, incore := c.core[key]
@@ -165,12 +165,9 @@ func (c *Settings) Write() ([]ItemChange, error) {
 	sort.Sort(itemChangeSlice(changes))
 	ops := []txn.Op{{
 		C:      settingsC,
-		Id:     c.key,
+		Id:     c.st.docID(c.key),
 		Assert: txn.DocExists,
-		Update: bson.D{
-			{"$set", updates},
-			{"$unset", deletions},
-		},
+		Update: setUnsetUpdate(updates, deletions),
 	}}
 	err := c.st.runTransaction(ops)
 	if err == txn.ErrAborted {
@@ -191,9 +188,10 @@ func newSettings(st *State, key string) *Settings {
 	}
 }
 
-// cleanSettingsMap cleans the map of version and _id fields and also unescapes
-// keys coming out of MongoDB.
+// cleanSettingsMap cleans the map of version, env-uuid and _id fields and
+// also unescapes keys coming out of MongoDB.
 func cleanSettingsMap(in map[string]interface{}) {
+	delete(in, "env-uuid")
 	delete(in, "_id")
 	delete(in, "txn-revno")
 	delete(in, "txn-queue")
@@ -245,12 +243,22 @@ func (c *Settings) Read() error {
 // readSettingsDoc reads the settings with the given
 // key. It returns the settings and the current rxnRevno.
 func readSettingsDoc(st *State, key string) (map[string]interface{}, int64, error) {
-	settings, closer := st.getCollection(settingsC)
+	settings, closer := st.getRawCollection(settingsC)
 	defer closer()
 
 	config := map[string]interface{}{}
-	err := settings.FindId(key).One(config)
-	if err != nil {
+
+	err := settings.FindId(st.docID(key)).One(config)
+
+	// This is required to allow loading of environ settings before the
+	// environment UUID migration has been applied to the settings collection.
+	// Without this, an agent's version cannot be read, blocking the upgrade.
+	if err == mgo.ErrNotFound && key == environGlobalKey {
+		err := settings.FindId(environGlobalKey).One(config)
+		if err != nil {
+			return nil, 0, err
+		}
+	} else if err != nil {
 		return nil, 0, err
 	}
 	txnRevno := config["txn-revno"].(int64)
@@ -271,9 +279,10 @@ var errSettingsExist = fmt.Errorf("cannot overwrite existing settings")
 
 func createSettingsOp(st *State, key string, values map[string]interface{}) txn.Op {
 	newValues := copyMap(values, escapeReplacer.Replace)
+	newValues["env-uuid"] = st.EnvironUUID()
 	return txn.Op{
 		C:      settingsC,
-		Id:     key,
+		Id:     st.docID(key),
 		Assert: txn.DocMissing,
 		Insert: newValues,
 	}
@@ -315,7 +324,7 @@ func replaceSettingsOp(st *State, key string, values map[string]interface{}) (tx
 	if err != nil {
 		return txn.Op{}, nil, err
 	}
-	deletes := map[string]int{}
+	deletes := bson.M{}
 	for k := range s.disk {
 		if _, found := values[k]; !found {
 			deletes[escapeReplacer.Replace(k)] = 1
@@ -323,10 +332,7 @@ func replaceSettingsOp(st *State, key string, values map[string]interface{}) (tx
 	}
 	newValues := copyMap(values, escapeReplacer.Replace)
 	op := s.assertUnchangedOp()
-	op.Update = bson.D{
-		{"$set", newValues},
-		{"$unset", deletes},
-	}
+	op.Update = setUnsetUpdate(bson.M(newValues), deletes)
 	assertFailed := func() (bool, error) {
 		latest, err := readSettings(st, key)
 		if err != nil {
@@ -340,7 +346,22 @@ func replaceSettingsOp(st *State, key string, values map[string]interface{}) (tx
 func (s *Settings) assertUnchangedOp() txn.Op {
 	return txn.Op{
 		C:      settingsC,
-		Id:     s.key,
+		Id:     s.st.docID(s.key),
 		Assert: bson.D{{"txn-revno", s.txnRevno}},
 	}
 }
+
+// setUnsetUpdate returns a bson.D for use in
+// a txn.Op's Update field, containing $set and
+// $unset operators if the corresponding operands
+// are non-empty.
+func setUnsetUpdate(set, unset bson.M) bson.D {
+	var update bson.D
+	if len(set) > 0 {
+		update = append(update, bson.DocElem{"$set", set})
+	}
+	if len(unset) > 0 {
+		update = append(update, bson.DocElem{"$unset", unset})
+	}
+	return update
+}
