@@ -1,4 +1,4 @@
-// Copyright 2012, 2013 Canonical Ltd.
+// Copyright 2012-2015 Canonical Ltd.
 // Licensed under the AGPLv3, see LICENCE file for details.
 
 package state
@@ -8,12 +8,12 @@ import (
 	"fmt"
 	"time"
 
-	"github.com/juju/charm"
 	"github.com/juju/errors"
 	"github.com/juju/loggo"
 	"github.com/juju/names"
 	jujutxn "github.com/juju/txn"
 	"github.com/juju/utils"
+	"gopkg.in/juju/charm.v4"
 	"gopkg.in/mgo.v2"
 	"gopkg.in/mgo.v2/bson"
 	"gopkg.in/mgo.v2/txn"
@@ -21,7 +21,6 @@ import (
 	"github.com/juju/juju/constraints"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/network"
-	"github.com/juju/juju/state/api/params"
 	"github.com/juju/juju/state/presence"
 	"github.com/juju/juju/tools"
 	"github.com/juju/juju/version"
@@ -56,6 +55,7 @@ const (
 // are resolved.
 type ResolvedMode string
 
+// These are available ResolvedMode values.
 const (
 	ResolvedNone       ResolvedMode = ""
 	ResolvedRetryHooks ResolvedMode = "retry-hooks"
@@ -63,9 +63,11 @@ const (
 )
 
 // unitDoc represents the internal state of a unit in MongoDB.
-// Note the correspondence with UnitInfo in state/api/params.
+// Note the correspondence with UnitInfo in apiserver/params.
 type unitDoc struct {
-	Name         string `bson:"_id"`
+	DocID        string `bson:"_id"`
+	Name         string `bson:"name"`
+	EnvUUID      string `bson:"env-uuid"`
 	Service      string
 	Series       string
 	CharmURL     *charm.URL
@@ -74,12 +76,12 @@ type unitDoc struct {
 	MachineId    string
 	Resolved     ResolvedMode
 	Tools        *tools.Tools `bson:",omitempty"`
-	Ports        []network.Port
 	Life         Life
 	TxnRevno     int64 `bson:"txn-revno"`
 	PasswordHash string
 
 	// No longer used - to be removed.
+	Ports          []network.Port
 	PublicAddress  string
 	PrivateAddress string
 }
@@ -99,7 +101,7 @@ func newUnit(st *State, udoc *unitDoc) *Unit {
 	}
 	unit.annotator = annotator{
 		globalKey: unit.globalKey(),
-		tag:       unit.Tag().String(),
+		tag:       unit.Tag(),
 		st:        st,
 	}
 	return unit
@@ -182,14 +184,14 @@ func (u *Unit) AgentTools() (*tools.Tools, error) {
 // SetAgentVersion sets the version of juju that the agent is
 // currently running.
 func (u *Unit) SetAgentVersion(v version.Binary) (err error) {
-	defer errors.Maskf(&err, "cannot set agent version for unit %q", u)
+	defer errors.DeferredAnnotatef(&err, "cannot set agent version for unit %q", u)
 	if err = checkVersionValidity(v); err != nil {
 		return err
 	}
 	tools := &tools.Tools{Version: v}
 	ops := []txn.Op{{
 		C:      unitsC,
-		Id:     u.doc.Name,
+		Id:     u.doc.DocID,
 		Assert: notDeadDoc,
 		Update: bson.D{{"$set", bson.D{{"tools", tools}}}},
 	}}
@@ -200,13 +202,6 @@ func (u *Unit) SetAgentVersion(v version.Binary) (err error) {
 	return nil
 }
 
-// SetMongoPassword sets the password the agent responsible for the unit
-// should use to communicate with the state servers.  Previous passwords
-// are invalidated.
-func (u *Unit) SetMongoPassword(password string) error {
-	return u.st.setMongoPassword(u.Tag().String(), password)
-}
-
 // SetPassword sets the password for the machine's agent.
 func (u *Unit) SetPassword(password string) error {
 	if len(password) < utils.MinAgentPasswordLength {
@@ -221,7 +216,7 @@ func (u *Unit) SetPassword(password string) error {
 func (u *Unit) setPasswordHash(passwordHash string) error {
 	ops := []txn.Op{{
 		C:      unitsC,
-		Id:     u.doc.Name,
+		Id:     u.doc.DocID,
 		Assert: notDeadDoc,
 		Update: bson.D{{"$set", bson.D{{"passwordhash", passwordHash}}}},
 	}}
@@ -301,6 +296,13 @@ func (u *Unit) Destroy() (err error) {
 	return err
 }
 
+var unitNotInstalled = bson.D{
+	{"$or", []bson.D{
+		{{"status", StatusPending}},
+		{{"status", StatusAllocating}},
+		{{"status", StatusInstalling}},
+	}}}
+
 // destroyOps returns the operations required to destroy the unit. If it
 // returns errRefresh, the unit should be refreshed and the destruction
 // operations recalculated.
@@ -334,7 +336,7 @@ func (u *Unit) destroyOps() ([]txn.Op, error) {
 	cleanupOp := u.st.newCleanupOp(cleanupDyingUnit, u.doc.Name)
 	setDyingOps := []txn.Op{{
 		C:      unitsC,
-		Id:     u.doc.Name,
+		Id:     u.doc.DocID,
 		Assert: isAliveDoc,
 		Update: bson.D{{"$set", bson.D{{"life", Dying}}}},
 	}, cleanupOp, minUnitsOp}
@@ -351,13 +353,13 @@ func (u *Unit) destroyOps() ([]txn.Op, error) {
 	} else if err != nil {
 		return nil, err
 	}
-	if sdoc.Status != params.StatusPending {
+	if sdoc.Status != StatusPending && sdoc.Status != StatusAllocating && sdoc.Status != StatusInstalling {
 		return setDyingOps, nil
 	}
 	ops := []txn.Op{{
 		C:      statusesC,
-		Id:     sdocId,
-		Assert: bson.D{{"status", params.StatusPending}},
+		Id:     u.st.docID(sdocId),
+		Assert: unitNotInstalled,
 	}, minUnitsOp}
 	removeAsserts := append(isAliveDoc, unitHasNoSubordinates...)
 	removeOps, err := u.removeOps(removeAsserts)
@@ -369,6 +371,97 @@ func (u *Unit) destroyOps() ([]txn.Op, error) {
 	return append(ops, removeOps...), nil
 }
 
+// destroyHostOps returns all necessary operations to destroy the service unit's host machine,
+// or ensure that the conditions preventing its destruction remain stable through the transaction.
+func (u *Unit) destroyHostOps(s *Service) (ops []txn.Op, err error) {
+	if s.doc.Subordinate {
+		return []txn.Op{{
+			C:      unitsC,
+			Id:     u.st.docID(u.doc.Principal),
+			Assert: txn.DocExists,
+			Update: bson.D{{"$pull", bson.D{{"subordinates", u.doc.Name}}}},
+		}}, nil
+	} else if u.doc.MachineId == "" {
+		unitLogger.Errorf("unit %v unassigned", u)
+		return nil, nil
+	}
+
+	machineUpdate := bson.D{{"$pull", bson.D{{"principals", u.doc.Name}}}}
+
+	m, err := u.st.Machine(u.doc.MachineId)
+	if err != nil {
+		if errors.IsNotFound(err) {
+			return nil, nil
+		}
+		return nil, err
+	}
+
+	containerCheck := true // whether container conditions allow destroying the host machine
+	containers, err := m.Containers()
+	if err != nil {
+		return nil, err
+	}
+	if len(containers) > 0 {
+		ops = append(ops, txn.Op{
+			C:      containerRefsC,
+			Id:     m.doc.DocID,
+			Assert: bson.D{{"children.0", bson.D{{"$exists", 1}}}},
+		})
+		containerCheck = false
+	} else {
+		ops = append(ops, txn.Op{
+			C:  containerRefsC,
+			Id: m.doc.DocID,
+			Assert: bson.D{{"$or", []bson.D{
+				{{"children", bson.D{{"$size", 0}}}},
+				{{"children", bson.D{{"$exists", false}}}},
+			}}},
+		})
+	}
+
+	machineCheck := true // whether host machine conditions allow destroy
+	if len(m.doc.Principals) != 1 || m.doc.Principals[0] != u.doc.Name {
+		machineCheck = false
+	} else if hasJob(m.doc.Jobs, JobManageEnviron) {
+		// Check that the machine does not have any responsibilities that
+		// prevent a lifecycle change.
+		machineCheck = false
+	} else if m.doc.HasVote {
+		machineCheck = false
+	}
+
+	// assert that the machine conditions pertaining to host removal conditions
+	// remain the same throughout the transaction.
+	var machineAssert bson.D
+	if machineCheck {
+		machineAssert = bson.D{{"$and", []bson.D{
+			bson.D{{"principals", []string{u.doc.Name}}},
+			bson.D{{"jobs", bson.D{{"$nin", []MachineJob{JobManageEnviron}}}}},
+			bson.D{{"hasvote", bson.D{{"$ne", true}}}},
+		}}}
+	} else {
+		machineAssert = bson.D{{"$or", []bson.D{
+			bson.D{{"principals", bson.D{{"$ne", []string{u.doc.Name}}}}},
+			bson.D{{"jobs", bson.D{{"$in", []MachineJob{JobManageEnviron}}}}},
+			bson.D{{"hasvote", true}},
+		}}}
+	}
+
+	// If removal conditions satisfied by machine & container docs, we can
+	// destroy it, in addition to removing the unit principal.
+	if machineCheck && containerCheck {
+		machineUpdate = append(machineUpdate, bson.D{{"$set", bson.D{{"life", Dying}}}}...)
+	}
+
+	ops = append(ops, txn.Op{
+		C:      machinesC,
+		Id:     m.doc.DocID,
+		Assert: machineAssert,
+		Update: machineUpdate,
+	})
+	return ops, nil
+}
+
 var errAlreadyRemoved = stderrors.New("entity has already been removed")
 
 // removeOps returns the operations necessary to remove the unit, assuming
@@ -384,6 +477,9 @@ func (u *Unit) removeOps(asserts bson.D) ([]txn.Op, error) {
 	return svc.removeUnitOps(u, asserts)
 }
 
+// ErrUnitHasSubordinates is a standard error to indicate that an a Unit
+// cannot complete an operation to end it's life because it still has
+// subordinate services
 var ErrUnitHasSubordinates = stderrors.New("unit has subordinates")
 
 var unitHasNoSubordinates = bson.D{{
@@ -407,14 +503,14 @@ func (u *Unit) EnsureDead() (err error) {
 	}()
 	ops := []txn.Op{{
 		C:      unitsC,
-		Id:     u.doc.Name,
+		Id:     u.doc.DocID,
 		Assert: append(notDeadDoc, unitHasNoSubordinates...),
 		Update: bson.D{{"$set", bson.D{{"life", Dead}}}},
 	}}
 	if err := u.st.runTransaction(ops); err != txn.ErrAborted {
 		return err
 	}
-	if notDead, err := isNotDead(u.st.db, unitsC, u.doc.Name); err != nil {
+	if notDead, err := isNotDead(u.st, unitsC, u.doc.DocID); err != nil {
 		return err
 	} else if !notDead {
 		return nil
@@ -426,7 +522,7 @@ func (u *Unit) EnsureDead() (err error) {
 // the service is Dying and no other references to it exist. It will fail if
 // the unit is not Dead.
 func (u *Unit) Remove() (err error) {
-	defer errors.Maskf(&err, "cannot remove unit %q", u)
+	defer errors.DeferredAnnotatef(&err, "cannot remove unit %q", u)
 	if u.doc.Life != Dead {
 		return stderrors.New("unit is not dead")
 	}
@@ -549,19 +645,27 @@ func (u *Unit) PrincipalName() (string, bool) {
 	return u.doc.Principal, u.doc.Principal != ""
 }
 
+// machine returns the unit's machine.
+func (u *Unit) machine() (*Machine, error) {
+	id, err := u.AssignedMachineId()
+	if err != nil {
+		return nil, errors.Annotatef(err, "unit %v cannot get assigned machine", u)
+	}
+	m, err := u.st.Machine(id)
+	if err != nil {
+		return nil, errors.Annotatef(err, "unit %v misses machine id %v", u)
+	}
+	return m, nil
+}
+
 // addressesOfMachine returns Addresses of the related machine if present.
 func (u *Unit) addressesOfMachine() []network.Address {
-	if id, err := u.AssignedMachineId(); err != nil {
-		unitLogger.Errorf("unit %v cannot get assigned machine: %v", u, err)
+	m, err := u.machine()
+	if err != nil {
+		unitLogger.Errorf("%v", err)
 		return nil
-	} else {
-		m, err := u.st.Machine(id)
-		if err == nil {
-			return m.Addresses()
-		}
-		unitLogger.Errorf("unit %v misses machine id %v", u, id)
 	}
-	return nil
+	return m.Addresses()
 }
 
 // PublicAddress returns the public address of the unit and whether it is valid.
@@ -584,6 +688,16 @@ func (u *Unit) PrivateAddress() (string, bool) {
 	return privateAddress, privateAddress != ""
 }
 
+// AvailabilityZone returns the name of the availability zone into which
+// the unit's machine instance was provisioned.
+func (u *Unit) AvailabilityZone() (string, error) {
+	m, err := u.machine()
+	if err != nil {
+		return "", errors.Trace(err)
+	}
+	return m.AvailabilityZone()
+}
+
 // Refresh refreshes the contents of the Unit from the underlying
 // state. It an error that satisfies errors.IsNotFound if the unit has
 // been removed.
@@ -591,7 +705,7 @@ func (u *Unit) Refresh() error {
 	units, closer := u.st.getCollection(unitsC)
 	defer closer()
 
-	err := units.FindId(u.doc.Name).One(&u.doc)
+	err := units.FindId(u.doc.DocID).One(&u.doc)
 	if err == mgo.ErrNotFound {
 		return errors.NotFoundf("unit %q", u)
 	}
@@ -602,7 +716,7 @@ func (u *Unit) Refresh() error {
 }
 
 // Status returns the status of the unit.
-func (u *Unit) Status() (status params.Status, info string, data params.StatusData, err error) {
+func (u *Unit) Status() (status Status, info string, data map[string]interface{}, err error) {
 	doc, err := getStatus(u.st, u.globalKey())
 	if err != nil {
 		return "", "", nil, err
@@ -613,86 +727,113 @@ func (u *Unit) Status() (status params.Status, info string, data params.StatusDa
 	return
 }
 
-// SetStatus sets the status of the unit. The optional values
+// SetStatus sets the status of the unit agent. The optional values
 // allow to pass additional helpful status data.
-func (u *Unit) SetStatus(status params.Status, info string, data params.StatusData) error {
-	doc := statusDoc{
-		Status:     status,
-		StatusInfo: info,
-		StatusData: data,
-	}
-	if err := doc.validateSet(false); err != nil {
+func (u *Unit) SetStatus(status Status, info string, data map[string]interface{}) error {
+	doc, err := newUnitAgentStatusDoc(status, info, data)
+	if err != nil {
 		return err
 	}
 	ops := []txn.Op{{
 		C:      unitsC,
-		Id:     u.doc.Name,
+		Id:     u.doc.DocID,
 		Assert: notDeadDoc,
 	},
-		updateStatusOp(u.st, u.globalKey(), doc),
+		updateStatusOp(u.st, u.globalKey(), doc.statusDoc),
 	}
-	err := u.st.runTransaction(ops)
+	err = u.st.runTransaction(ops)
 	if err != nil {
 		return fmt.Errorf("cannot set status of unit %q: %v", u, onAbort(err, ErrDead))
 	}
 	return nil
 }
 
-// OpenPort sets the policy of the port with protocol and number to be opened.
-func (u *Unit) OpenPort(protocol string, number int) (err error) {
-	port := network.Port{Protocol: protocol, Number: number}
-	defer errors.Maskf(&err, "cannot open port %v for unit %q", port, u)
-	ops := []txn.Op{{
-		C:      unitsC,
-		Id:     u.doc.Name,
-		Assert: notDeadDoc,
-		Update: bson.D{{"$addToSet", bson.D{{"ports", port}}}},
-	}}
-	err = u.st.runTransaction(ops)
+// OpenPorts opens the given port range and protocol for the unit, if
+// it does not conflict with another already opened range on the
+// unit's assigned machine.
+func (u *Unit) OpenPorts(protocol string, fromPort, toPort int) (err error) {
+	ports, err := NewPortRange(u.Name(), fromPort, toPort, protocol)
 	if err != nil {
-		return onAbort(err, ErrDead)
+		return errors.Annotatef(err, "invalid port range %v-%v/%v", fromPort, toPort, protocol)
 	}
-	found := false
-	for _, p := range u.doc.Ports {
-		if p == port {
-			break
-		}
+	defer errors.DeferredAnnotatef(&err, "cannot open ports %v for unit %q", ports, u)
+
+	machineId, err := u.AssignedMachineId()
+	if err != nil {
+		return errors.Annotatef(err, "unit %q has no assigned machine", u)
 	}
-	if !found {
-		u.doc.Ports = append(u.doc.Ports, port)
+
+	// TODO(dimitern) 2014-09-10 bug #1337804: network name is
+	// hard-coded until multiple network support lands
+	machinePorts, err := getOrCreatePorts(u.st, machineId, network.DefaultPublic)
+	if err != nil {
+		return errors.Annotatef(err, "cannot get or create ports for machine %q", machineId)
 	}
-	return nil
+
+	return machinePorts.OpenPorts(ports)
 }
 
-// ClosePort sets the policy of the port with protocol and number to be closed.
-func (u *Unit) ClosePort(protocol string, number int) (err error) {
-	port := network.Port{Protocol: protocol, Number: number}
-	defer errors.Maskf(&err, "cannot close port %v for unit %q", port, u)
-	ops := []txn.Op{{
-		C:      unitsC,
-		Id:     u.doc.Name,
-		Assert: notDeadDoc,
-		Update: bson.D{{"$pull", bson.D{{"ports", port}}}},
-	}}
-	err = u.st.runTransaction(ops)
+// ClosePorts closes the given port range and protocol for the unit.
+func (u *Unit) ClosePorts(protocol string, fromPort, toPort int) (err error) {
+	ports, err := NewPortRange(u.Name(), fromPort, toPort, protocol)
 	if err != nil {
-		return onAbort(err, ErrDead)
+		return errors.Annotatef(err, "invalid port range %v-%v/%v", fromPort, toPort, protocol)
 	}
-	newPorts := make([]network.Port, 0, len(u.doc.Ports))
-	for _, p := range u.doc.Ports {
-		if p != port {
-			newPorts = append(newPorts, p)
-		}
+	defer errors.DeferredAnnotatef(&err, "cannot close ports %v for unit %q", ports, u)
+
+	machineId, err := u.AssignedMachineId()
+	if err != nil {
+		return errors.Annotatef(err, "unit %q has no assigned machine", u)
 	}
-	u.doc.Ports = newPorts
-	return nil
+
+	// TODO(dimitern) 2014-09-10 bug #1337804: network name is
+	// hard-coded until multiple network support lands
+	machinePorts, err := getOrCreatePorts(u.st, machineId, network.DefaultPublic)
+	if err != nil {
+		return errors.Annotatef(err, "cannot get or create ports for machine %q", machineId)
+	}
+
+	return machinePorts.ClosePorts(ports)
 }
 
-// OpenedPorts returns a slice containing the open ports of the unit.
-func (u *Unit) OpenedPorts() []network.Port {
-	ports := append([]network.Port{}, u.doc.Ports...)
-	network.SortPorts(ports)
-	return ports
+// OpenPort opens the given port and protocol for the unit.
+func (u *Unit) OpenPort(protocol string, number int) error {
+	return u.OpenPorts(protocol, number, number)
+}
+
+// ClosePort closes the given port and protocol for the unit.
+func (u *Unit) ClosePort(protocol string, number int) error {
+	return u.ClosePorts(protocol, number, number)
+}
+
+// OpenedPorts returns a slice containing the open port ranges of the
+// unit.
+func (u *Unit) OpenedPorts() ([]network.PortRange, error) {
+	machineId, err := u.AssignedMachineId()
+	if err != nil {
+		return nil, errors.Annotatef(err, "unit %q has no assigned machine", u)
+	}
+
+	// TODO(dimitern) 2014-09-10 bug #1337804: network name is
+	// hard-coded until multiple network support lands
+	machinePorts, err := getPorts(u.st, machineId, network.DefaultPublic)
+	result := []network.PortRange{}
+	if err == nil {
+		ports := machinePorts.PortsForUnit(u.Name())
+		for _, port := range ports {
+			result = append(result, network.PortRange{
+				Protocol: port.Protocol,
+				FromPort: port.FromPort,
+				ToPort:   port.ToPort,
+			})
+		}
+	} else {
+		if !errors.IsNotFound(err) {
+			return nil, errors.Annotatef(err, "failed getting ports for unit %q", u)
+		}
+	}
+	network.SortPortRanges(result)
+	return result, nil
 }
 
 // CharmURL returns the charm URL this unit is currently using.
@@ -705,44 +846,47 @@ func (u *Unit) CharmURL() (*charm.URL, bool) {
 
 // SetCharmURL marks the unit as currently using the supplied charm URL.
 // An error will be returned if the unit is dead, or the charm URL not known.
-func (u *Unit) SetCharmURL(curl *charm.URL) (err error) {
-	defer func() {
-		if err == nil {
-			u.doc.CharmURL = curl
-		}
-	}()
+func (u *Unit) SetCharmURL(curl *charm.URL) error {
 	if curl == nil {
 		return fmt.Errorf("cannot set nil charm url")
 	}
 
 	db, closer := u.st.newDB()
 	defer closer()
-	units := db.C(unitsC)
-	charms := db.C(charmsC)
+	envUUID := u.st.EnvironUUID()
+	units := getCollectionFromDB(db, unitsC, envUUID)
+	charms := getCollectionFromDB(db, charmsC, envUUID)
 
 	buildTxn := func(attempt int) ([]txn.Op, error) {
-		if notDead, err := isNotDead(u.st.db, unitsC, u.doc.Name); err != nil {
-			return nil, err
-		} else if !notDead {
-			return nil, fmt.Errorf("unit %q is dead", u)
+		if attempt > 0 {
+			// NOTE: We're explicitly allowing SetCharmURL to succeed
+			// when the unit is Dying, because service/charm upgrades
+			// should still be allowed to apply to dying units, so
+			// that bugs in departed/broken hooks can be addressed at
+			// runtime.
+			if notDead, err := isNotDeadWithSession(units, u.doc.DocID); err != nil {
+				return nil, errors.Trace(err)
+			} else if !notDead {
+				return nil, ErrDead
+			}
 		}
-		sel := bson.D{{"_id", u.doc.Name}, {"charmurl", curl}}
+		sel := bson.D{{"_id", u.doc.DocID}, {"charmurl", curl}}
 		if count, err := units.Find(sel).Count(); err != nil {
-			return nil, err
+			return nil, errors.Trace(err)
 		} else if count == 1 {
 			// Already set
 			return nil, jujutxn.ErrNoOperations
 		}
-		if count, err := charms.FindId(curl).Count(); err != nil {
-			return nil, err
+		if count, err := charms.FindId(curl.String()).Count(); err != nil {
+			return nil, errors.Trace(err)
 		} else if count < 1 {
-			return nil, fmt.Errorf("unknown charm url %q", curl)
+			return nil, errors.Errorf("unknown charm url %q", curl)
 		}
 
 		// Add a reference to the service settings for the new charm.
 		incOp, err := settingsIncRefOp(u.st, u.doc.Service, curl, false)
 		if err != nil {
-			return nil, err
+			return nil, errors.Trace(err)
 		}
 
 		// Set the new charm URL.
@@ -751,7 +895,7 @@ func (u *Unit) SetCharmURL(curl *charm.URL) (err error) {
 			incOp,
 			{
 				C:      unitsC,
-				Id:     u.doc.Name,
+				Id:     u.doc.DocID,
 				Assert: append(notDeadDoc, differentCharm...),
 				Update: bson.D{{"$set", bson.D{{"charmurl", curl}}}},
 			}}
@@ -759,13 +903,17 @@ func (u *Unit) SetCharmURL(curl *charm.URL) (err error) {
 			// Drop the reference to the old charm.
 			decOps, err := settingsDecRefOps(u.st, u.doc.Service, u.doc.CharmURL)
 			if err != nil {
-				return nil, err
+				return nil, errors.Trace(err)
 			}
 			ops = append(ops, decOps...)
 		}
 		return ops, nil
 	}
-	return u.st.run(buildTxn)
+	err := u.st.run(buildTxn)
+	if err == nil {
+		u.doc.CharmURL = curl
+	}
+	return err
 }
 
 // AgentPresence returns whether the respective remote agent is alive.
@@ -777,12 +925,18 @@ func (u *Unit) AgentPresence() (bool, error) {
 // The returned name will be different from other Tag values returned by any
 // other entities from the same state.
 func (u *Unit) Tag() names.Tag {
+	return u.UnitTag()
+}
+
+// UnitTag returns a names.UnitTag representing this Unit, unless the
+// unit Name is invalid, in which case it will panic
+func (u *Unit) UnitTag() names.UnitTag {
 	return names.NewUnitTag(u.Name())
 }
 
 // WaitAgentPresence blocks until the respective agent is alive.
 func (u *Unit) WaitAgentPresence(timeout time.Duration) (err error) {
-	defer errors.Maskf(&err, "waiting for agent of unit %q", u)
+	defer errors.DeferredAnnotatef(&err, "waiting for agent of unit %q", u)
 	ch := make(chan presence.Change)
 	u.st.pwatcher.Watch(u.globalKey(), ch)
 	defer u.st.pwatcher.Unwatch(u.globalKey(), ch)
@@ -823,7 +977,7 @@ func (e *NotAssignedError) Error() string {
 
 // IsNotAssigned verifies that err is an instance of NotAssignedError
 func IsNotAssigned(err error) bool {
-	_, ok := err.(*NotAssignedError)
+	_, ok := errors.Cause(err).(*NotAssignedError)
 	return ok
 }
 
@@ -840,7 +994,7 @@ func (u *Unit) AssignedMachineId() (id string, err error) {
 	defer closer()
 
 	pudoc := unitDoc{}
-	err = units.Find(bson.D{{"_id", u.doc.Principal}}).One(&pudoc)
+	err = units.FindId(u.doc.Principal).One(&pudoc)
 	if err == mgo.ErrNotFound {
 		return "", errors.NotFoundf("principal unit %q of %q", u.doc.Principal, u)
 	} else if err != nil {
@@ -907,12 +1061,12 @@ func (u *Unit) assignToMachine(m *Machine, unused bool) (err error) {
 	}
 	ops := []txn.Op{{
 		C:      unitsC,
-		Id:     u.doc.Name,
+		Id:     u.doc.DocID,
 		Assert: assert,
 		Update: bson.D{{"$set", bson.D{{"machineid", m.doc.Id}}}},
 	}, {
 		C:      machinesC,
-		Id:     m.doc.Id,
+		Id:     m.doc.DocID,
 		Assert: massert,
 		Update: bson.D{{"$addToSet", bson.D{{"principals", u.doc.Name}}}, {"$set", bson.D{{"clean", false}}}},
 	}}
@@ -988,13 +1142,14 @@ func (u *Unit) assignToNewMachine(template MachineTemplate, parentId string, con
 	}
 	// Ensure the host machine is really clean.
 	if parentId != "" {
+		parentDocId := u.st.docID(parentId)
 		ops = append(ops, txn.Op{
 			C:      machinesC,
-			Id:     parentId,
+			Id:     parentDocId,
 			Assert: bson.D{{"clean", true}},
 		}, txn.Op{
 			C:      containerRefsC,
-			Id:     parentId,
+			Id:     parentDocId,
 			Assert: bson.D{hasNoContainersTerm},
 		})
 	}
@@ -1002,7 +1157,7 @@ func (u *Unit) assignToNewMachine(template MachineTemplate, parentId string, con
 	asserts := append(isAliveDoc, isUnassigned...)
 	ops = append(ops, txn.Op{
 		C:      unitsC,
-		Id:     u.doc.Name,
+		Id:     u.doc.DocID,
 		Assert: asserts,
 		Update: bson.D{{"$set", bson.D{{"machineid", mdoc.Id}}}},
 	})
@@ -1225,7 +1380,7 @@ func (u *Unit) findCleanMachineQuery(requireEmpty bool, cons *constraints.Value)
 		{"series", u.doc.Series},
 		{"jobs", []MachineJob{JobHostUnits}},
 		{"clean", true},
-		{"_id", bson.D{{"$nin", machinesWithContainers}}},
+		{"machineid", bson.D{{"$nin", machinesWithContainers}}},
 	}
 	// Add the container filter term if necessary.
 	var containerType instance.ContainerType
@@ -1273,7 +1428,7 @@ func (u *Unit) findCleanMachineQuery(requireEmpty bool, cons *constraints.Value)
 		}
 		var suitableIds = make([]string, len(suitableInstanceData))
 		for i, m := range suitableInstanceData {
-			suitableIds[i] = m.Id
+			suitableIds[i] = m.DocID
 		}
 		terms = append(terms, bson.DocElem{"_id", bson.D{{"$in", suitableIds}}})
 	}
@@ -1324,7 +1479,7 @@ func (u *Unit) assignToCleanMaybeEmptyMachine(requireEmpty bool) (m *Machine, er
 	for _, mdoc := range mdocs {
 		m := newMachine(u.st, mdoc)
 		instance, err := m.InstanceId()
-		if IsNotProvisionedError(err) {
+		if errors.IsNotProvisioned(err) {
 			unprovisioned = append(unprovisioned, m)
 		} else if err != nil {
 			assignContextf(&err, u, context)
@@ -1385,14 +1540,14 @@ func (u *Unit) UnassignFromMachine() (err error) {
 	// machine id is as expected.
 	ops := []txn.Op{{
 		C:      unitsC,
-		Id:     u.doc.Name,
+		Id:     u.doc.DocID,
 		Assert: txn.DocExists,
 		Update: bson.D{{"$set", bson.D{{"machineid", ""}}}},
 	}}
 	if u.doc.MachineId != "" {
 		ops = append(ops, txn.Op{
 			C:      machinesC,
-			Id:     u.doc.MachineId,
+			Id:     u.st.docID(u.doc.MachineId),
 			Assert: txn.DocExists,
 			Update: bson.D{{"$pull", bson.D{{"principals", u.doc.Name}}}},
 		})
@@ -1405,37 +1560,82 @@ func (u *Unit) UnassignFromMachine() (err error) {
 	return nil
 }
 
-// AddAction adds a new Action of type name and using arguments payload to
-// this Unit, and returns its ID
-func (u *Unit) AddAction(name string, payload map[string]interface{}) (string, error) {
-	actionId, err := newActionId(u.st, names.NewUnitTag(u.Name()).Id())
-	if err != nil {
-		return "", fmt.Errorf("cannot add action; error generating key: %v", err)
-	}
-	doc := actionDoc{Id: actionId, Name: name, Payload: payload}
-	ops := []txn.Op{{
-		C:      unitsC,
-		Id:     u.doc.Name,
-		Assert: notDeadDoc,
-	}, {
-		C:      actionsC,
-		Id:     doc.Id,
-		Assert: txn.DocMissing,
-		Insert: doc,
-	}}
+// ActionSpecsByName is a map of action names to their respective ActionSpec.
+type ActionSpecsByName map[string]charm.ActionSpec
 
-	buildTxn := func(attempt int) ([]txn.Op, error) {
-		if notDead, err := isNotDead(u.st.db, unitsC, u.doc.Name); err != nil {
-			return nil, err
-		} else if !notDead {
-			return nil, fmt.Errorf("unit %q is dead", u)
+// AddAction adds a new Action of type name and using arguments payload to
+// this Unit, and returns its ID.
+func (u *Unit) AddAction(name string, payload map[string]interface{}) (*Action, error) {
+	if len(name) == 0 {
+		return nil, errors.New("no action name given")
+	}
+	specs, err := u.ActionSpecs()
+	if err != nil {
+		return nil, err
+	}
+	spec, ok := specs[name]
+	if !ok {
+		return nil, errors.Errorf("action %q not defined on unit %q", name, u.Name())
+	}
+	_, err = spec.ValidateParams(payload)
+	if err != nil {
+		return nil, err
+	}
+	return u.st.EnqueueAction(u.Tag(), name, payload)
+}
+
+// ActionSpecs gets the ActionSpec map for the Unit's charm.
+func (u *Unit) ActionSpecs() (ActionSpecsByName, error) {
+	none := ActionSpecsByName{}
+	curl, _ := u.CharmURL()
+	if curl == nil {
+		// If unit charm URL is not yet set, fall back to service
+		svc, err := u.Service()
+		if err != nil {
+			return none, err
+		}
+		curl, _ = svc.CharmURL()
+		if curl == nil {
+			return none, errors.Errorf("no URL set for service %q", svc.Name())
 		}
-		return ops, nil
 	}
-	if err = u.st.run(buildTxn); err == nil {
-		return actionId, nil
+	ch, err := u.st.Charm(curl)
+	if err != nil {
+		return none, errors.Annotatef(err, "unable to get charm with URL %q", curl.String())
 	}
-	return "", err
+	chActions := ch.Actions()
+	if chActions == nil || len(chActions.ActionSpecs) == 0 {
+		return none, errors.Errorf("no actions defined on charm %q", ch.String())
+	}
+	return chActions.ActionSpecs, nil
+}
+
+// CancelAction removes a pending Action from the queue for this
+// ActionReceiver and marks it as cancelled.
+func (u *Unit) CancelAction(action *Action) (*Action, error) {
+	return action.Finish(ActionResults{Status: ActionCancelled})
+}
+
+// WatchActionNotifications starts and returns a StringsWatcher that
+// notifies when actions with Id prefixes matching this Unit are added
+func (u *Unit) WatchActionNotifications() StringsWatcher {
+	return u.st.watchEnqueuedActionsFilteredBy(u)
+}
+
+// Actions returns a list of actions pending or completed for this unit.
+func (u *Unit) Actions() ([]*Action, error) {
+	return u.st.matchingActions(u)
+}
+
+// CompletedActions returns a list of actions that have finished for
+// this unit.
+func (u *Unit) CompletedActions() ([]*Action, error) {
+	return u.st.matchingActionsCompleted(u)
+}
+
+// PendingActions returns a list of actions pending for this unit.
+func (u *Unit) PendingActions() ([]*Action, error) {
+	return u.st.matchingActionsPending(u)
 }
 
 // Resolve marks the unit as having had any previous state transition
@@ -1448,8 +1648,8 @@ func (u *Unit) Resolve(retryHooks bool) error {
 	if err != nil {
 		return err
 	}
-	if status != params.StatusError {
-		return fmt.Errorf("unit %q is not in an error state", u)
+	if status != StatusError {
+		return errors.Errorf("unit %q is not in an error state", u)
 	}
 	mode := ResolvedNoHooks
 	if retryHooks {
@@ -1464,7 +1664,7 @@ func (u *Unit) Resolve(retryHooks bool) error {
 // whether to attempt to reexecute previous failed hooks or to continue
 // as if they had succeeded before.
 func (u *Unit) SetResolved(mode ResolvedMode) (err error) {
-	defer errors.Maskf(&err, "cannot set resolved mode for unit %q", u)
+	defer errors.DeferredAnnotatef(&err, "cannot set resolved mode for unit %q", u)
 	switch mode {
 	case ResolvedRetryHooks, ResolvedNoHooks:
 	default:
@@ -1474,7 +1674,7 @@ func (u *Unit) SetResolved(mode ResolvedMode) (err error) {
 	resolvedNotSet := bson.D{{"resolved", ResolvedNone}}
 	ops := []txn.Op{{
 		C:      unitsC,
-		Id:     u.doc.Name,
+		Id:     u.doc.DocID,
 		Assert: append(notDeadDoc, resolvedNotSet...),
 		Update: bson.D{{"$set", bson.D{{"resolved", mode}}}},
 	}}
@@ -1484,7 +1684,7 @@ func (u *Unit) SetResolved(mode ResolvedMode) (err error) {
 	} else if err != txn.ErrAborted {
 		return err
 	}
-	if ok, err := isNotDead(u.st.db, unitsC, u.doc.Name); err != nil {
+	if ok, err := isNotDead(u.st, unitsC, u.doc.DocID); err != nil {
 		return err
 	} else if !ok {
 		return ErrDead
@@ -1497,7 +1697,7 @@ func (u *Unit) SetResolved(mode ResolvedMode) (err error) {
 func (u *Unit) ClearResolved() error {
 	ops := []txn.Op{{
 		C:      unitsC,
-		Id:     u.doc.Name,
+		Id:     u.doc.DocID,
 		Assert: txn.DocExists,
 		Update: bson.D{{"$set", bson.D{{"resolved", ResolvedNone}}}},
 	}}
@@ -1509,7 +1709,16 @@ func (u *Unit) ClearResolved() error {
 	return nil
 }
 
-// WatchActions starts and returns an ActionWatcher
-func (u *Unit) WatchActions() StringsWatcher {
-	return newActionWatcher(u.st, u.Tag().Id())
+// AddMetric adds a new batch of metrics to the database.
+// A UUID for the metric will be generated and the new MetricBatch will be returned
+func (u *Unit) AddMetrics(created time.Time, metrics []Metric) (*MetricBatch, error) {
+	charmUrl, ok := u.CharmURL()
+	if !ok {
+		return nil, stderrors.New("failed to add metrics, couldn't find charm url")
+	}
+	service, err := u.Service()
+	if err != nil {
+		return nil, errors.Annotatef(err, "couldn't retrieve service whilst adding metrics")
+	}
+	return u.st.addMetrics(u.UnitTag(), charmUrl, created, metrics, service.MetricCredentials())
 }
