@@ -7,7 +7,7 @@ import (
 	"os"
 
 	"github.com/juju/utils"
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/juju/osenv"
 	"github.com/juju/juju/testing"
