@@ -4,7 +4,7 @@
 package testing
 
 import (
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 )
 
 type GitSuite struct {
