@@ -4,7 +4,8 @@
 package upgrades_test
 
 import (
-	gc "launchpad.net/gocheck"
+	jc "github.com/juju/testing/checkers"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/environs/config"
 	jujutesting "github.com/juju/juju/juju/testing"
@@ -25,30 +26,30 @@ func (s *rsyslogPortSuite) SetUpTest(c *gc.C) {
 	s.ctx = &mockContext{
 		agentConfig: &mockAgentConfig{
 			dataDir:   s.DataDir(),
-			stateInfo: s.StateInfo(c),
+			mongoInfo: s.MongoInfo(c),
 		},
 		apiState: apiState,
 		state:    s.State,
 	}
 	cfg, err := s.State.EnvironConfig()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cfg.SyslogPort(), gc.Not(gc.Equals), config.DefaultSyslogPort)
 }
 
 func (s *rsyslogPortSuite) TestSyslogPortChanged(c *gc.C) {
 	err := upgrades.UpdateRsyslogPort(s.ctx)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	cfg, err := s.State.EnvironConfig()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cfg.SyslogPort(), gc.Equals, config.DefaultSyslogPort)
 }
 
 func (s *rsyslogPortSuite) TestIdempotent(c *gc.C) {
 	err := upgrades.UpdateRsyslogPort(s.ctx)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	err = upgrades.UpdateRsyslogPort(s.ctx)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	cfg, err := s.State.EnvironConfig()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	c.Assert(cfg.SyslogPort(), gc.Equals, config.DefaultSyslogPort)
 }
