@@ -24,22 +24,25 @@ import (
 // The presence and format of this constant is very important.
 // The debian/rules build recipe uses this value for the version
 // number of the release package.
-const version = "1.20.11"
+const version = "1.22.1"
 
 // The version that we switched over from old style numbering to new style.
 var switchOverVersion = MustParse("1.19.9")
 
-// lsbReleaseFile is the name of the file that is read in order to determine
-// the release version of ubuntu.
-var lsbReleaseFile = "/etc/lsb-release"
+// osReleaseFile is the name of the file that is read in order to determine
+// the linux type release version.
+var osReleaseFile = "/etc/os-release"
+
+var osVers = mustOSVersion()
 
 // Current gives the current version of the system.  If the file
 // "FORCE-VERSION" is present in the same directory as the running
 // binary, it will override this.
 var Current = Binary{
 	Number: MustParse(version),
-	Series: osVersion(),
+	Series: osVers,
 	Arch:   arch.HostArch(),
+	OS:     MustOSFromSeries(osVers),
 }
 
 var Compiler = runtime.Compiler
@@ -82,6 +85,7 @@ type Binary struct {
 	Number
 	Series string
 	Arch   string
+	OS     OSType
 }
 
 func (v Binary) String() string {
@@ -187,6 +191,11 @@ func ParseBinary(s string) (Binary, error) {
 	}
 	v.Series = m[7]
 	v.Arch = m[8]
+	operatingSystem, err := GetOSFromSeries(v.Series)
+	if err != nil {
+		return Binary{}, err
+	}
+	v.OS = operatingSystem
 	return v, nil
 }
 
@@ -337,21 +346,15 @@ func (v Number) IsDev() bool {
 	return v.Tag != "" || v.Build > 0
 }
 
-// ReleaseVersion looks for the value of DISTRIB_RELEASE in the content of
-// the lsbReleaseFile.  If the value is not found, the file is not found, or
+// ReleaseVersion looks for the value of VERSION_ID in the content of
+// the os-release.  If the value is not found, the file is not found, or
 // an error occurs reading the file, an empty string is returned.
 func ReleaseVersion() string {
-	content, err := ioutil.ReadFile(lsbReleaseFile)
+	release, err := readOSRelease()
 	if err != nil {
 		return ""
 	}
-	const prefix = "DISTRIB_RELEASE="
-	for _, line := range strings.Split(string(content), "\n") {
-		if strings.HasPrefix(line, prefix) {
-			return strings.Trim(line[len(prefix):], "\t '\"")
-		}
-	}
-	return ""
+	return release["VERSION_ID"]
 }
 
 // ParseMajorMinor takes an argument of the form "major.minor" and returns ints major and minor.
