@@ -8,13 +8,15 @@ import (
 	stdtesting "testing"
 	"time"
 
-	gc "launchpad.net/gocheck"
+	"github.com/juju/names"
+	jc "github.com/juju/testing/checkers"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/agent"
+	"github.com/juju/juju/api"
+	"github.com/juju/juju/api/keyupdater"
 	jujutesting "github.com/juju/juju/juju/testing"
 	"github.com/juju/juju/state"
-	"github.com/juju/juju/state/api"
-	"github.com/juju/juju/state/api/keyupdater"
 	coretesting "github.com/juju/juju/testing"
 	"github.com/juju/juju/utils/ssh"
 	sshtesting "github.com/juju/juju/utils/ssh/testing"
@@ -60,7 +62,7 @@ func (s *workerSuite) SetUpTest(c *gc.C) {
 	// Set up an existing key (which is not in the environment) in the ssh authorised_keys file.
 	s.existingKeys = []string{sshtesting.ValidKeyTwo.Key + " existinguser@host"}
 	err := ssh.AddKeys(authenticationworker.SSHUser, s.existingKeys...)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 
 	var apiRoot *api.State
 	apiRoot, s.machine = s.OpenAPIAsNewMachine(c)
@@ -76,21 +78,21 @@ func stop(c *gc.C, w worker.Worker) {
 type mockConfig struct {
 	agent.Config
 	c   *gc.C
-	tag string
+	tag names.Tag
 }
 
-func (mock *mockConfig) Tag() string {
+func (mock *mockConfig) Tag() names.Tag {
 	return mock.tag
 }
 
-func agentConfig(c *gc.C, tag string) *mockConfig {
+func agentConfig(c *gc.C, tag names.MachineTag) *mockConfig {
 	return &mockConfig{c: c, tag: tag}
 }
 
 func (s *workerSuite) setAuthorisedKeys(c *gc.C, keys ...string) {
 	keyStr := strings.Join(keys, "\n")
 	err := s.BackingState.UpdateEnvironConfig(map[string]interface{}{"authorized-keys": keyStr}, nil, nil)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	s.BackingState.StartSync()
 }
 
@@ -102,7 +104,7 @@ func (s *workerSuite) waitSSHKeys(c *gc.C, expected []string) {
 			c.Fatalf("timeout while waiting for authoirsed ssh keys to change")
 		case <-time.After(coretesting.ShortWait):
 			keys, err := ssh.ListKeys(authenticationworker.SSHUser, ssh.FullKeys)
-			c.Assert(err, gc.IsNil)
+			c.Assert(err, jc.ErrorIsNil)
 			keysStr := strings.Join(keys, "\n")
 			expectedStr := strings.Join(expected, "\n")
 			if expectedStr != keysStr {
@@ -114,7 +116,7 @@ func (s *workerSuite) waitSSHKeys(c *gc.C, expected []string) {
 }
 
 func (s *workerSuite) TestKeyUpdateRetainsExisting(c *gc.C) {
-	authWorker := authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().String()))
+	authWorker := authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().(names.MachineTag)))
 	defer stop(c, authWorker)
 
 	newKey := sshtesting.ValidKeyThree.Key + " user@host"
@@ -127,7 +129,7 @@ func (s *workerSuite) TestNewKeysInJujuAreSavedOnStartup(c *gc.C) {
 	newKey := sshtesting.ValidKeyThree.Key + " user@host"
 	s.setAuthorisedKeys(c, newKey)
 
-	authWorker := authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().String()))
+	authWorker := authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().(names.MachineTag)))
 	defer stop(c, authWorker)
 
 	newKeyWithCommentPrefix := sshtesting.ValidKeyThree.Key + " Juju:user@host"
@@ -135,7 +137,7 @@ func (s *workerSuite) TestNewKeysInJujuAreSavedOnStartup(c *gc.C) {
 }
 
 func (s *workerSuite) TestDeleteKey(c *gc.C) {
-	authWorker := authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().String()))
+	authWorker := authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().(names.MachineTag)))
 	defer stop(c, authWorker)
 
 	// Add another key
@@ -150,7 +152,7 @@ func (s *workerSuite) TestDeleteKey(c *gc.C) {
 }
 
 func (s *workerSuite) TestMultipleChanges(c *gc.C) {
-	authWorker := authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().String()))
+	authWorker := authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().(names.MachineTag)))
 	defer stop(c, authWorker)
 	s.waitSSHKeys(c, append(s.existingKeys, s.existingEnvKey))
 
@@ -163,7 +165,7 @@ func (s *workerSuite) TestMultipleChanges(c *gc.C) {
 }
 
 func (s *workerSuite) TestWorkerRestart(c *gc.C) {
-	authWorker := authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().String()))
+	authWorker := authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().(names.MachineTag)))
 	defer stop(c, authWorker)
 	s.waitSSHKeys(c, append(s.existingKeys, s.existingEnvKey))
 
@@ -174,7 +176,7 @@ func (s *workerSuite) TestWorkerRestart(c *gc.C) {
 	s.setAuthorisedKeys(c, sshtesting.ValidKeyThree.Key+" yetanother@host")
 
 	// Restart the worker and check that the ssh auth keys are as expected.
-	authWorker = authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().String()))
+	authWorker = authenticationworker.NewWorker(s.keyupdaterApi, agentConfig(c, s.machine.Tag().(names.MachineTag)))
 	defer stop(c, authWorker)
 
 	yetAnotherKeyWithCommentPrefix := sshtesting.ValidKeyThree.Key + " Juju:yetanother@host"
