@@ -5,7 +5,7 @@ package deployer
 
 import (
 	"github.com/juju/juju/agent"
-	"github.com/juju/juju/state/api/params"
+	"github.com/juju/juju/apiserver/params"
 )
 
 type fakeAPI struct{}
