@@ -9,8 +9,9 @@ import (
 	"reflect"
 	"time"
 
+	"github.com/juju/errors"
 	jc "github.com/juju/testing/checkers"
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/juju/testing"
@@ -52,10 +53,10 @@ func (s *workerSuite) TestWorker(c *gc.C) {
 	checkInstanceInfo := func(index int, m machine, expectedStatus string) bool {
 		isProvisioned := true
 		status, err := m.InstanceStatus()
-		if state.IsNotProvisionedError(err) {
+		if errors.IsNotProvisioned(err) {
 			isProvisioned = false
 		} else {
-			c.Assert(err, gc.IsNil)
+			c.Assert(err, jc.ErrorIsNil)
 		}
 		return reflect.DeepEqual(m.Addresses(), s.addressesForIndex(index)) && (!isProvisioned || status == expectedStatus)
 	}
@@ -75,7 +76,7 @@ func (s *workerSuite) TestWorker(c *gc.C) {
 			status, err := m.InstanceStatus()
 			if i%2 == 0 {
 				// Even machines not provisioned yet.
-				c.Assert(err, jc.Satisfies, state.IsNotProvisionedError)
+				c.Assert(err, jc.Satisfies, errors.IsNotProvisioned)
 			} else {
 				c.Assert(status, gc.Equals, "")
 			}
@@ -88,7 +89,7 @@ func (s *workerSuite) TestWorker(c *gc.C) {
 	for i := 0; i < len(insts)/2; i += 2 {
 		m := machines[i]
 		err := m.SetProvisioned(insts[i].Id(), "nonce", nil)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		dummy.SetInstanceAddresses(insts[i], s.addressesForIndex(i))
 		dummy.SetInstanceStatus(insts[i], "running")
 	}
@@ -104,7 +105,7 @@ func (s *workerSuite) TestWorker(c *gc.C) {
 			status, err := m.InstanceStatus()
 			if i%2 == 0 {
 				// Even machines not provisioned yet.
-				c.Assert(err, jc.Satisfies, state.IsNotProvisionedError)
+				c.Assert(err, jc.Satisfies, errors.IsNotProvisioned)
 			} else {
 				c.Assert(status, gc.Equals, "")
 			}
@@ -119,7 +120,7 @@ func (s *workerSuite) TestWorker(c *gc.C) {
 		if i%2 == 0 {
 			m := machines[i]
 			err := m.SetProvisioned(insts[i].Id(), "nonce", nil)
-			c.Assert(err, gc.IsNil)
+			c.Assert(err, jc.ErrorIsNil)
 		}
 		dummy.SetInstanceAddresses(insts[i], s.addressesForIndex(i))
 		dummy.SetInstanceStatus(insts[i], "running")
@@ -144,7 +145,7 @@ func (s *workerSuite) TestWorker(c *gc.C) {
 func machinesSatisfy(c *gc.C, machines []*state.Machine, f func(i int, m *state.Machine) bool) bool {
 	for i, m := range machines {
 		err := m.Refresh()
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		if !f(i, m) {
 			return false
 		}
@@ -157,16 +158,16 @@ func (s *workerSuite) setupScenario(c *gc.C) ([]*state.Machine, []instance.Insta
 	var insts []instance.Instance
 	for i := 0; i < 10; i++ {
 		m, err := s.State.AddMachine("series", state.JobHostUnits)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 		machines = append(machines, m)
-		inst, _ := testing.AssertStartInstance(c, s.Conn.Environ, m.Id())
+		inst, _ := testing.AssertStartInstance(c, s.Environ, m.Id())
 		insts = append(insts, inst)
 	}
 	// Associate the odd-numbered machines with an instance.
 	for i := 1; i < len(machines); i += 2 {
 		m := machines[i]
 		err := m.SetProvisioned(insts[i].Id(), "nonce", nil)
-		c.Assert(err, gc.IsNil)
+		c.Assert(err, jc.ErrorIsNil)
 	}
 	// Associate the first half of the instances with an address and status.
 	for i := 0; i < len(machines)/2; i++ {
