@@ -11,15 +11,15 @@ import (
 	"github.com/juju/utils/fslock"
 
 	"github.com/juju/juju/agent"
+	apiprovisioner "github.com/juju/juju/api/provisioner"
+	"github.com/juju/juju/api/watcher"
+	"github.com/juju/juju/apiserver/params"
 	"github.com/juju/juju/container"
 	"github.com/juju/juju/container/kvm"
 	"github.com/juju/juju/container/lxc"
 	"github.com/juju/juju/environs"
 	"github.com/juju/juju/instance"
 	"github.com/juju/juju/state"
-	"github.com/juju/juju/state/api/params"
-	apiprovisioner "github.com/juju/juju/state/api/provisioner"
-	"github.com/juju/juju/state/api/watcher"
 	"github.com/juju/juju/worker"
 )
 
@@ -29,6 +29,7 @@ import (
 type ContainerSetup struct {
 	runner              worker.Runner
 	supportedContainers []instance.ContainerType
+	imageURLGetter      container.ImageURLGetter
 	provisioner         *apiprovisioner.State
 	machine             *apiprovisioner.Machine
 	config              agent.Config
@@ -44,20 +45,30 @@ type ContainerSetup struct {
 	numberProvisioners int32
 }
 
+// ContainerSetupParams are used to initialise a container setup handler.
+type ContainerSetupParams struct {
+	Runner              worker.Runner
+	WorkerName          string
+	SupportedContainers []instance.ContainerType
+	ImageURLGetter      container.ImageURLGetter
+	Machine             *apiprovisioner.Machine
+	Provisioner         *apiprovisioner.State
+	Config              agent.Config
+	InitLock            *fslock.Lock
+}
+
 // NewContainerSetupHandler returns a StringsWatchHandler which is notified when
 // containers are created on the given machine.
-func NewContainerSetupHandler(runner worker.Runner, workerName string, supportedContainers []instance.ContainerType,
-	machine *apiprovisioner.Machine, provisioner *apiprovisioner.State,
-	config agent.Config, initLock *fslock.Lock) worker.StringsWatchHandler {
-
+func NewContainerSetupHandler(params ContainerSetupParams) worker.StringsWatchHandler {
 	return &ContainerSetup{
-		runner:              runner,
-		machine:             machine,
-		supportedContainers: supportedContainers,
-		provisioner:         provisioner,
-		config:              config,
-		workerName:          workerName,
-		initLock:            initLock,
+		runner:              params.Runner,
+		imageURLGetter:      params.ImageURLGetter,
+		machine:             params.Machine,
+		supportedContainers: params.SupportedContainers,
+		provisioner:         params.Provisioner,
+		config:              params.Config,
+		workerName:          params.WorkerName,
+		initLock:            params.InitLock,
 	}
 }
 
@@ -125,14 +136,15 @@ func (cs *ContainerSetup) initialiseAndStartProvisioner(containerType instance.C
 	}()
 
 	logger.Debugf("setup and start provisioner for %s containers", containerType)
-	if initialiser, broker, err := cs.getContainerArtifacts(containerType); err != nil {
+	toolsFinder := getToolsFinder(cs.provisioner)
+	initialiser, broker, toolsFinder, err := cs.getContainerArtifacts(containerType, toolsFinder)
+	if err != nil {
 		return errors.Annotate(err, "initialising container infrastructure on host machine")
-	} else {
-		if err := cs.runInitialiser(containerType, initialiser); err != nil {
-			return errors.Annotate(err, "setting up container dependencies on host machine")
-		}
-		return StartProvisioner(cs.runner, containerType, cs.provisioner, cs.config, broker)
 	}
+	if err := cs.runInitialiser(containerType, initialiser); err != nil {
+		return errors.Annotate(err, "setting up container dependencies on host machine")
+	}
+	return StartProvisioner(cs.runner, containerType, cs.provisioner, cs.config, broker, toolsFinder)
 }
 
 // runInitialiser runs the container initialiser with the initialisation hook held.
@@ -151,44 +163,58 @@ func (cs *ContainerSetup) TearDown() error {
 	return nil
 }
 
-func (cs *ContainerSetup) getContainerArtifacts(containerType instance.ContainerType) (container.Initialiser, environs.InstanceBroker, error) {
-	logger.Debugf("finding tools for %s containers", containerType)
-	tools, err := cs.provisioner.Tools(cs.config.Tag())
-	if err != nil {
-		logger.Errorf("cannot get tools from machine for %s container", containerType)
-		return nil, nil, err
-	}
+// getContainerArtifacts returns type-specific interfaces for
+// managing containers.
+//
+// The ToolsFinder passed in may be replaced or wrapped to
+// enforce container-specific constraints.
+func (cs *ContainerSetup) getContainerArtifacts(
+	containerType instance.ContainerType, toolsFinder ToolsFinder,
+) (
+	container.Initialiser,
+	environs.InstanceBroker,
+	ToolsFinder,
+	error,
+) {
 	var initialiser container.Initialiser
 	var broker environs.InstanceBroker
 
 	managerConfig, err := containerManagerConfig(containerType, cs.provisioner, cs.config)
 	if err != nil {
-		return nil, nil, err
+		return nil, nil, nil, err
 	}
 
 	switch containerType {
 	case instance.LXC:
 		series, err := cs.machine.Series()
 		if err != nil {
-			return nil, nil, err
+			return nil, nil, nil, err
 		}
 
 		initialiser = lxc.NewContainerInitialiser(series)
-		broker, err = NewLxcBroker(cs.provisioner, tools, cs.config, managerConfig)
+		broker, err = NewLxcBroker(cs.provisioner, cs.config, managerConfig, cs.imageURLGetter)
 		if err != nil {
-			return nil, nil, err
+			return nil, nil, nil, err
 		}
+
+		// LXC containers must have the same architecture as the host.
+		// We should call through to the finder since the version of
+		// tools running on the host may not match, but we want to
+		// override the arch constraint with the arch of the host.
+		toolsFinder = hostArchToolsFinder{toolsFinder}
+
 	case instance.KVM:
 		initialiser = kvm.NewContainerInitialiser()
-		broker, err = NewKvmBroker(cs.provisioner, tools, cs.config, managerConfig)
+		broker, err = NewKvmBroker(cs.provisioner, cs.config, managerConfig)
 		if err != nil {
 			logger.Errorf("failed to create new kvm broker")
-			return nil, nil, err
+			return nil, nil, nil, err
 		}
 	default:
-		return nil, nil, fmt.Errorf("unknown container type: %v", containerType)
+		return nil, nil, nil, fmt.Errorf("unknown container type: %v", containerType)
 	}
-	return initialiser, broker, nil
+
+	return initialiser, broker, toolsFinder, nil
 }
 
 func containerManagerConfig(
@@ -203,7 +229,7 @@ func containerManagerConfig(
 	if params.IsCodeNotImplemented(err) {
 		// We currently don't support upgrading;
 		// revert to the old configuration.
-		managerConfigResult.ManagerConfig = container.ManagerConfig{container.ConfigName: "juju"}
+		managerConfigResult.ManagerConfig = container.ManagerConfig{container.ConfigName: container.DefaultNamespace}
 	}
 	if err != nil {
 		return nil, err
@@ -221,13 +247,19 @@ var StartProvisioner = startProvisionerWorker
 
 // startProvisionerWorker kicks off a provisioner task responsible for creating containers
 // of the specified type on the machine.
-func startProvisionerWorker(runner worker.Runner, containerType instance.ContainerType,
-	provisioner *apiprovisioner.State, config agent.Config, broker environs.InstanceBroker) error {
+func startProvisionerWorker(
+	runner worker.Runner,
+	containerType instance.ContainerType,
+	provisioner *apiprovisioner.State,
+	config agent.Config,
+	broker environs.InstanceBroker,
+	toolsFinder ToolsFinder,
+) error {
 
 	workerName := fmt.Sprintf("%s-provisioner", containerType)
 	// The provisioner task is created after a container record has already been added to the machine.
 	// It will see that the container does not have an instance yet and create one.
 	return runner.StartWorker(workerName, func() (worker.Worker, error) {
-		return NewContainerProvisioner(containerType, provisioner, config, broker), nil
+		return NewContainerProvisioner(containerType, provisioner, config, broker, toolsFinder), nil
 	})
 }
