@@ -8,7 +8,8 @@ import (
 	"os/signal"
 	stdtesting "testing"
 
-	gc "launchpad.net/gocheck"
+	jc "github.com/juju/testing/checkers"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/juju/testing"
 	"github.com/juju/juju/worker"
@@ -44,16 +45,16 @@ func (s *TerminationWorkerSuite) TestStartStop(c *gc.C) {
 	w := terminationworker.NewWorker()
 	w.Kill()
 	err := w.Wait()
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 }
 
 func (s *TerminationWorkerSuite) TestSignal(c *gc.C) {
 	w := terminationworker.NewWorker()
 	proc, err := os.FindProcess(os.Getpid())
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	defer proc.Release()
 	err = proc.Signal(terminationworker.TerminationSignal)
-	c.Assert(err, gc.IsNil)
+	c.Assert(err, jc.ErrorIsNil)
 	err = w.Wait()
 	c.Assert(err, gc.Equals, worker.ErrTerminateAgent)
 }
