@@ -10,10 +10,14 @@ import (
 
 const NetworkTagKind = "network"
 
-var validNetwork = regexp.MustCompile("^([a-z0-9]+(-[a-z0-9]+)*)$")
+const (
+	NetworkSnippet = "(?:[a-z0-9]+(?:-[a-z0-9]+)*)"
+)
 
-// IsNetwork reports whether name is a valid network name.
-func IsNetwork(name string) bool {
+var validNetwork = regexp.MustCompile("^" + NetworkSnippet + "$")
+
+// IsValidNetwork reports whether name is a valid network name.
+func IsValidNetwork(name string) bool {
 	return validNetwork.MatchString(name)
 }
 
@@ -27,7 +31,7 @@ func (t NetworkTag) Id() string     { return t.name }
 
 // NewNetworkTag returns the tag of a network with the given name.
 func NewNetworkTag(name string) NetworkTag {
-	if !IsNetwork(name) {
+	if !IsValidNetwork(name) {
 		panic(fmt.Sprintf("%q is not a valid network name", name))
 	}
 	return NetworkTag{name: name}
