@@ -6,7 +6,7 @@ package testing_test
 import (
 	"os"
 
-	gc "launchpad.net/gocheck"
+	gc "gopkg.in/check.v1"
 
 	"github.com/juju/testing"
 )
