@@ -66,6 +66,14 @@ func (suite *TestServerSuite) TestHandlesNodeListingUnknownPath(c *C) {
 	c.Check(resp.StatusCode, Equals, http.StatusNotFound)
 }
 
+func (suite *TestServerSuite) TestHandlesNodegroupsInterfacesListingUnknownNodegroup(c *C) {
+	invalidPath := fmt.Sprintf("/api/%s/nodegroups/unknown/interfaces/", suite.server.version)
+	resp, err := http.Get(suite.server.Server.URL + invalidPath)
+
+	c.Check(err, IsNil)
+	c.Check(resp.StatusCode, Equals, http.StatusNotFound)
+}
+
 func (suite *TestServerSuite) TestNewNode(c *C) {
 	input := `{"system_id": "mysystemid"}`
 
@@ -357,6 +365,16 @@ func (suite *TestServerSuite) TestDeleteFile(c *C) {
 	c.Check(suite.server.Files(), DeepEquals, map[string]MAASObject{})
 }
 
+func (suite *TestServerSuite) TestListZonesNotSupported(c *C) {
+	// Older versions of MAAS do not support zones. We simulate
+	// this behaviour by returning 404 if no zones are defined.
+	zonesURL := getZonesEndpoint(suite.server.version)
+	resp, err := http.Get(suite.server.Server.URL + zonesURL)
+
+	c.Check(err, IsNil)
+	c.Check(resp.StatusCode, Equals, http.StatusNotFound)
+}
+
 // TestMAASObjectSuite validates that the object created by
 // NewTestMAAS can be used by the gomaasapi library as if it were a real
 // MAAS server.
@@ -619,7 +637,7 @@ func (suite *TestMAASObjectSuite) TestReleaseNodeReleasesAcquiredNode(c *C) {
 func (suite *TestMAASObjectSuite) TestGetNetworks(c *C) {
 	nodeJSON := `{"system_id": "mysystemid"}`
 	suite.TestMAASObject.TestServer.NewNode(nodeJSON)
-	networkJSON := `{"name": "mynetworkname"}`
+	networkJSON := `{"name": "mynetworkname", "ip": "0.1.2.0", "netmask": "255.255.255.0"}`
 	suite.TestMAASObject.TestServer.NewNetwork(networkJSON)
 	suite.TestMAASObject.TestServer.ConnectNodeToNetwork("mysystemid", "mynetworkname")
 
@@ -637,7 +655,13 @@ func (suite *TestMAASObjectSuite) TestGetNetworks(c *C) {
 
 	networkName, err := listNetworks.GetField("name")
 	c.Assert(err, IsNil)
+	ip, err := listNetworks.GetField("ip")
+	c.Assert(err, IsNil)
+	netmask, err := listNetworks.GetField("netmask")
+	c.Assert(err, IsNil)
 	c.Check(networkName, Equals, "mynetworkname")
+	c.Check(ip, Equals, "0.1.2.0")
+	c.Check(netmask, Equals, "255.255.255.0")
 }
 
 func (suite *TestMAASObjectSuite) TestGetNetworksNone(c *C) {
@@ -657,7 +681,7 @@ func (suite *TestMAASObjectSuite) TestGetNetworksNone(c *C) {
 func (suite *TestMAASObjectSuite) TestListNodesWithNetworks(c *C) {
 	nodeJSON := `{"system_id": "mysystemid"}`
 	suite.TestMAASObject.TestServer.NewNode(nodeJSON)
-	networkJSON := `{"name": "mynetworkname"}`
+	networkJSON := `{"name": "mynetworkname", "ip": "0.1.2.0", "netmask": "255.255.255.0"}`
 	suite.TestMAASObject.TestServer.NewNetwork(networkJSON)
 	suite.TestMAASObject.TestServer.ConnectNodeToNetworkWithMACAddress("mysystemid", "mynetworkname", "aa:bb:cc:dd:ee:ff")
 
@@ -699,8 +723,12 @@ func (suite *TestMAASObjectSuite) TestListNodesWithNetworks(c *C) {
 func (suite *TestMAASObjectSuite) TestListNetworkConnectedMACAddresses(c *C) {
 	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "node_1"}`)
 	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "node_2"}`)
-	suite.TestMAASObject.TestServer.NewNetwork(`{"name": "net_1"}`)
-	suite.TestMAASObject.TestServer.NewNetwork(`{"name": "net_2"}`)
+	suite.TestMAASObject.TestServer.NewNetwork(
+		`{"name": "net_1", "ip": "0.1.2.0", "netmask": "255.255.255.0"}`,
+	)
+	suite.TestMAASObject.TestServer.NewNetwork(
+		`{"name": "net_2", "ip": "0.2.2.0", "netmask": "255.255.255.0"}`,
+	)
 	suite.TestMAASObject.TestServer.ConnectNodeToNetworkWithMACAddress("node_2", "net_2", "aa:bb:cc:dd:ee:22")
 	suite.TestMAASObject.TestServer.ConnectNodeToNetworkWithMACAddress("node_1", "net_1", "aa:bb:cc:dd:ee:11")
 	suite.TestMAASObject.TestServer.ConnectNodeToNetworkWithMACAddress("node_2", "net_1", "aa:bb:cc:dd:ee:21")
@@ -760,10 +788,143 @@ func (suite *TestMAASObjectSuite) TestGetVersion(c *C) {
 	for _, capJSONName := range capArray {
 		capName, err := capJSONName.GetString()
 		c.Assert(err, IsNil)
-		c.Check(capName, Equals, "networks-management")
+		switch capName {
+		case "networks-management":
+		case "static-ipaddresses":
+		default:
+			c.Fatalf("unknown capability %q", capName)
+		}
 	}
 }
 
+func (suite *TestMAASObjectSuite) assertIPAmong(c *C, jsonObjIP JSONObject, expectIPs ...string) {
+	apiVersion := suite.TestMAASObject.TestServer.version
+	expectedURI := getIPAddressesEndpoint(apiVersion)
+
+	maasObj, err := jsonObjIP.GetMAASObject()
+	c.Assert(err, IsNil)
+	attrs := maasObj.GetMap()
+	uri, err := attrs["resource_uri"].GetString()
+	c.Assert(err, IsNil)
+	c.Assert(uri, Equals, expectedURI)
+	allocType, err := attrs["alloc_type"].GetFloat64()
+	c.Assert(err, IsNil)
+	c.Assert(allocType, Equals, 4.0)
+	created, err := attrs["created"].GetString()
+	c.Assert(err, IsNil)
+	c.Assert(created, Not(Equals), "")
+	ip, err := attrs["ip"].GetString()
+	c.Assert(err, IsNil)
+	if !contains(expectIPs, ip) {
+		c.Fatalf("expected IP in %v, got %q", expectIPs, ip)
+	}
+}
+
+func (suite *TestMAASObjectSuite) TestListIPAddresses(c *C) {
+	ipAddresses := suite.TestMAASObject.GetSubObject("ipaddresses")
+
+	// First try without any networks and IPs.
+	listIPObjects, err := ipAddresses.CallGet("", url.Values{})
+	c.Assert(err, IsNil)
+	items, err := listIPObjects.GetArray()
+	c.Assert(err, IsNil)
+	c.Assert(items, HasLen, 0)
+
+	// Add two networks and some addresses to each one.
+	suite.TestMAASObject.TestServer.NewNetwork(
+		`{"name": "net_1", "ip": "0.1.2.0", "netmask": "255.255.255.0"}`,
+	)
+	suite.TestMAASObject.TestServer.NewNetwork(
+		`{"name": "net_2", "ip": "0.2.2.0", "netmask": "255.255.255.0"}`,
+	)
+	suite.TestMAASObject.TestServer.NewIPAddress("0.1.2.3", "net_1")
+	suite.TestMAASObject.TestServer.NewIPAddress("0.1.2.4", "net_1")
+	suite.TestMAASObject.TestServer.NewIPAddress("0.1.2.5", "net_1")
+	suite.TestMAASObject.TestServer.NewIPAddress("0.2.2.3", "net_2")
+	suite.TestMAASObject.TestServer.NewIPAddress("0.2.2.4", "net_2")
+
+	// List all addresses and verify the needed response fields are set.
+	listIPObjects, err = ipAddresses.CallGet("", url.Values{})
+	c.Assert(err, IsNil)
+	items, err = listIPObjects.GetArray()
+	c.Assert(err, IsNil)
+	c.Assert(items, HasLen, 5)
+
+	for _, ipObj := range items {
+		suite.assertIPAmong(
+			c, ipObj,
+			"0.1.2.3", "0.1.2.4", "0.1.2.5", "0.2.2.3", "0.2.2.4",
+		)
+	}
+
+	// Remove all net_1 IPs.
+	removed := suite.TestMAASObject.TestServer.RemoveIPAddress("0.1.2.3")
+	c.Assert(removed, Equals, true)
+	removed = suite.TestMAASObject.TestServer.RemoveIPAddress("0.1.2.4")
+	c.Assert(removed, Equals, true)
+	removed = suite.TestMAASObject.TestServer.RemoveIPAddress("0.1.2.5")
+	c.Assert(removed, Equals, true)
+	// Remove the last IP twice, should be OK and return false.
+	removed = suite.TestMAASObject.TestServer.RemoveIPAddress("0.1.2.5")
+	c.Assert(removed, Equals, false)
+
+	// List again.
+	listIPObjects, err = ipAddresses.CallGet("", url.Values{})
+	c.Assert(err, IsNil)
+	items, err = listIPObjects.GetArray()
+	c.Assert(err, IsNil)
+	c.Assert(items, HasLen, 2)
+	for _, ipObj := range items {
+		suite.assertIPAmong(
+			c, ipObj,
+			"0.2.2.3", "0.2.2.4",
+		)
+	}
+}
+
+func (suite *TestMAASObjectSuite) TestReserveIPAddress(c *C) {
+	suite.TestMAASObject.TestServer.NewNetwork(
+		`{"name": "net_1", "ip": "0.1.2.0", "netmask": "255.255.255.0"}`,
+	)
+	ipAddresses := suite.TestMAASObject.GetSubObject("ipaddresses")
+	// First try "reserve" with requested_address set.
+	params := url.Values{"network": []string{"0.1.2.0/24"}, "requested_address": []string{"0.1.2.42"}}
+	res, err := ipAddresses.CallPost("reserve", params)
+	c.Assert(err, IsNil)
+	suite.assertIPAmong(c, res, "0.1.2.42")
+
+	// Now try "reserve" without requested_address.
+	delete(params, "requested_address")
+	res, err = ipAddresses.CallPost("reserve", params)
+	c.Assert(err, IsNil)
+	suite.assertIPAmong(c, res, "0.1.2.2")
+}
+
+func (suite *TestMAASObjectSuite) TestReleaseIPAddress(c *C) {
+	suite.TestMAASObject.TestServer.NewNetwork(
+		`{"name": "net_1", "ip": "0.1.2.0", "netmask": "255.255.255.0"}`,
+	)
+	suite.TestMAASObject.TestServer.NewIPAddress("0.1.2.3", "net_1")
+	ipAddresses := suite.TestMAASObject.GetSubObject("ipaddresses")
+
+	// Try with non-existing address - should return 404.
+	params := url.Values{"ip": []string{"0.2.2.1"}}
+	_, err := ipAddresses.CallPost("release", params)
+	c.Assert(err, ErrorMatches, `(\n|.)*404 Not Found(\n|.)*`)
+
+	// Now with existing one - all OK.
+	params = url.Values{"ip": []string{"0.1.2.3"}}
+	_, err = ipAddresses.CallPost("release", params)
+	c.Assert(err, IsNil)
+
+	// Ensure it got removed.
+	c.Assert(suite.TestMAASObject.TestServer.ipAddressesPerNetwork["net_1"], HasLen, 0)
+
+	// Try again, should return 404.
+	_, err = ipAddresses.CallPost("release", params)
+	c.Assert(err, ErrorMatches, `(\n|.)*404 Not Found(\n|.)*`)
+}
+
 const nodeDetailsXML = `<?xml version="1.0" standalone="yes" ?>
 <list>
 <node id="node2" claimed="true" class="system" handle="DMI:0001">
@@ -818,6 +979,74 @@ func (suite *TestMAASObjectSuite) TestListNodegroups(c *C) {
 	}
 }
 
+func (suite *TestMAASObjectSuite) TestListNodegroupsEmptyList(c *C) {
+	nodegroupListing := suite.TestMAASObject.GetSubObject("nodegroups")
+	result, err := nodegroupListing.CallGet("list", nil)
+	c.Assert(err, IsNil)
+
+	nodegroups, err := result.GetArray()
+	c.Assert(err, IsNil)
+	c.Check(nodegroups, HasLen, 0)
+}
+
+func (suite *TestMAASObjectSuite) TestListNodegroupInterfaces(c *C) {
+	suite.TestMAASObject.TestServer.AddBootImage("uuid-0", `{"architecture": "arm64", "release": "trusty"}`)
+	jsonText := `{
+            "ip_range_high": "172.16.0.128",
+            "ip_range_low": "172.16.0.2",
+            "broadcast_ip": "172.16.0.255",
+            "static_ip_range_low": "172.16.0.129",
+            "name": "eth0",
+            "ip": "172.16.0.2",
+            "subnet_mask": "255.255.255.0",
+            "management": 2,
+            "static_ip_range_high": "172.16.0.255",
+            "interface": "eth0"
+        }`
+
+	suite.TestMAASObject.TestServer.NewNodegroupInterface("uuid-0", jsonText)
+	nodegroupsInterfacesListing := suite.TestMAASObject.GetSubObject("nodegroups").GetSubObject("uuid-0").GetSubObject("interfaces")
+	result, err := nodegroupsInterfacesListing.CallGet("list", nil)
+	c.Assert(err, IsNil)
+
+	nodegroupsInterfaces, err := result.GetArray()
+	c.Assert(err, IsNil)
+	c.Check(nodegroupsInterfaces, HasLen, 1)
+
+	nodegroupsInterface, err := nodegroupsInterfaces[0].GetMap()
+	c.Assert(err, IsNil)
+
+	checkMember := func(member, expectedValue string) {
+		value, err := nodegroupsInterface[member].GetString()
+		c.Assert(err, IsNil)
+		c.Assert(value, Equals, expectedValue)
+	}
+	checkMember("ip_range_high", "172.16.0.128")
+	checkMember("ip_range_low", "172.16.0.2")
+	checkMember("broadcast_ip", "172.16.0.255")
+	checkMember("static_ip_range_low", "172.16.0.129")
+	checkMember("static_ip_range_high", "172.16.0.255")
+	checkMember("name", "eth0")
+	checkMember("ip", "172.16.0.2")
+	checkMember("subnet_mask", "255.255.255.0")
+	checkMember("interface", "eth0")
+
+	value, err := nodegroupsInterface["management"].GetFloat64()
+	c.Assert(err, IsNil)
+	c.Assert(value, Equals, 2.0)
+}
+
+func (suite *TestMAASObjectSuite) TestListNodegroupsInterfacesEmptyList(c *C) {
+	suite.TestMAASObject.TestServer.AddBootImage("uuid-0", `{"architecture": "arm64", "release": "trusty"}`)
+	nodegroupsInterfacesListing := suite.TestMAASObject.GetSubObject("nodegroups").GetSubObject("uuid-0").GetSubObject("interfaces")
+	result, err := nodegroupsInterfacesListing.CallGet("list", nil)
+	c.Assert(err, IsNil)
+
+	interfaces, err := result.GetArray()
+	c.Assert(err, IsNil)
+	c.Check(interfaces, HasLen, 0)
+}
+
 func (suite *TestMAASObjectSuite) TestListBootImages(c *C) {
 	suite.TestMAASObject.TestServer.AddBootImage("uuid-0", `{"architecture": "arm64", "release": "trusty"}`)
 	suite.TestMAASObject.TestServer.AddBootImage("uuid-1", `{"architecture": "amd64", "release": "precise"}`)
@@ -845,3 +1074,131 @@ func (suite *TestMAASObjectSuite) TestListBootImages(c *C) {
 	sort.Strings(bootImages)
 	c.Assert(bootImages, DeepEquals, expectedBootImages)
 }
+
+func (suite *TestMAASObjectSuite) TestListZones(c *C) {
+	expected := map[string]string{
+		"zone0": "zone0 is very nice",
+		"zone1": "zone1 is much nicer than zone0",
+	}
+	for name, desc := range expected {
+		suite.TestMAASObject.TestServer.AddZone(name, desc)
+	}
+
+	result, err := suite.TestMAASObject.GetSubObject("zones").CallGet("", nil)
+	c.Assert(err, IsNil)
+	c.Assert(result, NotNil)
+
+	list, err := result.GetArray()
+	c.Assert(err, IsNil)
+	c.Assert(list, HasLen, len(expected))
+
+	m := make(map[string]string)
+	for _, item := range list {
+		itemMap, err := item.GetMap()
+		c.Assert(err, IsNil)
+		name, err := itemMap["name"].GetString()
+		c.Assert(err, IsNil)
+		desc, err := itemMap["description"].GetString()
+		c.Assert(err, IsNil)
+		m[name] = desc
+	}
+	c.Assert(m, DeepEquals, expected)
+}
+
+func (suite *TestMAASObjectSuite) TestAcquireNodeZone(c *C) {
+	suite.TestMAASObject.TestServer.AddZone("z0", "rox")
+	suite.TestMAASObject.TestServer.AddZone("z1", "sux")
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "n0", "zone": "z0"}`)
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "n1", "zone": "z1"}`)
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "n2", "zone": "z1"}`)
+	nodesObj := suite.TestMAASObject.GetSubObject("nodes")
+
+	acquire := func(zone string) (string, string, error) {
+		var params url.Values
+		if zone != "" {
+			params = url.Values{"zone": []string{zone}}
+		}
+		jsonResponse, err := nodesObj.CallPost("acquire", params)
+		if err != nil {
+			return "", "", err
+		}
+		acquiredNode, err := jsonResponse.GetMAASObject()
+		c.Assert(err, IsNil)
+		systemId, err := acquiredNode.GetField("system_id")
+		c.Assert(err, IsNil)
+		assignedZone, err := acquiredNode.GetField("zone")
+		c.Assert(err, IsNil)
+		if zone != "" {
+			c.Assert(assignedZone, Equals, zone)
+		}
+		return systemId, assignedZone, nil
+	}
+
+	id, _, err := acquire("z0")
+	c.Assert(err, IsNil)
+	c.Assert(id, Equals, "n0")
+	id, _, err = acquire("z0")
+	c.Assert(err.(ServerError).StatusCode, Equals, http.StatusConflict)
+
+	id, zone, err := acquire("")
+	c.Assert(err, IsNil)
+	c.Assert(id, Not(Equals), "n0")
+	c.Assert(zone, Equals, "z1")
+}
+
+func (suite *TestMAASObjectSuite) TestAcquireFilterMemory(c *C) {
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "n0", "memory": 1024}`)
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "n1", "memory": 2048}`)
+	nodeListing := suite.TestMAASObject.GetSubObject("nodes")
+	jsonResponse, err := nodeListing.CallPost("acquire", url.Values{"mem": []string{"2048"}})
+	c.Assert(err, IsNil)
+	acquiredNode, err := jsonResponse.GetMAASObject()
+	c.Assert(err, IsNil)
+	mem, err := acquiredNode.GetMap()["memory"].GetFloat64()
+	c.Assert(err, IsNil)
+	c.Assert(mem, Equals, float64(2048))
+}
+
+func (suite *TestMAASObjectSuite) TestAcquireFilterCpuCores(c *C) {
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "n0", "cpu_count": 1}`)
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "n1", "cpu_count": 2}`)
+	nodeListing := suite.TestMAASObject.GetSubObject("nodes")
+	jsonResponse, err := nodeListing.CallPost("acquire", url.Values{"cpu-cores": []string{"2"}})
+	c.Assert(err, IsNil)
+	acquiredNode, err := jsonResponse.GetMAASObject()
+	c.Assert(err, IsNil)
+	cpucount, err := acquiredNode.GetMap()["cpu_count"].GetFloat64()
+	c.Assert(err, IsNil)
+	c.Assert(cpucount, Equals, float64(2))
+}
+
+func (suite *TestMAASObjectSuite) TestAcquireFilterArch(c *C) {
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "n0", "architecture": "amd64"}`)
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "n1", "architecture": "arm/generic"}`)
+	nodeListing := suite.TestMAASObject.GetSubObject("nodes")
+	jsonResponse, err := nodeListing.CallPost("acquire", url.Values{"arch": []string{"arm"}})
+	c.Assert(err, IsNil)
+	acquiredNode, err := jsonResponse.GetMAASObject()
+	c.Assert(err, IsNil)
+	arch, _ := acquiredNode.GetField("architecture")
+	c.Assert(arch, Equals, "arm/generic")
+}
+
+func (suite *TestMAASObjectSuite) TestDeploymentStatus(c *C) {
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "n0", "status": "6"}`)
+	suite.TestMAASObject.TestServer.NewNode(`{"system_id": "n1", "status": "1"}`)
+	nodes := suite.TestMAASObject.GetSubObject("nodes")
+	jsonResponse, err := nodes.CallGet("deployment_status", url.Values{"nodes": []string{"n0", "n1"}})
+	c.Assert(err, IsNil)
+	deploymentStatus, err := jsonResponse.GetMap()
+	c.Assert(err, IsNil)
+	c.Assert(deploymentStatus, HasLen, 2)
+	expectedStatus := map[string]string{
+		"n0": "Deployed", "n1": "Not in Deployment",
+	}
+	for systemId, status := range expectedStatus {
+		nodeStatus, err := deploymentStatus[systemId].GetString()
+		c.Assert(err, IsNil)
+		c.Assert(nodeStatus, Equals, status)
+	}
+}
