@@ -222,15 +222,13 @@ func (u *UserPass) ServeHTTP(w http.ResponseWriter, r *http.Request) {
 		u.ReturnFailure(w, http.StatusInternalServerError, err.Error())
 		return
 	}
-	if content, err := json.Marshal(res); err != nil {
+	content, err := json.Marshal(res)
+	if err != nil {
 		u.ReturnFailure(w, http.StatusInternalServerError, err.Error())
 		return
-	} else {
-		w.WriteHeader(http.StatusOK)
-		w.Write(content)
-		return
 	}
-	panic("All paths should have already returned")
+	w.WriteHeader(http.StatusOK)
+	w.Write(content)
 }
 
 func (u *UserPass) generateAccessResponse(userInfo *UserInfo) (*AccessResponse, error) {
