@@ -9,6 +9,7 @@ import (
 	"launchpad.net/goose/testservices/identityservice"
 	"net/url"
 	"regexp"
+	"sort"
 	"strings"
 )
 
@@ -19,18 +20,27 @@ var _ identityservice.ServiceProvider = (*Nova)(nil)
 // contains the service double's internal state.
 type Nova struct {
 	testservices.ServiceInstance
-	flavors      map[string]nova.FlavorDetail
-	servers      map[string]nova.ServerDetail
-	groups       map[string]nova.SecurityGroup
-	rules        map[string]nova.SecurityGroupRule
-	floatingIPs  map[string]nova.FloatingIP
-	networks     map[string]nova.Network
-	serverGroups map[string][]string
-	serverIPs    map[string][]string
-	nextServerId int
-	nextGroupId  int
-	nextRuleId   int
-	nextIPId     int
+	flavors           map[string]nova.FlavorDetail
+	servers           map[string]nova.ServerDetail
+	groups            map[string]nova.SecurityGroup
+	rules             map[string]nova.SecurityGroupRule
+	floatingIPs       map[string]nova.FloatingIP
+	networks          map[string]nova.Network
+	serverGroups      map[string][]string
+	serverIPs         map[string][]string
+	availabilityZones map[string]nova.AvailabilityZone
+	nextServerId      int
+	nextGroupId       int
+	nextRuleId        int
+	nextIPId          int
+}
+
+func errorJSONEncode(err error) (int, string) {
+	serverError, ok := err.(*testservices.ServerError)
+	if !ok {
+		serverError = testservices.NewInternalServerError(err.Error())
+	}
+	return serverError.Code(), serverError.AsJSON()
 }
 
 // endpoint returns either a versioned or non-versioned service
@@ -78,14 +88,15 @@ func New(hostURL, versionPath, tenantId, region string, identityService identity
 		{Id: "999", Name: "default", Description: "default group"},
 	}
 	novaService := &Nova{
-		flavors:      make(map[string]nova.FlavorDetail),
-		servers:      make(map[string]nova.ServerDetail),
-		groups:       make(map[string]nova.SecurityGroup),
-		rules:        make(map[string]nova.SecurityGroupRule),
-		floatingIPs:  make(map[string]nova.FloatingIP),
-		networks:     make(map[string]nova.Network),
-		serverGroups: make(map[string][]string),
-		serverIPs:    make(map[string][]string),
+		flavors:           make(map[string]nova.FlavorDetail),
+		servers:           make(map[string]nova.ServerDetail),
+		groups:            make(map[string]nova.SecurityGroup),
+		rules:             make(map[string]nova.SecurityGroupRule),
+		floatingIPs:       make(map[string]nova.FloatingIP),
+		networks:          make(map[string]nova.Network),
+		serverGroups:      make(map[string][]string),
+		serverIPs:         make(map[string][]string),
+		availabilityZones: make(map[string]nova.AvailabilityZone),
 		ServiceInstance: testservices.ServiceInstance{
 			IdentityService: identityService,
 			Scheme:          URL.Scheme,
@@ -122,6 +133,21 @@ func New(hostURL, versionPath, tenantId, region string, identityService identity
 	return novaService
 }
 
+// SetAvailabilityZones sets the availability zones for setting
+// availability zones.
+//
+// Note: this is implemented as a public method rather than as
+// an HTTP API for two reasons: availability zones are created
+// indirectly via "host aggregates", which are a cloud-provider
+// concept that we have not implemented, and because we want to
+// be able to synthesize zone state changes.
+func (n *Nova) SetAvailabilityZones(zones ...nova.AvailabilityZone) {
+	n.availabilityZones = make(map[string]nova.AvailabilityZone)
+	for _, z := range zones {
+		n.availabilityZones[z.Name] = z
+	}
+}
+
 // buildFlavorLinks populates the Links field of the passed
 // FlavorDetail as needed by OpenStack HTTP API. Call this
 // before addFlavor().
@@ -139,7 +165,7 @@ func (n *Nova) addFlavor(flavor nova.FlavorDetail) error {
 		return err
 	}
 	if _, err := n.flavor(flavor.Id); err == nil {
-		return fmt.Errorf("a flavor with id %q already exists", flavor.Id)
+		return testservices.NewAddFlavorError(flavor.Id)
 	}
 	n.flavors[flavor.Id] = flavor
 	return nil
@@ -152,7 +178,7 @@ func (n *Nova) flavor(flavorId string) (*nova.FlavorDetail, error) {
 	}
 	flavor, ok := n.flavors[flavorId]
 	if !ok {
-		return nil, fmt.Errorf("no such flavor %q", flavorId)
+		return nil, testservices.NewNoSuchFlavorError(flavorId)
 	}
 	return &flavor, nil
 }
@@ -224,7 +250,7 @@ func (n *Nova) addServer(server nova.ServerDetail) error {
 		return err
 	}
 	if _, err := n.server(server.Id); err == nil {
-		return fmt.Errorf("a server with id %q already exists", server.Id)
+		return testservices.NewServerAlreadyExistsError(server.Id)
 	}
 	n.servers[server.Id] = server
 	return nil
@@ -237,7 +263,7 @@ func (n *Nova) server(serverId string) (*nova.ServerDetail, error) {
 	}
 	server, ok := n.servers[serverId]
 	if !ok {
-		return nil, fmt.Errorf("no such server %q", serverId)
+		return nil, testservices.NewServerByIDNotFoundError(serverId)
 	}
 	return &server, nil
 }
@@ -252,7 +278,7 @@ func (n *Nova) serverByName(name string) (*nova.ServerDetail, error) {
 			return &server, nil
 		}
 	}
-	return nil, fmt.Errorf("no such server named %q", name)
+	return nil, testservices.NewServerByNameNotFoundError(name)
 }
 
 // serverAsEntity returns the stored ServerDetail as Entity.
@@ -378,7 +404,7 @@ func (n *Nova) addSecurityGroup(group nova.SecurityGroup) error {
 		return err
 	}
 	if _, err := n.securityGroup(group.Id); err == nil {
-		return fmt.Errorf("a security group with id %s already exists", group.Id)
+		return testservices.NewSecurityGroupAlreadyExistsError(group.Id)
 	}
 	group.TenantId = n.TenantId
 	if group.Rules == nil {
@@ -395,7 +421,7 @@ func (n *Nova) securityGroup(groupId string) (*nova.SecurityGroup, error) {
 	}
 	group, ok := n.groups[groupId]
 	if !ok {
-		return nil, fmt.Errorf("no such security group %s", groupId)
+		return nil, testservices.NewSecurityGroupByIDNotFoundError(groupId)
 	}
 	return &group, nil
 }
@@ -410,7 +436,7 @@ func (n *Nova) securityGroupByName(groupName string) (*nova.SecurityGroup, error
 			return &group, nil
 		}
 	}
-	return nil, fmt.Errorf("no such security group named %q", groupName)
+	return nil, testservices.NewSecurityGroupByNameNotFoundError(groupName)
 }
 
 // allSecurityGroups returns a list of all existing groups.
@@ -442,7 +468,7 @@ func (n *Nova) addSecurityGroupRule(ruleId string, rule nova.RuleInfo) error {
 		return err
 	}
 	if _, err := n.securityGroupRule(ruleId); err == nil {
-		return fmt.Errorf("a security group rule with id %s already exists", ruleId)
+		return testservices.NewSecurityGroupRuleAlreadyExistsError(ruleId)
 	}
 	group, err := n.securityGroup(rule.ParentGroupId)
 	if err != nil {
@@ -450,7 +476,7 @@ func (n *Nova) addSecurityGroupRule(ruleId string, rule nova.RuleInfo) error {
 	}
 	for _, ru := range group.Rules {
 		if ru.Id == ruleId {
-			return fmt.Errorf("cannot add twice rule %s to security group %s", ru.Id, group.Id)
+			return testservices.NewCannotAddTwiceRuleToGroupError(ru.Id, group.Id)
 		}
 	}
 	var zeroSecurityGroupRef nova.SecurityGroupRef
@@ -462,12 +488,19 @@ func (n *Nova) addSecurityGroupRule(ruleId string, rule nova.RuleInfo) error {
 	if rule.GroupId != nil {
 		sourceGroup, err := n.securityGroup(*rule.GroupId)
 		if err != nil {
-			return fmt.Errorf("unknown source security group %s", *rule.GroupId)
+			return testservices.NewUnknownSecurityGroupError(*rule.GroupId)
 		}
 		newrule.Group = nova.SecurityGroupRef{
 			TenantId: sourceGroup.TenantId,
 			Name:     sourceGroup.Name,
 		}
+	} else if rule.Cidr == "" {
+		// http://pad.lv/1226996
+		// It seems that if you don't supply Cidr or GroupId then
+		// Openstack treats the Cidr as 0.0.0.0/0
+		// However, since that is not clearly specified we just panic()
+		// because we don't want to rely on that behavior
+		panic(fmt.Sprintf("Neither Cidr nor GroupId are set for this SecurityGroup Rule: %v", rule))
 	}
 	if rule.FromPort != 0 {
 		newrule.FromPort = &rule.FromPort
@@ -504,7 +537,7 @@ func (n *Nova) securityGroupRule(ruleId string) (*nova.SecurityGroupRule, error)
 	}
 	rule, ok := n.rules[ruleId]
 	if !ok {
-		return nil, fmt.Errorf("no such security group rule %s", ruleId)
+		return nil, testservices.NewSecurityGroupRuleNotFoundError(ruleId)
 	}
 	return &rule, nil
 }
@@ -552,7 +585,7 @@ func (n *Nova) addServerSecurityGroup(serverId string, groupId string) error {
 	if ok {
 		for _, gid := range groups {
 			if gid == groupId {
-				return fmt.Errorf("server %q already belongs to group %s", serverId, groupId)
+				return testservices.NewServerBelongsToGroupError(serverId, groupId)
 			}
 		}
 	}
@@ -608,7 +641,7 @@ func (n *Nova) removeServerSecurityGroup(serverId string, groupId string) error
 	}
 	groups, ok := n.serverGroups[serverId]
 	if !ok {
-		return fmt.Errorf("server %q does not belong to any groups", serverId)
+		return testservices.NewServerDoesNotBelongToGroupsError(serverId)
 	}
 	idx := -1
 	for gi, gid := range groups {
@@ -618,7 +651,7 @@ func (n *Nova) removeServerSecurityGroup(serverId string, groupId string) error
 		}
 	}
 	if idx == -1 {
-		return fmt.Errorf("server %q does not belong to group %s", serverId, groupId)
+		return testservices.NewServerDoesNotBelongToGroupError(serverId, groupId)
 	}
 	groups = append(groups[:idx], groups[idx+1:]...)
 	n.serverGroups[serverId] = groups
@@ -631,7 +664,7 @@ func (n *Nova) addFloatingIP(ip nova.FloatingIP) error {
 		return err
 	}
 	if _, err := n.floatingIP(ip.Id); err == nil {
-		return fmt.Errorf("a floating IP with id %s already exists", ip.Id)
+		return testservices.NewFloatingIPExistsError(ip.Id)
 	}
 	n.floatingIPs[ip.Id] = ip
 	return nil
@@ -657,7 +690,7 @@ func (n *Nova) floatingIP(ipId string) (*nova.FloatingIP, error) {
 	}
 	ip, ok := n.floatingIPs[ipId]
 	if !ok {
-		return nil, fmt.Errorf("no such floating IP %s", ipId)
+		return nil, testservices.NewFloatingIPNotFoundError(ipId)
 	}
 	return &ip, nil
 }
@@ -672,7 +705,7 @@ func (n *Nova) floatingIPByAddr(address string) (*nova.FloatingIP, error) {
 			return &fip, nil
 		}
 	}
-	return nil, fmt.Errorf("no such floating IP with address %q", address)
+	return nil, testservices.NewFloatingIPNotFoundError(address)
 }
 
 // allFloatingIPs returns a list of all created floating IPs.
@@ -716,7 +749,7 @@ func (n *Nova) addServerFloatingIP(serverId string, ipId string) error {
 	if ok {
 		for _, fipId := range fips {
 			if fipId == ipId {
-				return fmt.Errorf("server %q already has floating IP %s", serverId, ipId)
+				return testservices.NewServerHasFloatingIPError(serverId, ipId)
 			}
 		}
 	}
@@ -760,7 +793,7 @@ func (n *Nova) removeServerFloatingIP(serverId string, ipId string) error {
 	}
 	fips, ok := n.serverIPs[serverId]
 	if !ok {
-		return fmt.Errorf("server %q does not have any floating IPs to remove", serverId)
+		return testservices.NewNoFloatingIPsToRemoveError(serverId)
 	}
 	idx := -1
 	for fi, fipId := range fips {
@@ -770,7 +803,7 @@ func (n *Nova) removeServerFloatingIP(serverId string, ipId string) error {
 		}
 	}
 	if idx == -1 {
-		return fmt.Errorf("server %q does not have floating IP %s", serverId, ipId)
+		return testservices.NewNoFloatingIPsError(serverId, ipId)
 	}
 	fips = append(fips[:idx], fips[idx+1:]...)
 	n.serverIPs[serverId] = fips
@@ -784,3 +817,27 @@ func (n *Nova) allNetworks() (networks []nova.Network) {
 	}
 	return networks
 }
+
+// allAvailabilityZones returns a list of all existing availability zones,
+// sorted by name.
+func (n *Nova) allAvailabilityZones() (zones []nova.AvailabilityZone) {
+	for _, zone := range n.availabilityZones {
+		zones = append(zones, zone)
+	}
+	sort.Sort(azByName(zones))
+	return zones
+}
+
+type azByName []nova.AvailabilityZone
+
+func (a azByName) Len() int {
+	return len(a)
+}
+
+func (a azByName) Less(i, j int) bool {
+	return a[i].Name < a[j].Name
+}
+
+func (a azByName) Swap(i, j int) {
+	a[i], a[j] = a[j], a[i]
+}
